Package: ggpicrust2
Type: Package
Title: Make 'PICRUSt2' Output Analysis and Visualization Easier
Version: 2.5.17
Author: Chen Yang [aut, cre],
  Liangliang Zhang [aut]
Maintainer: Chen Yang <cafferychen7850@gmail.com>
Authors@R: c(
    person("Chen", "Yang", email = "cafferychen7850@gmail.com", role = c("aut", "cre")),
    person("Liangliang", "Zhang", email = "lxz716@case.edu ", role = "aut"))
Description: Provides a convenient way to analyze and visualize 'PICRUSt2' output with pre-defined plots and functions. Allows for generating statistical plots about microbiome functional predictions and offers customization options. Features a one-click option for creating publication-level plots, saving time and effort in producing professional-grade figures. Streamlines the 'PICRUSt2' analysis and visualization process. For more details, see Yang et al. (2023) <doi:10.1093/bioinformatics/btad470>.
BugReports: https://github.com/cafferychen777/ggpicrust2/issues
URL: https://github.com/cafferychen777/ggpicrust2,
        https://cafferyang.com/ggpicrust2/
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.3
Imports: aplot, dplyr, ggplot2, grid, ggh4x, readr, tibble, tidyr,
        ggprism, patchwork, ggplotify, magrittr, progress, stats,
        methods, grDevices, tidygraph, ggraph, utils
Depends: R (>= 3.5.0)
Suggests: Biobase, ggdendro, KEGGREST, ComplexHeatmap, BiocGenerics,
        knitr, rmarkdown, testthat (>= 3.0.0), ALDEx2, DESeq2, edgeR,
        GGally, limma, MicrobiomeStat, SummarizedExperiment, circlize,
        lefser, Maaslin2, metagenomeSeq, fgsea, clusterProfiler,
        enrichplot, DOSE, ggVennDiagram, UpSetR, igraph, ggridges,
        ggrepel, logging
Config/testthat/edition: 3
biocViews: Microbiome, Metagenomics, Software
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-06-05 16:21:06 UTC; apple
Repository: CRAN
Date/Publication: 2026-06-05 16:50:02 UTC
Built: R 4.6.0; ; 2026-06-05 20:29:56 UTC; unix

# Generated by roxygen2: do not edit by hand

export(aggregate_taxa_contributions)
export(calculate_smart_text_size)
export(compare_daa_results)
export(compare_gsea_daa)
export(compare_metagenome_results)
export(create_gradient_colors)
export(create_legend_theme)
export(create_pathway_class_theme)
export(format_pvalue_smart)
export(get_available_themes)
export(get_color_theme)
export(get_significance_colors)
export(get_significance_stars)
export(ggpicrust2)
export(gsea_pathway_annotation)
export(import_MicrobiomeAnalyst_daa_results)
export(ko2kegg_abundance)
export(pathway_annotation)
export(pathway_daa)
export(pathway_errorbar)
export(pathway_errorbar_table)
export(pathway_gsea)
export(pathway_heatmap)
export(pathway_pca)
export(pathway_ridgeplot)
export(pathway_volcano)
export(prepare_gene_sets)
export(preview_color_theme)
export(read_contrib_file)
export(read_pathway_contrib_file)
export(read_strat_file)
export(resolve_annotation_overlaps)
export(safe_extract)
export(smart_color_selection)
export(taxa_contribution_bar)
export(taxa_contribution_heatmap)
export(visualize_gsea)
import(dplyr)
import(ggplot2)
import(tidyr)
importFrom(aplot,insert_right)
importFrom(aplot,insert_top)
importFrom(dplyr,arrange)
importFrom(dplyr,case_when)
importFrom(dplyr,filter)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,select)
importFrom(dplyr,slice_head)
importFrom(dplyr,slice_max)
importFrom(ggh4x,elem_list_rect)
importFrom(ggh4x,facet_nested)
importFrom(ggh4x,strip_nested)
importFrom(ggplot2,aes)
importFrom(ggplot2,coord_flip)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_line)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_density)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,stat_ellipse)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_classic)
importFrom(ggplot2,theme_minimal)
importFrom(ggplotify,as.ggplot)
importFrom(grDevices,col2rgb)
importFrom(grDevices,colorRampPalette)
importFrom(magrittr,"%>%")
importFrom(methods,new)
importFrom(stats,as.dist)
importFrom(stats,as.formula)
importFrom(stats,cor)
importFrom(stats,dist)
importFrom(stats,hclust)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,p.adjust)
importFrom(stats,prcomp)
importFrom(stats,relevel)
importFrom(stats,reorder)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,var)
importFrom(stats,wilcox.test)
importFrom(tidyr,pivot_longer)
importFrom(utils,data)
importFrom(utils,head)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# ggpicrust2

<img src="man/figures/logo.png" alt="ggpicrust2 logo" align="right" width="200">

🌟 **If you find `ggpicrust2` helpful, please consider giving us a star
on GitHub!** Your support greatly motivates us to improve and maintain
this project. 🌟

*ggpicrust2* is a comprehensive package designed to provide a seamless
and intuitive solution for analyzing and interpreting the results of
PICRUSt2 functional prediction. It offers a wide range of features,
including pathway name/description annotations, advanced differential
abundance (DA) methods, and visualization of DA results.

One of the newest additions to *ggpicrust2* is the capability to compare
the consistency and inconsistency across different DA methods applied to
the same dataset. This feature allows users to assess the agreement and
discrepancy between various methods when it comes to predicting and
sequencing the metagenome of a particular sample. It provides valuable
insights into the consistency of results obtained from different
approaches and helps users evaluate the robustness of their findings.

By leveraging this functionality, researchers, data scientists, and
bioinformaticians can gain a deeper understanding of the underlying
biological processes and mechanisms present in their PICRUSt2 output
data. This comparison of different methods enables them to make informed
decisions and draw reliable conclusions based on the consistency
evaluation of macrogenomic predictions or sequencing results for the
same sample.

If you are interested in exploring and analyzing your PICRUSt2 output
data, *ggpicrust2* is a powerful tool that provides a comprehensive set
of features, including the ability to assess the consistency and
evaluate the performance of different methods applied to the same
dataset.

[![CRAN
version](https://www.r-pkg.org/badges/version/ggpicrust2)](https://CRAN.R-project.org/package=ggpicrust2)
[![Downloads](https://cranlogs.r-pkg.org/badges/grand-total/ggpicrust2)](https://CRAN.R-project.org/package=ggpicrust2)
[![License:
MIT](https://img.shields.io/badge/License-MIT-yellow.svg)](https://opensource.org/license/mit)

## News

🔬 **Enhanced GSEA with Limma Camera/Fry Methods (v2.5.6)**

We’ve significantly enhanced the `pathway_gsea()` function with limma’s
competitive gene set testing methods:

- **`camera` method (new default)**: Accounts for inter-gene
  correlations, providing more reliable p-values than preranked methods
  (Wu et al., 2012)
- **`fry` method**: Fast rotation gene set test, efficient for large
  gene set collections
- **Covariate adjustment**: Both camera and fry support adjustment for
  confounding variables (age, sex, BMI, etc.)
- **Contrast specification**: Support for complex experimental designs
  with multiple contrasts

This addresses concerns in the literature that preranked GSEA methods
can produce “spectacularly wrong p-values” due to not accounting for
inter-gene correlations.

📊 **New Visualization Functions: Volcano Plot & Ridge Plot**

We’ve added two new visualization functions for enhanced analysis and
interpretation:

- **`pathway_volcano()`**: Creates publication-quality volcano plots for
  differential abundance analysis, with smart label placement and
  color-coded significance categories.
- **`pathway_ridgeplot()`**: Creates ridge plots (joy plots) for GSEA
  results interpretation, showing distribution of gene abundances within
  enriched pathways.

🔄 **Updated Reference Databases for Improved Pathway Annotation
(v2.1.4)**

We’ve significantly enhanced the reference databases used for pathway
annotation:

- **EC reference data**: Updated from 3,180 to 8,371 entries (163%
  increase)
- **KO reference data**: Updated from 23,917 to 27,531 unique KO IDs
  (15.4% increase)

These updates provide more comprehensive and accurate pathway
annotations, especially for recently discovered enzymes and KEGG
orthology entries. Users will experience improved coverage and precision
in pathway analysis without needing to change any code.

🌟 **New Feature: Gene Set Enrichment Analysis (GSEA) for PICRUSt2
Data**

We’re excited to announce the addition of GSEA functionality to the
ggpicrust2 package! This powerful new feature allows researchers to
perform Gene Set Enrichment Analysis on PICRUSt2 predicted functional
profiles, offering a more nuanced understanding of functional
differences between conditions.

The new GSEA module includes:

- `pathway_gsea()`: Performs GSEA analysis on PICRUSt2 data
- `visualize_gsea()`: Creates various visualizations including
  enrichment plots, dot plots, network plots, and heatmaps
- `compare_gsea_daa()`: Compares GSEA and differential abundance
  analysis results
- `gsea_pathway_annotation()`: Annotates GSEA results with pathway
  information

These new functions complement our existing differential abundance
analysis tools, providing researchers with multiple approaches to
analyze functional profiles.

🧫 **New Feature: Taxa Contribution Workflow for PICRUSt2 Per-sequence
Outputs**

ggpicrust2 now supports parsing and visualizing PICRUSt2 per-sequence
contribution outputs:

- `read_contrib_file()`: Reads `pred_metagenome_contrib.tsv`
- `read_strat_file()`: Reads `pred_metagenome_strat.tsv`
- `aggregate_taxa_contributions()`: Aggregates contributions by taxon,
  sample, and function
- `taxa_contribution_bar()`: Creates stacked bar plots of taxon-level
  contributions
- `taxa_contribution_heatmap()`: Summarizes contribution patterns across
  taxa and functions

This workflow makes it possible to move from pathway-level significance
to an interpretable answer for which taxa are driving those pathway
shifts.

🌟 **Also Check Out: `mLLMCelltype`**

We’re excited to introduce `mLLMCelltype`, our innovative framework for
**single-cell RNA sequencing data annotation**. This iterative multi-LLM
consensus framework leverages the collective intelligence of multiple
large language models (including GPT-4o/4.1, Claude-3.7/3.5, Gemini-2.0,
Grok-3, and others) to significantly improve cell type annotation
accuracy while providing transparent uncertainty quantification.

`mLLMCelltype` addresses critical challenges in scRNA-seq analysis
through its unique architecture:

- **Multi-LLM Consensus**: Overcomes single-model limitations by
  harnessing diverse LLMs’ collective intelligence
- **Structured Deliberation**: Enables models to share reasoning,
  evaluate evidence, and refine annotations through collaborative
  discussion
- **Transparent Uncertainty Metrics**: Provides quantitative measures to
  identify ambiguous cell populations requiring expert review
- **Hallucination Reduction**: Suppresses inaccurate predictions through
  cross-model critical evaluation
- **No Reference Dataset Required**: Performs accurate annotation
  without pre-training or reference data

For researchers working with single-cell data, `mLLMCelltype` offers a
powerful new approach to cell type annotation. Learn more about its
capabilities and methodology on GitHub: [mLLMCelltype
Repository](https://github.com/cafferychen777/mLLMCelltype).

We appreciate your support and interest in our tools and look forward to
seeing how they can enhance your research.

## Table of Contents

- [Citation](#citation)
- [Installation](#installation)
- [Project Links](#stay-updated)
- [Workflow](#workflow)
- [Output](#output)
- [Function Details](#function-details)
  - [ko2kegg_abundance()](#ko2kegg_abundance)
  - [pathway_daa()](#pathway_daa)
  - [compare_daa_results()](#compare_daa_results)
  - [pathway_annotation()](#pathway_annotation)
  - [pathway_errorbar()](#pathway_errorbar)
  - [pathway_heatmap()](#pathway_heatmap)
  - [pathway_pca()](#pathway_pca)
  - [compare_metagenome_results()](#compare_metagenome_results)
  - [taxa contribution workflow](#taxa_contribution)
  - [pathway_gsea()](#pathway_gsea)
  - [visualize_gsea()](#visualize_gsea)
  - [compare_gsea_daa()](#compare_gsea_daa)
  - [gsea_pathway_annotation()](#gsea_pathway_annotation)
- [FAQ](#faq)
- [Author’s Other Projects](#authors-other-projects)

## Citation

If you use *ggpicrust2* in your research, please cite the following
paper:

Chen Yang and others. (2023). ggpicrust2: an R package for PICRUSt2
predicted functional profile analysis and visualization.
*Bioinformatics*, btad470. [DOI
link](https://doi.org/10.1093/bioinformatics/btad470)

The package citation is also available directly in R:

``` r
citation("ggpicrust2")
```

## Installation

You can install the development version of *ggpicrust2* from GitHub
with:

``` r
# install.packages("devtools")
devtools::install_github("cafferychen777/ggpicrust2")
```

## Dependent CRAN Packages

| Package | Description |
|----|----|
| aplot | Create interactive plots |
| dplyr | A fast consistent tool for working with data frame like objects both in memory and out of memory |
| ggplot2 | An implementation of the Grammar of Graphics in R |
| grid | A rewrite of the graphics layout capabilities of R |
| MicrobiomeStat | Statistical analysis of microbiome data |
| readr | Read rectangular data (csv tsv fwf) into R |
| stats | The R Stats Package |
| tibble | Simple Data Frames |
| tidyr | Easily tidy data with spread() and gather() functions |
| ggprism | Interactive 3D plots with ‘prism’ graphics |
| cowplot | Streamlined Plot Theme and Plot Annotations for ‘ggplot2’ |
| ggforce | Easily add secondary axes, zooms, and image overlays to ‘ggplot2’ |
| ggplotify | Convert complex plots into ‘grob’ or ‘ggplot’ objects |
| magrittr | A Forward-Pipe Operator for R |
| utils | The R Utils Package |

## Optional Bioconductor Packages

The package works with a minimal CRAN installation, but several
workflows rely on Bioconductor packages that should be installed only
when you use the corresponding analysis methods or visualizations.

| Package | Description |
|----|----|
| phyloseq | Handling and analysis of high-throughput microbiome census data |
| ALDEx2 | Differential abundance analysis of taxonomic and functional features |
| SummarizedExperiment | SummarizedExperiment container for storing data and metadata together |
| Biobase | Base functions for Bioconductor |
| devtools | Tools to make developing R packages easier |
| ComplexHeatmap | Making Complex Heatmaps in R |
| BiocGenerics | S4 generic functions for Bioconductor |
| BiocManager | Access the Bioconductor Project Package Repositories |
| metagenomeSeq | Statistical analysis for sparse high-throughput sequencing |
| Maaslin2 | Tools for microbiome analysis |
| edgeR | Empirical Analysis of Digital Gene Expression Data in R |
| lefser | R implementation of the LEfSE method for microbiome biomarker discovery |
| limma | Linear Models for Microarray and RNA-Seq Data |
| KEGGREST | R Interface to KEGG REST API |
| DESeq2 | Differential gene expression analysis using RNA-seq data |

``` r
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

pkgs <- c("phyloseq", "ALDEx2", "SummarizedExperiment", "Biobase", "devtools",
          "ComplexHeatmap", "BiocGenerics", "BiocManager", "metagenomeSeq",
          "Maaslin2", "edgeR", "lefser", "limma", "KEGGREST", "DESeq2")

for (pkg in pkgs) {
  if (!requireNamespace(pkg, quietly = TRUE))
    BiocManager::install(pkg)
}
```

## Project Links

Use the project resources below for stable updates and support:

- Package website: <https://cafferyang.com/ggpicrust2/>
- Source repository: <https://github.com/cafferychen777/ggpicrust2>
- Issue tracker: <https://github.com/cafferychen777/ggpicrust2/issues>
- Release notes: [NEWS.md](NEWS.md)

## Workflow

The easiest way to analyze the PICRUSt2 output is using ggpicrust2()
function. The main pipeline can be run with ggpicrust2() function.

ggpicrust2() integrates ko abundance to kegg pathway abundance
conversion, annotation of pathway, differential abundance (DA) analysis,
part of DA results visualization. When you have trouble running
ggpicrust2(), you can debug it by running a separate function, which
will greatly increase the speed of your analysis and visualization.

<figure>
<img
src="https://raw.githubusercontent.com/cafferychen777/ggpicrust2_paper/main/paper_figure/Workflow.png"
alt="ggpicrust2 workflow diagram" />
<figcaption aria-hidden="true">ggpicrust2 workflow diagram</figcaption>
</figure>

### ggpicrust2()

You can download the example dataset from the provided [Github
link](https://github.com/cafferychen777/ggpicrust2_paper/tree/main/Dataset)
and [Google Drive
link](https://drive.google.com/drive/folders/1on4RKgm9NkaBCykMCCRvVJuEJeNVVqAF?usp=share_link)
or use the dataset included in the package.

``` r
# If you want to analyze the abundance of KEGG pathways instead of KO within the pathway, please set `ko_to_kegg` to TRUE.
# KEGG pathways typically have more descriptive explanations.

library(readr)
library(ggpicrust2)
library(tibble)
library(tidyverse)
library(ggprism)
library(patchwork)

# Load necessary data: abundance data and metadata
abundance_file <- "path/to/your/abundance_file.tsv"
metadata <- read_delim(
    "path/to/your/metadata.txt",
    delim = "\t",
    escape_double = FALSE,
    trim_ws = TRUE
)

# Run ggpicrust2 with input file path
results_file_input <- ggpicrust2(file = abundance_file,
                                 metadata = metadata,
                                 group = "your_group_column", # For example dataset, group = "Environment"
                                 pathway = "KO",
                                 daa_method = "LinDA",
                                 ko_to_kegg = TRUE,
                                 order = "pathway_class",
                                 p_values_bar = TRUE,
                                 x_lab = "pathway_name")

# Run ggpicrust2 with imported data.frame
abundance_data <- read_delim(abundance_file, delim = "\t", col_names = TRUE, trim_ws = TRUE)

# Run ggpicrust2 with input data
results_data_input <- ggpicrust2(data = abundance_data,
                                 metadata = metadata,
                                 group = "your_group_column", # For example dataset, group = "Environment"
                                 pathway = "KO",
                                 daa_method = "LinDA",
                                 ko_to_kegg = TRUE,
                                 order = "pathway_class",
                                 p_values_bar = TRUE,
                                 x_lab = "pathway_name")

# Access the plot and results dataframe for the first DA method
example_plot <- results_file_input[[1]]$plot
example_results <- results_file_input[[1]]$results

# Use the example data in ggpicrust2 package
data(ko_abundance)
data(metadata)
results_file_input <- ggpicrust2(data = ko_abundance,
                                 metadata = metadata,
                                 group = "Environment",
                                 pathway = "KO",
                                 daa_method = "LinDA",
                                 ko_to_kegg = TRUE,
                                 order = "pathway_class",
                                 p_values_bar = TRUE,
                                 x_lab = "pathway_name")

# Analyze the EC or MetaCyc pathway
data(metacyc_abundance)
results_file_input <- ggpicrust2(data = metacyc_abundance,
                                 metadata = metadata,
                                 group = "Environment",
                                 pathway = "MetaCyc",
                                 daa_method = "LinDA",
                                 ko_to_kegg = FALSE,
                                 order = "group",
                                 p_values_bar = TRUE,
                                 x_lab = "description")
results_file_input[[1]]$plot
results_file_input[[1]]$results
```

### If an error occurs with ggpicrust2, please use the following workflow.

``` r
library(readr)
library(ggpicrust2)
library(tibble)
library(tidyverse)
library(ggprism)
library(patchwork)

# If you want to analyze KEGG pathway abundance instead of KO within the pathway, turn ko_to_kegg to TRUE.
# KEGG pathways typically have more explainable descriptions.

# Load metadata as a tibble
# data(metadata)
metadata <- read_delim("path/to/your/metadata.txt", delim = "\t", escape_double = FALSE, trim_ws = TRUE)

# Load KEGG pathway abundance
# data(kegg_abundance)
kegg_abundance <- ko2kegg_abundance("path/to/your/pred_metagenome_unstrat.tsv")

# Perform pathway differential abundance analysis (DAA) using ALDEx2 method.
# Please change group to "your_group_column" if you are not using example dataset.
# From v2.5.14 ALDEx2 results include effect_size, diff_btw, log2_fold_change,
# rab_all, and overlap columns by default (via ALDEx2::aldex.effect()), matching
# the other DAA methods that return log2 fold changes by default. Pass
# include_effect_size = FALSE to skip that step.
daa_results_df <- pathway_daa(abundance = kegg_abundance, metadata = metadata, group = "Environment", daa_method = "ALDEx2", select = NULL, reference = NULL)

# Filter results for ALDEx2_Wilcoxon rank test method
# Please check the unique(daa_results_df$method) and choose one
daa_sub_method_results_df <- daa_results_df[daa_results_df$method == "ALDEx2_Wilcoxon rank test", ]

# Ranking by |log2_fold_change| is generally more biologically informative than
# ranking by p-value, especially for large datasets where small effects can
# reach statistical significance without being biologically meaningful.
top_hits <- daa_sub_method_results_df[order(-abs(daa_sub_method_results_df$log2_fold_change)), ]

# Annotate pathway results using KO to KEGG conversion
daa_annotated_sub_method_results_df <- pathway_annotation(pathway = "KO", daa_results_df = daa_sub_method_results_df, ko_to_kegg = TRUE)

# Generate pathway error bar plot
# Please change Group to metadata$your_group_column if you are not using example dataset
p <- pathway_errorbar(abundance = kegg_abundance, daa_results_df = daa_annotated_sub_method_results_df, Group = metadata$Environment, p_values_threshold = 0.05, order = "pathway_class", select = NULL, ko_to_kegg = TRUE, p_value_bar = TRUE, colors = NULL, x_lab = "pathway_name")

# If you want to analyze EC, MetaCyc, and KO without conversions, turn ko_to_kegg to FALSE.

# Load metadata as a tibble
# data(metadata)
metadata <- read_delim("path/to/your/metadata.txt", delim = "\t", escape_double = FALSE, trim_ws = TRUE)

# Load KO abundance as a data.frame
# data(ko_abundance)
ko_abundance <- read.delim("path/to/your/pred_metagenome_unstrat.tsv")

# Perform pathway DAA using ALDEx2 method
# Please change column_to_rownames() to the feature column if you are not using example dataset
# Please change group to "your_group_column" if you are not using example dataset
# ALDEx2 effect size columns (effect_size, diff_btw, log2_fold_change, rab_all,
# overlap) are included by default; see the section above.
daa_results_df <- pathway_daa(abundance = ko_abundance %>% column_to_rownames("#NAME"), metadata = metadata, group = "Environment", daa_method = "ALDEx2", select = NULL, reference = NULL)

# Filter results for ALDEx2_Wilcoxon rank test method
daa_sub_method_results_df <- daa_results_df[daa_results_df$method == "ALDEx2_Wilcoxon rank test", ]

# Annotate pathway results without KO to KEGG conversion
daa_annotated_sub_method_results_df <- pathway_annotation(pathway = "KO", daa_results_df = daa_sub_method_results_df, ko_to_kegg = FALSE)

# Generate pathway error bar plot
# Please change column_to_rownames() to the feature column
# Please change Group to metadata$your_group_column if you are not using example dataset
p <- pathway_errorbar(abundance = ko_abundance %>% column_to_rownames("#NAME"), daa_results_df = daa_annotated_sub_method_results_df, Group = metadata$Environment, p_values_threshold = 0.05, order = "group",
select = daa_annotated_sub_method_results_df %>% arrange(p_adjust) %>% slice(1:20) %>% dplyr::select(feature) %>% pull(),
ko_to_kegg = FALSE,
p_value_bar = TRUE,
colors = NULL,
x_lab = "description")

# Workflow for MetaCyc Pathway and EC

# Load MetaCyc pathway abundance and metadata
data("metacyc_abundance")
data("metadata")

# Perform pathway DAA using LinDA method
# Please change column_to_rownames() to the feature column if you are not using example dataset
# Please change group to "your_group_column" if you are not using example dataset
metacyc_daa_results_df <- pathway_daa(abundance = metacyc_abundance %>% column_to_rownames("pathway"), metadata = metadata, group = "Environment", daa_method = "LinDA")

# Annotate MetaCyc pathway results without KO to KEGG conversion
metacyc_daa_annotated_results_df <- pathway_annotation(pathway = "MetaCyc", daa_results_df = metacyc_daa_results_df, ko_to_kegg = FALSE)

# Generate pathway error bar plot
# Please change column_to_rownames() to the feature column
# Please change Group to metadata$your_group_column if you are not using example dataset
pathway_errorbar(abundance = metacyc_abundance %>% column_to_rownames("pathway"), daa_results_df = metacyc_daa_annotated_results_df, Group = metadata$Environment, ko_to_kegg = FALSE, p_values_threshold = 0.05, order = "group", select = NULL, p_value_bar = TRUE, colors = NULL, x_lab = "description")

# Generate pathway heatmap
# Please change column_to_rownames() to the feature column if you are not using example dataset
# Please change group to "your_group_column" if you are not using example dataset
feature_with_p_0.05 <- metacyc_daa_results_df %>% filter(p_adjust < 0.05)
pathway_heatmap(abundance = metacyc_abundance %>% filter(pathway %in% feature_with_p_0.05$feature) %>% column_to_rownames("pathway"), metadata = metadata, group = "Environment")

# Generate pathway PCA plot
# Please change column_to_rownames() to the feature column if you are not using example dataset
# Please change group to "your_group_column" if you are not using example dataset
pathway_pca(abundance = metacyc_abundance %>% column_to_rownames("pathway"), metadata = metadata, group = "Environment")

# Run pathway DAA for multiple methods
# Please change column_to_rownames() to the feature column if you are not using example dataset
# Please change group to "your_group_column" if you are not using example dataset
methods <- c("ALDEx2", "DESeq2", "edgeR")
daa_results_list <- lapply(methods, function(method) {
  pathway_daa(abundance = metacyc_abundance %>% column_to_rownames("pathway"), metadata = metadata, group = "Environment", daa_method = method)
})

# Compare results across different methods
comparison_results <- compare_daa_results(daa_results_list = daa_results_list, method_names = c("ALDEx2_Welch's t test", "ALDEx2_Wilcoxon rank test", "DESeq2", "edgeR"))
```

## Output

The typical output of the ggpicrust2 is like this.

<figure>
<img
src="https://cdn.jsdelivr.net/gh/cafferychen777/ggpicrust2_paper@main/paper_figure/figure1.jpg"
alt="ggpicrust2 pathway annotation overview" />
<figcaption aria-hidden="true">ggpicrust2 pathway annotation
overview</figcaption>
</figure>

## function details

### ko2kegg_abundance()

KEGG Orthology(KO) is a classification system developed by the Kyoto
Encyclopedia of Genes and Genomes (KEGG) data-base(Kanehisa et al.,
2022). It uses a hierarchical structure to classify enzymes based on the
reactions they catalyze. To better understand pathways’ role in
different groups and classify the pathways, the KO abundance table needs
to be converted to KEGG pathway abundance. But PICRUSt2 removes the
function from PICRUSt. ko2kegg_abundance() can help convert the table.

``` r
# Sample usage of the ko2kegg_abundance function
devtools::install_github('cafferychen777/ggpicrust2')

library(ggpicrust2)

# Assume that the KO abundance table is stored in a file named "ko_abundance.tsv"
ko_abundance_file <- "ko_abundance.tsv"

# Convert KO abundance to KEGG pathway abundance
kegg_abundance <- ko2kegg_abundance(file = ko_abundance_file)

# Alternatively, if the KO abundance data is already loaded as a data frame named "ko_abundance"
data("ko_abundance")
kegg_abundance <- ko2kegg_abundance(data = ko_abundance)

# The resulting kegg_abundance data frame can now be used for further analysis and visualization.
```

### pathway_daa()

Differential abundance (DA) analysis plays a major role in PICRUSt2
downstream analysis. `pathway_daa()` integrates the main DA methods used
for predicted functional profiles, excluding ANCOM and ANCOMBC. It
includes
[ALDEx2](https://www.bioconductor.org/packages/release/bioc/html/ALDEx2.html)
(Fernandes et al., 2013),
[DESeq2](https://bioconductor.org/packages/release/bioc/html/DESeq2.html)
(Love et al., 2014),
[Maaslin2](https://www.bioconductor.org/packages/release/bioc/html/Maaslin2.html)
(Mallick et al., 2021),
[LinDA](https://doi.org/10.1186/s13059-022-02655-5) (Zhou et al., 2022),
[edgeR](https://bioconductor.org/packages/release/bioc/html/edgeR.html)
(Robinson et al., 2010), [limma
voom](https://ucdavis-bioinformatics-training.github.io/2018-June-RNA-Seq-Workshop/thursday/DE.html)
(Ritchie et al., 2015),
[metagenomeSeq](https://www.bioconductor.org/packages/release/bioc/html/metagenomeSeq.html)
(Paulson et al., 2013), and
[Lefser](https://bioconductor.org/packages/release/bioc/html/lefser.html)
(Segata et al., 2011).

``` r
# The abundance table is recommended to be a data.frame rather than a tibble.
# The abundance table should have feature names or pathway names as row names, and sample names as column names.
# You can use the output of ko2kegg_abundance
ko_abundance_file <- "path/to/your/pred_metagenome_unstrat.tsv"
kegg_abundance <- ko2kegg_abundance(ko_abundance_file) # Or use data(kegg_abundance)

metadata <- read_delim("path/to/your/metadata.txt", delim = "\t", escape_double = FALSE, trim_ws = TRUE)

# The default DAA method is "ALDEx2"
# Please change group to "your_group_column" if you are not using example dataset
daa_results_df <- pathway_daa(abundance = kegg_abundance, metadata = metadata, group = "Environment", daa_method = "LinDA", select = NULL, p_adjust_method = "BH", reference = NULL)

# If you have more than 3 group levels and want to use the LinDA, limma voom, or Maaslin2 methods, you should provide a reference.
metadata <- read_delim("path/to/your/metadata.txt", delim = "\t", escape_double = FALSE, trim_ws = TRUE)

# Please change group to "your_group_column" if you are not using example dataset
daa_results_df <- pathway_daa(abundance = kegg_abundance, metadata = metadata, group = "Group", daa_method = "LinDA", select = NULL, p_adjust_method = "BH", reference = "Harvard BRI")

# Other example
data("metacyc_abundance")
data("metadata")
metacyc_daa_results_df <- pathway_daa(abundance = metacyc_abundance %>% column_to_rownames("pathway"), metadata = metadata, group = "Environment", daa_method = "LinDA", select = NULL, p_adjust_method = "BH", reference = NULL)
```

### compare_daa_results()

``` r
library(ggpicrust2)
library(tidyverse)
data("metacyc_abundance")
data("metadata")

# Run pathway_daa function for multiple methods
# Please change column_to_rownames() to the feature column if you are not using example dataset
# Please change group to "your_group_column" if you are not using example dataset
methods <- c("ALDEx2", "DESeq2", "edgeR")
daa_results_list <- lapply(methods, function(method) {
  pathway_daa(abundance = metacyc_abundance %>% column_to_rownames("pathway"), metadata = metadata, group = "Environment", daa_method = method)
})

method_names <- c("ALDEx2","DESeq2", "edgeR")
# Compare results across different methods
comparison_results <- compare_daa_results(daa_results_list = daa_results_list, method_names = method_names)
```

### pathway_annotation()

**If you are in China and you are using kegg pathway annotation, Please
make sure your internet can break through the firewall.**

**New Feature (v2.1.4)**: The `pathway_annotation()` function now
supports species-specific KEGG pathway annotation through the new
`organism` parameter. You can specify KEGG organism codes (e.g., “hsa”
for human, “eco” for E. coli) to get species-specific pathway
information. If no organism is specified (default), the function
retrieves generic KO information not specific to any organism.

**Note**: When `ko_to_kegg = TRUE`, only pathways with
`p_adjust < p_adjust_threshold` are sent to the KEGG API for annotation.
The `p_adjust_threshold` parameter defaults to 0.05 and can be
customized. When called from `ggpicrust2()`, this threshold is
automatically set to match the `p_values_threshold` parameter for
consistency.

``` r

# Make sure to check if the features in `daa_results_df` correspond to the selected pathway

# Annotate KEGG Pathway
data("kegg_abundance")
data("metadata")
# Please change group to "your_group_column" if you are not using example dataset
daa_results_df <- pathway_daa(abundance = kegg_abundance, metadata = metadata, group = "Environment", daa_method = "LinDA")

# Generic KO to KEGG pathway annotation (not specific to any organism)
daa_annotated_results_df <- pathway_annotation(pathway = "KO", daa_results_df = daa_results_df, ko_to_kegg = TRUE)

# Species-specific KEGG pathway annotation (e.g., for human)
human_annotated_results_df <- pathway_annotation(pathway = "KO", daa_results_df = daa_results_df, ko_to_kegg = TRUE, organism = "hsa")

# Species-specific KEGG pathway annotation (e.g., for E. coli)
ecoli_annotated_results_df <- pathway_annotation(pathway = "KO", daa_results_df = daa_results_df, ko_to_kegg = TRUE, organism = "eco")

# Annotate KO
data("ko_abundance")
data("metadata")
# Please change column_to_rownames() to the feature column if you are not using example dataset
# Please change group to "your_group_column" if you are not using example dataset
daa_results_df <- pathway_daa(abundance = ko_abundance %>% column_to_rownames("#NAME"), metadata = metadata, group = "Environment", daa_method = "LinDA")
daa_annotated_results_df <- pathway_annotation(pathway = "KO", daa_results_df = daa_results_df, ko_to_kegg = FALSE)

# Annotate KEGG
# daa_annotated_results_df <- pathway_annotation(pathway = "EC", daa_results_df = daa_results_df, ko_to_kegg = FALSE)

# Annotate MetaCyc Pathway
data("metacyc_abundance")
data("metadata")
# Please change column_to_rownames() to the feature column if you are not using example dataset
# Please change group to "your_group_column" if you are not using example dataset
metacyc_daa_results_df <- pathway_daa(abundance = metacyc_abundance %>% column_to_rownames("pathway"), metadata = metadata, group = "Environment", daa_method = "LinDA")
metacyc_daa_annotated_results_df <- pathway_annotation(pathway = "MetaCyc", daa_results_df = metacyc_daa_results_df, ko_to_kegg = FALSE)
```

### pathway_errorbar()

``` r
data("ko_abundance")
data("metadata")
kegg_abundance <- ko2kegg_abundance(data = ko_abundance) # Or use data(kegg_abundance)
# Please change group to "your_group_column" if you are not using example dataset
daa_results_df <- pathway_daa(kegg_abundance, metadata = metadata, group = "Environment", daa_method = "LinDA")
daa_annotated_results_df <- pathway_annotation(pathway = "KO", daa_results_df = daa_results_df, ko_to_kegg = TRUE)
# Please change Group to metadata$your_group_column if you are not using example dataset
p <- pathway_errorbar(abundance = kegg_abundance,
           daa_results_df = daa_annotated_results_df,
           Group = metadata$Environment,
           ko_to_kegg = TRUE,
           p_values_threshold = 0.05,
           order = "pathway_class",
           select = NULL,
           p_value_bar = TRUE,
           colors = NULL,
           x_lab = "pathway_name")

# If you want to analysis the EC. MetaCyc. KO without conversions.
data("metacyc_abundance")
data("metadata")
metacyc_daa_results_df <- pathway_daa(abundance = metacyc_abundance %>% column_to_rownames("pathway"), metadata = metadata, group = "Environment", daa_method = "LinDA")
metacyc_daa_annotated_results_df <- pathway_annotation(pathway = "MetaCyc", daa_results_df = metacyc_daa_results_df, ko_to_kegg = FALSE)
p <- pathway_errorbar(abundance = metacyc_abundance %>% column_to_rownames("pathway"),
           daa_results_df = metacyc_daa_annotated_results_df,
           Group = metadata$Environment,
           ko_to_kegg = FALSE,
           p_values_threshold = 0.05,
           order = "group",
           select = NULL,
           p_value_bar = TRUE,
           colors = NULL,
           x_lab = "description")
```

### pathway_heatmap()

In this section, we will demonstrate how to create a pathway heatmap
using the `pathway_heatmap` function in the ggpicrust2 package. This
function visualizes the relative abundance of pathways in different
samples.

Use the fake dataset

``` r
# Create example functional pathway abundance data
abundance_example <- matrix(rnorm(30), nrow = 3, ncol = 10)
colnames(abundance_example) <- paste0("Sample", 1:10)
rownames(abundance_example) <- c("PathwayA", "PathwayB", "PathwayC")

# Create example metadata
# Please change your sample id's column name to sample_name
metadata_example <- data.frame(sample_name = colnames(abundance_example),
                               group = factor(rep(c("Control", "Treatment"), each = 5)))

# Create a heatmap
pathway_heatmap(abundance_example, metadata_example, "group")
```

Use the real dataset

``` r
library(tidyverse)
library(ggh4x)
library(ggpicrust2)
# Load the data
data("metacyc_abundance")

# Load the metadata
data("metadata")

# Perform differential abundance analysis
metacyc_daa_results_df <- pathway_daa(
  abundance = metacyc_abundance %>% column_to_rownames("pathway"),
  metadata = metadata,
  group = "Environment",
  daa_method = "LinDA"
)

# Annotate the results
annotated_metacyc_daa_results_df <- pathway_annotation(
  pathway = "MetaCyc",
  daa_results_df = metacyc_daa_results_df,
  ko_to_kegg = FALSE
)

# Filter features with p < 0.05
feature_with_p_0.05 <- metacyc_daa_results_df %>%
  filter(p_adjust < 0.05)

# Create the heatmap
pathway_heatmap(
  abundance = metacyc_abundance %>%
    right_join(
      annotated_metacyc_daa_results_df %>% select(all_of(c("feature","description"))),
      by = c("pathway" = "feature")
    ) %>%
    filter(pathway %in% feature_with_p_0.05$feature) %>%
    select(-"pathway") %>%
    column_to_rownames("description"),
  metadata = metadata,
  group = "Environment"
)
```

### pathway_pca()

In this section, we will demonstrate how to perform Principal Component
Analysis (PCA) on functional pathway abundance data and create
visualizations of the PCA results using the `pathway_pca` function in
the ggpicrust2 package.

Use the fake dataset

``` r
# Create example functional pathway abundance data
abundance_example <- matrix(rnorm(30), nrow = 3, ncol = 10)
colnames(kegg_abundance_example) <- paste0("Sample", 1:10)
rownames(kegg_abundance_example) <- c("PathwayA", "PathwayB", "PathwayC")

# Create example metadata
metadata_example <- data.frame(sample_name = colnames(kegg_abundance_example),
                                group = factor(rep(c("Control", "Treatment"), each = 5)))
# Perform PCA and create visualizations
pathway_pca(abundance = abundance_example, metadata = metadata_example, "group")
```

Use the real dataset

``` r
# Create example functional pathway abundance data
data("metacyc_abundance")
data("metadata")

pathway_pca(abundance = metacyc_abundance %>% column_to_rownames("pathway"), metadata = metadata, group = "Environment")
```

### compare_metagenome_results()

``` r
library(ComplexHeatmap)
set.seed(123)
# First metagenome
metagenome1 <- abs(matrix(rnorm(1000), nrow = 100, ncol = 10))
rownames(metagenome1) <- paste0("KO", 1:100)
colnames(metagenome1) <- paste0("sample", 1:10)
# Second metagenome
metagenome2 <- abs(matrix(rnorm(1000), nrow = 100, ncol = 10))
rownames(metagenome2) <- paste0("KO", 1:100)
colnames(metagenome2) <- paste0("sample", 1:10)
# Put the metagenomes into a list
metagenomes <- list(metagenome1, metagenome2)
# Define names
names <- c("metagenome1", "metagenome2")
# Call the function
results <- compare_metagenome_results(metagenomes, names)
# Print the correlation matrix
print(results$correlation$cor_matrix)
# Print the p-value matrix
print(results$correlation$p_matrix)
```

### taxa contribution workflow

The taxa contribution workflow connects PICRUSt2 contribution outputs
with downstream pathway interpretation. Use `read_contrib_file()` for
`pred_metagenome_contrib.tsv`, `read_pathway_contrib_file()` for
pathway-level `path_abun_contrib.tsv`, or `read_strat_file()` for
`pred_metagenome_strat.tsv`, then aggregate to the taxonomic level you
want to visualize.

``` r
library(ggpicrust2)

# Parse PICRUSt2 per-sequence contribution output
contrib_data <- read_contrib_file("pred_metagenome_contrib.tsv")

# Or parse pathway-level contribution output without running DA analysis
# PICRUSt2 pathway contribution files are often gzipped and use MetaCyc IDs.
path_contrib_data <- read_pathway_contrib_file("path_abun_contrib.tsv.gz")

# Optional: use pathway-level DAA results to keep only significant pathways
data("kegg_abundance")
data("metadata")

daa_results <- pathway_daa(
  abundance = kegg_abundance,
  metadata = metadata,
  group = "Environment",
  daa_method = "ALDEx2"
)

# Aggregate contributions to genus level and keep top taxa
taxa_contrib <- aggregate_taxa_contributions(
  contrib_data = contrib_data,
  taxonomy = your_taxonomy_table,
  tax_level = "Genus",
  top_n = 10,
  daa_results_df = daa_results
)

# Visualize per-sample contributions
taxa_contribution_bar(
  contrib_agg = taxa_contrib,
  metadata = metadata,
  group = "Environment",
  facet_by = "function"
)

# Summarize mean contribution patterns across taxa and functions
taxa_contribution_heatmap(
  contrib_agg = taxa_contrib,
  n_functions = 20
)

# Pathway-level contribution workflow without DA filtering
path_taxa_contrib <- aggregate_taxa_contributions(
  contrib_data = path_contrib_data,
  taxonomy = your_taxonomy_table,
  tax_level = "Genus",
  top_n = 10
)

pathway_annotation_df <- pathway_annotation(
  data = data.frame(function_id = unique(path_taxa_contrib$function_id)),
  pathway = "MetaCyc"
)

taxa_contribution_heatmap(
  contrib_agg = path_taxa_contrib,
  annotation_data = pathway_annotation_df,
  n_functions = 20
)
```

### pathway_gsea()

The `pathway_gsea()` function performs Gene Set Enrichment Analysis
(GSEA) on PICRUSt2 predicted functional profiles. GSEA is a powerful
method for identifying enriched pathways between different conditions,
offering a more nuanced understanding of functional differences compared
to traditional differential abundance analysis.

**New in v2.5.6**: The function now supports limma’s `camera` and `fry`
methods, which provide more reliable p-values by accounting for
inter-gene correlations (Wu et al., 2012). The `camera` method is now
the default and recommended approach.

| Method | Type | Covariate Support | Description |
|----|----|----|----|
| `camera` (default) | Competitive | ✅ Yes | **Recommended.** Accounts for inter-gene correlations |
| `fry` | Self-contained | ✅ Yes | Fast rotation test, efficient for large gene set collections |
| `fgsea` | Preranked | ❌ No | Fast preranked GSEA (legacy) |
| `clusterProfiler` | Preranked | ❌ No | Traditional GSEA implementation (legacy) |

``` r
library(ggpicrust2)
library(tidyverse)

# Load example data
data("ko_abundance")
data("metadata")

# Perform GSEA analysis with camera (recommended)
gsea_results <- pathway_gsea(
  abundance = ko_abundance %>% column_to_rownames("#NAME"),
  metadata = metadata,
  group = "Environment",
  method = "camera",   # Recommended: accounts for inter-gene correlations
  pathway_type = "KEGG",
  min_size = 5,
  max_size = 500
)

# With covariate adjustment (powerful feature of camera/fry)
gsea_results_adjusted <- pathway_gsea(
  abundance = ko_abundance %>% column_to_rownames("#NAME"),
  metadata = metadata,
  group = "Disease",
  covariates = c("age", "sex"),  # Adjust for confounders
  method = "camera",
  pathway_type = "KEGG"
)

# View the results
head(gsea_results)
```

### visualize_gsea()

The `visualize_gsea()` function creates various visualizations for GSEA
results, including enrichment plots, dot plots, network plots, and
heatmaps.

``` r
library(ggpicrust2)
library(tidyverse)

# Load example data and perform GSEA
data("ko_abundance")
data("metadata")

gsea_results <- pathway_gsea(
  abundance = ko_abundance %>% column_to_rownames("#NAME"),
  metadata = metadata,
  group = "Environment"
)

# Create an enrichment plot for a specific pathway
# Annotate results so plots can use readable pathway names
annotated_results <- gsea_pathway_annotation(
  gsea_results = gsea_results,
  pathway_type = "KEGG"
)

# Create an enrichment-style summary plot
enrichment_plot <- visualize_gsea(
  gsea_results = annotated_results,
  plot_type = "enrichment_plot",
  n_pathways = 10
)

# Create a dot plot showing top enriched pathways
dot_plot <- visualize_gsea(
  gsea_results = annotated_results,
  plot_type = "dotplot",
  n_pathways = 20,  # Show top 20 pathways
  sort_by = "NES"   # Sort by Normalized Enrichment Score
)

# Create a network plot showing pathway relationships
network_plot <- visualize_gsea(
  gsea_results = gsea_results,
  plot_type = "network",
  n_pathways = 15,
  network_params = list(
    similarity_measure = "jaccard",
    similarity_cutoff = 0.2,
    layout = "fruchterman",
    node_color_by = "NES"
  )
)

# Create a heatmap showing pathway gene expression
heatmap_plot <- visualize_gsea(
  gsea_results = gsea_results,
  plot_type = "heatmap",
  abundance = ko_abundance %>% column_to_rownames("#NAME"),
  metadata = metadata,
  group = "Environment",
  n_pathways = 10,
  heatmap_params = list(
    cluster_rows = TRUE,
    cluster_columns = TRUE,
    show_column_names = TRUE,
    show_row_names = FALSE
  )
)
```

### compare_gsea_daa()

The `compare_gsea_daa()` function compares results from GSEA and
differential abundance analysis (DAA) to identify pathways that are
consistently identified by both methods or uniquely identified by each
method.

``` r
library(ggpicrust2)
library(tidyverse)

# Load example data
data("ko_abundance")
data("metadata")

# Prepare pathway-level abundance for DAA so identifiers match GSEA pathway IDs
kegg_pathway_abundance <- ko2kegg_abundance(data = ko_abundance)

# Perform GSEA analysis
gsea_results <- pathway_gsea(
  abundance = ko_abundance %>% column_to_rownames("#NAME"),
  metadata = metadata,
  group = "Environment"
)

# Perform DAA analysis
daa_results <- pathway_daa(
  abundance = kegg_pathway_abundance,
  metadata = metadata,
  group = "Environment",
  daa_method = "ALDEx2"
)

# Compare GSEA and DAA results
comparison <- compare_gsea_daa(
  gsea_results = gsea_results,
  daa_results = daa_results,
  p_threshold = 0.05,
  plot_type = "venn"  # Can be "venn", "upset", or "scatter"
)

# View the comparison plot
comparison$plot

# View the overlapping pathways
head(comparison$results$overlap)
```

### gsea_pathway_annotation()

The `gsea_pathway_annotation()` function annotates GSEA results with
pathway information, including pathway names, descriptions, and
classifications.

``` r
library(ggpicrust2)
library(tidyverse)

# Load example data and perform GSEA
data("ko_abundance")
data("metadata")

gsea_results <- pathway_gsea(
  abundance = ko_abundance %>% column_to_rownames("#NAME"),
  metadata = metadata,
  group = "Environment"
)

# Annotate GSEA results
annotated_results <- gsea_pathway_annotation(
  gsea_results = gsea_results,
  pathway_type = "KEGG"
)

# View the annotated results
head(annotated_results)
```

## FAQ

### Issue 1: pathway_errorbar error

When using `pathway_errorbar` with the following parameters:

``` r
pathway_errorbar(abundance = abundance,
                 daa_results_df = daa_results_df,
                 Group = metadata$Environment,
                 ko_to_kegg = TRUE,
                 p_values_threshold = 0.05,
                 order = "pathway_class",
                 select = NULL,
                 p_value_bar = TRUE,
                 colors = NULL,
                 x_lab = "pathway_name")
```

You may encounter an error:

    Error in `ggplot_add()`:
    ! Can't add `e2` to a <ggplot> object.
    Run `rlang::last_trace()` to see where the error occurred.

Make sure you have the `patchwork` package loaded:

``` r
library(patchwork)
```

### Issue 2: guide_train.prism_offset_minor error

You may encounter an error with `guide_train.prism_offset_minor`:

    Error in guide_train.prism_offset_minor(guide, panel_params[[aesthetic]]) :
      No minor breaks exist, guide_prism_offset_minor needs minor breaks to work

    Error in get(as.character(FUN)，mode = "function"object envir = envir)
    guide_prism_offset_minor' of mode'function' was not found

Ensure that the `ggprism` package is loaded:

``` r
library(ggprism)
```

### Issue 3: SSL certificate problem

When encountering the following error:

    SSL peer certificate or SSH remote key was not OK: [rest.kegg.jp] SSL certificate problem: certificate has expired

If you are in China, make sure your computer network can bypass the
firewall.

### Issue 4: Bad Request (HTTP 400)

When encountering the following error:

    Error in .getUrl(url, .flatFileParser) : Bad Request (HTTP 400).

Please restart R session.

### Issue 5: Error in grid.Call(C_textBounds, as.graphicsAnnot(xlabel),x$x, x$y, :

When encountering the following error:

    Error in grid.Call(C_textBounds, as.graphicsAnnot(xlabel),x$x, x$y, :

Please having some required fonts installed. You can refer to this
[thread](https://stackoverflow.com/questions/71362738/r-error-in-grid-callc-textbounds-as-graphicsannotxlabel-xx-xy-polygo).

### Issue 6: Visualization becomes cluttered when there are more than 30 features of statistical significance.

When faced with this issue, consider the following solutions:

**Solution 1: Utilize the ‘select’ parameter**

The ‘select’ parameter allows you to specify which features you wish to
visualize. Here’s an example of how you can apply this in your code:

    ggpicrust2::pathway_errorbar(
      abundance = kegg_abundance,
      daa_results_df = daa_results_df_annotated,
      Group = metadata$Day,
      p_values_threshold = 0.05,
      order = "pathway_class",
      select = c("ko05340", "ko00564", "ko00680", "ko00562", "ko03030", "ko00561", "ko00440", "ko00250", "ko00740", "ko04940", "ko00010", "ko00195", "ko00760", "ko00920", "ko00311", "ko00310", "ko04146", "ko00600", "ko04141", "ko04142", "ko00604", "ko04260", "ko00909", "ko04973", "ko00510", "ko04974"),
      ko_to_kegg = TRUE,
      p_value_bar = FALSE,
      colors = NULL,
      x_lab = "pathway_name"
    )

**Solution 2: Limit to the Top 20 features**

If there are too many significant features to visualize effectively, you
might consider limiting your visualization to the top 20 features with
the smallest adjusted p-values:

    daa_results_df_annotated <- daa_results_df_annotated[!is.na(daa_results_df_annotated$pathway_name),]

    daa_results_df_annotated$p_adjust <- round(daa_results_df_annotated$p_adjust,5)

    low_p_feature <- daa_results_df_annotated[order(daa_results_df_annotated$p_adjust), ]$feature[1:20]


    p <- ggpicrust2::pathway_errorbar(
      abundance = kegg_abundance,
      daa_results_df = daa_results_df_annotated,
      Group = metadata$Day,
      p_values_threshold = 0.05,
      order = "pathway_class",
      select = low_p_feature,
      ko_to_kegg = TRUE,
      p_value_bar = FALSE,
      colors = NULL,
      x_lab = "pathway_name")

### Issue 7: There are no statistically significant biomarkers

If you are not finding any statistically significant biomarkers in your
analysis, there could be several reasons for this:

1.  **The true difference between your groups is small or
    non-existent.** If the microbial communities or pathways you’re
    comparing are truly similar, then it’s correct and expected that you
    won’t find significant differences.

2.  **Your sample size might be too small to detect the differences.**
    Statistical power, the ability to detect differences if they exist,
    increases with sample size.

3.  **The variation within your groups might be too large.** If there’s
    a lot of variation in microbial communities within a single group,
    it can be hard to detect differences between groups.

Here are a few suggestions:

1.  **Increase your sample size**: If possible, adding more samples to
    your analysis can increase your statistical power, making it easier
    to detect significant differences.

2.  **Decrease intra-group variation**: If there’s a lot of variation
    within your groups, consider whether there are outliers or subgroups
    that are driving this variation. You might need to clean your data,
    or to stratify your analysis to account for these subgroups.

3.  **Change your statistical method or adjust parameters**: Depending
    on the nature of your data and your specific question, different
    statistical methods might be more or less powerful. If you’re
    currently using a parametric test, consider using a non-parametric
    test, or vice versa. Also, consider whether adjusting the parameters
    of your current test might help.

Remember, not finding significant results is also a result and can be
informative, as it might indicate that there are no substantial
differences between the groups you’re studying. It’s important to
interpret your results in the context of your specific study and not to
force statistical significance where there isn’t any.

With these strategies, you should be able to create a more readable and
informative visualization, even when dealing with a large number of
significant features.

## Author’s Other Projects

1.  [MicrobiomeStat](https://www.microbiomestat.wiki/): The
    MicrobiomeStat package is a dedicated R tool for exploring
    longitudinal microbiome data. It also accommodates multi-omics data
    and cross-sectional studies, valuing the collective efforts within
    the community. This tool aims to support researchers through their
    extensive biological inquiries over time, with a spirit of gratitude
    towards the community’s existing resources and a collaborative ethos
    for furthering microbiome research.

If you’re interested in helping to test and develop MicrobiomeStat,
please contact <cafferychen7850@gmail.com>.

2.  [MicrobiomeGallery](https://cafferyyang.shinyapps.io/MicrobiomeGallery/):
    This is a web-based platform currently under development, which aims
    to provide a space for sharing microbiome data visualization code
    and datasets.

<figure>
<img
src="https://raw.githubusercontent.com/cafferychen777/ggpicrust2_paper/main/paper_figure/MicrobiomeGallery_preview.jpg"
alt="Preview of related microbiome visualization tools" />
<figcaption aria-hidden="true">Preview of related microbiome
visualization tools</figcaption>
</figure>

We look forward to sharing more updates as these projects progress.

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)

## ----setup, eval=FALSE--------------------------------------------------------
# # Install ggpicrust2
# if (!requireNamespace("ggpicrust2", quietly = TRUE)) {
#   devtools::install_github("cafferychen777/ggpicrust2")
# }
# 
# # Install required Bioconductor packages
# if (!requireNamespace("BiocManager", quietly = TRUE)) {
#   install.packages("BiocManager")
# }
# 
# BiocManager::install(c("limma", "fgsea", "clusterProfiler", "enrichplot", "DOSE", "pathview"))
# 
# # Load the package
# library(ggpicrust2)
# library(dplyr)
# library(ggplot2)

## ----basic-gsea, eval=FALSE---------------------------------------------------
# # Load example data
# data(ko_abundance)
# data(metadata)
# 
# # Prepare abundance data
# abundance_data <- as.data.frame(ko_abundance)
# rownames(abundance_data) <- abundance_data[, "#NAME"]
# abundance_data <- abundance_data[, -1]
# 
# # Run GSEA analysis with camera (recommended)
# gsea_results <- pathway_gsea(
#   abundance = abundance_data,
#   metadata = metadata,
#   group = "Environment",
#   pathway_type = "KEGG",
#   method = "camera",
#   min_size = 5,
#   max_size = 500,
#   p.adjust = "BH"
# )
# 
# # View the top results
# head(gsea_results)

## ----covariate-gsea, eval=FALSE-----------------------------------------------
# # Example with covariate adjustment
# # Assuming metadata has columns: group, age, sex, BMI
# 
# gsea_results_adjusted <- pathway_gsea(
#   abundance = abundance_data,
#   metadata = metadata,
#   group = "Disease",
#   covariates = c("age", "sex", "BMI"),  # Adjust for these confounders
#   pathway_type = "KEGG",
#   method = "camera"
# )
# 
# # The results now reflect the group effect after adjusting for confounders
# head(gsea_results_adjusted)

## ----fry-gsea, eval=FALSE-----------------------------------------------------
# # Fast rotation gene set test
# gsea_results_fry <- pathway_gsea(
#   abundance = abundance_data,
#   metadata = metadata,
#   group = "Environment",
#   pathway_type = "KEGG",
#   method = "fry",
#   min_size = 5,
#   max_size = 500
# )
# 
# head(gsea_results_fry)

## ----fgsea, eval=FALSE--------------------------------------------------------
# # Note: Preranked methods don't account for inter-gene correlations
# # P-values may be less reliable (Wu et al., 2012)
# gsea_results_fgsea <- pathway_gsea(
#   abundance = abundance_data,
#   metadata = metadata,
#   group = "Environment",
#   pathway_type = "KEGG",
#   method = "fgsea",
#   rank_method = "signal2noise",
#   nperm = 1000,
#   min_size = 10,
#   max_size = 500,
#   p.adjust = "BH",
#   seed = 42
# )
# 
# # View the top results
# head(gsea_results_fgsea)

## ----annotate-gsea, eval=FALSE------------------------------------------------
# # Annotate GSEA results
# annotated_results <- gsea_pathway_annotation(
#   gsea_results = gsea_results,
#   pathway_type = "KEGG"
# )
# 
# # View the annotated results
# head(annotated_results)

## ----pathway-labels, eval=FALSE-----------------------------------------------
# # Option 1: Use raw GSEA results (shows pathway IDs)
# plot_with_ids <- visualize_gsea(
#   gsea_results = gsea_results,
#   plot_type = "barplot",
#   n_pathways = 10
# )
# 
# # Option 2: Use annotated results (automatically shows pathway names)
# plot_with_names <- visualize_gsea(
#   gsea_results = annotated_results,
#   plot_type = "barplot",
#   n_pathways = 10
# )
# 
# # Option 3: Explicitly specify which column to use for labels
# plot_custom_labels <- visualize_gsea(
#   gsea_results = annotated_results,
#   plot_type = "barplot",
#   pathway_label_column = "pathway_name",
#   n_pathways = 10
# )
# 
# # Compare the plots
# plot_with_ids
# plot_with_names
# plot_custom_labels

## ----barplot, eval=FALSE------------------------------------------------------
# # Create a barplot of the top enriched pathways
# barplot <- visualize_gsea(
#   gsea_results = annotated_results,
#   plot_type = "barplot",
#   n_pathways = 20,
#   sort_by = "p.adjust"
# )
# 
# # Display the plot
# barplot

## ----dotplot, eval=FALSE------------------------------------------------------
# # Create a dotplot of the top enriched pathways
# dotplot <- visualize_gsea(
#   gsea_results = annotated_results,
#   plot_type = "dotplot",
#   n_pathways = 20,
#   sort_by = "p.adjust"
# )
# 
# # Display the plot
# dotplot

## ----enrichment-plot, eval=FALSE----------------------------------------------
# # Create an enrichment plot for a specific pathway
# enrichment_plot <- visualize_gsea(
#   gsea_results = annotated_results,
#   plot_type = "enrichment_plot",
#   n_pathways = 10,
#   sort_by = "NES"
# )
# 
# # Display the plot
# enrichment_plot

## ----ridge-plot, eval=FALSE---------------------------------------------------
# # Create a ridge plot for GSEA results
# # Note: Requires ggridges package to be installed
# ridge_plot <- pathway_ridgeplot(
#   gsea_results = gsea_results,
#   abundance = abundance_data,
#   metadata = metadata,
#   group = "Environment",
#   pathway_type = "KEGG",
#   n_pathways = 10,
#   sort_by = "p.adjust",
#   show_direction = TRUE,
#   colors = c("Down" = "#3182bd", "Up" = "#de2d26")
# )
# 
# # Display the plot
# ridge_plot

## ----compare-gsea-daa, eval=FALSE---------------------------------------------
# # Run DAA analysis
# daa_results <- pathway_daa(
#   abundance = abundance_data,
#   metadata = metadata,
#   group = "Environment",
#   daa_method = "ALDEx2"
# )
# 
# # Annotate DAA results
# annotated_daa_results <- pathway_annotation(
#   pathway = "KO",
#   daa_results_df = daa_results,
#   ko_to_kegg = TRUE
# )
# 
# # Compare GSEA and DAA results
# comparison <- compare_gsea_daa(
#   gsea_results = annotated_results,
#   daa_results = annotated_daa_results,
#   plot_type = "venn",
#   p_threshold = 0.05
# )
# 
# # Display the comparison plot
# comparison$plot
# 
# # View the comparison results
# comparison$results

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)

## ----installation, eval = FALSE-----------------------------------------------
# # install.packages("devtools")
# devtools::install_github("cafferychen777/ggpicrust2")
# 
# library(ggpicrust2)
# library(tibble)

## ----one-command, eval = FALSE------------------------------------------------
# data("ko_abundance")
# data("metadata")
# 
# results <- ggpicrust2(
#   data = ko_abundance,
#   metadata = metadata,
#   group = "Environment",
#   pathway = "KO",
#   daa_method = "LinDA",
#   ko_to_kegg = TRUE,
#   order = "pathway_class",
#   p_values_bar = TRUE,
#   x_lab = "pathway_name"
# )
# 
# # Access the main outputs
# results[[1]]$plot
# head(results[[1]]$results)

## ----ko-to-kegg, eval = FALSE-------------------------------------------------
# kegg_pathway_abundance <- ko2kegg_abundance(data = ko_abundance)
# head(kegg_pathway_abundance[, 1:3])

## ----daa, eval = FALSE--------------------------------------------------------
# daa_results <- pathway_daa(
#   abundance = kegg_pathway_abundance,
#   metadata = metadata,
#   group = "Environment",
#   daa_method = "ALDEx2"
# )
# 
# head(daa_results)

## ----annotation, eval = FALSE-------------------------------------------------
# annotated_daa <- pathway_annotation(
#   pathway = "KO",
#   daa_results_df = daa_results,
#   ko_to_kegg = TRUE
# )
# 
# head(annotated_daa)

## ----errorbar, eval = FALSE---------------------------------------------------
# pathway_errorbar(
#   abundance = kegg_pathway_abundance,
#   daa_results_df = annotated_daa,
#   Group = "Environment"
# )

## ----heatmap-pca, eval = FALSE------------------------------------------------
# sig_pathways <- annotated_daa$feature[annotated_daa$p_adjust < 0.05]
# 
# if (length(sig_pathways) > 0) {
#   pathway_heatmap(
#     abundance = kegg_pathway_abundance[sig_pathways, , drop = FALSE],
#     metadata = metadata,
#     group = "Environment"
#   )
# }
# 
# pathway_pca(
#   abundance = kegg_pathway_abundance,
#   metadata = metadata,
#   group = "Environment"
# )

## ----contrib-readers, eval = FALSE--------------------------------------------
# # For pred_metagenome_contrib.tsv
# contrib_data <- read_contrib_file("pred_metagenome_contrib.tsv")
# 
# # For path_abun_contrib.tsv.gz; PICRUSt2 pathway output commonly uses MetaCyc IDs.
# path_contrib_data <- read_pathway_contrib_file("path_abun_contrib.tsv.gz")
# 
# # For pred_metagenome_strat.tsv
# strat_data <- read_strat_file("pred_metagenome_strat.tsv")

## ----contrib-aggregate, eval = FALSE------------------------------------------
# taxa_contrib <- aggregate_taxa_contributions(
#   contrib_data = contrib_data,
#   taxonomy = your_taxonomy_table,
#   tax_level = "Genus",
#   top_n = 10,
#   daa_results_df = daa_results
# )
# 
# head(taxa_contrib)

## ----pathway-contrib-no-daa, eval = FALSE-------------------------------------
# path_taxa_contrib <- aggregate_taxa_contributions(
#   contrib_data = path_contrib_data,
#   taxonomy = your_taxonomy_table,
#   tax_level = "Genus",
#   top_n = 10
# )
# 
# pathway_annotation_df <- pathway_annotation(
#   data = data.frame(function_id = unique(path_taxa_contrib$function_id)),
#   pathway = "MetaCyc"
# )

## ----contrib-plots, eval = FALSE----------------------------------------------
# taxa_contribution_bar(
#   contrib_agg = taxa_contrib,
#   metadata = metadata,
#   group = "Environment",
#   facet_by = "function"
# )
# 
# taxa_contribution_heatmap(
#   contrib_agg = taxa_contrib,
#   n_functions = 20
# )

## ----gsea, eval = FALSE-------------------------------------------------------
# gsea_results <- pathway_gsea(
#   abundance = ko_abundance %>% column_to_rownames("#NAME"),
#   metadata = metadata,
#   group = "Environment",
#   pathway_type = "KEGG",
#   method = "camera"
# )
# 
# annotated_gsea <- gsea_pathway_annotation(
#   gsea_results = gsea_results,
#   pathway_type = "KEGG"
# )
# 
# visualize_gsea(
#   gsea_results = annotated_gsea,
#   plot_type = "barplot",
#   n_pathways = 15
# )

#' DAA Results Dataset
#'
#' This dataset is the result of processing 'kegg_abundance' through the 'LinDA' method in the 'pathway_daa' function.
#' It includes information about the feature, groups compared, p values, and method used.
#'
#' @format A data frame with columns:
#' \describe{
#'   \item{adj_method}{Method used for p-value adjustment.}
#'   \item{feature}{The feature (pathway) being compared.}
#'   \item{group1}{The first group in the comparison.}
#'   \item{group2}{The second group in the comparison.}
#'   \item{method}{The method used for the comparison.}
#'   \item{p_adjust}{The adjusted p-value from the comparison.}
#'   \item{p_values}{The raw p-value from the comparison.}
#' }
#' @source From ggpicrust2 package demonstration.
#' @references Douglas GM, Maffei VJ, Zaneveld J, Yurgel SN, Brown JR, Taylor CM, Huttenhower C, Langille MGI. PICRUSt2 for prediction of metagenome functions. Nat Biotechnol. 2020.
"daa_results_df"

#' Data Utilities for ggpicrust2
#'
#' Internal utility functions for data preprocessing, sample matching,
#' and format standardization.
#'
#' @name data_utils
#' @keywords internal
NULL

# =============================================================================
# File I/O
# =============================================================================

#' Read abundance data from a delimited file
#'
#' Reads PICRUSt2-style abundance data from .tsv, .txt, or .csv files.
#' Delimiter is auto-detected from the file extension (csv -> comma, others -> tab).
#'
#' @param file_path Path to the input file
#' @return A data frame (tibble) with the parsed abundance data
#' @noRd
read_abundance_file <- function(file_path) {
  if (!is.character(file_path) || length(file_path) != 1) {
    stop("'file_path' must be a single file path string")
  }
  if (!file.exists(file_path)) {
    stop("File does not exist: ", file_path)
  }

  file_ext <- tolower(tools::file_ext(file_path))
  if (!file_ext %in% c("txt", "tsv", "csv")) {
    stop(
      "Unsupported file format '.", file_ext, "'. ",
      "Accepted formats: .tsv, .txt, .csv"
    )
  }

  delimiter <- if (file_ext == "csv") "," else "\t"

  abundance <- readr::read_delim(
    file_path,
    delim = delimiter,
    show_col_types = FALSE,
    progress = FALSE
  )

  if (ncol(abundance) < 2) {
    stop("Input file must contain at least 2 columns (IDs + samples)")
  }

  abundance
}

# =============================================================================
# KO Abundance Cleaning
# =============================================================================

#' Clean KO Abundance Data
#'
#' Standardizes KO ID format by removing common prefixes (e.g., "ko:" from
#' PICRUSt 2.6.2 format). This function is idempotent - calling it multiple
#' times produces the same result.
#'
#' @param abundance Data frame with KO IDs in first column
#' @param verbose Logical, whether to print messages (default: TRUE)
#' @return Data frame with cleaned KO IDs
#' @noRd
clean_ko_abundance <- function(abundance, verbose = TRUE) {
  if (!is.data.frame(abundance) || ncol(abundance) < 2) {
    return(abundance)
  }

  ko_ids <- abundance[[1]]
  original_ids <- ko_ids

  # Remove "ko:" prefix (PICRUSt 2.6.2 format)
  ko_ids <- gsub("^ko:", "", ko_ids)

  # Count changes
  n_cleaned <- sum(original_ids != ko_ids)

  if (n_cleaned > 0 && verbose) {
    message(sprintf("Standardized %d KO IDs (removed 'ko:' prefix)", n_cleaned))
  }

  abundance[[1]] <- ko_ids
  abundance
}

# =============================================================================
# Sample Matching Utilities
# =============================================================================
#' Check if Two Sample Vectors Have Sufficient Overlap
#'
#' @param vec1 First vector of sample identifiers
#' @param vec2 Second vector of sample identifiers
#' @param threshold Minimum proportion of overlap required (default: 0.5)
#' @return Logical indicating whether vectors have sufficient overlap
#' @noRd
samples_match <- function(vec1, vec2, threshold = 0.5) {
  vec1 <- as.character(vec1)
  vec2 <- as.character(vec2)
  n_common <- length(intersect(vec1, vec2))
  min_length <- min(length(vec1), length(vec2))

  if (min_length == 0) return(FALSE)
  n_common / min_length >= threshold
}

#' Find Sample Column in Metadata
#'
#' Identifies which column in metadata contains sample identifiers that match
#' the abundance data columns. Searches in priority order: standard names,
#' then all columns, then rownames.
#'
#' @param metadata Data frame containing sample metadata
#' @param abundance_samples Character vector of sample names from abundance data
#' @return Column name if found, or ".rownames" if rownames match, or NULL
#' @noRd
find_sample_column <- function(metadata, abundance_samples) {
  abundance_samples <- as.character(abundance_samples)

  # Priority 1: Standard column names (case variations)
  standard_names <- c(
    "sample", "Sample", "SAMPLE",
    "sample_name", "Sample_Name", "SampleName", "samplename",
    "sample_id", "Sample_ID", "SampleID", "sampleid",
    "samples", "Samples"
  )

  for (col in standard_names) {
    if (col %in% colnames(metadata)) {
      if (samples_match(metadata[[col]], abundance_samples)) {
        return(col)
      }
    }
  }

  # Priority 2: Any column that matches
  for (col in colnames(metadata)) {
    if (samples_match(metadata[[col]], abundance_samples)) {
      return(col)
    }
  }

  # Priority 3: Rownames
  if (samples_match(rownames(metadata), abundance_samples)) {
    return(".rownames")
  }

  NULL
}

#' Align Abundance Data and Metadata by Samples
#'
#' Ensures abundance columns and metadata rows are in the same sample order.
#' Handles sample column detection, validation, and reordering in one step.
#'
#' @param abundance Data frame or matrix with samples as columns (and features as rows)
#' @param metadata Data frame with sample metadata
#' @param sample_col Column name containing sample IDs (auto-detected if NULL)
#' @param verbose Logical, whether to print messages (default: TRUE)
#' @return List containing:
#'   \item{abundance}{Aligned abundance data}
#'   \item{metadata}{Aligned metadata}
#'   \item{sample_col}{Name of the sample column used}
#'   \item{n_samples}{Number of aligned samples}
#' @noRd
align_samples <- function(abundance, metadata, sample_col = NULL, verbose = TRUE) {
  # Get abundance sample names
  abundance_samples <- colnames(abundance)

  if (length(abundance_samples) == 0) {
    stop("Abundance data has no column names (sample identifiers)")
  }

  # Find sample column if not provided
  if (is.null(sample_col)) {
    sample_col <- find_sample_column(metadata, abundance_samples)

    if (is.null(sample_col)) {
      stop(
        "Cannot find matching sample identifiers between abundance and metadata.\n",
        "  Abundance samples (first 5): ",
        paste(head(abundance_samples, 5), collapse = ", "),
        if (length(abundance_samples) > 5) "..." else "", "\n",
        "  Metadata columns: ", paste(colnames(metadata), collapse = ", "), "\n",
        "  Metadata rownames (first 5): ",
        paste(head(rownames(metadata), 5), collapse = ", "),
        if (nrow(metadata) > 5) "..." else ""
      )
    }

    if (verbose) {
      if (sample_col == ".rownames") {
        message("Using metadata rownames as sample identifiers")
      } else {
        message(sprintf("Using column '%s' as sample identifier", sample_col))
      }
    }
  }

  # Handle rownames case
  if (sample_col == ".rownames") {
    metadata <- as.data.frame(metadata)
    metadata$.sample_id <- rownames(metadata)
    sample_col <- ".sample_id"
  }

  # Validate sample_col exists
 if (!sample_col %in% colnames(metadata)) {
    stop(sprintf("Sample column '%s' not found in metadata", sample_col))
  }

  metadata_samples <- as.character(metadata[[sample_col]])

  # Find common samples
  common_samples <- intersect(abundance_samples, metadata_samples)

  if (length(common_samples) == 0) {
    stop(
      "No common samples found between abundance and metadata.\n",
      "  Abundance samples: ", paste(head(abundance_samples, 3), collapse = ", "), "...\n",
      "  Metadata samples: ", paste(head(metadata_samples, 3), collapse = ", "), "..."
    )
  }

  # Report mismatches if any
  missing_in_abundance <- setdiff(metadata_samples, abundance_samples)
  missing_in_metadata <- setdiff(abundance_samples, metadata_samples)

  if (verbose && (length(missing_in_abundance) > 0 || length(missing_in_metadata) > 0)) {
    message(sprintf(
      "Aligned %d samples (dropped %d from metadata, %d from abundance)",
      length(common_samples),
      length(missing_in_abundance),
      length(missing_in_metadata)
    ))
  }

  # Align data: subset and reorder to common samples
  abundance_aligned <- abundance[, common_samples, drop = FALSE]
  row_idx <- match(common_samples, metadata_samples)
  metadata_aligned <- as.data.frame(metadata[row_idx, , drop = FALSE])

  # Set rownames to sample names for downstream compatibility
  rownames(metadata_aligned) <- common_samples

  # Verify alignment
  if (!identical(colnames(abundance_aligned), as.character(metadata_aligned[[sample_col]]))) {
    stop("Internal error: sample alignment failed")
  }

  list(
    abundance = abundance_aligned,
    metadata = metadata_aligned,
    sample_col = sample_col,
    n_samples = length(common_samples)
  )
}

# =============================================================================
# Reference Data Loading
# =============================================================================

#' Load Reference Data from Package
#'
#' Unified function to load reference data from the package's data/ directory.
#' All reference data is lazy-loaded, so this function simply retrieves it
#' from the package namespace.
#'
#' @param ref_type Character string specifying the reference type:
#'   \itemize{
#'     \item "KO": KO annotation reference (id, description, pathway info)
#'     \item "EC": EC annotation reference (id, description)
#'     \item "MetaCyc": MetaCyc pathway reference (id, description)
#'     \item "KEGG": KEGG pathway reference (pathway, pathway_name)
#'     \item "ko_to_kegg": KO to KEGG pathway mapping
#'     \item "ko_to_go": KO to GO term mapping
#'     \item "metacyc_to_ec": MetaCyc to EC mapping
#'   }
#' @return Data frame with reference data
#' @noRd
load_reference_data <- function(ref_type) {
  # Mapping from user-friendly names to actual data object names

  ref_map <- list(
    "KO" = "ko_reference",
    "EC" = "ec_reference",
    "MetaCyc" = "metacyc_reference",
    "KEGG" = "kegg_pathway_reference",
    "ko_to_kegg" = "ko_to_kegg_reference",
    "ko_to_go" = "ko_to_go_reference",
    "metacyc_to_ec" = "metacyc_to_ec_reference"
  )

  ref_name <- ref_map[[ref_type]]
  if (is.null(ref_name)) {
    stop(sprintf(
      "Unknown reference type: '%s'. Valid options: %s",
      ref_type, paste(names(ref_map), collapse = ", ")
    ))
  }

  # Get from package namespace (lazy-loaded data)
  pkg_ns <- asNamespace("ggpicrust2")
  if (exists(ref_name, envir = pkg_ns)) {
    return(as.data.frame(get(ref_name, envir = pkg_ns)))
  }

  # Fallback: try using data() function

  env <- new.env()
 tryCatch({
    data(list = ref_name, package = "ggpicrust2", envir = env)
    if (exists(ref_name, envir = env)) {
      return(as.data.frame(get(ref_name, envir = env)))
    }
  }, error = function(e) NULL)

  stop(sprintf(
    "Reference data '%s' not found. Please reinstall the package.",
    ref_name
  ))
}

# =============================================================================
# Statistical Calculation Utilities
# =============================================================================

#' Calculate data-driven pseudocount for log transformation
#'
#' Calculates a pseudocount based on the data to avoid log(0) issues.
#' Uses half of the minimum non-zero value to ensure the pseudocount
#' is smaller than any real value in the data.
#'
#' @param values Numeric vector of abundance values
#' @return Pseudocount value (half of minimum non-zero value, or 1e-6 fallback)
#' @keywords internal
calculate_pseudocount <- function(values) {
  values <- as.numeric(values)
  non_zero <- values[values > 0 & !is.na(values) & !is.infinite(values)]
  if (length(non_zero) > 0) {
    min(non_zero) * 0.5
  } else {
    1e-6  # fallback for edge cases
 }
}

#' Calculate log2 fold change with consistent pseudocount handling
#'
#' Calculates log2 fold change between two groups with proper handling
#' of zero values using a data-driven pseudocount approach.
#'
#' @param mean1 Numeric. Mean abundance of group 1 (reference/control)
#' @param mean2 Numeric. Mean abundance of group 2 (comparison/treatment)
#' @param pseudocount Optional numeric. If NULL, calculated from reference_values
#' @param reference_values Optional numeric vector for calculating pseudocount
#' @return log2(mean2/mean1) with pseudocount protection
#' @keywords internal
#'
#' @details
#' The fold change direction is group2/group1, so:
#' - Positive values indicate higher abundance in group2
#' - Negative values indicate higher abundance in group1
calculate_log2_fold_change <- function(mean1, mean2, pseudocount = NULL,
                                        reference_values = NULL) {
  # Calculate pseudocount if not provided
  if (is.null(pseudocount)) {
    if (!is.null(reference_values)) {
      pseudocount <- calculate_pseudocount(reference_values)
    } else {
      # Use the means themselves to estimate pseudocount
      pseudocount <- calculate_pseudocount(c(mean1, mean2))
    }
  }

  # Calculate log2 fold change: group2 / group1
  log2((mean2 + pseudocount) / (mean1 + pseudocount))
}

# =============================================================================
# Input Validation Utilities
# =============================================================================

#' Validate Abundance Data
#'
#' Validates that abundance data is in the correct format for analysis.
#'
#' @param abundance Data frame or matrix to validate
#' @param min_samples Minimum number of samples required (default: 2)
#' @param require_numeric Whether all non-ID columns must be numeric (default: TRUE)
#' @return TRUE if valid, otherwise stops with error
#' @noRd
validate_abundance <- function(abundance, min_samples = 2, require_numeric = TRUE) {
  if (!is.data.frame(abundance) && !is.matrix(abundance)) {
    stop("'abundance' must be a data frame or matrix")
  }

  if (ncol(abundance) < min_samples) {
    stop(sprintf("At least %d samples are required, found %d", min_samples, ncol(abundance)))
  }

  if (require_numeric && is.data.frame(abundance)) {
    # Check if numeric columns exist (excluding potential ID column)
    numeric_cols <- sapply(abundance, is.numeric)
    if (sum(numeric_cols) == 0) {
      stop("Abundance data must contain numeric values")
    }
  }

  TRUE
}

#' Validate Metadata
#'
#' Validates that metadata is in the correct format.
#'
#' @param metadata Data frame to validate
#' @param required_cols Character vector of required column names (optional)
#' @return TRUE if valid, otherwise stops with error
#' @noRd
validate_metadata <- function(metadata, required_cols = NULL) {
  if (!is.data.frame(metadata)) {
    stop("'metadata' must be a data frame")
  }

  if (nrow(metadata) == 0) {
    stop("'metadata' cannot be empty")
  }

  if (!is.null(required_cols)) {
    missing_cols <- setdiff(required_cols, colnames(metadata))
    if (length(missing_cols) > 0) {
      stop(sprintf("Required column(s) not found in metadata: %s",
                   paste(missing_cols, collapse = ", ")))
    }
  }

  TRUE
}

#' Validate Group Column
#'
#' Validates that a group column exists and has appropriate values.
#'
#' @param metadata Data frame containing group column
#' @param group Column name for grouping variable
#' @param min_groups Minimum number of groups required (default: 2)
#' @param min_per_group Minimum samples per group (default: 1)
#' @return TRUE if valid, otherwise stops with error
#' @noRd
validate_group <- function(metadata, group, min_groups = 2, min_per_group = 1) {
  if (!group %in% colnames(metadata)) {
    stop(sprintf("Group column '%s' not found in metadata. Available columns: %s",
                 group, paste(colnames(metadata), collapse = ", ")))
  }

  group_values <- metadata[[group]]
  group_levels <- unique(group_values[!is.na(group_values)])

  if (length(group_levels) < min_groups) {
    stop(sprintf("At least %d groups are required, found %d: %s",
                 min_groups, length(group_levels),
                 paste(group_levels, collapse = ", ")))
  }

  if (min_per_group > 1) {
    group_counts <- table(group_values)
    small_groups <- names(group_counts)[group_counts < min_per_group]
    if (length(small_groups) > 0) {
      stop(sprintf("Groups with fewer than %d samples: %s",
                   min_per_group, paste(small_groups, collapse = ", ")))
    }
  }

  TRUE
}

# =============================================================================
# Extended Validation Utilities
# =============================================================================

#' Validate Numeric Matrix Quality
#'
#' Deep validation of numeric matrix including negative values, NA, and duplicates.
#'
#' @param mat Numeric matrix to validate
#' @param check_negative Check for negative values (default: TRUE)
#' @param check_na Check for NA values (default: TRUE, issues warning)
#' @param check_duplicates Check for duplicate column names (default: TRUE)
#' @return TRUE if valid, otherwise stops with error
#' @noRd
validate_numeric_matrix <- function(mat, check_negative = TRUE,
                                    check_na = TRUE, check_duplicates = TRUE) {
  if (!is.numeric(mat)) {
    stop("Abundance data contains non-numeric values. All sample columns must be numeric.")
  }

  if (check_negative) {
    neg_cols <- apply(mat, 2, function(x) any(x < 0, na.rm = TRUE))
    if (any(neg_cols)) {
      stop(sprintf("Negative values found in columns: %s",
                   paste(colnames(mat)[neg_cols], collapse = ", ")))
    }
  }

  if (check_na) {
    na_count <- sum(is.na(mat))
    if (na_count > 0) {
      warning(sprintf("Found %d NA values in data", na_count))
    }
  }

  if (check_duplicates && !is.null(colnames(mat))) {
    if (anyDuplicated(colnames(mat))) {
      stop("Duplicate column names found in data")
    }
  }

  TRUE
}

#' Validate Zero Abundance Issues
#'
#' Checks for empty data, all-zero data, and excessive zero rows.
#' Returns information about zero rows for downstream filtering.
#'
#' @param mat Numeric matrix (features x samples)
#' @param warn_threshold Proportion of zero rows to trigger warning (default: 0.9)
#' @return List with: valid (logical), zero_rows (logical vector), n_nonzero (integer)
#' @noRd
validate_zero_abundance <- function(mat, warn_threshold = 0.9) {
  # Check dimensions

  if (nrow(mat) == 0) {
    stop("No features in abundance data")
  }
  if (ncol(mat) == 0) {
    stop("No samples in abundance data")
  }

  # Check all-zero
  total <- sum(mat, na.rm = TRUE)
  if (total == 0) {
    stop("All abundance values are zero")
  }

  # Identify zero rows

  zero_rows <- rowSums(mat, na.rm = TRUE) == 0
  n_nonzero <- sum(!zero_rows)
  zero_prop <- mean(zero_rows)

  if (zero_prop > warn_threshold) {
    warning(sprintf("%.0f%% of features have zero abundance across all samples",
                    zero_prop * 100))
  }

  list(valid = TRUE, zero_rows = zero_rows, n_nonzero = n_nonzero)
}

#' Validate Feature ID Format
#'
#' Checks if feature IDs match expected format patterns.
#'
#' @param ids Character vector of feature IDs
#' @param type Expected ID type: "KO", "EC", "pathway", or "auto" (default: "auto")
#' @return TRUE (issues warning if format doesn't match)
#' @noRd
validate_feature_ids <- function(ids, type = "auto") {
  patterns <- list(
    KO = "^K\\d{5}$",
    EC = "^(EC:)?\\d+\\.\\d+\\.\\d+\\.(\\d+|-)$",
    pathway = "^(ko|map|ec)?\\d{5}$"
  )

  # Auto-detect type
  if (type == "auto") {
    for (t in names(patterns)) {
      if (mean(grepl(patterns[[t]], ids)) > 0.5) {
        type <- t
        break
      }
    }
    if (type == "auto") return(TRUE)  # Unknown format, skip validation
  }

  if (type %in% names(patterns)) {
    invalid <- ids[!grepl(patterns[[type]], ids)]
    if (length(invalid) > 0 && length(invalid) < length(ids)) {
      warning(sprintf("%d %s IDs don't match expected format (e.g., %s)",
                      length(invalid), type, paste(head(invalid, 2), collapse = ", ")))
    }
  }

  TRUE
}

#' Validate Input for DAA Analysis
#'
#' Comprehensive validation for differential abundance analysis input.
#' Combines zero-abundance checks with method-specific requirements.
#'
#' @param mat Numeric matrix (features x samples)
#' @param method DAA method name (optional, for method-specific checks)
#' @param min_features Minimum required non-zero features (default: 2)
#' @param filter_zero Whether to return filtered matrix (default: FALSE)
#' @return If filter_zero=FALSE: TRUE. If filter_zero=TRUE: filtered matrix.
#' @noRd
validate_daa_input <- function(mat, method = NULL, min_features = 2, filter_zero = FALSE) {
  # Basic zero checks
  result <- validate_zero_abundance(mat)

  # Method-specific requirements
  if (!is.null(method)) {
    if (method == "ALDEx2" && result$n_nonzero < 2) {
      stop(sprintf("ALDEx2 requires at least 2 non-zero features (found %d)", result$n_nonzero))
    }
    if (method == "LinDA" && result$n_nonzero < 10) {
      warning(sprintf("Only %d non-zero features for LinDA; results may be unreliable", result$n_nonzero))
    }
  }

  # General minimum check
  if (result$n_nonzero < min_features) {
    stop(sprintf("At least %d non-zero features required (found %d)", min_features, result$n_nonzero))
  }

  if (filter_zero) {
    return(mat[!result$zero_rows, , drop = FALSE])
  }

  TRUE
}

#' Require Package with Consistent Error Message
#'
#' Checks if a package is available and provides consistent installation instructions.
#'
#' @param pkg Package name
#' @param purpose Optional description of why the package is needed
#' @param bioc Whether it's a Bioconductor package (default: TRUE)
#' @return TRUE if available, otherwise stops with error
#' @noRd
require_package <- function(pkg, purpose = NULL, bioc = TRUE) {
  if (!requireNamespace(pkg, quietly = TRUE)) {
    install_cmd <- if (bioc) {
      sprintf("BiocManager::install('%s')", pkg)
    } else {
      sprintf("install.packages('%s')", pkg)
    }
    msg <- sprintf("Package '%s' is required", pkg)
    if (!is.null(purpose)) {
      msg <- sprintf("%s for %s", msg, purpose)
    }
    stop(sprintf("%s. Install with: %s", msg, install_cmd))
  }
  TRUE
}

#' Validate Choice Parameter
#'
#' Validates that a value is one of the allowed choices.
#'
#' @param value The value to validate
#' @param choices Vector of allowed choices
#' @param param_name Name of the parameter (for error message)
#' @return TRUE if valid, otherwise stops with error
#' @noRd
validate_choice <- function(value, choices, param_name = "value") {
  if (!value %in% choices) {
    stop(sprintf("%s must be one of: %s", param_name, paste(choices, collapse = ", ")))
  }
  TRUE
}

#' Validate Data Frame Input
#'
#' Validates that input is a data frame with required columns.
#'
#' @param df The data frame to validate
#' @param required_cols Character vector of required column names (optional)
#' @param param_name Name of the parameter (for error message)
#' @return TRUE if valid, otherwise stops with error
#' @noRd
validate_dataframe <- function(df, required_cols = NULL, param_name = "data") {
  if (!is.data.frame(df)) {
    stop(sprintf("'%s' must be a data frame", param_name))
  }
  if (!is.null(required_cols)) {
    missing <- setdiff(required_cols, colnames(df))
    if (length(missing) > 0) {
      stop(sprintf("Missing required columns in %s: %s", param_name, paste(missing, collapse = ", ")))
    }
  }
  TRUE
}

# =============================================================================
# Empty Result Frame Factories
# =============================================================================

#' Create Empty DAA Result Data Frame
#'
#' Creates an empty data frame with the correct structure for DAA results.
#'
#' @param method Character string specifying the DAA method used
#' @param include_log2fc Logical. If TRUE, includes log2_fold_change column
#'   instead of p_adjust and adj_method (for LinDA-style results)
#' @return Empty data frame with standard DAA result columns
#' @noRd
create_empty_daa_result <- function(method = "unknown", include_log2fc = FALSE) {
  if (include_log2fc) {
    data.frame(
      feature = character(0),
      method = character(0),
      group1 = character(0),
      group2 = character(0),
      p_values = numeric(0),
      log2_fold_change = numeric(0),
      stringsAsFactors = FALSE
    )
  } else {
    data.frame(
      feature = character(0),
      method = character(0),
      group1 = character(0),
      group2 = character(0),
      p_values = numeric(0),
      p_adjust = numeric(0),
      adj_method = character(0),
      stringsAsFactors = FALSE
    )
  }
}

#' Create Empty GSEA Result Data Frame
#'
#' Creates an empty data frame with the correct structure for GSEA results.
#'
#' @param method Character string specifying the GSEA method used
#' @param full Logical. If TRUE, includes ES, NES, leading_edge columns
#' @return Empty data frame with standard GSEA result columns
#' @noRd
create_empty_gsea_result <- function(method = "unknown", full = FALSE) {
  if (full) {
    data.frame(
      pathway_id = character(0),
      pathway_name = character(0),
      size = integer(0),
      ES = numeric(0),
      NES = numeric(0),
      pvalue = numeric(0),
      p.adjust = numeric(0),
      leading_edge = character(0),
      method = character(0),
      stringsAsFactors = FALSE
    )
  } else {
    data.frame(
      pathway_id = character(0),
      pathway_name = character(0),
      size = integer(0),
      direction = character(0),
      pvalue = numeric(0),
      p.adjust = numeric(0),
      method = character(0),
      NES = numeric(0),
      leading_edge = character(0),
      stringsAsFactors = FALSE
    )
  }
}

#' Validate DAA results data frame
#'
#' Validates that a DAA results data frame meets requirements for visualization
#' functions (single method, single group pair).
#'
#' @param daa_results_df Data frame containing DAA results
#' @param require_single_method Logical. If TRUE, requires exactly one method
#' @param require_single_group_pair Logical. If TRUE, requires exactly one group1/group2 pair
#' @return Invisible TRUE if validation passes
#' @keywords internal
validate_daa_results <- function(daa_results_df,
                                  require_single_method = TRUE,
                                  require_single_group_pair = TRUE) {
  if (require_single_method && length(unique(daa_results_df$method)) != 1) {
    stop("daa_results_df contains multiple methods. Filter to one method first.")
  }
  if (require_single_group_pair) {
    if (length(unique(daa_results_df$group1)) != 1 ||
        length(unique(daa_results_df$group2)) != 1) {
      stop("daa_results_df contains multiple group pairs. Filter to one pair first.")
    }
  }
  invisible(TRUE)
}

#' Require a column exists in a data frame
#'
#' Simple check that a single column exists. For checking multiple columns,
#' use validate_dataframe() with required_cols parameter.
#'
#' @param df Data frame to check
#' @param col Column name to require
#' @param param_name Name of the data frame parameter (for error message)
#' @return Invisible TRUE if column exists, otherwise stops with error
#' @keywords internal
require_column <- function(df, col, param_name = deparse(substitute(df))) {
  if (!col %in% colnames(df)) {
    stop(sprintf("Column '%s' not found in %s", col, param_name))
  }
  invisible(TRUE)
}

# Helper: create standard DAA test data
# 3 pathways, configurable samples/groups. Values are deterministic (no randomness).
create_daa_test_data <- function(n_samples = 4, n_groups = 2) {
  # Pool of abundance values per pathway (cycled if n_samples > 6)
  pool <- list(
    c(10, 20, 15, 30, 35, 25),
    c(20, 30, 25, 40, 45, 35),
    c(30, 40, 35, 50, 55, 45)
  )
  abundance <- data.frame(
    lapply(seq_len(n_samples), function(i) {
      sapply(pool, function(p) p[((i - 1) %% length(p)) + 1])
    })
  )
  colnames(abundance) <- paste0("sample", seq_len(n_samples))
  rownames(abundance) <- paste0("pathway", 1:3)

  spg <- n_samples %/% n_groups
  groups <- rep(c("control", "treatment", "other")[seq_len(n_groups)], each = spg)
  if (length(groups) < n_samples) groups <- c(groups, rep("other", n_samples - length(groups)))
  metadata <- data.frame(
    sample = colnames(abundance),
    group = groups,
    stringsAsFactors = FALSE
  )
  list(abundance = abundance, metadata = metadata)
}

test_that("pathway_daa works with basic inputs", {
  td <- create_daa_test_data(n_samples = 6)

  result <- pathway_daa(td$abundance, td$metadata, "group", daa_method = "ALDEx2")
  expect_s3_class(result, "data.frame")
  expect_true(all(c("feature", "method", "p_values", "adj_method", "p_adjust") %in% colnames(result)))
  expect_gte(nrow(result), nrow(td$abundance))
  expect_true(all(grepl("ALDEx2", result$method)))
  expect_true(all(!is.na(result$p_values)))
  expect_true(all(result$p_values >= 0 & result$p_values <= 1))
})

test_that("pathway_daa validates inputs correctly", {
  abundance <- data.frame(
    sample1 = c(10, 20, 30), sample2 = c(20, 30, 40),
    sample3 = c(30, 40, 50), sample4 = c(40, 50, 60),
    row.names = c("pathway1", "pathway2", "pathway3")
  )

  # Mismatched sample names
  expect_error(
    pathway_daa(abundance, data.frame(wrong = paste0("x", 1:4), group = rep(c("a","b"), each = 2)), "group"),
    "Cannot find matching sample identifiers between abundance and metadata"
  )

  # Single group
  expect_error(
    pathway_daa(abundance, data.frame(sample = paste0("sample", 1:4), group = rep("control", 4)), "group"),
    "At least 2 groups are required"
  )

  # Insufficient sample size
  small_abundance <- abundance[, 1:3, drop = FALSE]
  expect_error(
    pathway_daa(small_abundance, data.frame(sample = paste0("sample", 1:3), group = c("a","a","b")), "group"),
    "At least 4 samples are required"
  )
})

test_that("pathway_daa core methods produce expected results", {
  n_samples <- 10
  n_features <- 3

  set.seed(123)
  abundance <- as.data.frame(matrix(
    rpois(n_samples * n_features, lambda = 20),
    nrow = n_features, ncol = n_samples,
    dimnames = list(paste0("pathway", 1:n_features), paste0("sample", 1:n_samples))
  ))

  metadata <- data.frame(
    sample = paste0("sample", 1:n_samples),
    group = rep(c("control", "treatment"), each = n_samples / 2)
  )

  core_methods <- c("ALDEx2", "limma voom", "edgeR")
  for (method in core_methods) {
    result <- suppressWarnings(pathway_daa(abundance, metadata, "group", daa_method = method))

    expect_true(is.data.frame(result))
    expect_true(all(c("feature", "method", "p_values") %in% colnames(result)))

    if (method == "ALDEx2") {
      expect_gte(nrow(result), n_features)
    } else {
      expect_equal(nrow(result), n_features)
    }

    expect_true(all(result$p_values >= 0 & result$p_values <= 1))
  }
})

test_that("pathway_daa extended methods run when explicitly enabled", {
  skip_if(
    Sys.getenv("GGPICRUST2_RUN_EXTENDED_DAA_TESTS", "false") != "true",
    "Set GGPICRUST2_RUN_EXTENDED_DAA_TESTS=true to run extended DAA method tests."
  )

  n_samples <- 10
  n_features <- 3
  set.seed(123)
  abundance <- as.data.frame(matrix(
    rpois(n_samples * n_features, lambda = 20),
    nrow = n_features, ncol = n_samples,
    dimnames = list(paste0("pathway", 1:n_features), paste0("sample", 1:n_samples))
  ))
  metadata <- data.frame(
    sample = paste0("sample", 1:n_samples),
    group = rep(c("control", "treatment"), each = n_samples / 2)
  )

  method_pkg <- c(
    "DESeq2" = "DESeq2",
    "metagenomeSeq" = "metagenomeSeq",
    "Maaslin2" = "Maaslin2"
  )
  extended_methods <- c("DESeq2", "metagenomeSeq", "LinDA", "Maaslin2")

  for (method in extended_methods) {
    if (method %in% names(method_pkg)) {
      skip_if_not_installed(method_pkg[[method]])
    }

    # Capture noisy method output to keep default test logs readable.
    captured <- capture.output({
      result <- suppressWarnings(pathway_daa(abundance, metadata, "group", daa_method = method))
    }, type = "output")
    ignore <- captured

    expect_true(is.data.frame(result))
    expect_true(all(c("feature", "method", "p_values") %in% colnames(result)))
  }
})

test_that("pathway_daa handles sample selection correctly", {
  # Use 6 samples so selecting 4 still meets the minimum requirement
  td <- create_daa_test_data(n_samples = 6)

  # Select a true subset (4 of 6): 2 control + 2 treatment
  selected <- c("sample1", "sample2", "sample4", "sample5")
  result <- pathway_daa(td$abundance, td$metadata, "group",
                       daa_method = "ALDEx2", select = selected)
  expect_s3_class(result, "data.frame")

  # Invalid sample selection
  expect_error(
    pathway_daa(td$abundance, td$metadata, "group",
                daa_method = "ALDEx2",
                select = c("sample1", "invalid_sample")),
    "Some selected samples not in abundance data"
  )
})

test_that("pathway_daa select= keeps metadata rows aligned with abundance columns", {
  # Regression: previously the select= branch reordered abundance columns to
  # match `select`, but only filtered metadata rows without reordering them.
  # Group labels then drifted relative to abundance, producing wrong p-values
  # (and sometimes significance flips) when `select` was not in natural order.
  #
  # Build a strong-signal dataset and compare results against a pre-subset
  # baseline. Internal select= must equal the pre-subset path.
  set.seed(7)
  n_features <- 3
  ctl <- matrix(rpois(n_features * 3, lambda = 5),  nrow = n_features)
  trt <- matrix(rpois(n_features * 3, lambda = 50), nrow = n_features)
  abundance <- as.data.frame(cbind(ctl, trt))
  colnames(abundance) <- paste0("S", 1:6)
  rownames(abundance) <- paste0("p", seq_len(n_features))

  metadata <- data.frame(
    sample = colnames(abundance),
    group  = rep(c("ctl", "trt"), each = 3),
    stringsAsFactors = FALSE
  )

  # Pick samples in a deliberately non-natural order that mixes the two groups.
  selected <- c("S4", "S2", "S6", "S1")

  via_select <- pathway_daa(abundance, metadata, "group",
                            daa_method = "ALDEx2", select = selected)
  via_preset <- pathway_daa(abundance[, selected, drop = FALSE],
                            metadata[metadata$sample %in% selected, ],
                            "group",
                            daa_method = "ALDEx2")

  # Same features, same method rows, same (group1, group2) assignment.
  expect_equal(via_select$feature, via_preset$feature)
  expect_equal(via_select$method,  via_preset$method)
  expect_equal(via_select$group1,  via_preset$group1)
  expect_equal(via_select$group2,  via_preset$group2)

  # p-values should agree closely (ALDEx2 is Monte Carlo-based but
  # deterministic enough here for a tight tolerance).
  expect_true(cor(via_select$p_values, via_preset$p_values) > 0.99)
})

test_that("limma voom multi-group labels align with p-values and coefficients", {
  # Regression: group2 was assigned a length-(k-1) vector and R's recycling
  # produced interleaved B,C,B,C,... labels for a 3-group design. The correct
  # labeling is B repeated N_features times, then C repeated N_features times,
  # matching as.vector()'s column-major flattening of fit$p.value[,-1].
  n_features <- 4
  n_per_group <- 4
  set.seed(42)
  abundance <- as.data.frame(matrix(
    rpois(n_features * n_per_group * 3, lambda = 20),
    nrow = n_features,
    dimnames = list(paste0("p", seq_len(n_features)),
                    paste0("S", seq_len(n_per_group * 3)))
  ))
  metadata <- data.frame(
    sample = colnames(abundance),
    group  = rep(c("A", "B", "C"), each = n_per_group),
    stringsAsFactors = FALSE
  )

  res <- suppressWarnings(
    pathway_daa(abundance, metadata, "group",
                daa_method = "limma voom", reference = "A")
  )

  # 4 features * 2 non-reference contrasts = 8 rows.
  expect_equal(nrow(res), n_features * 2)

  # Within each contrast block, every feature must appear exactly once
  # (not interleaved across contrasts).
  expect_equal(sort(res$feature[res$group2 == "B"]), paste0("p", 1:n_features))
  expect_equal(sort(res$feature[res$group2 == "C"]), paste0("p", 1:n_features))

  # group1 must be the reference for every row.
  expect_true(all(res$group1 == "A"))
})

test_that("DESeq2 respects the user-supplied reference level", {
  skip_if_not_installed("DESeq2")
  td <- create_daa_test_data(n_samples = 6)

  captured <- capture.output({
    res_default   <- suppressWarnings(
      pathway_daa(td$abundance, td$metadata, "group", daa_method = "DESeq2")
    )
    res_reference <- suppressWarnings(
      pathway_daa(td$abundance, td$metadata, "group",
                  daa_method = "DESeq2", reference = "treatment")
    )
  }, type = "output")
  ignore <- captured

  # Default behavior unchanged: alphabetical first level is the reference.
  expect_true(all(res_default$group1 == "control"))
  expect_true(all(res_default$group2 == "treatment"))

  # Explicit reference inverts the contrast direction.
  expect_true(all(res_reference$group1 == "treatment"))
  expect_true(all(res_reference$group2 == "control"))

  # Flipping the reference flips the sign of log2 fold changes. DESeq2
  # re-fits dispersion per call so the magnitudes are not bit-identical,
  # but should match to several decimals.
  merged <- merge(
    res_default[, c("feature", "log2_fold_change")],
    res_reference[, c("feature", "log2_fold_change")],
    by = "feature", suffixes = c("_def", "_ref")
  )
  finite <- is.finite(merged$log2_fold_change_def) &
            is.finite(merged$log2_fold_change_ref)
  expect_true(all(abs(
    merged$log2_fold_change_def[finite] + merged$log2_fold_change_ref[finite]
  ) < 1e-4))
})

test_that("DESeq2 handles multi-group input", {
  skip_if_not_installed("DESeq2")

  td <- create_daa_test_data(n_samples = 9, n_groups = 3)

  captured <- capture.output({
    res <- suppressWarnings(
      pathway_daa(td$abundance, td$metadata, "group",
                  daa_method = "DESeq2", reference = "control")
    )
  }, type = "output")
  ignore <- captured

  # 3 features * 2 non-reference contrasts = 6 rows.
  expect_equal(nrow(res), nrow(td$abundance) * 2)
  expect_true(all(res$group1 == "control"))
  expect_setequal(unique(res$group2), c("treatment", "other"))
})

test_that("Maaslin2 multi-group produces one row per (feature, contrast)", {
  skip_if_not_installed("Maaslin2")

  n_features <- 4
  n_per_group <- 4
  set.seed(123)
  abundance <- as.data.frame(matrix(
    rpois(n_features * n_per_group * 3, lambda = 30),
    nrow = n_features,
    dimnames = list(paste0("p", seq_len(n_features)),
                    paste0("S", seq_len(n_per_group * 3)))
  ))
  metadata <- data.frame(
    sample = colnames(abundance),
    group  = rep(c("A", "B", "C"), each = n_per_group),
    stringsAsFactors = FALSE
  )

  captured <- capture.output({
    res <- suppressWarnings(
      pathway_daa(abundance, metadata, "group",
                  daa_method = "Maaslin2", reference = "A")
    )
  }, type = "output")
  ignore <- captured

  # Was flattened to n_features rows pre-fix; expect n_features * (k-1).
  expect_equal(nrow(res), n_features * 2)
  expect_true(all(res$group1 == "A"))
  expect_setequal(unique(res$group2), c("B", "C"))

  # Features preserved after Maaslin2's hyphen-to-dot renaming round-trip.
  expect_true(all(res$feature %in% rownames(abundance)))
})

test_that("metagenomeSeq works with non-default sample column name", {
  skip_if_not_installed("metagenomeSeq")

  # cumNormStatFast() needs enough inter-sample variance to estimate a
  # scaling quantile, so use Poisson-generated counts with n=10 samples
  # (matching the existing extended-method test pattern) rather than the
  # tiny deterministic pool from create_daa_test_data().
  n_samples  <- 10
  n_features <- 3
  set.seed(321)
  abundance <- as.data.frame(matrix(
    rpois(n_samples * n_features, lambda = 20),
    nrow = n_features,
    dimnames = list(paste0("p", seq_len(n_features)),
                    paste0("S", seq_len(n_samples)))
  ))
  # Exercise the branch that previously hardcoded metadata$sample by
  # naming the sample column something other than "sample".
  metadata <- data.frame(
    SampleID = colnames(abundance),
    group    = rep(c("control", "treatment"), each = n_samples / 2),
    stringsAsFactors = FALSE
  )

  captured <- capture.output({
    res <- suppressWarnings(
      pathway_daa(abundance, metadata, "group", daa_method = "metagenomeSeq")
    )
  }, type = "output")
  ignore <- captured

  expect_s3_class(res, "data.frame")
  expect_true(all(c("feature", "method", "p_values") %in% colnames(res)))
  expect_equal(nrow(res), n_features)
})

test_that("metagenomeSeq survives degenerate cumNormStat input", {
  skip_if_not_installed("metagenomeSeq")

  # Minimal-but-legal input: 4 samples, 3 features, monotonic counts.
  # metagenomeSeq::cumNormStatFast() returns NaN for this shape because the
  # per-sample quantile search has nothing to stabilize on, and the package
  # then aborts with the cryptic "missing value where TRUE/FALSE needed".
  # We pre-compute the normalization factor with a fallback to p = 0.5,
  # which is metagenomeSeq's own documented default.
  abundance <- data.frame(
    S1 = c(10, 20, 30), S2 = c(15, 25, 35),
    S3 = c(30, 40, 50), S4 = c(35, 45, 55),
    row.names = paste0("p", 1:3)
  )
  metadata <- data.frame(
    sample = paste0("S", 1:4),
    group  = c("A", "A", "B", "B"),
    stringsAsFactors = FALSE
  )

  captured <- capture.output({
    res <- suppressWarnings(
      pathway_daa(abundance, metadata, "group", daa_method = "metagenomeSeq")
    )
  }, type = "output")
  ignore <- captured

  expect_s3_class(res, "data.frame")
  expect_equal(nrow(res), nrow(abundance))
  expect_true(all(c("feature", "method", "p_values") %in% colnames(res)))
})

test_that("pathway_daa rejects negative abundance values", {
  # Regression: validate_daa_input() previously skipped numeric-matrix quality
  # checks, so negative entries silently flowed into downstream methods and
  # produced cryptic failures or nonsense results.
  abundance <- data.frame(
    S1 = c(10, 20, 30), S2 = c(20, -5, 40),
    S3 = c(30, 40, 50), S4 = c(40, 50, 60),
    row.names = paste0("p", 1:3)
  )
  metadata <- data.frame(
    sample = paste0("S", 1:4),
    group  = c("a", "a", "b", "b"),
    stringsAsFactors = FALSE
  )
  expect_error(
    pathway_daa(abundance, metadata, "group", daa_method = "ALDEx2"),
    "Negative values found"
  )
})

test_that("pathway_daa handles factor levels correctly with subset", {
  # GitHub issue #158: 3 groups, select only 2
  abundance <- data.frame(
    sample1 = c(10, 20, 30), sample2 = c(20, 30, 40),
    sample3 = c(15, 25, 35), sample4 = c(30, 40, 50),
    sample5 = c(25, 35, 45),
    row.names = paste0("pathway", 1:3)
  )

  metadata <- data.frame(
    sample = paste0("sample", 1:5),
    group = c("A", "A", "B", "B", "C")
  )

  selected_samples <- c("sample1", "sample2", "sample3", "sample4")

  # Internal subsetting via select=
  set.seed(42)
  result1 <- pathway_daa(abundance, metadata, "group",
                        daa_method = "ALDEx2", select = selected_samples)

  # Pre-subsetting
  set.seed(42)
  result2 <- pathway_daa(abundance[, selected_samples, drop = FALSE],
                        metadata[metadata$sample %in% selected_samples, ],
                        "group", daa_method = "ALDEx2")

  expect_equal(nrow(result1), nrow(result2))
  expect_equal(result1$feature, result2$feature)
  expect_equal(result1$method, result2$method)

  # Groups should only include A and B, not C
  expect_true(all(result1$group1 %in% c("A", "B")))
  expect_true(all(result1$group2 %in% c("A", "B")))
  expect_false(any(c(result1$group1, result1$group2) == "C"))

  p_correlation <- cor(result1$p_values, result2$p_values, use = "complete.obs")
  expect_true(p_correlation > 0.9)
})

test_that("pathway_daa handles multiple groups correctly", {
  td <- create_daa_test_data(n_samples = 6, n_groups = 3)

  suppressWarnings({
    result <- pathway_daa(td$abundance, td$metadata, "group",
                         daa_method = "limma voom", reference = "control")
  })

  expect_s3_class(result, "data.frame")
  expect_true(all(!is.na(result$p_values)))

  # A correctly wired multi-group result should have one row per
  # (feature, non-reference level) pair -- `N_features * (k - 1)` total --
  # with `group1` pinned to the reference and `group2` covering every
  # other level. Asserting only `!is.na(p_values)` misses label/row-count
  # regressions like the one fixed in 2.5.15 where group2 was recycled
  # into interleaved B,C,B,C,... labels.
  expected_contrasts <- setdiff(unique(td$metadata$group), "control")
  expect_equal(nrow(result), nrow(td$abundance) * length(expected_contrasts))
  expect_true(all(result$group1 == "control"))
  expect_setequal(unique(result$group2), expected_contrasts)
  # Every feature must appear exactly once within each contrast block,
  # confirming `group2` labels align with per-feature p-values instead
  # of being recycled across rows.
  for (lvl in expected_contrasts) {
    expect_equal(sort(result$feature[result$group2 == lvl]),
                 sort(rownames(td$abundance)))
  }
})

test_that("pathway_daa handles p-value adjustment correctly", {
  td <- create_daa_test_data(n_samples = 4)

  # ALDEx2 uses its own pre-computed BH correction (Monte Carlo-based),
  # which is more accurate than simple p.adjust. The user's p_adjust_method
  # choice is not applied for ALDEx2. Other methods still honor the user's choice.
  result <- pathway_daa(td$abundance, td$metadata, "group",
                       daa_method = "ALDEx2", p_adjust_method = "BH")

  expect_true(all(result$adj_method == "BH (method-specific)"))
  expect_true(all(!is.na(result$p_adjust)))
  expect_true(all(result$p_adjust >= 0 & result$p_adjust <= 1))
})

test_that("pathway_daa include_abundance_stats parameter works correctly", {
  td <- create_daa_test_data(n_samples = 4)

  # Columns contributed by abundance stats (relative-abundance means/SDs).
  # log2_fold_change is intentionally excluded here: when the DAA method
  # already provides a log2_fold_change column (ALDEx2 with effect size on),
  # the relative-abundance ratio is suppressed to avoid conflating two
  # different effect-size definitions. That behavior is covered by a
  # dedicated test below.
  abundance_cols <- c("mean_rel_abundance_group1", "sd_rel_abundance_group1",
                     "mean_rel_abundance_group2", "sd_rel_abundance_group2")

  # Without abundance stats (default). Disable effect size too so this test
  # isolates the abundance-stats toggle from the ALDEx2 effect-size toggle.
  result_basic <- pathway_daa(td$abundance, td$metadata, "group",
                             daa_method = "ALDEx2",
                             include_abundance_stats = FALSE,
                             include_effect_size = FALSE)

  expect_false(any(abundance_cols %in% colnames(result_basic)))
  expect_false("log2_fold_change" %in% colnames(result_basic))

  # With abundance stats, still isolating from effect size.
  result_enhanced <- pathway_daa(td$abundance, td$metadata, "group",
                                daa_method = "ALDEx2",
                                include_abundance_stats = TRUE,
                                include_effect_size = FALSE)

  expect_true(all(abundance_cols %in% colnames(result_enhanced)))
  # With no method-native log2FC in the result, abundance stats should
  # contribute its relative-abundance-ratio log2_fold_change column.
  expect_true("log2_fold_change" %in% colnames(result_enhanced))

  for (col in c(abundance_cols, "log2_fold_change")) {
    expect_true(is.numeric(result_enhanced[[col]]))
  }
  expect_true(all(is.finite(result_enhanced$log2_fold_change)))
  expect_true(all(result_enhanced$sd_rel_abundance_group1 >= 0, na.rm = TRUE))
  expect_true(all(result_enhanced$sd_rel_abundance_group2 >= 0, na.rm = TRUE))
})

test_that("ALDEx2 returns effect size columns by default", {
  td <- create_daa_test_data(n_samples = 4)

  # Default call: no explicit include_effect_size.
  result_default <- pathway_daa(td$abundance, td$metadata, "group",
                                daa_method = "ALDEx2")

  effect_cols <- c("effect_size", "diff_btw", "log2_fold_change",
                   "rab_all", "overlap")
  expect_true(all(effect_cols %in% colnames(result_default)))
  for (col in effect_cols) {
    expect_true(is.numeric(result_default[[col]]))
  }

  # Opt-out recovers the minimal p-value-only output.
  result_opt_out <- pathway_daa(td$abundance, td$metadata, "group",
                                daa_method = "ALDEx2",
                                include_effect_size = FALSE)
  expect_false(any(effect_cols %in% colnames(result_opt_out)))
})

test_that("include_abundance_stats does not collide with method-native log2FC", {
  td <- create_daa_test_data(n_samples = 4)

  # ALDEx2 with both flags on: method-native log2_fold_change (CLR-space
  # diff.btw) must be preserved, and merging abundance stats must not
  # introduce .x / .y suffixes.
  result <- pathway_daa(td$abundance, td$metadata, "group",
                        daa_method = "ALDEx2",
                        include_abundance_stats = TRUE,
                        include_effect_size = TRUE)

  expect_true("log2_fold_change" %in% colnames(result))
  expect_false(any(c("log2_fold_change.x", "log2_fold_change.y") %in%
                     colnames(result)))
  # The retained log2_fold_change should equal ALDEx2's diff_btw (CLR space),
  # not the relative-abundance ratio.
  expect_equal(result$log2_fold_change, result$diff_btw)
})

test_that("format_linda_output handles malformed LinDA outputs robustly", {
  malformed_output <- list(
    groupB = data.frame(
      pvalue = I(matrix(c(0.01, 0.02, 0.03, 0.04, 0.05, 0.06), nrow = 3)),
      log2FoldChange = c(1.2, 0.8, 0.5),
      row.names = c("path1", "path2", "path3")
    ),
    groupC = data.frame(
      some_other_col = c(1, 2),
      row.names = c("path4", "path5")
    )
  )

  result <- suppressWarnings(
    ggpicrust2:::format_linda_output(
      linda_output = malformed_output,
      group = "group",
      reference = "A",
      Level = c("A", "B", "C")
    )
  )

  expect_s3_class(result, "data.frame")
  expect_equal(nrow(result), 5)
  expect_true(all(c("feature", "method", "group1", "group2", "p_values", "log2_fold_change") %in% colnames(result)))

  group_b_rows <- result$group2 == "B"
  expect_true(any(group_b_rows))
  expect_true(all(is.na(result$p_values[group_b_rows])))
  expect_true(all(!is.na(result$log2_fold_change[group_b_rows])))

  group_c_rows <- result$group2 == "C"
  expect_true(any(group_c_rows))
  expect_true(all(is.na(result$p_values[group_c_rows])))
  expect_true(all(is.na(result$log2_fold_change[group_c_rows])))
})

test_that("pathway_daa Lefser fails fast for multi-group input", {
  skip_if_not_installed("lefser")

  td <- create_daa_test_data(n_samples = 6, n_groups = 3)
  expect_error(
    pathway_daa(
      abundance = td$abundance,
      metadata = td$metadata,
      group = "group",
      daa_method = "Lefser"
    ),
    "requires exactly 2 groups"
  )
})

test_that("pathway_daa LinDA honors user-specified reference", {
  # Regression: LinDA used `~ group` without releveling, so the factor's
  # natural first level was always used as reference while the result's
  # group1 column was labeled with the user-supplied `reference`. That
  # produced rows with group1 == group2 and log2FC that did not reflect
  # the requested contrast direction.
  skip_if_not_installed("MicrobiomeStat")

  set.seed(42)
  n_feat <- 12; n_samp <- 12
  abund <- matrix(rpois(n_feat * n_samp, 40),
                  nrow = n_feat, ncol = n_samp,
                  dimnames = list(paste0("f", seq_len(n_feat)),
                                  paste0("S", seq_len(n_samp))))
  meta <- data.frame(
    sample = paste0("S", seq_len(n_samp)),
    Env = c(rep("control", 6), rep("treatment", 6)),
    stringsAsFactors = FALSE
  )

  r_ctrl <- suppressMessages(suppressWarnings(
    pathway_daa(abundance = abund, metadata = meta, group = "Env",
                daa_method = "LinDA", reference = "control")
  ))
  r_trt <- suppressMessages(suppressWarnings(
    pathway_daa(abundance = abund, metadata = meta, group = "Env",
                daa_method = "LinDA", reference = "treatment")
  ))

  expect_true(all(r_ctrl$group1 == "control"))
  expect_true(all(r_ctrl$group2 == "treatment"))
  expect_true(all(r_trt$group1 == "treatment"))
  expect_true(all(r_trt$group2 == "control"))
  # Flipping the reference must flip the sign of log2 fold change.
  expect_equal(r_ctrl$log2_fold_change, -r_trt$log2_fold_change, tolerance = 1e-6)
})

test_that("pathway_daa Maaslin2 honors user-specified reference in 2-group case", {
  # Regression: the 2-group branch passed `reference = NULL` to Maaslin2,
  # so Maaslin2 used its alphabetical default regardless of the user's
  # `reference` argument. Combined with the result-labeling that used
  # the user's `reference` for group1, this produced group1 == group2
  # and a coefficient whose sign did not flip when the user flipped
  # the reference.
  skip_if_not_installed("Maaslin2")

  set.seed(42)
  n_feat <- 12; n_samp <- 12
  abund <- matrix(rpois(n_feat * n_samp, 40),
                  nrow = n_feat, ncol = n_samp,
                  dimnames = list(paste0("f", seq_len(n_feat)),
                                  paste0("S", seq_len(n_samp))))
  meta <- data.frame(
    sample = paste0("S", seq_len(n_samp)),
    Env = c(rep("control", 6), rep("treatment", 6)),
    stringsAsFactors = FALSE
  )

  m_ctrl <- suppressMessages(suppressWarnings(
    pathway_daa(abundance = abund, metadata = meta, group = "Env",
                daa_method = "Maaslin2", reference = "control")
  ))
  m_trt <- suppressMessages(suppressWarnings(
    pathway_daa(abundance = abund, metadata = meta, group = "Env",
                daa_method = "Maaslin2", reference = "treatment")
  ))

  expect_true(all(m_ctrl$group1 == "control"))
  expect_true(all(m_ctrl$group2 == "treatment"))
  expect_true(all(m_trt$group1 == "treatment"))
  expect_true(all(m_trt$group2 == "control"))
  # Align rows by feature before comparing — Maaslin2 may reorder.
  ord_ctrl <- order(m_ctrl$feature)
  ord_trt <- order(m_trt$feature)
  expect_equal(m_ctrl$log2_fold_change[ord_ctrl],
               -m_trt$log2_fold_change[ord_trt],
               tolerance = 1e-6)
})

test_that("pathway_daa edgeR honors user-specified reference", {
  # Regression: edgeR's exactTest() used raw factor order (pair = c(1, 2)),
  # so the `reference` argument was silently ignored and result labels
  # were always Level[1]/Level[2]. Relevel the grouping factor so edgeR
  # and the labels agree with the documented semantics.
  skip_if_not_installed("edgeR")

  set.seed(7)
  n_feat <- 10; n_samp <- 12
  abund <- matrix(rpois(n_feat * n_samp, 40),
                  nrow = n_feat, ncol = n_samp,
                  dimnames = list(paste0("f", seq_len(n_feat)),
                                  paste0("S", seq_len(n_samp))))
  meta <- data.frame(
    sample = paste0("S", seq_len(n_samp)),
    Env = c(rep("control", 6), rep("treatment", 6)),
    stringsAsFactors = FALSE
  )

  e_ctrl <- suppressWarnings(suppressMessages(
    pathway_daa(abundance = abund, metadata = meta, group = "Env",
                daa_method = "edgeR", reference = "control")
  ))
  e_trt <- suppressWarnings(suppressMessages(
    pathway_daa(abundance = abund, metadata = meta, group = "Env",
                daa_method = "edgeR", reference = "treatment")
  ))

  expect_true(all(e_ctrl$group1 == "control"))
  expect_true(all(e_ctrl$group2 == "treatment"))
  expect_true(all(e_trt$group1 == "treatment"))
  expect_true(all(e_trt$group2 == "control"))
  expect_equal(e_ctrl$log2_fold_change, -e_trt$log2_fold_change, tolerance = 1e-6)
})

test_that("pathway_daa metagenomeSeq honors user-specified reference", {
  # Regression: metagenomeSeq labels were fixed to Level[1]/Level[2] and
  # the model matrix used raw factor order, so `reference = "treatment"`
  # left both labels and p-values unchanged.
  skip_if_not_installed("metagenomeSeq")

  set.seed(7)
  n_feat <- 10; n_samp <- 12
  abund <- matrix(rpois(n_feat * n_samp, 40),
                  nrow = n_feat, ncol = n_samp,
                  dimnames = list(paste0("f", seq_len(n_feat)),
                                  paste0("S", seq_len(n_samp))))
  meta <- data.frame(
    sample = paste0("S", seq_len(n_samp)),
    Env = c(rep("control", 6), rep("treatment", 6)),
    stringsAsFactors = FALSE
  )

  m_ctrl <- suppressWarnings(suppressMessages(
    pathway_daa(abundance = abund, metadata = meta, group = "Env",
                daa_method = "metagenomeSeq", reference = "control")
  ))
  m_trt <- suppressWarnings(suppressMessages(
    pathway_daa(abundance = abund, metadata = meta, group = "Env",
                daa_method = "metagenomeSeq", reference = "treatment")
  ))

  expect_true(all(m_ctrl$group1 == "control"))
  expect_true(all(m_ctrl$group2 == "treatment"))
  expect_true(all(m_trt$group1 == "treatment"))
  expect_true(all(m_trt$group2 == "control"))
})

test_that("pathway_daa metagenomeSeq emits one block per non-reference level for >=3 groups", {
  # Regression: metagenomeSeq previously built a full k-column model matrix,
  # called fit_feature_model() once, read `coef = 2`, and hard-coded the
  # output as `group1 = Level[1] / group2 = Level[2]` regardless of how
  # many groups existed. Contrasts against any level beyond the first
  # non-reference one were silently dropped. The result should now have
  # `(k - 1) * n_features` rows, one (ref, non-ref) block per contrast,
  # with both the coefficient and the labels redone per pair (since
  # fitFeatureModel is documented as a two-group entry point).
  skip_if_not_installed("metagenomeSeq")

  set.seed(11)
  n_feat <- 8
  per_group <- 6
  groups <- rep(c("A", "B", "C"), each = per_group)
  n_samp <- length(groups)
  abund <- matrix(rpois(n_feat * n_samp, 50),
                  nrow = n_feat, ncol = n_samp,
                  dimnames = list(paste0("f", seq_len(n_feat)),
                                  paste0("S", seq_len(n_samp))))
  meta <- data.frame(
    sample = paste0("S", seq_len(n_samp)),
    Env = groups,
    stringsAsFactors = FALSE
  )

  res <- suppressWarnings(suppressMessages(
    pathway_daa(abundance = abund, metadata = meta, group = "Env",
                daa_method = "metagenomeSeq")
  ))

  # Shape: k = 3 groups -> 2 non-reference contrasts -> 2 * n_feat rows.
  expect_equal(nrow(res), 2 * n_feat)

  # Reference defaults to the first level ("A"). group1 must always be A,
  # group2 must cover exactly {B, C} once each per feature.
  expect_true(all(res$group1 == "A"))
  expect_setequal(unique(res$group2), c("B", "C"))
  expect_equal(sum(res$group2 == "B"), n_feat)
  expect_equal(sum(res$group2 == "C"), n_feat)

  # reference = "B" must flip group1 to B and restrict group2 to {A, C}.
  res_refB <- suppressWarnings(suppressMessages(
    pathway_daa(abundance = abund, metadata = meta, group = "Env",
                daa_method = "metagenomeSeq", reference = "B")
  ))
  expect_equal(nrow(res_refB), 2 * n_feat)
  expect_true(all(res_refB$group1 == "B"))
  expect_setequal(unique(res_refB$group2), c("A", "C"))

  # p_values and log2_fold_change columns must both be present across the
  # full output even if individual MRcoefs() calls fall back to NA.
  expect_true("p_values" %in% colnames(res))
  expect_true("log2_fold_change" %in% colnames(res))
})

test_that("pathway_daa re-validates group count after align/select", {
  # Regression: validate_group() only checks the raw metadata. If
  # align_samples() or a narrow `select =` filter removes every sample
  # of a level, the single-group leftover could propagate into backends
  # with a confusing downstream error.
  td <- create_daa_test_data(n_samples = 6, n_groups = 2)

  expect_error(
    pathway_daa(
      abundance = td$abundance,
      metadata = td$metadata,
      group = "group",
      daa_method = "ALDEx2",
      select = td$metadata$sample[td$metadata$group == "control"]
    ),
    "at least 2 groups with samples"
  )
})

test_that("pathway_daa rejects unsupported daa_method with a typo suggestion", {
  # Regression: pathway_daa() used to dispatch via switch() with no
  # default branch, so a misspelled method like "linDA" fell through and
  # returned NULL silently. Now we whitelist-validate up front and hint
  # the canonical spelling for common typos.
  td <- create_daa_test_data(n_samples = 6, n_groups = 2)

  expect_error(
    pathway_daa(
      abundance = td$abundance,
      metadata = td$metadata,
      group = "group",
      daa_method = "linDA"
    ),
    "Did you mean 'LinDA'"
  )

  expect_error(
    pathway_daa(
      abundance = td$abundance,
      metadata = td$metadata,
      group = "group",
      daa_method = "Lefse"
    ),
    "Did you mean 'Lefser'"
  )

  expect_error(
    pathway_daa(
      abundance = td$abundance,
      metadata = td$metadata,
      group = "group",
      daa_method = "totally_unknown"
    ),
    "Unsupported daa_method"
  )
})

test_that("pathway_daa rejects samples with zero total abundance", {
  # Regression: `x / sum(x)` inside calculate_abundance_stats() produced
  # NaN for zero-sum sample columns; the surrounding `mean(..., na.rm =
  # TRUE)` then silently dropped those NaN values, so the reported group
  # means were computed from fewer samples than the user had supplied.
  # The zero-sum column must now surface as an actionable error that
  # names the offending sample.
  skip_if_not_installed("ALDEx2")

  td <- create_daa_test_data(n_samples = 6, n_groups = 2)
  bad_abundance <- td$abundance
  zero_sample <- colnames(bad_abundance)[3]
  bad_abundance[, zero_sample] <- 0

  expect_error(
    pathway_daa(
      abundance = bad_abundance,
      metadata = td$metadata,
      group = "group",
      daa_method = "ALDEx2",
      include_abundance_stats = TRUE
    ),
    regexp = zero_sample
  )
})

# Tests for abundance-quality helpers in data_utils.R:
#   - compute_relative_abundance()
#   - validate_abundance(check_zero_columns = TRUE)
#
# Regression: x / sum(x) used to produce NaN on zero-sum sample columns,
# which then got silently dropped by downstream mean(..., na.rm = TRUE)
# aggregations. That computed group stats from the wrong sample size
# without any warning. Both the helper and the input validator now
# refuse zero-sum columns with an actionable error that names the
# offending sample(s).

test_that("compute_relative_abundance matches the old apply idiom on clean input", {
  cra <- getFromNamespace("compute_relative_abundance", "ggpicrust2")

  set.seed(1)
  M <- matrix(sample(1:20, 12), nrow = 3, ncol = 4,
              dimnames = list(paste0("f", 1:3), paste0("S", 1:4)))

  old_way <- apply(t(M), 1, function(x) x / sum(x))
  new_way <- cra(M)
  expect_equal(new_way, old_way)
  # Columns sum to 1 after normalization.
  expect_equal(unname(colSums(new_way)), rep(1, ncol(M)))
})

test_that("compute_relative_abundance rejects zero-sum columns with a named error", {
  cra <- getFromNamespace("compute_relative_abundance", "ggpicrust2")

  M <- matrix(c(10, 20, 0, 40,
                15, 25, 0, 45,
                12, 22, 0, 42),
              nrow = 3, byrow = TRUE,
              dimnames = list(paste0("f", 1:3), c("S1", "S2", "BAD", "S4")))

  expect_error(cra(M, context = "unit-test"),
               regexp = "BAD",
               class = "simpleError")
  expect_error(cra(M), regexp = "unit-test|abundance", fixed = FALSE)
})

test_that("compute_relative_abundance rejects columns that sum to NA", {
  cra <- getFromNamespace("compute_relative_abundance", "ggpicrust2")

  M <- matrix(c(10, NA, 30,
                15, NA, 35),
              nrow = 2, byrow = TRUE,
              dimnames = list(paste0("f", 1:2), c("S1", "S2", "S3")))

  expect_error(cra(M), regexp = "S2")
})

test_that("compute_relative_abundance falls back to index labels when colnames are missing", {
  cra <- getFromNamespace("compute_relative_abundance", "ggpicrust2")

  M <- matrix(c(1, 0, 3,
                2, 0, 4),
              nrow = 2, byrow = TRUE)  # no colnames

  # Position 2 is zero-sum; error must still identify it.
  expect_error(cra(M), regexp = "\\b2\\b")
})

test_that("validate_abundance rejects zero-sum sample columns by default", {
  va <- getFromNamespace("validate_abundance", "ggpicrust2")

  M <- matrix(c(10, 20, 0, 40,
                15, 25, 0, 45),
              nrow = 2, byrow = TRUE,
              dimnames = list(paste0("f", 1:2), c("S1", "S2", "BAD", "S4")))

  expect_error(va(M), regexp = "BAD")

  # Data-frame form (abundance-only columns) also checked.
  df <- as.data.frame(M)
  expect_error(va(df), regexp = "BAD")
})

test_that("validate_abundance can be asked to skip the zero-column check", {
  va <- getFromNamespace("validate_abundance", "ggpicrust2")

  M <- matrix(c(10, 0,
                15, 0),
              nrow = 2, byrow = TRUE,
              dimnames = list(paste0("f", 1:2), c("S1", "BAD")))

  expect_true(va(M, check_zero_columns = FALSE))
})

test_that("validate_abundance tolerates a non-numeric ID column in data frames", {
  # abundance_to_numeric_matrix() must strip a character ID column before
  # summing, otherwise data.frame inputs from earlier pipeline stages
  # would trigger spurious "not numeric" errors.
  va <- getFromNamespace("validate_abundance", "ggpicrust2")

  df <- data.frame(
    feature = paste0("K", 1:3),
    S1 = c(10, 20, 30),
    S2 = c(5, 15, 25),
    stringsAsFactors = FALSE
  )
  expect_true(va(df))
})

# Unit tests for summarize_abundance_by_group(), the per-feature/per-group
# mean/sd helper that pathway_errorbar() and calculate_abundance_stats()
# both route through. Locks down contract so future refactors do not
# silently break one caller while leaving the other intact.
test_that("summarize_abundance_by_group matches manual mean/sd per group", {
  sbg <- getFromNamespace("summarize_abundance_by_group", "ggpicrust2")

  set.seed(42)
  M <- matrix(runif(12), nrow = 3, ncol = 4,
              dimnames = list(c("f1", "f2", "f3"),
                              c("S1", "S2", "S3", "S4")))
  g <- c("A", "A", "B", "B")

  out <- sbg(M, g)
  # Feature-major order: (f1,A), (f1,B), (f2,A), ... — required for
  # downstream compatibility with pathway_errorbar's match()/order() code.
  expect_equal(out$name, rep(c("f1", "f2", "f3"), each = 2))
  expect_equal(out$group, rep(c("A", "B"), times = 3))

  # Numerics must match manual computation exactly.
  for (feat in rownames(M)) {
    manual_A_mean <- mean(M[feat, g == "A"])
    manual_A_sd   <- stats::sd(M[feat, g == "A"])
    manual_B_mean <- mean(M[feat, g == "B"])
    manual_B_sd   <- stats::sd(M[feat, g == "B"])
    row <- out[out$name == feat, ]
    expect_equal(row$mean[row$group == "A"], manual_A_mean)
    expect_equal(row$sd[row$group == "A"], manual_A_sd)
    expect_equal(row$mean[row$group == "B"], manual_B_mean)
    expect_equal(row$sd[row$group == "B"], manual_B_sd)
  }
})

test_that("summarize_abundance_by_group handles NA in abundance via na.rm = TRUE", {
  sbg <- getFromNamespace("summarize_abundance_by_group", "ggpicrust2")

  M <- matrix(c(1, 2, NA, 4,
                5, 6, 7, 8),
              nrow = 2, byrow = TRUE,
              dimnames = list(c("f1", "f2"), c("S1", "S2", "S3", "S4")))
  g <- c("A", "A", "B", "B")
  out <- sbg(M, g)

  # f1 in group B: only S4 is non-NA, so mean = 4, sd = NA (stats::sd on
  # length-1 vector returns NA). This matches calculate_abundance_stats().
  f1_B <- out[out$name == "f1" & out$group == "B", ]
  expect_equal(f1_B$mean, 4)
  expect_true(is.na(f1_B$sd))

  # f2 across all samples is intact.
  f2_A <- out[out$name == "f2" & out$group == "A", ]
  expect_equal(f2_A$mean, mean(c(5, 6)))
  expect_equal(f2_A$sd, stats::sd(c(5, 6)))
})

test_that("summarize_abundance_by_group errors on length mismatch", {
  sbg <- getFromNamespace("summarize_abundance_by_group", "ggpicrust2")

  M <- matrix(1:6, nrow = 2, ncol = 3,
              dimnames = list(c("f1", "f2"), c("S1", "S2", "S3")))
  expect_error(sbg(M, c("A", "B")), regexp = "length")
})

test_that("align_samples is idempotent on already-aligned inputs", {
  # Contract: ggpicrust2() pre-aligns inputs at its top, then calls
  # pathway_daa(), which independently re-aligns. Both sites use the
  # same helper with identical arguments, so the duplicate is bounded
  # by the invariant that running align_samples() twice on the result
  # of the first call is a no-op. This test locks that invariant --
  # any future change to align_samples() that breaks idempotency
  # would silently reintroduce drift between the two paths.
  set.seed(11)
  abund <- matrix(runif(20), nrow = 5, ncol = 4,
                  dimnames = list(paste0("K", 1:5),
                                  paste0("S", 1:4)))
  meta <- data.frame(
    sample = paste0("S", 1:4),
    Env    = c("A", "A", "B", "B"),
    stringsAsFactors = FALSE
  )

  first  <- align_samples(abund, meta, verbose = FALSE)
  second <- align_samples(first$abundance, first$metadata, verbose = FALSE)

  expect_identical(first$abundance, second$abundance)
  expect_identical(first$metadata,  second$metadata)
  expect_identical(first$sample_col, second$sample_col)
  expect_identical(first$n_samples,  second$n_samples)
})

# Test for gsea_pathway_annotation function
# create_gsea_test_results() is defined in helper-gsea.R

test_that("gsea_pathway_annotation annotates KEGG pathways correctly", {
  gsea_results <- create_gsea_test_results()

  kegg_ref <- ggpicrust2:::load_reference_data("KEGG")
  gsea_results$pathway_id <- head(kegg_ref$pathway, 10)

  annotated <- gsea_pathway_annotation(gsea_results, pathway_type = "KEGG")

  expect_s3_class(annotated, "data.frame")
  expect_equal(nrow(annotated), nrow(gsea_results))
  expect_true("pathway_name" %in% colnames(annotated))
  expect_true(any(annotated$pathway_name != annotated$pathway_id))
})

test_that("gsea_pathway_annotation annotates MetaCyc pathways correctly", {
  mock_results <- data.frame(
    pathway_id = c("1CMET2-PWY", "ANAGLYCOLYSIS-PWY", "UNKNOWN-PATHWAY-TEST"),
    NES = c(1.5, -1.2, 0.2),
    pvalue = c(0.02, 0.08, 0.45),
    stringsAsFactors = FALSE
  )

  annotated <- gsea_pathway_annotation(mock_results, pathway_type = "MetaCyc")

  expect_s3_class(annotated, "data.frame")
  expect_equal(nrow(annotated), 3)
  expect_true("pathway_name" %in% colnames(annotated))
  expect_true(all(!is.na(annotated$pathway_name)))

  # Unknown pathway should keep its ID as name
  unknown_row <- annotated[annotated$pathway_id == "UNKNOWN-PATHWAY-TEST", ]
  expect_equal(unknown_row$pathway_name, "UNKNOWN-PATHWAY-TEST")
})

test_that("gsea_pathway_annotation annotates GO pathways", {
  gsea_results <- create_gsea_test_results()
  gsea_results$pathway_id <- paste0("GO:", sprintf("%07d", 1:10))

  annotated <- gsea_pathway_annotation(gsea_results, pathway_type = "GO")

  expect_s3_class(annotated, "data.frame")
  expect_equal(nrow(annotated), nrow(gsea_results))
  expect_true("pathway_name" %in% colnames(annotated))
})

test_that("gsea_pathway_annotation validates inputs correctly", {
  gsea_results <- create_gsea_test_results()

  expect_error(gsea_pathway_annotation(gsea_results = "invalid"), "'gsea_results' must be a data frame")
  expect_error(gsea_pathway_annotation(gsea_results, pathway_type = "invalid"), "pathway_type must be one of")

  gsea_results_missing <- gsea_results[, !names(gsea_results) %in% c("pathway_id")]
  expect_error(gsea_pathway_annotation(gsea_results_missing), "missing required column: pathway_id")
})

create_followup_errorbar_test_data <- function(n_features = 5, p_adjust = NULL) {
  set.seed(123)
  n_samples <- 10
  abundance <- matrix(runif(n_features * n_samples), nrow = n_features, ncol = n_samples)
  rownames(abundance) <- paste0("pathway", seq_len(n_features))
  colnames(abundance) <- paste0("sample", seq_len(n_samples))

  if (is.null(p_adjust)) p_adjust <- rep(0.01, n_features)

  daa_results_df <- data.frame(
    feature = paste0("pathway", seq_len(n_features)),
    pathway_name = paste0("Pathway ", seq_len(n_features)),
    p_adjust = p_adjust,
    method = rep("ALDEx2_Welch's t test", n_features),
    group1 = rep("GroupA", n_features),
    group2 = rep("GroupB", n_features),
    stringsAsFactors = FALSE
  )

  group <- factor(rep(c("GroupA", "GroupB"), each = n_samples / 2))
  names(group) <- colnames(abundance)

  list(abundance = abundance, daa_results_df = daa_results_df, Group = group)
}

test_that("import_MicrobiomeAnalyst_daa_results defaults and validates shape", {
  daa <- data.frame(
    feature = c("p1", "p2"),
    p_values = c(0.01, 0.02),
    p_adjust = c(0.03, 0.04),
    Statistics = c(1.2, -0.5),
    stringsAsFactors = FALSE
  )

  imported <- import_MicrobiomeAnalyst_daa_results(data = daa)
  expect_equal(imported$group1, c("control", "control"))
  expect_equal(imported$group2, c("treatment", "treatment"))
  expect_false(anyNA(names(imported)))

  too_short <- daa[, 1:3]
  expect_error(
    import_MicrobiomeAnalyst_daa_results(data = too_short),
    "at least four columns"
  )

  extra_col <- cbind(daa, extra = 1)
  imported_extra <- import_MicrobiomeAnalyst_daa_results(data = extra_col)
  expect_false(anyNA(names(imported_extra)))
  expect_true("extra" %in% names(imported_extra))
})

test_that("import_MicrobiomeAnalyst_daa_results warns that data wins over file_path", {
  daa <- data.frame(
    feature = "p1",
    p_values = 0.01,
    p_adjust = 0.03,
    Statistics = 1,
    stringsAsFactors = FALSE
  )

  expect_warning(
    imported <- import_MicrobiomeAnalyst_daa_results(
      file_path = tempfile(fileext = ".csv"),
      data = daa
    ),
    "Using data and ignoring file_path"
  )
  expect_equal(imported$feature, "p1")
})

test_that("pathway_errorbar order='group' resolves tied groups deterministically", {
  abundance <- matrix(
    c(10, 20, 30, 40,
      20, 10, 40, 30),
    nrow = 2,
    byrow = TRUE,
    dimnames = list(c("f1", "f2"), paste0("S", 1:4))
  )
  daa <- data.frame(
    feature = c("f1", "f2"),
    method = "m",
    group1 = "A",
    group2 = "B",
    p_adjust = c(0.01, 0.02),
    description = c("d1", "d2"),
    stringsAsFactors = FALSE
  )
  group <- c("A", "A", "B", "B")
  names(group) <- colnames(abundance)

  expect_warning(
    plot <- pathway_errorbar(
      abundance,
      daa,
      group,
      order = "group",
      x_lab = "description",
      p_value_bar = "FALSE"
    ),
    NA
  )
  expect_s3_class(plot, "patchwork")
})

test_that("pathway_errorbar normalizes logical flags and rejects invalid p_adjust", {
  td <- create_followup_errorbar_test_data(n_features = 2, p_adjust = c(0, 0.01))
  expect_error(
    plot <- pathway_errorbar(
      td$abundance,
      td$daa_results_df,
      td$Group,
      x_lab = "pathway_name",
      p_value_bar = "FALSE",
      smart_colors = "FALSE",
      accessibility_mode = "FALSE"
    ),
    NA
  )
  expect_s3_class(plot, "patchwork")

  td$daa_results_df$p_adjust[1] <- -0.1
  expect_error(
    pathway_errorbar(td$abundance, td$daa_results_df, td$Group, x_lab = "pathway_name"),
    "non-negative"
  )
})

test_that("pathway_errorbar_table gives ko_to_kegg an explicit contract", {
  td <- create_followup_errorbar_test_data(n_features = 2, p_adjust = c(0.01, 0.02))

  expect_error(
    pathway_errorbar_table(
      abundance = td$abundance,
      daa_results_df = td$daa_results_df,
      Group = td$Group,
      ko_to_kegg = TRUE
    ),
    "pathway_class"
  )

  td$daa_results_df$pathway_class <- c("Metabolism", "Metabolism")
  expect_error(
    res <- pathway_errorbar_table(
      abundance = td$abundance,
      daa_results_df = td$daa_results_df,
      Group = td$Group,
      ko_to_kegg = "TRUE"
    ),
    NA
  )
  expect_true("pathway_class" %in% names(res))
})

test_that("gsea_pathway_annotation preserves input order", {
  kegg_ref <- ggpicrust2:::load_reference_data("KEGG")
  ids <- rev(head(kegg_ref$pathway, 3))
  gsea_results <- data.frame(
    pathway_id = ids,
    NES = c(1, 2, 3),
    pvalue = c(0.03, 0.02, 0.01),
    p.adjust = c(0.03, 0.02, 0.01),
    stringsAsFactors = FALSE
  )

  annotated <- gsea_pathway_annotation(gsea_results, pathway_type = "KEGG")
  expect_equal(annotated$pathway_id, ids)
})

test_that("visualize_gsea validates network contract before delayed plot build", {
  gsea_results <- create_gsea_test_results()

  missing_leading_edge <- gsea_results[, setdiff(names(gsea_results), "leading_edge")]
  expect_error(
    visualize_gsea(missing_leading_edge, plot_type = "network"),
    "leading_edge"
  )

  expect_error(
    visualize_gsea(
      gsea_results,
      plot_type = "network",
      network_params = list(similarity_measure = "bad")
    ),
    "network_params\\$similarity_measure"
  )

  expect_error(
    visualize_gsea(
      gsea_results,
      plot_type = "network",
      network_params = list(node_color_by = "missing")
    ),
    "network_params\\$node_color_by"
  )
})

test_that("visualize_gsea honors edge_width_by", {
  skip_if_not_installed("igraph")
  skip_if_not_installed("ggraph")
  skip_if_not_installed("tidygraph")

  gsea_results <- create_gsea_test_results(n_pathways = 3)
  gsea_results$leading_edge <- c("K1;K2", "K2;K3", "K3;K4")

  similarity_plot <- visualize_gsea(
    gsea_results,
    plot_type = "network",
    network_params = list(edge_width_by = "similarity", similarity_cutoff = 0)
  )
  constant_plot <- visualize_gsea(
    gsea_results,
    plot_type = "network",
    network_params = list(edge_width_by = "constant", similarity_cutoff = 0)
  )

  expect_true("edge_width" %in% names(similarity_plot$layers[[1]]$mapping))
  expect_false("edge_width" %in% names(constant_plot$layers[[1]]$mapping))
})

test_that("visualize_gsea heatmap treats annotation_colors=NULL as default", {
  skip_if_not_installed("ComplexHeatmap")
  skip_if_not_installed("circlize")

  gsea_results <- create_gsea_test_results(n_pathways = 2)
  gsea_results$pathway_id <- c("p1", "p2")
  gsea_results$leading_edge <- c("K00001;K00002", "K00002;K00003")
  abundance <- matrix(
    1:12,
    nrow = 3,
    dimnames = list(c("K00001", "K00002", "K00003"), paste0("S", 1:4))
  )
  metadata <- data.frame(
    sample = paste0("S", 1:4),
    group = c("A", "A", "B", "B"),
    stringsAsFactors = FALSE
  )

  expect_error(
    heatmap <- visualize_gsea(
      gsea_results,
      plot_type = "heatmap",
      abundance = abundance,
      metadata = metadata,
      group = "group",
      heatmap_params = list(annotation_colors = NULL)
    ),
    NA
  )
  expect_s4_class(heatmap, "Heatmap")
})

test_that("compare_metagenome_results checks heatmap namespaces explicitly", {
  body_src <- paste(deparse(body(ggpicrust2::compare_metagenome_results)), collapse = "\n")
  expect_true(grepl("require_package\\(\"circlize\"", body_src))
  expect_true(grepl("require_package\\(\"ComplexHeatmap\"", body_src))
})

# Regression tests for 8 issues fixed in the first-principles audit pass.
# Each `test_that()` block corresponds to one issue in that audit and
# reproduces the original broken behavior to prove the fix sticks.

# -----------------------------------------------------------------------------
# Issue 1: align_samples() must reject duplicate sample IDs when the caller
# passes `sample_col` explicitly. Before the fix, match() silently collapsed
# duplicates to the first hit and dropped other rows without warning.
# -----------------------------------------------------------------------------
test_that("align_samples() rejects duplicate IDs under explicit sample_col", {
  align <- getFromNamespace("align_samples", "ggpicrust2")

  abundance <- matrix(
    c(1, 2, 3, 4, 5, 6, 7, 8, 9),
    nrow = 3,
    dimnames = list(paste0("f", 1:3), c("S1", "S2", "S3"))
  )
  metadata <- data.frame(
    sample_id = c("S1", "S1", "S2"),   # duplicated "S1"
    group = c("A", "B", "C"),
    stringsAsFactors = FALSE
  )

  expect_error(
    align(abundance, metadata, sample_col = "sample_id", verbose = FALSE),
    regexp = "S1"
  )
  expect_error(
    align(abundance, metadata, sample_col = "sample_id", verbose = FALSE),
    regexp = "duplicated"
  )
})

test_that("align_samples() still succeeds on unique explicit sample_col", {
  align <- getFromNamespace("align_samples", "ggpicrust2")

  abundance <- matrix(
    c(1, 2, 3, 4, 5, 6),
    nrow = 2,
    dimnames = list(paste0("f", 1:2), c("S1", "S2", "S3"))
  )
  metadata <- data.frame(
    sample_id = c("S1", "S2", "S3"),
    group = c("A", "B", "C"),
    stringsAsFactors = FALSE
  )

  res <- align(abundance, metadata, sample_col = "sample_id", verbose = FALSE)
  expect_equal(res$n_samples, 3)
  expect_equal(colnames(res$abundance), c("S1", "S2", "S3"))
})


# -----------------------------------------------------------------------------
# Issue 2: aggregate_taxa_contributions() used to treat every DAA feature as
# a pathway ID, then remap to KOs. That silently zeroed-out KO-level DAA
# results. The fix dispatches on ID shape: KO -> direct match, pathway ->
# expand via ko_to_kegg.
# -----------------------------------------------------------------------------

make_contrib <- function() {
  data.frame(
    sample = rep(c("S1", "S2"), each = 4),
    function_id = rep(c("K00001", "K00002", "K00003", "K00004"), 2),
    taxon = rep(c("ASV1", "ASV2"), each = 2, times = 2),
    taxon_function_abun = seq_len(8),
    norm_taxon_function_contrib = seq(0.1, 0.8, length.out = 8),
    stringsAsFactors = FALSE
  )
}

test_that("aggregate_taxa_contributions() accepts KO-level DAA results directly", {
  contrib <- make_contrib()
  daa <- data.frame(
    feature = c("K00001", "K00002"),
    p_adjust = c(0.01, 0.02),
    stringsAsFactors = FALSE
  )
  # Before the fix this call errored with "No data remaining after filtering".
  agg <- aggregate_taxa_contributions(contrib, daa_results_df = daa, top_n = 2)
  expect_true(nrow(agg) > 0)
  expect_true(all(agg$function_id %in% c("K00001", "K00002")))
})

test_that("aggregate_taxa_contributions() errors clearly on unrecognized feature IDs", {
  contrib <- make_contrib()
  daa <- data.frame(
    feature = c("GENE_X", "GENE_Y"),
    p_adjust = c(0.01, 0.02),
    stringsAsFactors = FALSE
  )
  expect_error(
    aggregate_taxa_contributions(contrib, daa_results_df = daa),
    regexp = "KO|pathway"
  )
})

test_that("aggregate_taxa_contributions() still honors pathway-level features", {
  # Use a pathway ID that ko_to_kegg maps to at least one KO present in contrib.
  ref <- getFromNamespace("load_reference_data", "ggpicrust2")("ko_to_kegg")
  kos <- unique(ref$ko_id[ref$pathway_id == ref$pathway_id[1]])
  skip_if(length(kos) == 0, "ko_to_kegg reference unavailable")

  contrib <- data.frame(
    sample = rep(c("S1", "S2"), each = length(kos)),
    function_id = rep(kos, 2),
    taxon = rep(c("ASV1", "ASV2"), length.out = 2 * length(kos)),
    taxon_function_abun = seq_len(2 * length(kos)),
    norm_taxon_function_contrib = seq(0.01, 1, length.out = 2 * length(kos)),
    stringsAsFactors = FALSE
  )
  daa <- data.frame(
    feature = ref$pathway_id[1],
    p_adjust = 0.01,
    stringsAsFactors = FALSE
  )
  agg <- aggregate_taxa_contributions(contrib, daa_results_df = daa, top_n = 2)
  expect_true(nrow(agg) > 0)
})


# -----------------------------------------------------------------------------
# Issue 3: pathway_heatmap() with cluster_rows = TRUE crashed on constant
# rows because scale() turned them into NA, and dist()/hclust() then
# received NAs. The fix coerces post-scale NA back to 0.
# -----------------------------------------------------------------------------
test_that("pathway_heatmap() survives constant pathway rows with clustering", {
  abundance <- matrix(
    c(
      10, 20, 30, 40,   # varying
       5,  5,  5,  5,   # constant - would have NA'd after scale()
       8, 18, 28, 38    # varying
    ),
    nrow = 3, byrow = TRUE,
    dimnames = list(paste0("Pathway", 1:3), paste0("S", 1:4))
  )
  metadata <- data.frame(
    sample = paste0("S", 1:4),
    group = c("A", "A", "B", "B"),
    stringsAsFactors = FALSE
  )

  expect_silent_run <- tryCatch({
    p <- pathway_heatmap(
      abundance = abundance,
      metadata = metadata,
      group = "group",
      cluster_rows = TRUE
    )
    TRUE
  }, error = function(e) e)

  expect_true(isTRUE(expect_silent_run) || inherits(expect_silent_run, "ggplot"))
  # Check the data went through with zero (not NA) for the constant row.
  # Extract the long data directly for assertion.
  df <- p$data
  const_vals <- df$Value[df$rowname == "Pathway2"]
  expect_true(all(const_vals == 0))
  expect_false(any(is.na(df$Value)))
})


# -----------------------------------------------------------------------------
# Issue 4: compare_gsea_daa(plot_type = "scatter") crashed deep inside merge()
# when DAA results lacked log2_fold_change. The fix validates required
# columns up front with an actionable error.
# -----------------------------------------------------------------------------
test_that("compare_gsea_daa('scatter') errors when effect-size columns missing", {
  gsea <- data.frame(
    pathway_id = c("ko00010", "ko00020"),
    p.adjust = c(0.01, 0.2),
    stringsAsFactors = FALSE
  )
  daa <- data.frame(
    feature = c("ko00010", "ko00020"),
    p_adjust = c(0.02, 0.5),
    stringsAsFactors = FALSE
  )
  expect_error(
    compare_gsea_daa(gsea, daa, plot_type = "scatter"),
    regexp = "log2_fold_change|NES"
  )
})

test_that("compare_gsea_daa('venn') still works without effect-size columns", {
  gsea <- data.frame(
    pathway_id = c("ko00010", "ko00020"),
    p.adjust = c(0.01, 0.2),
    stringsAsFactors = FALSE
  )
  daa <- data.frame(
    feature = c("ko00010", "ko00030"),
    p_adjust = c(0.02, 0.01),
    stringsAsFactors = FALSE
  )
  res <- compare_gsea_daa(gsea, daa, plot_type = "venn")
  expect_equal(res$results$n_overlap, 1)
  expect_equal(res$results$overlap, "ko00010")
})

test_that("compare_gsea_daa() counts significant pathways with set semantics", {
  gsea <- data.frame(
    pathway_id = c("ko00010", "ko00020"),
    p.adjust = c(0.01, 0.2),
    stringsAsFactors = FALSE
  )
  daa <- data.frame(
    feature = c("ko00010", "ko00010", "ko00030"),
    p_adjust = c(0.02, 0.03, 0.01),
    stringsAsFactors = FALSE
  )

  res <- compare_gsea_daa(gsea, daa, plot_type = "venn")
  expect_equal(res$results$n_gsea_total, 1)
  expect_equal(res$results$n_daa_total, 2)
  expect_equal(res$results$n_overlap, 1)
  expect_equal(res$results$daa_only, "ko00030")
})


# -----------------------------------------------------------------------------
# Issue 5: compare_daa_results() used to double-filter diff_features so
# that a feature present in 2 of 3 methods was silently collapsed to 0
# disagreement. The fix defines diff = method_features \ intersection.
# -----------------------------------------------------------------------------
test_that("compare_daa_results() counts partial-agreement features as diffs", {
  # Feature A is in all three methods -> common to all.
  # Feature B is in methods 1 and 2 only -> disagreement from method 3's
  # perspective, and should appear in method 1 & 2 diff lists.
  m1 <- data.frame(feature = c("A", "B"),
                   group1 = "g1", group2 = "g2",
                   p_adjust = c(0.01, 0.02),
                   stringsAsFactors = FALSE)
  m2 <- data.frame(feature = c("A", "B"),
                   group1 = "g1", group2 = "g2",
                   p_adjust = c(0.01, 0.03),
                   stringsAsFactors = FALSE)
  m3 <- data.frame(feature = c("A"),
                   group1 = "g1", group2 = "g2",
                   p_adjust = c(0.04),
                   stringsAsFactors = FALSE)

  suppressMessages(
    out <- compare_daa_results(
      list(m1, m2, m3),
      method_names = c("m1", "m2", "m3"),
      p_values_threshold = 0.05
    )
  )

  # A is the single unanimous feature.
  expect_equal(unique(out$num_common_features), 1)

  # m1 and m2 each report B as a diff (not endorsed by m3).
  # m3 has no diffs because A is unanimous and B is absent.
  diffs <- setNames(out$num_diff_features, out$method)
  expect_equal(unname(diffs["m1"]), 1)
  expect_equal(unname(diffs["m2"]), 1)
  expect_equal(unname(diffs["m3"]), 0)

  # String column reflects the same thing.
  expect_equal(out$diff_features[out$method == "m1"], "B")
  expect_equal(out$diff_features[out$method == "m2"], "B")
})


# -----------------------------------------------------------------------------
# Issue 6: visualize_gsea() heatmap branch used to rely solely on
# rownames(metadata). When metadata had a sample column + default
# integer rownames, column annotation came out as all-NA. The fix
# routes through align_samples() so it works the same as every other
# entry point in the package.
# -----------------------------------------------------------------------------
test_that("visualize_gsea('heatmap') aligns via sample column, not just rownames", {
  skip_if_not_installed("ComplexHeatmap")
  skip_if_not_installed("circlize")

  gsea_results <- create_gsea_test_results(n_pathways = 5)

  set.seed(42)
  abundance <- as.data.frame(matrix(
    runif(50 * 8, 1, 100), nrow = 50, ncol = 8,
    dimnames = list(paste0("K", sprintf("%05d", 1:50)), paste0("Sample", 1:8))
  ))

  # Metadata uses sample_name column; rownames are default integers.
  metadata <- data.frame(
    sample_name = paste0("Sample", 1:8),
    group = rep(c("G1", "G2"), each = 4),
    stringsAsFactors = FALSE
  )
  stopifnot(identical(rownames(metadata), as.character(1:8)))

  hm <- visualize_gsea(
    gsea_results = gsea_results,
    plot_type = "heatmap",
    abundance = abundance,
    metadata = metadata,
    group = "group"
  )
  expect_s4_class(hm, "Heatmap")

  # Extract the HeatmapAnnotation and confirm Group values are not all-NA.
  # Access through top_annotation slot for ComplexHeatmap objects.
  top_anno <- hm@top_annotation
  anno_df <- top_anno@anno_list[[1]]@color_mapping@levels
  expect_true(all(c("G1", "G2") %in% anno_df))
})


# -----------------------------------------------------------------------------
# Issue 7: visualize_gsea() unnecessarily required enrichplot for
# enrichment_plot/dotplot/barplot. Those branches are pure ggplot2.
# -----------------------------------------------------------------------------
test_that("visualize_gsea() pure-ggplot branches do not require enrichplot", {
  # We cannot actually uninstall enrichplot in the test harness, so
  # instead confirm the visualize_gsea() body no longer names enrichplot
  # for these plot types. This guards against accidental re-introduction
  # of the requireNamespace() check.
  body_src <- paste(deparse(body(ggpicrust2::visualize_gsea)), collapse = "\n")
  # The helpful inline comment still mentions enrichplot, but the
  # active check -- requireNamespace("enrichplot", ...) -- should be gone
  # for enrichment_plot/dotplot/barplot.
  expect_false(grepl("requireNamespace\\(\\s*\"enrichplot\"", body_src))
})


# -----------------------------------------------------------------------------
# Issue 8: run_limma_gsea() voom fallback used <<- to inject v into the
# caller's frame. Verify the source no longer does that.
# -----------------------------------------------------------------------------
test_that("run_limma_gsea() no longer uses <<- in its voom fallback", {
  run_limma_gsea <- getFromNamespace("run_limma_gsea", "ggpicrust2")
  body_src <- paste(deparse(body(run_limma_gsea)), collapse = "\n")
  expect_false(grepl("v\\s*<<-", body_src))
  expect_false(grepl("class\\(v\\)\\s*<<-", body_src))
})

# Tests for reference data structure

test_that("ko_to_kegg_reference has correct structure and coverage", {
  ko_to_kegg_reference <- ggpicrust2:::load_reference_data("ko_to_kegg")

  # Structure
  expect_s3_class(ko_to_kegg_reference, "data.frame")
  required_cols <- c("pathway_id", "ko_id", "pathway_name")
  expect_true(all(required_cols %in% names(ko_to_kegg_reference)))

  # Coverage (thresholds based on actual KEGG pathway maps,
  # excluding BRITE classification nodes)
  expect_gte(nrow(ko_to_kegg_reference), 50000)
  expect_gte(length(unique(ko_to_kegg_reference$pathway_id)), 500)
  expect_gte(length(unique(ko_to_kegg_reference$ko_id)), 20000)

  # No NA in key columns
  expect_equal(sum(is.na(ko_to_kegg_reference$pathway_id)), 0)
  expect_equal(sum(is.na(ko_to_kegg_reference$ko_id)), 0)

  # ID format (5-digit pathway IDs only, no BRITE classification nodes)
  expect_true(all(grepl("^ko\\d{5}$", ko_to_kegg_reference$pathway_id)))
  expect_true(all(grepl("^K\\d", trimws(ko_to_kegg_reference$ko_id))))
})

create_kegg_reference_test_data <- function(n_kos, n_samples, seed = 123) {
  set.seed(seed)
  ko_to_kegg_reference <- ggpicrust2:::load_reference_data("ko_to_kegg")
  real_kos <- head(unique(ko_to_kegg_reference$ko_id), n_kos)

  data <- data.frame(function. = real_kos, stringsAsFactors = FALSE)
  for (i in seq_len(n_samples)) {
    data[[paste0("S", i)]] <- rpois(n_kos, lambda = 100)
  }
  data
}

test_that("ko2kegg_abundance works with real reference data", {
  test_data <- create_kegg_reference_test_data(100, 5)
  result <- suppressMessages(ko2kegg_abundance(data = test_data))

  expect_s3_class(result, "data.frame")
  expect_gt(nrow(result), 0)
  expect_equal(ncol(result), 5)
  expect_true(all(result >= 0))
  expect_false(any(is.na(result)))
  expect_equal(colnames(result), colnames(test_data)[-1])

  # Row names should be valid pathway IDs
  expect_true(all(grepl("^ko\\d{5}$", rownames(result))))
})

test_that("ko2kegg_abundance is deterministic", {
  test_data <- create_kegg_reference_test_data(50, 3, seed = 456)

  result1 <- suppressMessages(ko2kegg_abundance(data = test_data))
  result2 <- suppressMessages(ko2kegg_abundance(data = test_data))

  expect_identical(result1, result2)
})

test_that("ko2kegg_abundance fails fast on total KO/KEGG mismatch", {
  # Since 2.5.13 (commit 72eb140, "Fail fast on ko_to_kegg/pathway misuse"),
  # ko2kegg_abundance() errors out instead of silently returning an empty
  # matrix when no input KO maps into any KEGG pathway. Silent empties
  # cascade into cryptic downstream DAA failures ("No features in abundance
  # data") that are hard to diagnose, so the strict error is intentional.

  # Well-formed KO IDs that are not present in the KEGG reference.
  fake_data <- data.frame(function. = c("K99999", "K88888"),
                          S1 = c(100, 200),
                          stringsAsFactors = FALSE)
  expect_error(
    suppressMessages(ko2kegg_abundance(data = fake_data)),
    "No KO IDs in the input matched any KEGG pathway"
  )

  # Real KO IDs, but all-zero abundances collapse every pathway row to zero,
  # which the function treats as the same "nothing to analyze" condition.
  ko_to_kegg_reference <- ggpicrust2:::load_reference_data("ko_to_kegg")
  real_kos <- head(unique(ko_to_kegg_reference$ko_id), 10)
  zero_data <- data.frame(function. = real_kos,
                          S1 = rep(0, 10),
                          stringsAsFactors = FALSE)
  expect_error(
    suppressMessages(ko2kegg_abundance(data = zero_data)),
    "No KO IDs in the input matched any KEGG pathway"
  )
})

test_that("prepare_gene_sets creates valid gene sets from reference data", {
  gene_sets <- prepare_gene_sets("KEGG")

  expect_type(gene_sets, "list")
  expect_gt(length(gene_sets), 400)
  expect_false("ko01001" %in% names(gene_sets))
  expect_false("ko99980" %in% names(gene_sets))

  set_sizes <- sapply(gene_sets, length)
  expect_gt(min(set_sizes), 0)
  expect_lt(max(set_sizes), 5000)
})

# Tests for pathway_errorbar_table

test_that("pathway_errorbar_table auto-detects the sample column from metadata", {
  # Regression: pathway_errorbar_table() used to hardcode
  # `sample_col = "sample_name"`, so users whose metadata used the far more
  # common `sample` column got an immediate
  # "Column 'sample_name' not found in metadata". Align behavior with the
  # rest of the package (align_samples / find_sample_column) by
  # auto-detecting the sample column when `sample_col = NULL`.

  abundance <- matrix(
    c(10, 20, 30, 40,
      15, 25, 35, 45,
      12, 22, 32, 42),
    nrow = 3, byrow = TRUE,
    dimnames = list(paste0("p", 1:3), paste0("S", 1:4))
  )

  metadata <- data.frame(
    sample = paste0("S", 1:4),
    Env    = c("A", "A", "B", "B"),
    stringsAsFactors = FALSE
  )

  daa_results_df <- data.frame(
    feature   = paste0("p", 1:3),
    method    = "ALDEx2",
    group1    = "A",
    group2    = "B",
    p_values  = c(0.01, 0.2, 0.04),
    p_adjust  = c(0.03, 0.2, 0.04),
    stringsAsFactors = FALSE
  )

  # Sample column auto-detection should succeed without passing sample_col.
  res <- pathway_errorbar_table(
    abundance      = abundance,
    daa_results_df = daa_results_df,
    Group          = metadata$Env,
    metadata       = metadata,
    p_values_threshold = 0.05
  )

  expect_s3_class(res, "data.frame")
  expect_true(all(c("feature", "group1", "group2",
                    "mean_rel_abundance_group1",
                    "mean_rel_abundance_group2",
                    "p_adjust") %in% colnames(res)))
  # Only the features passing p_values_threshold should appear.
  expect_setequal(res$feature, c("p1", "p3"))
})

test_that("pathway_errorbar_table honors an explicit sample_col", {
  abundance <- matrix(
    c(10, 20, 30, 40,
      15, 25, 35, 45,
      12, 22, 32, 42),
    nrow = 3, byrow = TRUE,
    dimnames = list(paste0("p", 1:3), paste0("S", 1:4))
  )

  # Non-standard column name -- user must pass it explicitly.
  metadata <- data.frame(
    my_sample_id = paste0("S", 1:4),
    Env          = c("A", "A", "B", "B"),
    stringsAsFactors = FALSE
  )

  daa_results_df <- data.frame(
    feature   = paste0("p", 1:3),
    method    = "ALDEx2",
    group1    = "A",
    group2    = "B",
    p_values  = c(0.01, 0.2, 0.04),
    p_adjust  = c(0.03, 0.2, 0.04),
    stringsAsFactors = FALSE
  )

  res <- pathway_errorbar_table(
    abundance      = abundance,
    daa_results_df = daa_results_df,
    Group          = metadata$Env,
    metadata       = metadata,
    sample_col     = "my_sample_id",
    p_values_threshold = 0.05
  )

  expect_s3_class(res, "data.frame")
  expect_true("feature" %in% colnames(res))
})

test_that("pathway_errorbar_table rejects multi-contrast input instead of silently truncating", {
  # Regression: the previous implementation derived the two group names via
  #   group1_name <- unique(daa_results_filtered_sub_df$group1)[1]
  #   group2_name <- unique(daa_results_filtered_sub_df$group2)[1]
  # which LOOKS like "silently pick the first of many" behavior. In reality
  # the upstream validate_daa_results() was supposed to guarantee a single
  # pair, but the idiom would mask any future validator bypass by quietly
  # collapsing a multi-contrast DAA result down to its first (group1, group2)
  # block -- a dangerous "default success" where the returned table reflects
  # only one contrast out of several without any warning.
  #
  # The multi-contrast shape is exactly what pathway_daa() now emits for
  # >=3 groups (one row-block per non-reference level), so this is a real
  # user input, not a synthetic edge case.
  abundance <- matrix(
    c(10, 20, 30, 40, 50, 60,
      15, 25, 35, 45, 55, 65,
      12, 22, 32, 42, 52, 62),
    nrow = 3, byrow = TRUE,
    dimnames = list(paste0("p", 1:3), paste0("S", 1:6))
  )

  Group <- c("A", "A", "B", "B", "C", "C")

  # Two contrasts against reference "A": (A vs B) and (A vs C).
  daa_results_df <- data.frame(
    feature  = rep(paste0("p", 1:3), times = 2),
    method   = "DESeq2",
    group1   = "A",
    group2   = rep(c("B", "C"), each = 3),
    p_values = c(0.01, 0.2, 0.04, 0.03, 0.5, 0.02),
    p_adjust = c(0.02, 0.2, 0.05, 0.04, 0.5, 0.03),
    stringsAsFactors = FALSE
  )

  # Must hard-fail, not silently return a table for only the first contrast.
  expect_error(
    pathway_errorbar_table(
      abundance      = abundance,
      daa_results_df = daa_results_df,
      Group          = Group,
      p_values_threshold = 0.05
    ),
    regexp = "multiple group pairs"
  )
})

#' Compare Metagenome Results
#'
#' @param metagenomes A list of metagenomes matrices with rows as KOs and columns as samples.
#' Each matrix in the list should correspond to a different metagenome.
#' @param names A vector of names for the metagenomes in the same order as in the `metagenomes` list.
#' @param daa_method A character specifying the method for differential abundance analysis (DAA).
#' Possible choices are: "ALDEx2", "DESeq2", "edgeR", "limma voom", "metagenomeSeq", "LinDA",
#' "Maaslin2", and "Lefse". The default is "ALDEx2".
#' @param p_adjust_method A character specifying the method for p-value adjustment.
#' Possible choices are: "BH" (Benjamini-Hochberg), "holm", "bonferroni", "hochberg", "fdr", and "none".
#' The default is "BH".
#' @param reference A character specifying the reference group level for DAA.
#' This parameter is used when there are more than two groups. The default is NULL.
#' @param p.adjust Deprecated. Use \code{p_adjust_method} instead.
#' @name compare_metagenome_results
#' @return A list containing three elements:
#' \itemize{
#' \item "daa": a data frame of results from the `pathway_daa` function
#' containing the differential abundance analysis results.
#' \item "correlation": a list with two elements: "cor_matrix" and "p_matrix", which are
#' matrices of per-feature median Spearman correlation coefficients and their
#' corresponding p-values, respectively, between every pair of metagenomes.
#' \item "heatmap": a ComplexHeatmap object visualizing the correlation matrix.
#' Use \code{print()} or \code{draw()} to display it.
#' }
#'
#' @examples
#' \donttest{
#' library(dplyr)
#' library(ComplexHeatmap)
#' # Generate example data
#' set.seed(123)
#' # First metagenome
#' metagenome1 <- abs(matrix(rnorm(1000), nrow = 100, ncol = 10))
#' rownames(metagenome1) <- paste0("KO", 1:100)
#' colnames(metagenome1) <- paste0("sample", 1:10)
#' # Second metagenome
#' metagenome2 <- abs(matrix(rnorm(1000), nrow = 100, ncol = 10))
#' rownames(metagenome2) <- paste0("KO", 1:100)
#' colnames(metagenome2) <- paste0("sample", 1:10)
#' # Put the metagenomes into a list
#' metagenomes <- list(metagenome1, metagenome2)
#' # Define names
#' names <- c("metagenome1", "metagenome2")
#' # Call the function
#' results <- compare_metagenome_results(metagenomes, names, daa_method = "LinDA")
#' # Print the correlation matrix
#' print(results$correlation$cor_matrix)
#' # Display the heatmap
#' print(results$heatmap)
#' }
#' @importFrom stats cor median wilcox.test
#' @export
compare_metagenome_results <- function(metagenomes, names, daa_method = "ALDEx2",
                                       p_adjust_method = "BH", reference = NULL,
                                       p.adjust = NULL) {
  # Backward compatibility for deprecated parameter
  if (!is.null(p.adjust)) {
    warning("'p.adjust' parameter is deprecated. Use 'p_adjust_method' instead.", call. = FALSE)
    p_adjust_method <- p.adjust
  }

  if (length(metagenomes) != length(names)) {
    stop("The length of 'metagenomes' must match the length of 'names'")
  }

  # Concatenate metagenomes with pseudonym column names
  pseudonym_metagenomes <- lapply(seq_along(metagenomes), function(i) {
    new_metagenome <- metagenomes[[i]]
    colnames(new_metagenome) <- paste0(names[i], "_", colnames(new_metagenome))
    return(new_metagenome)
  })
  concatenated_metagenomes <- do.call(cbind, pseudonym_metagenomes)

  # Create a new metadata with original sample names
  new_metadata <- data.frame(sample = colnames(concatenated_metagenomes),
                             group = rep(names, sapply(metagenomes, ncol)))

  # Perform DAA
  daa_results <- pathway_daa(abundance = concatenated_metagenomes,
                             metadata = new_metadata,
                             group = "group",
                             daa_method = daa_method,
                             p_adjust_method = p_adjust_method,
                             reference = reference)

  # Compute per-feature Spearman correlations between metagenomes
  # For each pair, compute correlation per feature row, then summarize with median
  n_metagenomes <- length(names)
  cor_matrix <- matrix(NA, nrow = n_metagenomes, ncol = n_metagenomes)
  p_matrix <- matrix(NA, nrow = n_metagenomes, ncol = n_metagenomes)

  for (i in seq_along(names)) {
    for (j in seq(from = i, to = n_metagenomes)) {
      # Per-feature correlation: correlate sample profiles for each feature
      feature_cors <- sapply(seq_len(nrow(metagenomes[[i]])), function(k) {
        stats::cor(metagenomes[[i]][k, ], metagenomes[[j]][k, ], method = "spearman")
      })
      cor_matrix[i, j] <- median(feature_cors, na.rm = TRUE)
      cor_matrix[j, i] <- cor_matrix[i, j]

      # Test if median correlation differs from 0
      p_matrix[i, j] <- stats::wilcox.test(feature_cors, mu = 0)$p.value
      p_matrix[j, i] <- p_matrix[i, j]
    }
  }

  rownames(cor_matrix) <- names
  colnames(cor_matrix) <- names
  rownames(p_matrix) <- names
  colnames(p_matrix) <- names

  cor_results <- list(cor_matrix = cor_matrix, p_matrix = p_matrix)

  # Build heatmap object (return it, don't print — let the user decide)
  color_mapping <- circlize::colorRamp2(c(-1, 0, 1), c("#0571b0", "white", "#ca0020"))
  heatmap_obj <- ComplexHeatmap::Heatmap(cor_matrix, name = "correlation",
    show_row_names = TRUE, show_column_names = TRUE, col = color_mapping)

  list(daa = daa_results, correlation = cor_results, heatmap = heatmap_obj)
}

# Tests for compare_metagenome_results alignment semantics.
#
# Regression: the per-feature Spearman correlation used to index rows by
# position (metagenomes[[i]][k, ] vs metagenomes[[j]][k, ]). Two
# metagenomes with identical row names in different orders therefore
# compared *different* features and produced nonsense (often negative)
# correlations. We now align all metagenomes on the shared feature set by
# name before both the DAA cbind step and the correlation loop.

test_that("compare_metagenome_results aligns metagenomes by feature name, not by row position", {
  set.seed(1)
  n_feat <- 20
  n_samp <- 6

  m1 <- abs(matrix(rnorm(n_feat * n_samp), nrow = n_feat, ncol = n_samp))
  rownames(m1) <- paste0("K", sprintf("%04d", seq_len(n_feat)))
  colnames(m1) <- paste0("S", seq_len(n_samp))

  # m2 is m1 with rows reversed -- identical *content per feature name*,
  # but different row positions. A by-name aligned correlation must be
  # exactly 1.0 on the diagonal comparison.
  m2 <- m1[rev(rownames(m1)), , drop = FALSE]

  skip_if_not_installed("MicrobiomeStat")
  skip_if_not_installed("ComplexHeatmap")

  res <- suppressWarnings(suppressMessages(
    compare_metagenome_results(list(m1, m2),
                               names = c("m1", "m2"),
                               daa_method = "LinDA")
  ))

  # Diagonal is trivially 1.
  expect_equal(res$correlation$cor_matrix["m1", "m1"], 1)
  expect_equal(res$correlation$cor_matrix["m2", "m2"], 1)
  # Off-diagonal should be 1 (identical features), not the ~ -0.5 the
  # by-position bug produced.
  expect_equal(res$correlation$cor_matrix["m1", "m2"], 1)
  expect_equal(res$correlation$cor_matrix["m2", "m1"], 1)
})

test_that("compare_metagenome_results errors cleanly when metagenomes have no row names", {
  set.seed(2)
  m1 <- abs(matrix(rnorm(40), nrow = 10, ncol = 4))
  m2 <- abs(matrix(rnorm(40), nrow = 10, ncol = 4))
  colnames(m1) <- paste0("S", 1:4)
  colnames(m2) <- paste0("S", 1:4)
  # deliberately no rownames on m2
  rownames(m1) <- paste0("K", 1:10)

  expect_error(
    compare_metagenome_results(list(m1, m2),
                               names = c("a", "b"),
                               daa_method = "LinDA"),
    regexp = "row names"
  )
})

test_that("compare_metagenome_results errors when metagenomes share no features", {
  set.seed(3)
  m1 <- abs(matrix(rnorm(40), nrow = 10, ncol = 4))
  m2 <- abs(matrix(rnorm(40), nrow = 10, ncol = 4))
  rownames(m1) <- paste0("K", 1:10)
  rownames(m2) <- paste0("J", 1:10)  # disjoint feature namespace
  colnames(m1) <- paste0("S", 1:4)
  colnames(m2) <- paste0("S", 1:4)

  expect_error(
    compare_metagenome_results(list(m1, m2),
                               names = c("a", "b"),
                               daa_method = "LinDA"),
    regexp = "No shared feature"
  )
})

test_that("compare_metagenome_results aligns metagenomes by sample name, not by column position", {
  # Regression: the per-feature Spearman correlation used to compute
  #   cor(m[i][k, ], m[j][k, ])
  # indexing columns by POSITION. Two metagenomes with identical
  # feature/sample content but columns reordered therefore correlated
  # "sample 1 of metagenome A" against "sample 10 of metagenome B" --
  # biologically meaningless -- and produced median correlations as low
  # as -0.6 for matrices that are really identical under by-name
  # alignment. Sample columns are now intersected by name before the
  # correlation loop so this is guaranteed to be correlation = 1.
  skip_if_not_installed("MicrobiomeStat")
  skip_if_not_installed("ComplexHeatmap")

  set.seed(4)
  n_feat <- 20
  n_samp <- 6
  m1 <- abs(matrix(rnorm(n_feat * n_samp), nrow = n_feat, ncol = n_samp))
  rownames(m1) <- paste0("K", sprintf("%04d", seq_len(n_feat)))
  colnames(m1) <- paste0("S", seq_len(n_samp))

  # m2 is m1 with columns in reverse order. Same biological content,
  # only the sample-axis positions differ.
  m2 <- m1[, rev(colnames(m1)), drop = FALSE]

  res <- suppressWarnings(suppressMessages(
    compare_metagenome_results(list(m1, m2),
                               names = c("m1", "m2"),
                               daa_method = "LinDA")
  ))

  # By-name alignment must recover correlation = 1, not the ~-0.6 the
  # position-indexed bug produced on identical content.
  expect_equal(res$correlation$cor_matrix["m1", "m2"], 1)
  expect_equal(res$correlation$cor_matrix["m2", "m1"], 1)
})

test_that("compare_metagenome_results errors cleanly when metagenomes have no sample names", {
  # Parallel to the missing-rownames test: without colnames we cannot
  # align samples by name, and position alignment is the bug we are
  # preventing.
  set.seed(5)
  m1 <- abs(matrix(rnorm(40), nrow = 10, ncol = 4))
  m2 <- abs(matrix(rnorm(40), nrow = 10, ncol = 4))
  rownames(m1) <- paste0("K", 1:10)
  rownames(m2) <- paste0("K", 1:10)
  colnames(m1) <- paste0("S", 1:4)
  # deliberately no colnames on m2

  expect_error(
    compare_metagenome_results(list(m1, m2),
                               names = c("a", "b"),
                               daa_method = "LinDA"),
    regexp = "column names"
  )
})

test_that("compare_metagenome_results rejects metagenomes whose sample sets do not overlap", {
  # Mismatched column counts used to fall through to stats::cor() and
  # abort mid-loop with "incompatible dimensions". Disjoint sample sets
  # now fail fast at the boundary with an actionable message.
  set.seed(6)
  m1 <- abs(matrix(rnorm(40), nrow = 10, ncol = 4))
  m2 <- abs(matrix(rnorm(60), nrow = 10, ncol = 6))
  rownames(m1) <- paste0("K", 1:10)
  rownames(m2) <- paste0("K", 1:10)
  colnames(m1) <- paste0("S",  1:4)
  colnames(m2) <- paste0("S2_", 1:6)  # disjoint sample namespace

  expect_error(
    compare_metagenome_results(list(m1, m2),
                               names = c("a", "b"),
                               daa_method = "LinDA"),
    regexp = "No shared sample"
  )
})

test_that("compare_metagenome_results warns when sample intersection drops samples", {
  # A partial sample overlap is almost always a user mistake (parallel
  # quantifications should share the same biological samples). Proceed
  # on the shared set but make the drop visible.
  skip_if_not_installed("MicrobiomeStat")
  skip_if_not_installed("ComplexHeatmap")

  set.seed(7)
  # m1 has samples S1..S4, m2 has samples S1..S3 plus a unique S_extra.
  # Shared set is S1..S3; m1 drops S4, m2 drops S_extra.
  n_feat <- 15
  feat_names <- paste0("K", sprintf("%03d", seq_len(n_feat)))

  m1 <- abs(matrix(rnorm(n_feat * 4), nrow = n_feat, ncol = 4))
  rownames(m1) <- feat_names
  colnames(m1) <- c(paste0("S", 1:3), "S4")

  m2 <- abs(matrix(rnorm(n_feat * 4), nrow = n_feat, ncol = 4))
  rownames(m2) <- feat_names
  colnames(m2) <- c(paste0("S", 1:3), "S_extra")

  expect_warning(
    suppressMessages(
      compare_metagenome_results(list(m1, m2),
                                 names = c("m1", "m2"),
                                 daa_method = "LinDA")
    ),
    regexp = "sample"
  )
})

#' This function integrates pathway name/description annotations, ten of the most advanced differential abundance (DA) methods, and visualization of DA results.
#'
#' @param file A character string representing the file path of the input file containing KO abundance data in picrust2 export format. The input file should have KO identifiers in the first column and sample identifiers in the first row. The remaining cells should contain the abundance values for each KO-sample pair.
#' @param data An optional data.frame containing KO abundance data in the same format as the input file. If provided, the function will use this data instead of reading from the file. By default, this parameter is set to NULL.
#' @param metadata A tibble, consisting of sample information
#' @param group A character, name of the group
#' @param pathway A character, consisting of "EC", "KO", "MetaCyc"
#' @param daa_method a character specifying the method for differential abundance analysis, default is "ALDEx2", choices are:
#' - "ALDEx2": ANOVA-Like Differential Expression tool for high throughput sequencing data
#' - "DESeq2": Differential expression analysis based on the negative binomial distribution using DESeq2
#' - "edgeR": Exact test for differences between two groups of negative-binomially distributed counts using edgeR
#' - "limma voom": Limma-voom framework for the analysis of RNA-seq data
#' - "metagenomeSeq": Fit logistic regression models to test for differential abundance between groups using metagenomeSeq
#' - "LinDA": Linear models for differential abundance analysis of microbiome compositional data
#' - "Maaslin2": Multivariate Association with Linear Models (MaAsLin2) for differential abundance analysis
#' @param ko_to_kegg Logical or logical-like string controlling conversion of
#'   KO abundance to KEGG pathway abundance.
#' @param filter_for_prokaryotes Logical. If TRUE (default), filters out KEGG pathways
#'   that are specific to eukaryotes (e.g., human diseases, organismal systems) when
#'   ko_to_kegg = TRUE. Set to FALSE to include all KEGG pathways.
#' @param p_adjust_method A character specifying the method for p-value adjustment,
#'   default is "BH".
#' @param p.adjust a character specifying the method for p-value adjustment, default is "BH", choices are:
#'- "BH": Benjamini-Hochberg correction
#'- "holm": Holm's correction
#'- "bonferroni": Bonferroni correction
#'- "hochberg": Hochberg's correction
#'- "fdr": False discovery rate correction
#'- "none": No p-value adjustment.
#' @param order A character to control the order of the main plot rows
#' @param p_values_bar A character to control if the main plot has the p_values bar
#' @param x_lab A character to control the x-axis label name, you can choose from "feature","pathway_name" and "description"
#' @param select A vector consisting of pathway names to be selected
#' @param reference A character, a reference group level for several DA methods
#' @param colors A vector consisting of colors number
#' @param p_values_threshold A numeric value specifying the threshold for statistical
#'   significance of differential abundance. Pathways with adjusted p-values below this
#'   threshold will be displayed in the plot. Default is 0.05.
#'
#' @return A list containing:
#' \itemize{
#'   \item Numbered elements (1, 2, ...): Sub-lists for each DA method, each containing:
#'     \itemize{
#'       \item \code{plot}: A ggplot2 error bar plot visualizing the differential abundance results
#'       \item \code{results}: A data frame of differential abundance results for that method
#'     }
#'   \item \code{abundance}: The processed abundance data (KEGG pathway or original) for downstream analysis
#'   \item \code{metadata}: The metadata data frame
#'   \item \code{group}: The group variable name used in the analysis
#'   \item \code{daa_results_df}: The complete annotated DAA results data frame
#'   \item \code{ko_to_kegg}: Logical indicating whether KO to KEGG conversion was performed
#' }
#' These additional fields allow seamless integration with \code{\link{pathway_pca}} and
#' \code{\link{pathway_heatmap}} for further visualization without re-preparing data.
#' @export
#' @examples
#' \dontrun{
#' # Load necessary data: abundance data and metadata
#' abundance_file <- "path/to/your/abundance_file.tsv"
#' metadata <- read.csv("path/to/your/metadata.csv")
#'
#' # Run ggpicrust2 with input file path
#' results_file_input <- ggpicrust2(file = abundance_file,
#'                                  metadata = metadata,
#'                                  group = "your_group_column",
#'                                  pathway = "KO",
#'                                  daa_method = "LinDA",
#'                                  ko_to_kegg = "TRUE",
#'                                  order = "pathway_class",
#'                                  p_values_bar = TRUE,
#'                                  x_lab = "pathway_name")
#'
#' # Run ggpicrust2 with imported data.frame
#' abundance_data <- read_delim(abundance_file, delim="\t", col_names=TRUE, trim_ws=TRUE)
#'
#' # Run ggpicrust2 with input data
#' results_data_input <- ggpicrust2(data = abundance_data,
#'                                  metadata = metadata,
#'                                  group = "your_group_column",
#'                                  pathway = "KO",
#'                                  daa_method = "LinDA",
#'                                  ko_to_kegg = "TRUE",
#'                                  order = "pathway_class",
#'                                  p_values_bar = TRUE,
#'                                  x_lab = "pathway_name")
#'
#' # Access the plot and results dataframe for the first DA method
#' example_plot <- results_file_input[[1]]$plot
#' example_results <- results_file_input[[1]]$results
#'
#' # Use the example data in ggpicrust2 package
#' data(ko_abundance)
#' data(metadata)
#' results_file_input <- ggpicrust2(data = ko_abundance,
#'                                  metadata = metadata,
#'                                  group = "Environment",
#'                                  pathway = "KO",
#'                                  daa_method = "LinDA",
#'                                  ko_to_kegg = TRUE,
#'                                  order = "pathway_class",
#'                                  p_values_bar = TRUE,
#'                                  x_lab = "pathway_name")
#' # Analyze the EC or MetaCyc pathway
#' data(metacyc_abundance)
#' results_file_input <- ggpicrust2(data = metacyc_abundance,
#'                                  metadata = metadata,
#'                                  group = "Environment",
#'                                  pathway = "MetaCyc",
#'                                  daa_method = "LinDA",
#'                                  ko_to_kegg = FALSE,
#'                                  order = "group",
#'                                  p_values_bar = TRUE,
#'                                  x_lab = "description")
#'
#' # Use the returned data for PCA analysis (no need to re-prepare data)
#' pca_plot <- pathway_pca(
#'   abundance = results_file_input$abundance,
#'   metadata = results_file_input$metadata,
#'   group = results_file_input$group
#' )
#'
#' # Use the returned data for heatmap (filter significant pathways first)
#' sig_features <- results_file_input$daa_results_df %>%
#'   dplyr::filter(p_adjust < 0.05) %>%
#'   dplyr::pull(feature)
#' if (length(sig_features) > 0) {
#'   heatmap_plot <- pathway_heatmap(
#'     abundance = results_file_input$abundance[sig_features, , drop = FALSE],
#'     metadata = results_file_input$metadata,
#'     group = results_file_input$group
#'   )
#' }
#'}
ggpicrust2 <- function(file = NULL,
                       data = NULL,
                       metadata,
                       group,
                       pathway,
                       daa_method = "ALDEx2",
                       ko_to_kegg = FALSE,
                       filter_for_prokaryotes = TRUE,
                       p_adjust_method = "BH",
                       order = "group",
                       p_values_bar = TRUE,
                       x_lab = NULL,
                       select = NULL,
                       reference = NULL,
                       colors = NULL,
                       p_values_threshold = 0.05,
                       p.adjust = NULL) {
  # Backward compatibility for the deprecated `p.adjust` parameter. We
  # keep it accepted (with NULL default so we can distinguish unset from
  # set) but migrate users onto `p_adjust_method` to match the rest of
  # the package. Only warn when the caller actually supplied it.
  if (!is.null(p.adjust)) {
    warning("'p.adjust' parameter is deprecated. Use 'p_adjust_method' instead.",
            call. = FALSE)
    p_adjust_method <- p.adjust
  }
  ko_to_kegg <- normalize_logical_flag(ko_to_kegg, "ko_to_kegg")
  filter_for_prokaryotes <- normalize_logical_flag(filter_for_prokaryotes, "filter_for_prokaryotes")

  # Input validation
  if (is.null(file) && is.null(data)) {
    stop("Error: Please provide either a 'file' path (character string) or a 'data' data frame.")
  }

  if (!is.null(file) && !is.null(data)) {
    warning("Both 'file' and 'data' provided. Using 'data' and ignoring 'file'.")
    file <- NULL  # Set file to NULL to use data
  }

  # Validate file parameter if provided
  if (!is.null(file)) {
    if (!is.character(file) || length(file) != 1) {
      stop("Error: 'file' parameter must be a character string representing a file path. If you have already loaded your data into R, please use the 'data' parameter instead.")
    }
    if (!file.exists(file)) {
      stop("Error: File does not exist: ", file)
    }
  }

  # Validate data parameter if provided
  if (!is.null(data)) {
    if (!is.data.frame(data)) {
      stop("Error: 'data' parameter must be a data frame. If you want to provide a file path, please use the 'file' parameter instead.")
    }
  }

  # Validate daa_method early. Lefser returns LDA effect sizes, not
  # p-values per pathway, so it cannot drive the downstream
  # pathway_errorbar() step that ggpicrust2() produces. Accept the
  # historical misspelling "Lefse" for backward compatibility but
  # forward it through as an error with the canonical name.
  if (daa_method %in% c("Lefser", "Lefse")) {
    stop("The 'Lefser' method is not suitable for ggpicrust2() as it does not output p-values.")
  }

  # `ko_to_kegg = TRUE` aggregates KO abundances into KEGG pathways, so it is
  # only meaningful when the input is KO abundance (pathway = "KO"). Pairing it
  # with EC/MetaCyc produces an empty matrix and a cryptic downstream error.
  if (ko_to_kegg && !identical(pathway, "KO")) {
    stop(sprintf(
      "`ko_to_kegg = TRUE` requires `pathway = \"KO\"`, but got pathway = \"%s\".\n  - For EC or MetaCyc data, set `ko_to_kegg = FALSE`.\n  - For KO -> KEGG pathway analysis, set `pathway = \"KO\"`.",
      pathway))
  }

  # Step 1: Load abundance data
  if (ko_to_kegg) {
    message("Converting KO to KEGG...\n")
    abundance <- if (!is.null(file)) {
      ko2kegg_abundance(file, filter_for_prokaryotes = filter_for_prokaryotes)
    } else {
      ko2kegg_abundance(data = data, filter_for_prokaryotes = filter_for_prokaryotes)
    }
  } else {
    message("Reading input data...\n")
    abundance <- if (!is.null(file)) {
      read_abundance_file(file)
    } else {
      data
    }
    abundance <- as.data.frame(abundance)
    abundance <- clean_ko_abundance(abundance)
    rownames(abundance) <- abundance[, 1]
    abundance <- abundance[, -1]
  }

  # Align once at the wrapper boundary. Both DAA and plotting consume this
  # same sample order, so pathway_daa() is called with an explicit
  # pre-aligned contract below instead of redoing the same alignment.
  aligned <- align_samples(abundance, metadata, verbose = FALSE)
  abundance <- aligned$abundance
  metadata <- aligned$metadata

  # Step 2: Differential abundance analysis
  #
  # Do NOT forward `select` to pathway_daa() here. In this wrapper `select`
  # is documented as pathway/feature names (see @param select -- "A vector
  # consisting of pathway names to be selected"), whereas pathway_daa()'s
  # own `select` argument means sample names. Passing pathway names into
  # pathway_daa() immediately failed with "Some selected samples not in
  # abundance data". The user-supplied `select` is forwarded to
  # pathway_errorbar() below, which is where feature-level filtering for
  # the plot actually happens; pathway_daa() runs on the full sample set.
  daa_results_df <- pathway_daa(
    abundance = abundance,
    metadata = metadata,
    group = group,
    daa_method = daa_method,
    p_adjust_method = p_adjust_method,
    reference = reference,
    .pre_aligned = TRUE,
    .sample_col = aligned$sample_col
  )

  # Check for significant biomarkers
  num_significant <- sum(daa_results_df$p_adjust < p_values_threshold, na.rm = TRUE)
  if (num_significant == 0) {
    warning(sprintf("No statistically significant biomarkers found (p_adjust < %g). ", p_values_threshold),
            "Analysis will continue for visualization purposes.", call. = FALSE)
  }

  # Step 3: Pathway annotation
  message("Annotating pathways...\n")
  daa_results_df <- pathway_annotation(
    pathway = pathway,
    ko_to_kegg = ko_to_kegg,
    daa_results_df = daa_results_df,
    p_adjust_threshold = p_values_threshold
  )

  # Step 4: Create plots for each method
  message("Creating pathway error bar plots...\n")
  plot_result_list <- list()
  Group_vec <- metadata[[group]]
  names(Group_vec) <- colnames(abundance)

  for (i in seq_along(unique(daa_results_df$method))) {
    method_name <- unique(daa_results_df$method)[i]
    daa_sub_method_results_df <- daa_results_df[daa_results_df$method == method_name, ]

    combination_bar_plot <- pathway_errorbar(
      abundance = abundance,
      daa_results_df = daa_sub_method_results_df,
      Group = Group_vec,
      ko_to_kegg = ko_to_kegg,
      p_value_bar = p_values_bar,
      order = order,
      colors = colors,
      select = select,
      x_lab = x_lab,
      p_values_threshold = p_values_threshold
    )

    if (is.null(combination_bar_plot)) {
      message(sprintf("Plot %d skipped (no data for method: %s)\n", i, method_name))
    } else {
      message(sprintf("Plot %d created.\n", i))
    }

    plot_result_list[[i]] <- list(plot = combination_bar_plot, results = daa_sub_method_results_df)
  }

  message("ggpicrust2 analysis completed.\n")

  # Step 5: Add data for downstream analysis

  plot_result_list$abundance <- abundance
  plot_result_list$metadata <- metadata
  plot_result_list$group <- group
  plot_result_list$daa_results_df <- daa_results_df
  plot_result_list$ko_to_kegg <- ko_to_kegg

  return(plot_result_list)
}

# Tests for ggpicrust2() return structure

test_that("ggpicrust2 returns all expected fields", {
  skip_if(
    Sys.getenv("GGPICRUST2_RUN_E2E_TESTS", "false") != "true",
    "Set GGPICRUST2_RUN_E2E_TESTS=true to run full ggpicrust2 end-to-end tests."
  )
  skip_on_cran()
  skip_if_not_installed("ALDEx2")

  data(ko_abundance, package = "ggpicrust2")
  data(metadata, package = "ggpicrust2")

  result <- suppressMessages(ggpicrust2(
    data = ko_abundance,
    metadata = metadata,
    group = "Environment",
    pathway = "KO",
    daa_method = "ALDEx2",
    ko_to_kegg = TRUE
  ))

  # Check structure
  expect_type(result, "list")
  expect_true(all(c("abundance", "metadata", "group", "daa_results_df", "ko_to_kegg") %in% names(result)))

  # Check field types
  expect_true(is.data.frame(result$abundance) || is.matrix(result$abundance))
  expect_true(is.data.frame(result$metadata))
  expect_true(is.data.frame(result$daa_results_df))
  expect_equal(result$group, "Environment")
  expect_true(result$ko_to_kegg)

  # Backward compatibility: old API returned unnamed list elements accessible
  # by position. result[[1]] is the first DA method's plot+results.
  expect_true(!is.null(result[[1]]))
  expect_true(all(c("plot", "results") %in% names(result[[1]])))
})

test_that("ggpicrust2 aligns Group vector to abundance sample order before plotting", {
  captured_group <- NULL

  mock_abundance <- data.frame(
    S_B = c(10, 20),
    S_A = c(30, 40),
    row.names = c("K00001", "K00002"),
    check.names = FALSE
  )

  metadata <- data.frame(
    sample = c("S_A", "S_B"),
    Environment = c("A", "B"),
    stringsAsFactors = FALSE
  )

  mock_pathway_daa <- function(abundance, metadata, group, daa_method, select = NULL,
                               p_adjust_method, reference, .pre_aligned = FALSE,
                               .sample_col = NULL) {
    expect_true(.pre_aligned)
    expect_true(.sample_col %in% colnames(metadata))
    data.frame(
      feature = rownames(abundance)[1],
      method = "mock_method",
      p_values = 0.01,
      adj_method = "BH",
      p_adjust = 0.01,
      group1 = "A",
      group2 = "B",
      stringsAsFactors = FALSE
    )
  }

  mock_pathway_annotation <- function(pathway, ko_to_kegg, daa_results_df, p_adjust_threshold) {
    daa_results_df$pathway_name <- daa_results_df$feature
    daa_results_df$pathway_class <- "Class1"
    daa_results_df
  }

  mock_pathway_errorbar <- function(abundance, daa_results_df, Group, ko_to_kegg, p_value_bar, order, colors, select, x_lab, p_values_threshold) {
    captured_group <<- Group
    NULL
  }

  result <- testthat::with_mocked_bindings(
    ggpicrust2(
      data = data.frame(function. = rownames(mock_abundance), mock_abundance, check.names = FALSE),
      metadata = metadata,
      group = "Environment",
      pathway = "KO",
      daa_method = "ALDEx2",
      ko_to_kegg = FALSE
    ),
    pathway_daa = mock_pathway_daa,
    pathway_annotation = mock_pathway_annotation,
    pathway_errorbar = mock_pathway_errorbar
  )

  expect_type(result, "list")
  expect_equal(names(captured_group), c("S_B", "S_A"))
  expect_equal(unname(captured_group), c("B", "A"))
})

test_that("ggpicrust2 rejects inconsistent ko_to_kegg / pathway combinations", {
  # Catch the user-intent conflict up front: ko_to_kegg = TRUE aggregates KO
  # counts into KEGG pathways, so it is only meaningful when pathway = "KO".
  # Previously this combination produced a cryptic "No features in abundance
  # data" error after the matrix collapsed to zero rows downstream.
  minimal_data <- data.frame(
    function. = c("1.1.1.1", "1.1.1.2"),
    S1 = c(10, 20),
    S2 = c(15, 25),
    stringsAsFactors = FALSE
  )
  minimal_metadata <- data.frame(
    sample = c("S1", "S2"),
    Condition = c("A", "B"),
    stringsAsFactors = FALSE
  )

  expect_error(
    ggpicrust2(
      data = minimal_data,
      metadata = minimal_metadata,
      group = "Condition",
      pathway = "EC",
      daa_method = "LinDA",
      ko_to_kegg = TRUE
    ),
    "requires .pathway = .KO."
  )

  expect_error(
    ggpicrust2(
      data = minimal_data,
      metadata = minimal_metadata,
      group = "Condition",
      pathway = "MetaCyc",
      daa_method = "LinDA",
      ko_to_kegg = TRUE
    ),
    "requires .pathway = .KO."
  )

  expect_error(
    ggpicrust2(
      data = minimal_data,
      metadata = minimal_metadata,
      group = "Condition",
      pathway = "MetaCyc",
      daa_method = "LinDA",
      ko_to_kegg = "TRUE"
    ),
    "requires .pathway = .KO."
  )
})

test_that("ggpicrust2 does not emit p.adjust deprecation warning on a normal call", {
  # Regression: ggpicrust2() had been renamed to accept `p_adjust_method`,
  # but its internal pathway_daa() call still passed `p.adjust = p.adjust`.
  # That made every normal ggpicrust2() call trigger the deprecation
  # warning that is supposed to fire only when a user explicitly supplies
  # the legacy argument.
  skip_if_not_installed("MicrobiomeStat")

  set.seed(42)
  n_ko <- 15; n_samp <- 8
  abund <- matrix(rpois(n_ko * n_samp, lambda = 50),
                  nrow = n_ko, ncol = n_samp)
  rownames(abund) <- paste0("K", sprintf("%05d", seq_len(n_ko)))
  colnames(abund) <- paste0("S", seq_len(n_samp))
  abund_df <- cbind(
    `function` = rownames(abund),
    as.data.frame(abund)
  )
  meta <- data.frame(
    sample_name = colnames(abund),
    Env = rep(c("A", "B"), each = n_samp / 2),
    stringsAsFactors = FALSE
  )

  dep_msgs <- character(0)
  tryCatch(
    withCallingHandlers(
      suppressMessages(ggpicrust2(
        data = abund_df,
        metadata = meta,
        group = "Env",
        pathway = "KO",
        daa_method = "LinDA",
        ko_to_kegg = FALSE
      )),
      warning = function(w) {
        if (grepl("p.adjust.*deprecated", conditionMessage(w))) {
          dep_msgs <<- c(dep_msgs, conditionMessage(w))
        }
        invokeRestart("muffleWarning")
      }
    ),
    error = function(e) invisible(NULL)   # unrelated downstream errors OK
  )
  expect_length(dep_msgs, 0)
})

test_that("ggpicrust2 does not forward `select` (pathway names) to pathway_daa's sample-name `select`", {
  # Regression: `select` is documented on ggpicrust2() as a vector of
  # pathway names for plot-time feature selection (@param select). The
  # wrapper, however, forwarded it unchanged to pathway_daa() where
  # `select` means SAMPLE names, so `ggpicrust2(..., select =
  # pathway_names)` aborted immediately inside pathway_daa() with
  # "Some selected samples not in abundance data". The same value was
  # also forwarded to pathway_errorbar() where its documented meaning
  # (feature selection) is correct; the fix is to stop sending it into
  # pathway_daa() so its only downstream consumer is the errorbar step.
  captured_daa_select   <- "SENTINEL"
  captured_errbar_select <- "SENTINEL"

  mock_abundance <- data.frame(
    S_A = c(10, 20, 30),
    S_B = c(15, 25, 35),
    row.names = c("K00001", "K00002", "K00003"),
    check.names = FALSE
  )

  metadata <- data.frame(
    sample = c("S_A", "S_B"),
    Environment = c("A", "B"),
    stringsAsFactors = FALSE
  )

  mock_pathway_daa <- function(abundance, metadata, group, daa_method,
                               p_adjust_method, reference, .pre_aligned = FALSE,
                               .sample_col = NULL, ...) {
    # Capture whether a `select` arg made it through ... (it should not).
    extra <- list(...)
    captured_daa_select <<- if ("select" %in% names(extra)) extra$select else NULL
    expect_true(.pre_aligned)
    expect_equal(.sample_col, "sample")
    data.frame(
      feature  = rownames(abundance),
      method   = "mock_method",
      p_values = c(0.001, 0.5, 0.001),
      adj_method = "BH",
      p_adjust = c(0.001, 0.5, 0.001),
      group1   = "A",
      group2   = "B",
      stringsAsFactors = FALSE
    )
  }

  mock_pathway_annotation <- function(pathway, ko_to_kegg, daa_results_df,
                                      p_adjust_threshold) {
    daa_results_df$pathway_name  <- daa_results_df$feature
    daa_results_df$pathway_class <- "Class1"
    daa_results_df
  }

  mock_pathway_errorbar <- function(abundance, daa_results_df, Group,
                                    ko_to_kegg, p_value_bar, order,
                                    colors, select, x_lab,
                                    p_values_threshold) {
    captured_errbar_select <<- select
    NULL
  }

  pathway_ids <- rownames(mock_abundance)

  testthat::with_mocked_bindings(
    ggpicrust2(
      data = data.frame(function. = rownames(mock_abundance),
                        mock_abundance, check.names = FALSE),
      metadata   = metadata,
      group      = "Environment",
      pathway    = "KO",
      daa_method = "ALDEx2",
      ko_to_kegg = FALSE,
      select     = pathway_ids  # pathway names, per the docstring
    ),
    pathway_daa         = mock_pathway_daa,
    pathway_annotation  = mock_pathway_annotation,
    pathway_errorbar    = mock_pathway_errorbar
  )

  # pathway_daa must receive no `select` at all (so it runs on the full
  # sample set), while pathway_errorbar must receive the pathway names
  # the user passed in -- that is where feature-level filtering belongs.
  expect_null(captured_daa_select)
  expect_equal(captured_errbar_select, pathway_ids)
})

test_that("ggpicrust2 still warns when caller passes legacy p.adjust argument", {
  skip_if_not_installed("MicrobiomeStat")

  set.seed(42)
  n_ko <- 10; n_samp <- 6
  abund <- matrix(rpois(n_ko * n_samp, lambda = 50),
                  nrow = n_ko, ncol = n_samp)
  rownames(abund) <- paste0("K", sprintf("%05d", seq_len(n_ko)))
  colnames(abund) <- paste0("S", seq_len(n_samp))
  abund_df <- cbind(
    `function` = rownames(abund),
    as.data.frame(abund)
  )
  meta <- data.frame(
    sample_name = colnames(abund),
    Env = rep(c("A", "B"), each = n_samp / 2),
    stringsAsFactors = FALSE
  )

  got_dep <- FALSE
  tryCatch(
    withCallingHandlers(
      suppressMessages(ggpicrust2(
        data = abund_df,
        metadata = meta,
        group = "Env",
        pathway = "KO",
        daa_method = "LinDA",
        ko_to_kegg = FALSE,
        p.adjust = "BH"
      )),
      warning = function(w) {
        if (grepl("p.adjust.*deprecated", conditionMessage(w))) {
          got_dep <<- TRUE
        }
        invokeRestart("muffleWarning")
      }
    ),
    error = function(e) invisible(NULL)
  )
  expect_true(got_dep)
})

#' @importFrom magrittr %>%
#' @importFrom stats wilcox.test setNames
NULL

#' Differential Abundance Analysis for Predicted Functional Pathways
#'
#' @name pathway_daa
#'
#' @description
#' Performs differential abundance analysis on predicted functional pathway data using various statistical methods.
#' This function supports multiple methods for analyzing differences in pathway abundance between groups,
#' including popular approaches like ALDEx2, DESeq2, edgeR, and others.
#'
#' @param abundance A data frame or matrix containing predicted functional pathway abundance,
#'        with pathways/features as rows and samples as columns.
#'        The column names should match the sample names in metadata.
#'        Values should be counts or abundance measurements.
#'
#' @param metadata A data frame or tibble containing sample information.
#'        Must include a 'sample' column with sample identifiers matching the column names in abundance data.
#'
#' @param group Character string specifying the column name in metadata that contains group information
#'        for differential abundance analysis.
#'
#' @param daa_method Character string specifying the method for differential abundance analysis.
#'        Available choices are:
#'        \itemize{
#'          \item \code{"ALDEx2"}: ANOVA-Like Differential Expression tool
#'          \item \code{"DESeq2"}: Differential expression analysis based on negative binomial distribution
#'          \item \code{"edgeR"}: Exact test for differences between groups using negative binomial model
#'          \item \code{"limma voom"}: Limma-voom framework for RNA-seq analysis
#'          \item \code{"metagenomeSeq"}: Zero-inflated Gaussian mixture model
#'          \item \code{"LinDA"}: Linear models for differential abundance analysis
#'          \item \code{"Maaslin2"}: Multivariate Association with Linear Models
#'          \item \code{"Lefser"}: Linear discriminant analysis effect size
#'        }
#'        Default is "ALDEx2".
#'
#' @param select Vector of sample names to include in the analysis.
#'        If NULL (default), all samples are included.
#'
#' @param p_adjust_method Character string specifying the method for p-value adjustment.
#'        Choices are:
#'        \itemize{
#'          \item \code{"BH"}: Benjamini-Hochberg procedure (default)
#'          \item \code{"holm"}: Holm's step-down method
#'          \item \code{"bonferroni"}: Bonferroni correction
#'          \item \code{"hochberg"}: Hochberg's step-up method
#'          \item \code{"fdr"}: False Discovery Rate
#'          \item \code{"none"}: No adjustment
#'        }
#'
#' @param reference Character string specifying the reference level for the group comparison.
#'        If NULL (default), the first level is used as reference.
#'
#' @param include_abundance_stats Logical value indicating whether to include
#'        abundance statistics (mean relative abundance and standard deviation
#'        per group) in the output. Default is FALSE. When the selected
#'        \code{daa_method} already provides a \code{log2_fold_change} column
#'        (ALDEx2 with effect size, DESeq2, edgeR, limma voom, LinDA, Maaslin2,
#'        metagenomeSeq), the method-native log2 fold change is preserved and
#'        the relative-abundance ratio is not recomputed.
#'
#' @param include_effect_size Logical value indicating whether to compute
#'        ALDEx2 effect size information via \code{ALDEx2::aldex.effect()}.
#'        When TRUE, adds \code{effect_size}, \code{diff_btw},
#'        \code{log2_fold_change}, \code{rab_all}, and \code{overlap} columns,
#'        aligning ALDEx2 output with the other DAA methods that return log2
#'        fold changes by default. Only applicable for two-group comparisons
#'        with the ALDEx2 method; ignored otherwise. Default is TRUE; set to
#'        FALSE to skip the extra \code{aldex.effect()} computation.
#'
#' @param p.adjust Deprecated. Use \code{p_adjust_method} instead.
#' @param .pre_aligned Internal logical. Set to TRUE only when the caller has
#'        already aligned abundance columns and metadata rows in identical
#'        sample order.
#' @param .sample_col Internal character. Sample identifier column used when
#'        \code{.pre_aligned = TRUE}.
#'
#' @param ... Additional arguments passed to the specific DAA method
#'
#' @return A data frame containing the differential abundance analysis results. The structure of the results
#' depends on the chosen DAA method. For methods that support multi-group comparisons (like LinDA),
#' when there are more than two groups, the results will contain separate rows for each feature in each
#' pairwise comparison between the reference group and each non-reference group. The data frame includes
#' the following columns:
#' \itemize{
#'   \item \code{feature}: Feature/pathway identifier
#'   \item \code{method}: The DAA method used
#'   \item \code{group1}: Reference group
#'   \item \code{group2}: Comparison group
#'   \item \code{p_values}: P-values for the comparison
#'   \item \code{p_adjust}: Adjusted p-values
#'   \item \code{adj_method}: Method used for p-value adjustment
#' }
#'
#' Methods that fit a model on the abundance data (DESeq2, edgeR, limma voom,
#' LinDA, Maaslin2, metagenomeSeq) return a \code{log2_fold_change} column
#' computed in the method's own model space. ALDEx2 returns
#' \code{log2_fold_change} (plus \code{effect_size}, \code{diff_btw},
#' \code{rab_all}, \code{overlap}) when \code{include_effect_size = TRUE}
#' (the default), derived from \code{ALDEx2::aldex.effect()} in CLR space.
#' Lefser returns an \code{lda_score} column instead, which is its native
#' effect-size metric.
#'
#' When \code{include_abundance_stats = TRUE}, the following additional columns
#' are included:
#' \itemize{
#'   \item \code{mean_rel_abundance_group1}: Mean relative abundance for group1
#'   \item \code{sd_rel_abundance_group1}: Standard deviation of relative
#'         abundance for group1
#'   \item \code{mean_rel_abundance_group2}: Mean relative abundance for group2
#'   \item \code{sd_rel_abundance_group2}: Standard deviation of relative
#'         abundance for group2
#' }
#' A \code{log2_fold_change} column from relative abundance is only added when
#' the DAA method does not already provide one, to avoid conflating model-based
#' and ratio-based effect sizes.
#'
#' @examples
#' \donttest{
#' # Load example data
#' data(ko_abundance)
#' data(metadata)
#'
#' # Prepare abundance data
#' abundance_data <- as.data.frame(ko_abundance)
#' rownames(abundance_data) <- abundance_data[, "#NAME"]
#' abundance_data <- abundance_data[, -1]
#'
#' # Run differential abundance analysis using ALDEx2
#' results <- pathway_daa(
#'   abundance = abundance_data,
#'   metadata = metadata,
#'   group = "Environment"
#' )
#'
#' # Using a different method (DESeq2)
#' deseq_results <- pathway_daa(
#'   abundance = abundance_data,
#'   metadata = metadata,
#'   group = "Environment",
#'   daa_method = "DESeq2"
#' )
#'
#' # Create example data with more samples
#' abundance <- data.frame(
#'   sample1 = c(10, 20, 30),
#'   sample2 = c(20, 30, 40),
#'   sample3 = c(30, 40, 50),
#'   sample4 = c(40, 50, 60),
#'   sample5 = c(50, 60, 70),
#'   row.names = c("pathway1", "pathway2", "pathway3")
#' )
#'
#' metadata <- data.frame(
#'   sample = c("sample1", "sample2", "sample3", "sample4", "sample5"),
#'   group = c("control", "control", "treatment", "treatment", "treatment")
#' )
#'
#' # Run differential abundance analysis using ALDEx2
#' results <- pathway_daa(abundance, metadata, "group")
#'
#' # Using a different method (limma voom instead of DESeq2 for this small example)
#' limma_results <- pathway_daa(abundance, metadata, "group",
#'                             daa_method = "limma voom")
#'
#' # Analyze specific samples only
#' subset_results <- pathway_daa(abundance, metadata, "group",
#'                              select = c("sample1", "sample2", "sample3", "sample4"))
#'
#' # ALDEx2 returns effect size columns by default
#' # (effect_size, diff_btw, log2_fold_change, rab_all, overlap).
#' # Ranking by |log2_fold_change| is generally more biologically informative
#' # than ranking by p-value, especially for large datasets where small effects
#' # can reach statistical significance without being biologically meaningful.
#' aldex2_res <- pathway_daa(abundance, metadata, "group", daa_method = "ALDEx2")
#' head(aldex2_res)
#'
#' # Opt out of the extra aldex.effect() computation if only p-values are needed
#' aldex2_pvals_only <- pathway_daa(abundance, metadata, "group",
#'                                 daa_method = "ALDEx2",
#'                                 include_effect_size = FALSE)
#' }
#'
#' @references
#' \itemize{
#'   \item ALDEx2: Fernandes et al. (2014) Unifying the analysis of high-throughput sequencing datasets:
#'         characterizing RNA-seq, 16S rRNA gene sequencing and selective growth experiments by
#'         compositional data analysis. Microbiome.
#'   \item DESeq2: Love et al. (2014) Moderated estimation of fold change and dispersion for
#'         RNA-seq data with DESeq2. Genome Biology.
#'   \item edgeR: Robinson et al. (2010) edgeR: a Bioconductor package for differential expression
#'         analysis of digital gene expression data. Bioinformatics.
#'   \item limma-voom: Law et al. (2014) voom: precision weights unlock linear model analysis tools
#'         for RNA-seq read counts. Genome Biology.
#'   \item metagenomeSeq: Paulson et al. (2013) Differential abundance analysis for microbial
#'         marker-gene surveys. Nature Methods.
#'   \item Maaslin2: Mallick et al. (2021) Multivariable Association Discovery in Population-scale
#'         Meta-omics Studies.
#' }
NULL

#' Helper function to calculate abundance statistics for differential analysis
#'
#' This function calculates mean relative abundance, standard deviation, and log2 fold change
#' for each feature between two groups.
#'
#' @param abundance A matrix or data frame with features as rows and samples as columns
#' @param metadata A data frame containing sample information
#' @param group Character string specifying the group column name in metadata
#' @param features Character vector of feature names to calculate statistics for
#' @param group1 Character string specifying the first group name
#' @param group2 Character string specifying the second group name
#'
#' @return A data frame with columns:
#'   \item{feature}{Feature identifier}
#'   \item{mean_rel_abundance_group1}{Mean relative abundance for group1}
#'   \item{sd_rel_abundance_group1}{Standard deviation of relative abundance for group1}
#'   \item{mean_rel_abundance_group2}{Mean relative abundance for group2}
#'   \item{sd_rel_abundance_group2}{Standard deviation of relative abundance for group2}
#'   \item{log2_fold_change}{Log2 fold change (group2/group1)}
#'
#' @keywords internal
calculate_abundance_stats <- function(abundance, metadata, group, features, group1, group2) {
  # Validate inputs using unified functions
  validate_abundance(abundance)
  validate_metadata(metadata)
  validate_group(metadata, group)

  # Convert abundance to matrix if needed
  abundance_mat <- as.matrix(abundance)

  # Align samples between abundance and metadata
  aligned <- align_samples(abundance_mat, metadata, verbose = FALSE)
  abundance_filtered <- aligned$abundance
  metadata_ordered <- aligned$metadata
  sample_col <- aligned$sample_col

  # Get group assignments
  group_assignments <- metadata_ordered[[group]]

  # Filter for the two groups of interest
  group1_samples <- colnames(abundance_filtered)[group_assignments == group1]
  group2_samples <- colnames(abundance_filtered)[group_assignments == group2]

  if (length(group1_samples) == 0) {
    stop("No samples found for group1: ", group1)
  }
  if (length(group2_samples) == 0) {
    stop("No samples found for group2: ", group2)
  }

  # Convert to relative abundance via the shared helper so zero-sum sample
  # columns fail fast here instead of producing NaN that later gets
  # dropped by `na.rm = TRUE` inside the mean()/sd() aggregations,
  # silently changing the effective sample size.
  relative_abundance <- compute_relative_abundance(
    abundance_filtered,
    context = "calculate_abundance_stats()"
  )

  # Filter for specified features
  if (!is.null(features)) {
    features_available <- intersect(features, rownames(relative_abundance))
    if (length(features_available) == 0) {
      stop("None of the specified features found in abundance data")
    }
    relative_abundance <- relative_abundance[features_available, , drop = FALSE]
  }

  # Per-feature mean and sd for the two groups of interest. Restrict the
  # matrix to group1 + group2 columns before aggregation so the helper does
  # not spend time on samples that wouldn't show up in the output anyway.
  # This is also the only entry point that computes the mean/sd inputs to
  # the log2 fold change: `pathway_errorbar()` reuses the same helper, so
  # any future change to the aggregation rule (e.g. na.rm strategy,
  # trimmed means, etc.) only needs to happen in one place.
  keep <- group_assignments %in% c(group1, group2)
  long_stats <- summarize_abundance_by_group(
    relative_abundance[, keep, drop = FALSE],
    group_assignments[keep]
  )

  feature_order <- rownames(relative_abundance)
  g1 <- long_stats[long_stats$group == group1, , drop = FALSE]
  g2 <- long_stats[long_stats$group == group2, , drop = FALSE]
  g1 <- g1[match(feature_order, g1$name), , drop = FALSE]
  g2 <- g2[match(feature_order, g2$name), , drop = FALSE]

  # Data-driven pseudocount must be computed from the full relative-
  # abundance matrix (same scope as before the refactor), not from the
  # per-group means -- otherwise the pseudocount would track fold-change
  # scale and inflate artificially for small-sample comparisons.
  pseudocount <- calculate_pseudocount(as.vector(relative_abundance))

  results <- data.frame(
    feature = feature_order,
    mean_rel_abundance_group1 = g1$mean,
    sd_rel_abundance_group1 = g1$sd,
    mean_rel_abundance_group2 = g2$mean,
    sd_rel_abundance_group2 = g2$sd,
    log2_fold_change = calculate_log2_fold_change(
      g1$mean, g2$mean, pseudocount = pseudocount
    ),
    stringsAsFactors = FALSE
  )

  return(results)
}

#' @rdname pathway_daa
#' @export
pathway_daa <- function(abundance, metadata, group, daa_method = "ALDEx2",
                       select = NULL, p_adjust_method = "BH", reference = NULL,
                       include_abundance_stats = FALSE, include_effect_size = TRUE,
                       p.adjust = NULL, .pre_aligned = FALSE,
                       .sample_col = NULL, ...) {
  # Backward compatibility for deprecated parameter
  if (!is.null(p.adjust)) {
    warning("'p.adjust' parameter is deprecated. Use 'p_adjust_method' instead.", call. = FALSE)
    p_adjust_method <- p.adjust
  }

  # Single source of truth for supported DAA methods: this list drives
  # validation, the method->package mapping for optional-dependency
  # loading, and documents what the switch() below will accept.
  method_packages <- list(
    "ALDEx2" = "ALDEx2", "DESeq2" = "DESeq2", "edgeR" = "edgeR",
    "limma voom" = "limma", "metagenomeSeq" = "metagenomeSeq",
    "LinDA" = "MicrobiomeStat",
    "Maaslin2" = "Maaslin2", "Lefser" = "lefser"
  )

  # Validate daa_method up front. switch() below has no default, so an
  # unrecognized value silently yielded NULL; worse, both README and
  # some internal helpers historically referenced the misspellings
  # "linDA" and "Lefse", so users copy-pasted invalid values and got
  # a confusing NULL back. Fail fast with a message that shows the
  # canonical set.
  if (length(daa_method) != 1 || !is.character(daa_method) ||
      is.na(daa_method) || !nzchar(daa_method)) {
    stop("'daa_method' must be a single non-empty character string. ",
         "Supported methods: ",
         paste(names(method_packages), collapse = ", "), ".")
  }
  if (!daa_method %in% names(method_packages)) {
    # Common misspellings -> canonical form. Users frequently hit
    # "linDA" (from older docs) or "Lefse" (the LEfSe upstream name,
    # but our backend is the Bioconductor "Lefser" package); point
    # them at the right spelling instead of generic "not supported".
    suggestion <- switch(
      tolower(daa_method),
      "linda" = "LinDA",
      "lefse" = "Lefser",
      "aldex" = "ALDEx2",
      "aldex2" = "ALDEx2",
      "maaslin"  = "Maaslin2",
      "maaslin2" = "Maaslin2",
      "deseq" = "DESeq2",
      "deseq2" = "DESeq2",
      NULL
    )
    stop(
      sprintf("Unsupported daa_method: '%s'. ", daa_method),
      if (!is.null(suggestion)) sprintf("Did you mean '%s'? ", suggestion) else "",
      "Supported methods: ",
      paste(names(method_packages), collapse = ", "), "."
    )
  }

  require_package(method_packages[[daa_method]], purpose = daa_method)

  # Input validation using unified functions
  validate_abundance(abundance, min_samples = 4)
  validate_metadata(metadata)
  validate_group(metadata, group, min_groups = 2)

  # Alignment ownership is explicit. Standalone callers use the public
  # contract and get automatic sample matching here. ggpicrust2() aligns
  # once at the wrapper boundary because it also needs the same aligned
  # objects for plotting, then calls this function with `.pre_aligned = TRUE`
  # so we validate the invariant instead of repeating the alignment.
  .pre_aligned <- normalize_logical_flag(.pre_aligned, ".pre_aligned")
  if (.pre_aligned) {
    if (is.null(colnames(abundance))) {
      stop("Pre-aligned abundance data must have sample column names.", call. = FALSE)
    }
    metadata <- as.data.frame(metadata)
    sample_col <- .sample_col
    if (is.null(sample_col) || !sample_col %in% colnames(metadata)) {
      sample_col <- find_sample_column(metadata, colnames(abundance))
    }
    if (is.null(sample_col)) {
      stop("Pre-aligned metadata must contain sample identifiers matching abundance columns.",
           call. = FALSE)
    }
    if (sample_col == ".rownames") {
      metadata$.sample_id <- rownames(metadata)
      sample_col <- ".sample_id"
    }
    if (!identical(colnames(abundance), as.character(metadata[[sample_col]]))) {
      stop("Pre-aligned abundance and metadata are not in the same sample order.",
           call. = FALSE)
    }
    aligned <- list(
      abundance = abundance,
      metadata = metadata,
      sample_col = sample_col,
      n_samples = ncol(abundance)
    )
  } else {
    aligned <- align_samples(abundance, metadata, verbose = FALSE)
  }
  abundance <- aligned$abundance
  metadata <- tibble::as_tibble(aligned$metadata)
  sample_col <- aligned$sample_col

  if (aligned$n_samples < 4) {
    stop("At least 4 samples required for DAA (after alignment)")
  }

  # Handle sample selection
  if (!is.null(select)) {
    if (!all(select %in% colnames(abundance))) {
      stop("Some selected samples not in abundance data")
    }
    abundance <- abundance[, select, drop = FALSE]
    # Keep metadata rows in the exact order of `select` so row i of metadata
    # corresponds to column i of abundance. Simple %in% filtering preserves
    # the original order and can silently desynchronize Group vs samples.
    metadata <- metadata[match(select, metadata[[sample_col]]), , drop = FALSE]
  }

  # Validate abundance matrix
  abundance_mat <- as.matrix(abundance)
  validate_daa_input(abundance_mat, method = daa_method)

  Group <- factor(metadata[[group]])

  # Ensure factor levels only include groups present in the data
  # This is defensive programming to handle edge cases with unused factor levels
  Group <- droplevels(Group)
  Level <- levels(Group)
  length_Level <- length(Level)

  # Re-validate group counts after align_samples()/select have pruned samples.
  # The up-front validate_group() check sees the raw metadata, so sample
  # alignment or a narrow `select =` that removes every row of a level can
  # leave a single-group dataset that would crash inside the backends with a
  # less actionable error.
  if (length_Level < 2) {
    stop(
      "DAA requires at least 2 groups with samples after alignment",
      if (!is.null(select)) " and `select` filtering" else "",
      "; found ", length_Level,
      if (length_Level == 1) paste0(" ('", Level, "')") else "",
      "."
    )
  }

  # Perform differential analysis
  result <- switch(
    daa_method,
    "ALDEx2" = perform_aldex2_analysis(abundance_mat, Group, Level, length_Level, include_effect_size),
    "DESeq2" = perform_deseq2_analysis(abundance_mat, metadata, group, reference, Level, length_Level),
    "LinDA" = perform_linda_analysis(abundance, metadata, group, reference, Level, length_Level),
    "limma voom" = perform_limma_voom_analysis(abundance_mat, Group, reference, Level, length_Level),
    "edgeR" = perform_edger_analysis(abundance_mat, Group, reference, Level, length_Level),
    "metagenomeSeq" = perform_metagenomeseq_analysis(abundance_mat, metadata, group, reference, Level),
    "Maaslin2" = perform_maaslin2_analysis(abundance_mat, metadata, group, reference, Level, length_Level),
    "Lefser" = perform_lefser_analysis(abundance_mat, metadata, group, Level)
  )


  # Add multiple testing correction
  # Note: Some methods (e.g., ALDEx2) provide pre-computed BH-corrected p-values

  # which are more accurate as they account for Monte Carlo sampling uncertainty
  if (!is.null(result) && "p_values" %in% colnames(result) && nrow(result) > 0) {
    if ("p_adjust" %in% colnames(result)) {
      # p_adjust already computed by the method (e.g., ALDEx2)
      # Just add the adjustment method indicator
      result$adj_method <- "BH (method-specific)"
    } else {
      # Apply standard p-value adjustment for methods without pre-computed values
      result$p_adjust <- stats::p.adjust(result$p_values, method = p_adjust_method)
      result$adj_method <- p_adjust_method
    }
  }

  # Add abundance statistics if requested
  if (include_abundance_stats && !is.null(result) && nrow(result) > 0) {
    # Check if we have the required columns for abundance stats
    if (all(c("feature", "group1", "group2") %in% colnames(result))) {
      tryCatch({
        # Get unique group pairs for calculation
        unique_comparisons <- unique(result[, c("group1", "group2")])

        # Calculate abundance stats for each unique comparison
        all_abundance_stats <- data.frame()

        for (i in seq_len(nrow(unique_comparisons))) {
          group1_name <- unique_comparisons$group1[i]
          group2_name <- unique_comparisons$group2[i]

          # Get features for this comparison
          comparison_features <- result$feature[
            result$group1 == group1_name & result$group2 == group2_name
          ]

          if (length(comparison_features) > 0) {
            abundance_stats <- calculate_abundance_stats(
              abundance = abundance_mat,
              metadata = metadata,
              group = group,
              features = comparison_features,
              group1 = group1_name,
              group2 = group2_name
            )

            # Add group information for merging
            abundance_stats$group1 <- group1_name
            abundance_stats$group2 <- group2_name

            all_abundance_stats <- rbind(all_abundance_stats, abundance_stats)
          }
        }

        # Merge abundance stats with results. If the DAA method already
        # provides a method-native log2_fold_change (e.g. ALDEx2 effect size
        # in CLR space, DESeq2 shrunk log2FC), keep it and drop the
        # relative-abundance-ratio version to avoid a .x/.y merge collision
        # and to avoid conflating two different effect-size definitions.
        if (nrow(all_abundance_stats) > 0) {
          if ("log2_fold_change" %in% colnames(result) &&
              "log2_fold_change" %in% colnames(all_abundance_stats)) {
            all_abundance_stats$log2_fold_change <- NULL
          }
          result <- merge(
            result,
            all_abundance_stats,
            by = c("feature", "group1", "group2"),
            all.x = TRUE
          )
        }
      }, error = function(e) {
        warning("Failed to calculate abundance statistics: ", e$message)
      })
    } else {
      warning("Cannot calculate abundance statistics: required columns (feature, group1, group2) not found in results")
    }
  }

  return(result)
}

# Helper function: Perform ALDEx2 analysis
perform_aldex2_analysis <- function(abundance_mat, Group, Level, length_Level, include_effect_size = FALSE) {
  # Filter zero-abundance features (ALDEx2 requirement)
  abundance_mat <- validate_daa_input(abundance_mat, method = "ALDEx2", filter_zero = TRUE)
  abundance_mat <- round(abundance_mat)

  # Convert Group to numeric for ALDEx2
  Group <- as.numeric(Group)

  # Run analysis based on number of groups
  if (length_Level == 2) {
    ALDEx2_object <- tryCatch(
      ALDEx2::aldex.clr(abundance_mat, Group, mc.samples = 256, denom = "all", verbose = FALSE),
      error = function(e) stop("ALDEx2 CLR failed: ", e$message)
    )

    results <- tryCatch(
      ALDEx2::aldex.ttest(ALDEx2_object, paired.test = FALSE, verbose = FALSE),
      error = function(e) stop("ALDEx2 t-test failed: ", e$message)
    )

    # Get effect size results if requested
    effect_results <- NULL
    if (include_effect_size) {
      effect_results <- tryCatch({
        # In newer versions of ALDEx2, aldex.effect() only needs the clr object
        # The conditions are already stored in the clr object
        ALDEx2::aldex.effect(
          ALDEx2_object,
          verbose = FALSE
        )
      }, error = function(e) {
        warning("Failed to calculate ALDEx2 effect sizes: ", e$message,
                ". Proceeding without effect size information.")
        return(NULL)
      })
    }

    # Build result dataframe using original Level names
    # Use ALDEx2's pre-computed BH-corrected p-values (Monte Carlo-based)
    # These are more accurate than simple p.adjust() as they account for
    # the uncertainty in the CLR transformation
    base_df <- data.frame(
      feature = rep(rownames(results), 2),
      method = c(
        rep("ALDEx2_Welch's t test", nrow(results)),
        rep("ALDEx2_Wilcoxon rank test", nrow(results))
      ),
      group1 = rep(Level[1], 2 * nrow(results)),
      group2 = rep(Level[2], 2 * nrow(results)),
      p_values = c(results$we.ep, results$wi.ep),
      p_adjust = c(results$we.eBH, results$wi.eBH),
      stringsAsFactors = FALSE
    )

    # Add effect size columns if available
    if (!is.null(effect_results)) {
      # Simple approach: add the most important effect size columns
      # Replicate effect size data for both test methods (Welch and Wilcoxon)
      n_features <- nrow(results)

      # Add basic effect size columns
      if ("effect" %in% colnames(effect_results)) {
        base_df$effect_size <- rep(effect_results$effect, 2)
      }
      if ("diff.btw" %in% colnames(effect_results)) {
        base_df$diff_btw <- rep(effect_results$diff.btw, 2)
        base_df$log2_fold_change <- rep(effect_results$diff.btw, 2)
      }
      if ("rab.all" %in% colnames(effect_results)) {
        base_df$rab_all <- rep(effect_results$rab.all, 2)
      }
      if ("overlap" %in% colnames(effect_results)) {
        base_df$overlap <- rep(effect_results$overlap, 2)
      }
    }

    return(base_df)
    
  } else {
    # Multi-group analysis. ALDEx2::aldex.effect() only supports two-group
    # comparisons, so include_effect_size is silently ignored here regardless
    # of its value.
    ALDEx2_object <- tryCatch(
      ALDEx2::aldex.clr(abundance_mat, Group, mc.samples = 256, denom = "all", verbose = FALSE),
      error = function(e) stop("ALDEx2 CLR failed: ", e$message)
    )

    results <- ALDEx2::aldex.kw(ALDEx2_object)

    result_df <- data.frame(
      feature = rep(rownames(results), 2),
      method = c(rep("ALDEx2_Kruskal-Wallace test", nrow(results)),
                 rep("ALDEx2_glm test", nrow(results))),
      p_values = c(results$kw.ep, results$glm.ep),
      p_adjust = c(results$kw.eBH, results$glm.eBH),
      stringsAsFactors = FALSE
    )

    # Add group columns
    group_cols <- matrix(rep(Level, each = nrow(result_df)), nrow = nrow(result_df))
    colnames(group_cols) <- paste0("group", seq_along(Level))
    result_df <- cbind(result_df[, c("feature", "method")], group_cols,
                       result_df[, c("p_values", "p_adjust"), drop = FALSE])

    return(result_df)
  }
}

# Helper function: Perform DESeq2 analysis
perform_deseq2_analysis <- function(abundance_mat, metadata, group, reference, Level, length_Level) {
  counts <- round(as.matrix(abundance_mat))

  # Resolve the reference level. When the user supplies one we honor it,
  # otherwise fall back to the first level so behavior stays identical to
  # the previous default.
  ref_level <- if (!is.null(reference) && reference %in% Level) reference else Level[1]
  non_ref_levels <- setdiff(Level, ref_level)
  factor_levels <- c(ref_level, non_ref_levels)

  # Ensure group column is a factor with reference in the first position so
  # DESeq2's default contrasts match the semantics expected downstream.
  metadata[[group]] <- factor(metadata[[group]], levels = factor_levels)

  # Create DESeqDataSet object
  se <- SummarizedExperiment::SummarizedExperiment(
    assays = list(counts = counts),
    colData = metadata
  )

  result <- tryCatch({
    suppressWarnings({
      dds <- DESeq2::DESeqDataSet(se, design = as.formula(paste0("~", group)))

      dds <- DESeq2::estimateSizeFactors(dds)

      # Estimate dispersions with fallback chain: parametric -> local -> mean -> gene-wise
      # Note: fitType choice should NOT be based on sample size (DESeq2 documentation);
      # parametric is the default and recommended for most cases.
      dds <- tryCatch({
        DESeq2::estimateDispersions(dds, fitType = "parametric")
      }, error = function(e1) {
        tryCatch({
          DESeq2::estimateDispersions(dds, fitType = "local")
        }, error = function(e2) {
          tryCatch({
            DESeq2::estimateDispersions(dds, fitType = "mean")
          }, error = function(e3) {
            message("All dispersion fitting methods failed, using gene-wise estimates...")
            dds <- DESeq2::estimateDispersionsGeneEst(dds)
            DESeq2::dispersions(dds) <- SummarizedExperiment::mcols(dds)$dispGeneEst
            return(dds)
          })
        })
      })

      dds <- DESeq2::nbinomWaldTest(dds)

      # One row-block per non-reference level, identical shape to
      # edgeR's multi-group output.
      build_block <- function(lvl) {
        res <- DESeq2::results(dds, contrast = c(group, lvl, ref_level))
        data.frame(
          feature = rownames(abundance_mat),
          method = "DESeq2",
          group1 = ref_level,
          group2 = lvl,
          p_values = res$pvalue,
          log2_fold_change = res$log2FoldChange,
          stringsAsFactors = FALSE
        )
      }

      if (length_Level == 2) {
        build_block(non_ref_levels)
      } else {
        do.call(rbind, lapply(non_ref_levels, build_block))
      }
    })
  }, error = function(e) {
    stop("DESeq2 analysis failed: ", e$message)
  })

  if (is.null(result)) {
    stop("DESeq2 analysis failed to produce results")
  }

  return(result)
}

# Helper function: Perform limma voom analysis
perform_limma_voom_analysis <- function(abundance_mat, Group, reference, Level, length_Level) {
  
  # Ensure Group is a factor type
  Group <- factor(Group)
  if (!is.null(reference)) {
    Group <- relevel(Group, ref = reference)
  }

  # Create design matrix
  design <- model.matrix(~Group)

  # Perform voom transformation and analysis
  dge <- edgeR::DGEList(counts = abundance_mat, group = Group)
  dge <- edgeR::calcNormFactors(dge)
  v <- limma::voom(dge, design)
  fit <- limma::lmFit(v, design)
  fit <- limma::eBayes(fit)

  # Extract results
  if (length_Level == 2) {
    # Two-group comparison - ensure consistent format with other methods
    group_levels <- levels(Group)
    results <- data.frame(
      feature = rownames(abundance_mat),
      method = "limma voom",
      group1 = group_levels[1],
      group2 = group_levels[2],
      p_values = fit$p.value[,2],
      log2_fold_change = fit$coefficients[,2],
      stringsAsFactors = FALSE
    )
  } else {
    # Multi-group comparison handling.
    # fit$p.value columns [-1] correspond to levels(Group)[-1] in order
    # (since Group was releveled so the reference sits first). as.vector()
    # of that matrix stacks columns head-to-tail: all features for
    # contrast 1, then all features for contrast 2, etc. group2 must be
    # repeated *each = n_features* to stay aligned; plain recycling
    # produces interleaved B,C,B,C labels that scramble the contrast.
    contrasts <- levels(Group)[-1]
    n_features <- nrow(abundance_mat)
    group1_label <- if (!is.null(reference)) reference else levels(Group)[1]
    results <- data.frame(
      feature = rep(rownames(abundance_mat), length(contrasts)),
      method = "limma voom",
      group1 = group1_label,
      group2 = rep(contrasts, each = n_features),
      p_values = as.vector(fit$p.value[, -1]),
      log2_fold_change = as.vector(fit$coefficients[, -1]),
      stringsAsFactors = FALSE
    )
  }

  return(results)
}

# Helper function: Perform edgeR analysis
perform_edger_analysis <- function(abundance_mat, Group, reference, Level, length_Level) {

  # Resolve the reference level. Without this, edgeR's exactTest() used the
  # raw factor order (level 1 vs level 2), so the user-supplied `reference`
  # was silently ignored and the result labels were always `Level[1]` /
  # `Level[2]` regardless of intent. Relevel once so both the model and the
  # labels align with the requested contrast direction.
  ref_level <- if (!is.null(reference) && reference %in% Level) reference else Level[1]
  Group <- stats::relevel(factor(Group, levels = Level), ref = ref_level)
  Level <- levels(Group)

  # Create DGEList object
  dge <- edgeR::DGEList(counts = round(abundance_mat), group = Group)
  dge <- edgeR::calcNormFactors(dge)
  dge <- edgeR::estimateCommonDisp(dge, verbose = TRUE)

  if (length_Level == 2) {
    # Two-group comparison: reference (level 1) vs non-reference (level 2).
    # exactTest(pair = c(1, 2)) is `log(level2 / level1)`, matching
    # group1 = ref, group2 = non-ref.
    et <- edgeR::exactTest(dge, pair = c(1, 2))
    results <- data.frame(
      feature = rownames(abundance_mat),
      method = "edgeR",
      group1 = Level[1],
      group2 = Level[2],
      p_values = et$table$PValue,
      log2_fold_change = et$table$logFC,
      stringsAsFactors = FALSE
    )
  } else {
    # Multi-group: emit one block per (reference, non-reference) contrast so
    # the shape matches DESeq2 / limma voom / LinDA / Maaslin2. Previously
    # we enumerated every pair via combn(), which produced k*(k-1)/2 rows
    # per feature without any privileged reference -- inconsistent with the
    # other backends and with the documented `reference` semantics.
    non_ref_levels <- Level[-1]
    results_list <- lapply(non_ref_levels, function(lvl) {
      et <- edgeR::exactTest(dge, pair = c(Level[1], lvl))
      data.frame(
        feature = rownames(abundance_mat),
        method = "edgeR",
        group1 = Level[1],
        group2 = lvl,
        p_values = et$table$PValue,
        log2_fold_change = et$table$logFC,
        stringsAsFactors = FALSE
      )
    })
    results <- do.call(rbind, results_list)
  }

  return(results)
}

# Helper function: Perform metagenomeSeq analysis
perform_metagenomeseq_analysis <- function(abundance_mat, metadata, group, reference, Level) {
  new_mrexperiment <- getExportedValue("metagenomeSeq", "newMRexperiment")
  cum_norm <- getExportedValue("metagenomeSeq", "cumNorm")
  cum_norm_stat_fast <- getExportedValue("metagenomeSeq", "cumNormStatFast")
  fit_feature_model <- getExportedValue("metagenomeSeq", "fitFeatureModel")
  mrcoefs <- getExportedValue("metagenomeSeq", "MRcoefs")

  # Convert metadata to data.frame. align_samples() has already ordered
  # metadata rows to match abundance_mat columns, so we set rownames from
  # the (authoritative) abundance column names rather than trusting a
  # hardcoded "sample" column that may not exist.
  metadata <- as.data.frame(metadata)
  rownames(metadata) <- colnames(abundance_mat)

  # Resolve the reference level and relevel the grouping factor so the
  # ~group model matrix uses the user-specified level as the intercept.
  # Without this, metagenomeSeq always contrasted against the natural
  # first level and our group1/group2 labels (Level[1]/Level[2]) were
  # fixed regardless of the `reference` argument, so flipping the
  # reference left both labels and coefficients unchanged.
  ref_level <- if (!is.null(reference) && reference %in% Level) reference else Level[1]
  metadata[[group]] <- stats::relevel(
    factor(metadata[[group]], levels = Level),
    ref = ref_level
  )
  Level <- levels(metadata[[group]])

  counts_all <- round(as.matrix(abundance_mat))

  # Single two-group fit. `fitFeatureModel()` is metagenomeSeq's documented
  # entry point for two-group comparisons (it tests a single coefficient and
  # returns one p-value per feature in `fit@pvalues`). For each pairwise
  # contrast we fit this two-group model on the subset of samples in the
  # two levels of interest and read `coef = 2` -- the non-reference dummy.
  # Returns a list of two length-nrow(counts_sub) vectors so the shape is
  # stable across contrasts regardless of log-fold-change extraction
  # success.
  fit_pair <- function(counts_sub, meta_sub, group_col) {
    phenoData <- new("AnnotatedDataFrame",
                     data = meta_sub,
                     varMetadata = data.frame(
                       labelDescription = colnames(meta_sub),
                       row.names = colnames(meta_sub)
                     ))
    obj <- try(
      new_mrexperiment(counts = counts_sub, phenoData = phenoData,
                       featureData = NULL, libSize = NULL, normFactors = NULL),
      silent = TRUE
    )
    if (inherits(obj, "try-error")) {
      stop("Failed to create MRexperiment object: ",
           attr(obj, "condition")$message)
    }
    # cumNormStatFast() chooses a normalization quantile by comparing
    # per-sample CDFs; with very small or near-uniform matrices (e.g. the
    # minimum 4-sample input) the underlying quantile math returns NaN,
    # and metagenomeSeq then aborts inside an `if (x <= 0.5)` check with
    # "missing value where TRUE/FALSE needed". Precompute the factor
    # ourselves and fall back to metagenomeSeq's documented default
    # (p = 0.5) whenever it is degenerate.
    p_norm <- tryCatch(cum_norm_stat_fast(obj), error = function(e) NA_real_)
    if (!is.finite(p_norm)) p_norm <- 0.5
    obj <- cum_norm(obj, p = p_norm)

    mod <- stats::model.matrix(as.formula(paste0("~", group_col)),
                               data = meta_sub)
    fit <- fit_feature_model(obj, mod)

    coef_table <- tryCatch(
      mrcoefs(fit, coef = 2),
      error = function(e) {
        warning("Failed to extract coefficients from metagenomeSeq: ",
                e$message)
        NULL
      }
    )
    log2fc <- if (!is.null(coef_table) && "logFC" %in% colnames(coef_table)) {
      coef_table$logFC
    } else {
      rep(NA_real_, nrow(counts_sub))
    }
    list(p_values = fit@pvalues, log2_fold_change = log2fc)
  }

  if (length(Level) == 2) {
    res <- fit_pair(counts_all, metadata, group)
    return(data.frame(
      feature = rownames(abundance_mat),
      method = "metagenomeSeq",
      group1 = Level[1],
      group2 = Level[2],
      p_values = res$p_values,
      log2_fold_change = res$log2_fold_change,
      stringsAsFactors = FALSE
    ))
  }

  # Multi-group: emit one (reference, non-reference) contrast block per
  # non-reference level. Previously the function built a full k-column
  # model matrix, called fit_feature_model() once, extracted coef = 2,
  # and hard-coded group1 = Level[1] / group2 = Level[2] -- which
  # silently dropped all contrasts beyond the first non-reference level
  # and shaped the output as if only two groups existed. fitFeatureModel
  # is also documented as a two-group entry point, so re-fitting per pair
  # on the subset of samples in the two levels of interest is the correct
  # multi-group handling and matches the shape returned by edgeR / LinDA
  # / Maaslin2 / DESeq2 / limma voom.
  non_ref_levels <- Level[-1]
  results_list <- lapply(non_ref_levels, function(lvl) {
    keep <- metadata[[group]] %in% c(Level[1], lvl)
    meta_sub <- metadata[keep, , drop = FALSE]
    # Relevel within the subset so the 2-column model matrix has the
    # reference level first and the current non-reference level second.
    meta_sub[[group]] <- factor(as.character(meta_sub[[group]]),
                                levels = c(Level[1], lvl))
    counts_sub <- counts_all[, keep, drop = FALSE]
    res <- fit_pair(counts_sub, meta_sub, group)
    data.frame(
      feature = rownames(abundance_mat),
      method = "metagenomeSeq",
      group1 = Level[1],
      group2 = lvl,
      p_values = res$p_values,
      log2_fold_change = res$log2_fold_change,
      stringsAsFactors = FALSE
    )
  })
  do.call(rbind, results_list)
}

# Helper function: Perform Maaslin2 analysis
perform_maaslin2_analysis <- function(abundance_mat, metadata, group, reference, Level, length_Level) {

  # Prepare metadata first
  metadata <- as.data.frame(metadata)

  # Ensure metadata rownames match the column names of abundance_mat
  # This is crucial because the main function has already reordered metadata to match abundance
  rownames(metadata) <- colnames(abundance_mat)

  # Transpose abundance matrix (samples become rows, features become columns)
  abundance_mat_t <- t(abundance_mat)

  # Use a run-specific directory to avoid stale files from previous Maaslin2 runs.
  output_dir <- tempfile(pattern = "ggpicrust2_maaslin2_")
  dir.create(output_dir, recursive = TRUE, showWarnings = FALSE)
  on.exit(unlink(output_dir, recursive = TRUE, force = TRUE), add = TRUE)

  # Maaslin2 emits an early logging::logwarn() before it sets up its own
  # log handler. If a prior Maaslin2 invocation in the same R session left
  # a writeToFile handler pointing at a now-deleted directory, that early
  # warning blows up with "cannot open the connection". Clear stale
  # handlers up front so repeat calls stay safe.
  if (requireNamespace("logging", quietly = TRUE)) {
    root_logger <- logging::getLogger()
    for (h_name in names(root_logger$handlers)) {
      try(logging::removeHandler(h_name), silent = TRUE)
    }
  }

  # Resolve the reference level up front so we can (a) pass it to Maaslin2
  # regardless of group count and (b) label group1 consistently. Without
  # forwarding reference in the 2-group case, Maaslin2 falls back to its
  # alphabetical-first-level default -- if the user asked for a different
  # reference, the resulting contrast direction silently disagrees with the
  # group1/group2 labels we emit.
  ref_level <- if (!is.null(reference) && reference %in% Level) reference else Level[1]
  maaslin_reference <- paste0(group, ",", ref_level)

  # Run Maaslin2 analysis via dynamic lookup so the package remains optional
  maaslin2_fn <- getExportedValue("Maaslin2", "Maaslin2")
  fit_data <- maaslin2_fn(
    input_data = abundance_mat_t,
    input_metadata = metadata,
    output = output_dir,
    transform = "AST",
    fixed_effects = group,
    reference = maaslin_reference,
    normalization = "TSS",
    standardize = TRUE,
    min_prevalence = 0.1,
    cores = 1,
    plot_heatmap = FALSE,
    plot_scatter = FALSE
  )

  # Read all results instead of significant results
  results_file <- file.path(output_dir, "all_results.tsv")

  if (!file.exists(results_file)) {
    stop("Maaslin2 analysis failed to produce results file")
  }

  maaslin2_results <- utils::read.table(results_file,
                                        header = TRUE,
                                        sep = "\t",
                                        stringsAsFactors = FALSE)

  # Keep only rows that correspond to contrasts on `group`. Maaslin2 emits
  # one row per (feature, non-reference level) for a categorical fixed effect,
  # which gives us the multi-group shape natively: N_features rows in the
  # 2-group case, N_features * (k - 1) in a k-group case.
  maaslin2_results <- maaslin2_results[maaslin2_results$metadata == group, , drop = FALSE]

  # Maaslin2 replaces hyphens (-) with dots (.) in feature names, so we
  # translate Maaslin2's feature column back to the original IDs rather
  # than trying to match original IDs into Maaslin2 output (which loses
  # multi-group rows via match()'s first-hit semantics).
  original_features <- rownames(abundance_mat)
  translated_features <- original_features[match(maaslin2_results$feature,
                                                 gsub("-", ".", original_features))]
  # Fall back to the raw Maaslin2 name if the reverse map missed.
  translated_features[is.na(translated_features)] <-
    maaslin2_results$feature[is.na(translated_features)]

  results <- data.frame(
    feature = translated_features,
    method = "Maaslin2",
    group1 = ref_level,
    group2 = maaslin2_results$value,
    p_values = maaslin2_results$pval,
    log2_fold_change = maaslin2_results$coef,
    stringsAsFactors = FALSE
  )

  return(results)
}

# Helper function: Perform Lefser analysis
perform_lefser_analysis <- function(abundance_mat, metadata, group, Level) {

  # Check if we only have 2 groups (lefser requirement)
  if (length(Level) != 2) {
    stop("Lefser requires exactly 2 groups. Found ", length(Level), " groups.")
  }

  # Create SummarizedExperiment object
  se <- SummarizedExperiment::SummarizedExperiment(
    assays = list(counts = abundance_mat),
    colData = metadata
  )

  # Perform Lefser analysis to get LDA scores (effect sizes)
  lefser_results <- tryCatch({
    lefser::lefser(se, classCol = group)
  }, error = function(e) {
    message("Lefser analysis failed: ", e$message)
    NULL
  })

  # Get all features and group assignments
  all_features <- rownames(abundance_mat)
  group_vector <- metadata[[group]]

  # Calculate Wilcoxon p-values for all features
  # This is consistent with LEfSe methodology which uses Kruskal-Wallis/Wilcoxon internally
  p_values <- sapply(seq_len(nrow(abundance_mat)), function(i) {
    feature_values <- abundance_mat[i, ]
    group1_values <- feature_values[group_vector == Level[1]]
    group2_values <- feature_values[group_vector == Level[2]]

    # Handle edge cases
    if (length(unique(c(group1_values, group2_values))) <= 1) {
      return(1.0)  # No variation, not significant
    }

    tryCatch({
      wilcox.test(group1_values, group2_values, exact = FALSE)$p.value
    }, error = function(e) {
      1.0  # Return 1.0 if test fails
    })
  })

  # Initialize results with Wilcoxon p-values
  results <- data.frame(
    feature = all_features,
    method = "Lefser",
    group1 = Level[1],
    group2 = Level[2],
    p_values = p_values,
    lda_score = rep(NA_real_, length(all_features)),  # LDA effect size from lefser
    stringsAsFactors = FALSE
  )

  # Add LDA scores for features identified by lefser
  if (!is.null(lefser_results) && nrow(lefser_results) > 0) {
    sig_indices <- match(lefser_results$features, all_features)
    valid_indices <- !is.na(sig_indices)
    results$lda_score[sig_indices[valid_indices]] <- lefser_results$scores[valid_indices]
    message(sprintf("Lefser identified %d features with LDA score > threshold", sum(valid_indices)))
  } else {
    message("No significant features found by Lefser LDA analysis.")
  }

  return(results)
}

# Helper function: Perform LinDA analysis
format_linda_output <- function(linda_output, group, reference, Level) {
  # Create an empty list to store results for each comparison
  results_list <- list()

  # Get all non-reference groups
  non_ref_groups <- Level[Level != reference]

  # Process each output element (each corresponds to a group comparison)
  for (i in seq_along(linda_output)) {
    output_name <- names(linda_output)[i]

    # Extract non-reference group name from output name
    group_prefix <- paste0(group, "")
    if (!is.null(output_name) && startsWith(output_name, group_prefix)) {
      comparison_group <- substring(output_name, nchar(group_prefix) + 1)
    } else {
      comparison_group <- if (i <= length(non_ref_groups)) non_ref_groups[i] else paste0("Group", i)
    }
    comparison_group <- as.character(comparison_group)[1]

    comparison_df <- linda_output[[i]]
    if (!is.data.frame(comparison_df) || nrow(comparison_df) == 0) {
      next
    }

    n_features <- nrow(comparison_df)
    feature_ids <- rownames(comparison_df)
    if (is.null(feature_ids)) {
      feature_ids <- paste0("feature_", seq_len(n_features))
    }

    pvals <- comparison_df$pvalue
    if (is.null(pvals)) {
      warning("LinDA output missing 'pvalue' column; filling with NA for this comparison.")
      pvals <- rep(NA_real_, n_features)
    } else if (length(pvals) != n_features) {
      warning(
        "LinDA output has inconsistent 'pvalue' length (",
        length(pvals), " vs ", n_features, "); filling with NA for this comparison."
      )
      pvals <- rep(NA_real_, n_features)
    }

    lfc <- comparison_df$log2FoldChange
    if (is.null(lfc)) {
      warning("LinDA output missing 'log2FoldChange' column; filling with NA for this comparison.")
      lfc <- rep(NA_real_, n_features)
    } else if (length(lfc) != n_features) {
      warning(
        "LinDA output has inconsistent 'log2FoldChange' length (",
        length(lfc), " vs ", n_features, "); filling with NA for this comparison."
      )
      lfc <- rep(NA_real_, n_features)
    }

    # Create a results data frame for this comparison
    results_list[[length(results_list) + 1]] <- data.frame(
      feature = feature_ids,
      method = "LinDA",
      group1 = reference,
      group2 = comparison_group,
      p_values = as.numeric(pvals),
      log2_fold_change = as.numeric(lfc),
      stringsAsFactors = FALSE
    )
  }

  # Combine all results
  if (length(results_list) > 0) {
    do.call(rbind, results_list)
  } else {
    create_empty_daa_result(include_log2fc = TRUE)
  }
}

perform_linda_analysis <- function(abundance, metadata, group, reference, Level, length_Level) {
  # Filter zero-abundance features using unified validation
  feature.dat <- validate_daa_input(as.matrix(abundance), method = "LinDA", filter_zero = TRUE)
  meta.dat <- metadata

  # Handle reference level
  if (!is.null(reference) && !(reference %in% Level)) {
    warning(sprintf("Reference '%s' not in group levels; using '%s'", reference, Level[1]))
    reference <- Level[1]
  } else if (is.null(reference)) {
    reference <- Level[1]
  }

  # Relevel the grouping factor so that LinDA's `~ group` formula actually
  # treats the user-specified level as the reference. Without this, LinDA
  # silently keeps the natural first level as reference while our result
  # labeling uses `reference` -- producing rows with group1 == group2 and
  # statistics that don't reflect the requested contrast direction.
  meta.dat[[group]] <- stats::relevel(
    factor(meta.dat[[group]], levels = Level),
    ref = reference
  )
  # Keep Level in sync with the releveled factor so downstream lookups
  # (e.g. non-reference level enumeration) agree with the model fit.
  Level <- levels(meta.dat[[group]])

  formula <- paste0("~ ", group)

  # Run LinDA
  linda_result <- tryCatch({
    # Run LinDA analysis
    linda_obj <- MicrobiomeStat::linda(
      feature.dat = feature.dat,
      meta.dat = meta.dat,
      formula = formula,
      feature.dat.type = "count",
      prev.filter = 0,
      mean.abund.filter = 0,
      adaptive = TRUE,
      zero.handling = "pseudo-count",
      pseudo.cnt = 0.5,
      p.adj.method = "BH",
      alpha = 0.05,
      n.cores = 1,
      verbose = TRUE    # Changed to TRUE to provide more detailed output
    )
    
    # Check the structure of linda_obj
    if (is.null(linda_obj) || is.null(linda_obj$output) || length(linda_obj$output) == 0) {
      message("LinDA analysis returned empty results. This might be due to insufficient variation in the data.")
      return(create_empty_daa_result(include_log2fc = TRUE))
    }
    
    format_linda_output(
      linda_output = linda_obj$output,
      group = group,
      reference = reference,
      Level = Level
    )
  }, error = function(e) {
    # Catch and report errors, but return an empty data frame instead of NULL
    message(sprintf("Error in LinDA analysis: %s", e$message))
    create_empty_daa_result(include_log2fc = TRUE)
  })
  
  return(linda_result)
}

# Helper: create standard volcano test data
create_volcano_test_data <- function() {
  set.seed(123)
  data.frame(
    feature = paste0("ko", sprintf("%05d", 1:20)),
    pathway_name = paste0("Pathway ", 1:20),
    log2FoldChange = rnorm(20, 0, 2),
    p_adjust = runif(20, 0, 0.1),
    stringsAsFactors = FALSE
  )
}

test_that("pathway_volcano creates a ggplot object", {
  skip_if_not_installed("ggrepel")

  p <- pathway_volcano(create_volcano_test_data())
  expect_s3_class(p, "ggplot")
})

test_that("pathway_volcano handles custom column names", {
  skip_if_not_installed("ggrepel")

  set.seed(123)
  daa_results <- data.frame(
    id = paste0("ko", sprintf("%05d", 1:20)),
    name = paste0("Pathway ", 1:20),
    fc = rnorm(20, 0, 2),
    pval = runif(20, 0, 0.1),
    stringsAsFactors = FALSE
  )

  p <- pathway_volcano(daa_results, fc_col = "fc", p_col = "pval", label_col = "name")
  expect_s3_class(p, "ggplot")
})

test_that("pathway_volcano handles custom thresholds", {
  skip_if_not_installed("ggrepel")

  p <- pathway_volcano(
    create_volcano_test_data(),
    fc_threshold = 0.5,
    p_threshold = 0.01,
    label_top_n = 5
  )
  expect_s3_class(p, "ggplot")
})

test_that("pathway_volcano handles custom colors", {
  skip_if_not_installed("ggrepel")

  p <- pathway_volcano(
    create_volcano_test_data(),
    colors = c("Down" = "blue", "Not Significant" = "gray", "Up" = "red")
  )
  expect_s3_class(p, "ggplot")
})

test_that("pathway_volcano handles no labels", {
  skip_if_not_installed("ggrepel")

  p <- pathway_volcano(create_volcano_test_data(), label_top_n = 0)
  expect_s3_class(p, "ggplot")
})

test_that("pathway_volcano errors on missing columns", {
  set.seed(123)
  daa_results <- data.frame(
    feature = paste0("ko", sprintf("%05d", 1:20)),
    some_value = rnorm(20),
    stringsAsFactors = FALSE
  )

  expect_error(pathway_volcano(daa_results), "not found")
})

test_that("pathway_volcano errors on non-data.frame input", {
  expect_error(pathway_volcano("not a data frame"), "must be a data frame")
})

test_that("pathway_volcano keeps all-zero p-values finite", {
  daa_results <- data.frame(
    feature = c("ko00001", "ko00002"),
    pathway_name = c("Pathway 1", "Pathway 2"),
    log2_fold_change = c(2, -2),
    p_adjust = c(0, 0),
    stringsAsFactors = FALSE
  )

  expect_warning(
    p <- pathway_volcano(daa_results),
    regexp = NA
  )
  expect_true(all(is.finite(p$data$neg_log10_p)))
  expect_true(all(p$data$neg_log10_p > 0))
})

test_that("pathway_volcano rejects negative p-values", {
  daa_results <- data.frame(
    feature = "ko00001",
    pathway_name = "Pathway 1",
    log2_fold_change = 2,
    p_adjust = -0.01,
    stringsAsFactors = FALSE
  )

  expect_error(pathway_volcano(daa_results), "negative")
})

test_that("pathway_volcano works with real DAA workflow", {
  skip_if_not_installed("ggrepel")
  skip_on_cran()

  data(ko_abundance, package = "ggpicrust2")
  data(metadata, package = "ggpicrust2")

  kegg_abundance <- ko2kegg_abundance(data = ko_abundance)

  daa_results <- suppressMessages(pathway_daa(
    abundance = kegg_abundance,
    metadata = metadata,
    group = "Environment",
    daa_method = "LinDA"
  ))

  daa_annotated <- suppressMessages(pathway_annotation(
    pathway = "KO",
    ko_to_kegg = TRUE,
    daa_results_df = daa_results
  ))

  p <- pathway_volcano(daa_annotated, label_top_n = 5)
  expect_s3_class(p, "ggplot")
})

# Helper: create standard errorbar test data
create_errorbar_test_data <- function(n_features = 5, p_adjust = NULL) {
  set.seed(123)
  n_samples <- 10

  abundance <- matrix(runif(n_features * n_samples), nrow = n_features, ncol = n_samples)
  rownames(abundance) <- paste0("pathway", 1:n_features)
  colnames(abundance) <- paste0("sample", 1:n_samples)

  if (is.null(p_adjust)) p_adjust <- rep(0.01, n_features)

  daa_results_df <- data.frame(
    feature = paste0("pathway", 1:n_features),
    pathway_name = paste0("Pathway ", 1:n_features),
    p_adjust = p_adjust,
    method = rep("ALDEx2_Welch's t test", n_features),
    group1 = rep("GroupA", n_features),
    group2 = rep("GroupB", n_features),
    stringsAsFactors = FALSE
  )

  Group <- factor(rep(c("GroupA", "GroupB"), each = n_samples / 2))
  names(Group) <- paste0("sample", 1:n_samples)

  list(abundance = abundance, daa_results_df = daa_results_df, Group = Group)
}

test_that("pathway_errorbar basic functionality works", {
  td <- create_errorbar_test_data(
    n_features = 10,
    p_adjust = c(0.01, 0.02, 0.03, 0.04, 0.05, 0.06, 0.07, 0.08, 0.09, 0.1)
  )

  p <- pathway_errorbar(
    abundance = td$abundance,
    daa_results_df = td$daa_results_df,
    Group = td$Group,
    p_values_threshold = 0.05,
    select = paste0("pathway", 1:5),
    x_lab = "pathway_name"
  )

  expect_s3_class(p, "patchwork")
})

test_that("pathway_errorbar pathway_names_text_size parameter works", {
  td <- create_errorbar_test_data(
    n_features = 10,
    p_adjust = c(0.01, 0.02, 0.03, 0.04, 0.05, 0.06, 0.07, 0.08, 0.09, 0.1)
  )

  # Test with auto text size
  p1 <- pathway_errorbar(
    abundance = td$abundance,
    daa_results_df = td$daa_results_df,
    Group = td$Group,
    p_values_threshold = 0.05,
    select = paste0("pathway", 1:5),
    x_lab = "pathway_name",
    pathway_names_text_size = "auto"
  )

  expect_s3_class(p1, "patchwork")

  # Test with custom text size
  p2 <- pathway_errorbar(
    abundance = td$abundance,
    daa_results_df = td$daa_results_df,
    Group = td$Group,
    p_values_threshold = 0.05,
    select = paste0("pathway", 1:5),
    x_lab = "pathway_name",
    pathway_names_text_size = 12
  )

  expect_s3_class(p2, "patchwork")
})

test_that("pathway_errorbar handles missing annotations", {
  td <- create_errorbar_test_data(n_features = 2, p_adjust = c(0.01, 0.02))
  td$daa_results_df$pathway_name <- c(NA, "Pathway 2")

  p <- pathway_errorbar(
    abundance = td$abundance,
    daa_results_df = td$daa_results_df,
    Group = td$Group,
    x_lab = "pathway_name"
  )
  expect_s3_class(p, "patchwork")
})

test_that("pathway_errorbar handles too many features", {
  td <- create_errorbar_test_data(n_features = 31)

  expect_warning(
    pathway_errorbar(
      abundance = td$abundance,
      daa_results_df = td$daa_results_df,
      Group = td$Group,
      x_lab = "pathway_name"
    ),
    regexp = "Found \\d+ significant features"
  )
})

test_that("pathway_errorbar handles custom colors correctly", {
  td <- create_errorbar_test_data()

  p <- pathway_errorbar(
    abundance = td$abundance,
    daa_results_df = td$daa_results_df,
    Group = td$Group,
    colors = c("#FF0000", "#0000FF"),
    x_lab = "pathway_name"
  )

  expect_s3_class(p, "patchwork")
})

test_that("pathway_errorbar handles different ordering options", {
  td <- create_errorbar_test_data(p_adjust = c(0.04, 0.01, 0.03, 0.02, 0.05))
  td$daa_results_df$pathway_class <- c("Class1", "Class1", "Class2", "Class2", "Class3")

  for (order_type in c("p_values", "name", "group", "pathway_class")) {
    p <- pathway_errorbar(
      abundance = td$abundance,
      daa_results_df = td$daa_results_df,
      Group = td$Group,
      order = order_type,
      x_lab = "pathway_name"
    )
    expect_s3_class(p, "patchwork")
  }

  # Invalid order type (function lacks upfront validation; crashes downstream)
  expect_error(
    pathway_errorbar(
      abundance = td$abundance,
      daa_results_df = td$daa_results_df,
      Group = td$Group,
      order = "invalid_order",
      x_lab = "pathway_name"
    )
  )
})

test_that("pathway_errorbar regression: ko_to_kegg TRUE with pathway_class order", {
  td <- create_errorbar_test_data(
    n_features = 6,
    p_adjust = c(0.001, 0.002, 0.003, 0.004, 0.005, 0.006)
  )
  td$daa_results_df$pathway_class <- c("Class2", "Class1", "Class2", "Class1", "Class3", "Class3")

  # Regression guard for Issue #177/#196 path:
  # ko_to_kegg=TRUE + order='pathway_class' used to fail with
  # `tibble::column_to_rownames()` / "Can't find column `.`".
  expect_error(
    pathway_errorbar(
      abundance = td$abundance,
      daa_results_df = td$daa_results_df,
      Group = td$Group,
      ko_to_kegg = TRUE,
      order = "pathway_class",
      p_values_threshold = 0.05,
      x_lab = "pathway_name"
    ),
    NA
  )
})

test_that("pathway_errorbar aligns Group by names when provided", {
  td <- create_errorbar_test_data(
    n_features = 4,
    p_adjust = c(0.001, 0.002, 0.003, 0.004)
  )
  td$daa_results_df$pathway_class <- c("Class1", "Class1", "Class2", "Class2")

  # Intentionally shuffle Group order but keep sample names.
  shuffled_group <- td$Group[c(10, 9, 8, 7, 6, 5, 4, 3, 2, 1)]

  expect_error(
    pathway_errorbar(
      abundance = td$abundance,
      daa_results_df = td$daa_results_df,
      Group = shuffled_group,
      ko_to_kegg = TRUE,
      order = "pathway_class",
      p_values_threshold = 0.05,
      x_lab = "pathway_name"
    ),
    NA
  )
})

test_that("pathway_errorbar handles p_value_bar parameter correctly", {
  td <- create_errorbar_test_data()

  p1 <- pathway_errorbar(
    abundance = td$abundance,
    daa_results_df = td$daa_results_df,
    Group = td$Group,
    p_value_bar = TRUE,
    x_lab = "pathway_name"
  )

  p2 <- pathway_errorbar(
    abundance = td$abundance,
    daa_results_df = td$daa_results_df,
    Group = td$Group,
    p_value_bar = FALSE,
    x_lab = "pathway_name"
  )

  expect_s3_class(p1, "patchwork")
  expect_s3_class(p2, "patchwork")
  expect_false(identical(p1, p2))
})

test_that("pathway_errorbar_table function works correctly", {
  td <- create_errorbar_test_data(n_features = 3, p_adjust = c(0.01, 0.02, 0.03))

  metadata <- data.frame(
    sample = colnames(td$abundance),
    group = td$Group,
    stringsAsFactors = FALSE
  )
  daa_results <- pathway_daa(
    abundance = td$abundance,
    metadata = metadata,
    group = "group",
    daa_method = "ALDEx2"
  )
  daa_single_method <- daa_results[daa_results$method == "ALDEx2_Welch's t test", ]

  result <- pathway_errorbar_table(
    abundance = td$abundance,
    daa_results_df = daa_single_method,
    Group = td$Group,
    p_values_threshold = 1.0
  )

  expect_s3_class(result, "data.frame")
  expect_true(nrow(result) > 0)

  expected_cols <- c("feature", "group1", "group2",
                    "mean_rel_abundance_group1", "sd_rel_abundance_group1",
                    "mean_rel_abundance_group2", "sd_rel_abundance_group2",
                    "log2_fold_change", "p_adjust")
  expect_true(all(expected_cols %in% colnames(result)))
})

#' @importFrom grDevices col2rgb
#' @importFrom methods new
#' @importFrom stats as.formula model.matrix relevel p.adjust reorder var prcomp sd
#' @importFrom utils head data

.onLoad <- function(libname, pkgname) {
  op <- options()
  op.ggpicrust2 <- list(
    ggpicrust2.cache_enabled = TRUE,
    ggpicrust2.verbose = TRUE,
    ggpicrust2.max_retries = 3,
    ggpicrust2.timeout = 300
  )
  toset <- !(names(op.ggpicrust2) %in% names(op))
  if (any(toset)) options(op.ggpicrust2[toset])
  
  invisible()
}

.onAttach <- function(libname, pkgname) {
  packageStartupMessage("Loading required package: ggpicrust2")
  packageStartupMessage("To cite ggpicrust2 in publications use:")
  packageStartupMessage("Chen Yang, Jiahao Mai, Xuan Cao, Aaron Burberry, Fabio Cominelli, Liangliang Zhang, ggpicrust2: an R package for PICRUSt2 predicted functional profile analysis and visualization, Bioinformatics, Volume 39, Issue 8, August 2023, btad470, https://doi.org/10.1093/bioinformatics/btad470")
} 

#' Legend and Annotation Utilities for ggpicrust2
#'
#' This module provides enhanced legend and annotation functionality for ggpicrust2
#' visualizations, including intelligent p-value formatting, significance marking,
#' and customizable legend styling.
#'
#' @name legend_annotation_utils
NULL

#' Smart P-value Formatting
#'
#' @param p_values Numeric vector of p-values
#' @param format Character string specifying format type
#' @param stars Logical, whether to include star symbols
#' @param thresholds Numeric vector of significance thresholds
#' @param star_symbols Character vector of star symbols
#' @return Character vector of formatted p-values
#' @export
format_pvalue_smart <- function(p_values, 
                               format = "smart", 
                               stars = TRUE,
                               thresholds = c(0.001, 0.01, 0.05),
                               star_symbols = c("***", "**", "*")) {
  
  # Input validation
  if (!is.numeric(p_values)) {
    stop("p_values must be numeric")
  }
  
  format <- match.arg(format, c("numeric", "scientific", "smart", "stars_only", "combined"))
  
  # Base formatting
  formatted <- switch(format,
    "numeric" = sprintf("%.3f", p_values),
    "scientific" = sprintf("%.2e", p_values),
    "smart" = ifelse(p_values < 0.001, 
                    sprintf("p < 0.001"), 
                    ifelse(p_values < 0.01,
                          sprintf("p = %.3f", p_values),
                          sprintf("p = %.2f", p_values))),
    "stars_only" = rep("", length(p_values)),
    "combined" = sprintf("%.3f", p_values)
  )
  
  # Add significance stars if requested
  if (stars && format != "numeric") {
    stars_vec <- get_significance_stars(p_values, thresholds, star_symbols)
    
    if (format == "stars_only") {
      formatted <- stars_vec
    } else if (format == "combined") {
      formatted <- paste(formatted, stars_vec)
    } else {
      # For smart format, add stars after the p-value
      formatted <- ifelse(stars_vec != "", 
                         paste(formatted, stars_vec), 
                         formatted)
    }
  }
  
  return(formatted)
}

#' Get Significance Stars
#'
#' @param p_values Numeric vector of p-values
#' @param thresholds Numeric vector of significance thresholds
#' @param symbols Character vector of star symbols
#' @return Character vector of star symbols
#' @export
get_significance_stars <- function(p_values, 
                                 thresholds = c(0.001, 0.01, 0.05),
                                 symbols = c("***", "**", "*")) {
  
  if (length(thresholds) != length(symbols)) {
    stop("thresholds and symbols must have the same length")
  }
  
  stars <- character(length(p_values))
  
  for (i in seq_along(thresholds)) {
    stars[p_values < thresholds[i]] <- symbols[i]
  }
  
  return(stars)
}

#' Get Significance Colors
#'
#' @param p_values Numeric vector of p-values
#' @param thresholds Numeric vector of significance thresholds  
#' @param colors Character vector of colors for each significance level
#' @param default_color Default color for non-significant values
#' @return Character vector of colors
#' @export
get_significance_colors <- function(p_values,
                                  thresholds = c(0.001, 0.01, 0.05),
                                  colors = c("#d73027", "#fc8d59", "#fee08b"),
                                  default_color = "#999999") {
  
  if (length(thresholds) != length(colors)) {
    stop("thresholds and colors must have the same length")
  }
  
  result_colors <- rep(default_color, length(p_values))
  
  for (i in seq_along(thresholds)) {
    result_colors[p_values < thresholds[i]] <- colors[i]
  }
  
  return(result_colors)
}

#' Create Enhanced Legend Theme
#'
#' @param position Legend position ("top", "bottom", "left", "right", "none")
#' @param direction Legend direction ("horizontal", "vertical")
#' @param title Legend title
#' @param title_size Title font size
#' @param text_size Text font size
#' @param key_size Key size in cm
#' @param key_width Key width
#' @param key_height Key height
#' @param ncol Number of columns
#' @param nrow Number of rows
#' @param box_just Legend box justification
#' @param margin Legend margin
#' @return ggplot2 theme elements
#' @export
create_legend_theme <- function(position = "top",
                               direction = "horizontal", 
                               title = NULL,
                               title_size = 12,
                               text_size = 10,
                               key_size = 0.8,
                               key_width = NULL,
                               key_height = NULL,
                               ncol = NULL,
                               nrow = NULL,
                               box_just = "center",
                               margin = ggplot2::margin(0, 0, 0, 0)) {
  
  # Validate inputs
  position <- match.arg(position, c("top", "bottom", "left", "right", "none"))
  direction <- match.arg(direction, c("horizontal", "vertical"))
  box_just <- match.arg(box_just, c("center", "top", "bottom", "left", "right"))
  
  # Auto-adjust direction based on position if not explicitly set
  if (missing(direction)) {
    direction <- ifelse(position %in% c("top", "bottom"), "horizontal", "vertical")
  }
  
  # Create legend theme
  legend_theme <- ggplot2::theme(
    legend.position = position,
    legend.direction = direction,
    legend.title = if (!is.null(title)) {
      ggplot2::element_text(size = title_size, face = "bold")
    } else {
      ggplot2::element_blank()
    },
    legend.text = ggplot2::element_text(size = text_size),
    legend.key.size = ggplot2::unit(key_size, "cm"),
    legend.box.just = box_just,
    legend.margin = margin
  )
  
  # Add key dimensions if specified
  if (!is.null(key_width)) {
    legend_theme$legend.key.width <- ggplot2::unit(key_width, "cm")
  }
  if (!is.null(key_height)) {
    legend_theme$legend.key.height <- ggplot2::unit(key_height, "cm")
  }
  
  # Add guide specifications for multi-column/row layouts
  guide_specs <- list()
  if (!is.null(ncol)) {
    guide_specs$ncol <- ncol
  }
  if (!is.null(nrow)) {
    guide_specs$nrow <- nrow
  }
  
  return(list(theme = legend_theme, guide_specs = guide_specs))
}

#' Smart Text Size Calculator
#'
#' @param n_items Number of items to display
#' @param base_size Base text size
#' @param min_size Minimum text size
#' @param max_size Maximum text size
#' @return Calculated text size
#' @export
calculate_smart_text_size <- function(n_items, base_size = 10, min_size = 8, max_size = 14) {
  
  if (n_items <= 5) {
    size <- base_size
  } else if (n_items <= 10) {
    size <- base_size * 0.9
  } else if (n_items <= 20) {
    size <- base_size * 0.8
  } else {
    size <- base_size * 0.7
  }
  
  # Ensure within bounds
  size <- max(min_size, min(max_size, size))
  
  return(size)
}

#' Create Pathway Class Annotation Theme
#'
#' @param text_size Text size
#' @param text_color Text color
#' @param text_face Text face ("plain", "bold", "italic")
#' @param text_family Text family
#' @param text_angle Text angle in degrees
#' @param text_hjust Horizontal justification (0-1)
#' @param text_vjust Vertical justification (0-1)
#' @param bg_color Background color
#' @param bg_alpha Background alpha
#' @param position Annotation position ("left", "right", "none")
#' @return List of annotation styling parameters
#' @export
create_pathway_class_theme <- function(text_size = "auto",
                                     text_color = "black",
                                     text_face = "bold",
                                     text_family = "sans",
                                     text_angle = 0,
                                     text_hjust = 0.5,
                                     text_vjust = 0.5,
                                     bg_color = NULL,
                                     bg_alpha = 0.2,
                                     position = "left") {
  
  # Validate inputs
  text_face <- match.arg(text_face, c("plain", "bold", "italic"))
  position <- match.arg(position, c("left", "right", "none"))
  
  return(list(
    text_size = text_size,
    text_color = text_color,
    text_face = text_face,
    text_family = text_family,
    text_angle = text_angle,
    text_hjust = text_hjust,
    text_vjust = text_vjust,
    bg_color = bg_color,
    bg_alpha = bg_alpha,
    position = position
  ))
}

#' Detect and Resolve Annotation Overlaps
#'
#' @param labels Character vector of labels
#' @param positions Numeric vector of positions
#' @param min_distance Minimum distance between labels
#' @return Adjusted positions
#' @export
resolve_annotation_overlaps <- function(labels, positions, min_distance = 1) {
  
  if (length(labels) != length(positions)) {
    stop("labels and positions must have the same length")
  }
  
  if (length(positions) <= 1) {
    return(positions)
  }
  
  # Sort by position
  order_idx <- order(positions)
  sorted_positions <- positions[order_idx]
  
  # Adjust overlapping positions
  adjusted_positions <- sorted_positions
  
  for (i in 2:length(adjusted_positions)) {
    if (adjusted_positions[i] - adjusted_positions[i-1] < min_distance) {
      adjusted_positions[i] <- adjusted_positions[i-1] + min_distance
    }
  }
  
  # Restore original order
  result_positions <- numeric(length(positions))
  result_positions[order_idx] <- adjusted_positions
  
  return(result_positions)
}

# Export utility functions
utils::globalVariables(c("p_values", "thresholds", "symbols", "colors"))

#' Color Theme System for ggpicrust2
#'
#' This module provides a comprehensive color theme system for ggpicrust2 visualizations,
#' including journal-specific themes, colorblind-friendly palettes, and intelligent
#' color selection based on data characteristics.
#'
#' @name color_themes
#' @importFrom grDevices colorRampPalette
NULL

#' Get Available Color Themes
#' 
#' @return A character vector of available theme names
#' @export
get_available_themes <- function() {
  return(c("default", "nature", "science", "cell", "nejm", "lancet", 
           "colorblind_friendly", "viridis", "plasma", "minimal", 
           "high_contrast", "pastel", "bold"))
}

#' Get Color Theme
#'
#' @param theme_name Character string specifying the theme name
#' @param n_colors Integer specifying the number of colors needed
#' @return A list containing theme colors and settings
#' @export
get_color_theme <- function(theme_name = "default", n_colors = 8) {
  
  # Define base themes
  themes <- list(
    
    # Default theme (current ggpicrust2 colors)
    default = list(
      group_colors = c("#d93c3e", "#3685bc", "#6faa3e", "#e8a825", "#c973e6", "#ee6b3d", "#2db0a7", "#f25292"),
      pathway_class_colors = c("#D51F26","#272E6A","#208A42","#89288F","#F47D2B",
                              "#FEE500","#8A9FD1","#C06CAB","#E6C2DC","#90D5E4",
                              "#89C75F","#F37B7D","#9983BD","#D24B27","#3BBCA8",
                              "#6E4B9E","#0C727C", "#7E1416","#D8A767","#3D3D3D"),
      fold_change_colors = c(negative = "#2166ac", neutral = "#f7f7f7", positive = "#b2182b"),
      fold_change_single = "#87ceeb",
      description = "Default ggpicrust2 color scheme"
    ),
    
    # Nature journal style
    nature = list(
      group_colors = c("#E64B35", "#4DBBD5", "#00A087", "#3C5488", "#F39B7F", "#8491B4", "#91D1C2", "#DC0000"),
      pathway_class_colors = c("#E64B35", "#4DBBD5", "#00A087", "#3C5488", "#F39B7F", "#8491B4", "#91D1C2", "#DC0000",
                              "#B09C85", "#F39B7F", "#FFDC91", "#8491B4", "#91D1C2", "#7E6148", "#B2DF8A", "#33A02C",
                              "#CAB2D6", "#6A3D9A", "#FFFF99", "#B15928"),
      fold_change_colors = c(negative = "#3C5488", neutral = "#F7F7F7", positive = "#E64B35"),
      fold_change_single = "#4DBBD5",
      description = "Nature journal inspired colors - professional and clear"
    ),
    
    # Science journal style  
    science = list(
      group_colors = c("#1f77b4", "#ff7f0e", "#2ca02c", "#d62728", "#9467bd", "#8c564b", "#e377c2", "#7f7f7f"),
      pathway_class_colors = c("#1f77b4", "#ff7f0e", "#2ca02c", "#d62728", "#9467bd", "#8c564b", "#e377c2", "#7f7f7f",
                              "#bcbd22", "#17becf", "#aec7e8", "#ffbb78", "#98df8a", "#ff9896", "#c5b0d5", "#c49c94",
                              "#f7b6d3", "#c7c7c7", "#dbdb8d", "#9edae5"),
      fold_change_colors = c(negative = "#1f77b4", neutral = "#f0f0f0", positive = "#d62728"),
      fold_change_single = "#2ca02c",
      description = "Science journal style - classic and authoritative"
    ),
    
    # Cell journal style
    cell = list(
      group_colors = c("#c51b7d", "#4d9221", "#2166ac", "#762a83", "#f1a340", "#998ec3", "#1b7837", "#e08214"),
      pathway_class_colors = c("#c51b7d", "#4d9221", "#2166ac", "#762a83", "#f1a340", "#998ec3", "#1b7837", "#e08214",
                              "#f7f7f7", "#d9f0d3", "#a6dba0", "#5aae61", "#1b7837", "#00441b", "#f4a582", "#d6604d",
                              "#b2182b", "#67001f", "#053061", "#2166ac"),
      fold_change_colors = c(negative = "#2166ac", neutral = "#f7f7f7", positive = "#c51b7d"),
      fold_change_single = "#4d9221",
      description = "Cell journal style - vibrant and distinctive"
    ),
    
    # NEJM style
    nejm = list(
      group_colors = c("#BC3C29", "#0072B5", "#E18727", "#20854E", "#7876B1", "#6F99AD", "#FFDC91", "#EE4C97"),
      pathway_class_colors = c("#BC3C29", "#0072B5", "#E18727", "#20854E", "#7876B1", "#6F99AD", "#FFDC91", "#EE4C97",
                              "#B09C85", "#F39B7F", "#8491B4", "#91D1C2", "#DC0000", "#7E6148", "#B2DF8A", "#33A02C",
                              "#CAB2D6", "#6A3D9A", "#FFFF99", "#B15928"),
      fold_change_colors = c(negative = "#0072B5", neutral = "#F5F5F5", positive = "#BC3C29"),
      fold_change_single = "#20854E",
      description = "New England Journal of Medicine style - medical professional"
    ),
    
    # Lancet style
    lancet = list(
      group_colors = c("#00468B", "#ED0000", "#42B540", "#0099B4", "#925E9F", "#FDAF91", "#AD002A", "#ADB6B6"),
      pathway_class_colors = c("#00468B", "#ED0000", "#42B540", "#0099B4", "#925E9F", "#FDAF91", "#AD002A", "#ADB6B6",
                              "#1B1919", "#FF6A6A", "#90EE90", "#87CEEB", "#DDA0DD", "#F0E68C", "#FFB6C1", "#D3D3D3",
                              "#8FBC8F", "#20B2AA", "#9370DB", "#CD853F"),
      fold_change_colors = c(negative = "#00468B", neutral = "#F8F8FF", positive = "#ED0000"),
      fold_change_single = "#42B540",
      description = "The Lancet style - authoritative medical colors"
    ),
    
    # Colorblind friendly (using ColorBrewer)
    colorblind_friendly = list(
      group_colors = c("#1b9e77", "#d95f02", "#7570b3", "#e7298a", "#66a61e", "#e6ab02", "#a6761d", "#666666"),
      pathway_class_colors = c("#1b9e77", "#d95f02", "#7570b3", "#e7298a", "#66a61e", "#e6ab02", "#a6761d", "#666666",
                              "#a6cee3", "#1f78b4", "#b2df8a", "#33a02c", "#fb9a99", "#e31a1c", "#fdbf6f", "#ff7f00",
                              "#cab2d6", "#6a3d9a", "#ffff99", "#b15928"),
      fold_change_colors = c(negative = "#7570b3", neutral = "#f7f7f7", positive = "#d95f02"),
      fold_change_single = "#1b9e77",
      description = "Colorblind friendly palette - accessible to all users"
    ),
    
    # Viridis-inspired
    viridis = list(
      group_colors = c("#440154", "#31688e", "#35b779", "#fde725", "#482777", "#287d8e", "#95d840", "#c2df23"),
      pathway_class_colors = c("#440154", "#482777", "#3f4a8a", "#31688e", "#26838f", "#1f9d8a", "#6cce5a", "#b6de2b",
                              "#fee825", "#35b779", "#1fa187", "#4fbc73", "#85d54a", "#c2df23", "#f0f921", "#fcffa4",
                              "#2d6a8c", "#3d4a8a", "#482777", "#440154"),
      fold_change_colors = c(negative = "#440154", neutral = "#35b779", positive = "#fde725"),
      fold_change_single = "#31688e",
      description = "Viridis-inspired - perceptually uniform colors"
    ),
    
    # Plasma-inspired
    plasma = list(
      group_colors = c("#0d0887", "#6a00a8", "#b12a90", "#e16462", "#fca636", "#f0f921", "#9c179e", "#5302a3"),
      pathway_class_colors = c("#0d0887", "#2e049c", "#4c02a1", "#6a00a8", "#8b0aa5", "#a82296", "#c43c75", "#dd513a",
                              "#f1605d", "#feaa3a", "#feca57", "#f0f921", "#b12a90", "#e16462", "#fca636", "#fcffa4",
                              "#9c179e", "#5302a3", "#320597", "#0d0887"),
      fold_change_colors = c(negative = "#0d0887", neutral = "#dd513a", positive = "#f0f921"),
      fold_change_single = "#b12a90",
      description = "Plasma-inspired - vibrant and energetic"
    ),
    
    # Minimal/clean style
    minimal = list(
      group_colors = c("#2c3e50", "#e74c3c", "#3498db", "#2ecc71", "#f39c12", "#9b59b6", "#1abc9c", "#34495e"),
      pathway_class_colors = c("#2c3e50", "#34495e", "#3498db", "#2980b9", "#2ecc71", "#27ae60", "#e74c3c", "#c0392b",
                              "#f39c12", "#e67e22", "#9b59b6", "#8e44ad", "#1abc9c", "#16a085", "#f1c40f", "#95a5a6",
                              "#ecf0f1", "#bdc3c7", "#7f8c8d", "#2c3e50"),
      fold_change_colors = c(negative = "#3498db", neutral = "#ecf0f1", positive = "#e74c3c"),
      fold_change_single = "#2ecc71",
      description = "Minimal clean style - modern and professional"
    ),
    
    # High contrast
    high_contrast = list(
      group_colors = c("#000000", "#E31A1C", "#1F78B4", "#33A02C", "#FF7F00", "#6A3D9A", "#B15928", "#A6CEE3"),
      pathway_class_colors = c("#000000", "#E31A1C", "#1F78B4", "#33A02C", "#FF7F00", "#6A3D9A", "#B15928", "#A6CEE3",
                              "#FDBF6F", "#FF7F00", "#CAB2D6", "#6A3D9A", "#FFFF99", "#B15928", "#FFFFFF", "#999999",
                              "#FB9A99", "#E31A1C", "#A6CEE3", "#1F78B4"),
      fold_change_colors = c(negative = "#1F78B4", neutral = "#FFFFFF", positive = "#E31A1C"),
      fold_change_single = "#33A02C",
      description = "High contrast - maximum visibility and clarity"
    ),
    
    # Pastel theme
    pastel = list(
      group_colors = c("#FFB3BA", "#BAFFC9", "#BAE1FF", "#FFFFBA", "#FFD1FF", "#E1BAFF", "#BAFFE1", "#FFBABA"),
      pathway_class_colors = c("#FFB3BA", "#FFDFBA", "#FFFFBA", "#BAFFBA", "#BAFFFF", "#BABDFF", "#DBBAFF", "#FFBAD1",
                              "#FFC9BA", "#D1FFBA", "#BAFFE1", "#BAF7FF", "#BAD1FF", "#E1BAFF", "#FFBADB", "#F7FFBA",
                              "#C9BAFF", "#FFBAC9", "#BADBFF", "#BAFFBA"),
      fold_change_colors = c(negative = "#BAE1FF", neutral = "#FFFFBA", positive = "#FFB3BA"),
      fold_change_single = "#BAFFC9",
      description = "Pastel theme - soft and gentle colors"
    ),
    
    # Bold theme
    bold = list(
      group_colors = c("#FF0000", "#00FF00", "#0000FF", "#FFFF00", "#FF00FF", "#00FFFF", "#FF8000", "#8000FF"),
      pathway_class_colors = c("#FF0000", "#00FF00", "#0000FF", "#FFFF00", "#FF00FF", "#00FFFF", "#FF8000", "#8000FF",
                              "#FF4000", "#80FF00", "#0080FF", "#FF0080", "#8000FF", "#FF8040", "#40FF80", "#4080FF",
                              "#FF4080", "#80FF40", "#8040FF", "#FF0040"),
      fold_change_colors = c(negative = "#0000FF", neutral = "#808080", positive = "#FF0000"),
      fold_change_single = "#00FF00",
      description = "Bold theme - maximum impact and visibility"
    )
  )
  
  # Get the requested theme
  if (!theme_name %in% names(themes)) {
    warning(paste("Theme", theme_name, "not found. Using default theme."))
    theme_name <- "default"
  }
  
  theme <- themes[[theme_name]]
  
  # Adjust colors to requested number
  if (n_colors > 0) {
    theme$group_colors <- rep(theme$group_colors, length.out = n_colors)
    theme$pathway_class_colors <- rep(theme$pathway_class_colors, length.out = max(n_colors, 20))
  }
  
  return(theme)
}

#' Smart Color Selection
#'
#' Intelligently selects colors based on data characteristics
#'
#' @param n_groups Number of groups in the data
#' @param has_pathway_class Whether pathway class information is available
#' @param data_type Type of data ("abundance", "pvalue", "foldchange")
#' @param accessibility_mode Whether to use accessibility-friendly colors
#' @return A list with suggested theme and colors
#' @export
smart_color_selection <- function(n_groups, has_pathway_class = FALSE, 
                                 data_type = "abundance", accessibility_mode = FALSE) {
  
  # Base recommendation logic
  if (accessibility_mode) {
    recommended_theme <- "colorblind_friendly"
  } else if (n_groups <= 3) {
    recommended_theme <- "nature"  # Clean and professional for few groups
  } else if (n_groups <= 6) {
    recommended_theme <- "science"  # Good balance for medium number of groups
  } else {
    recommended_theme <- "viridis"  # Better for many groups
  }
  
  # Adjust based on data type
  if (data_type == "pvalue") {
    recommended_theme <- "high_contrast"  # Better for significance visualization
  } else if (data_type == "foldchange") {
    recommended_theme <- "cell"  # Good diverging colors for fold changes
  }
  
  # Get the theme
  theme <- get_color_theme(recommended_theme, n_groups)
  
  # Add recommendation explanation
  theme$recommendation_reason <- paste(
    "Selected", recommended_theme, "theme for", n_groups, "groups,",
    "data type:", data_type,
    if (accessibility_mode) ", with accessibility considerations" else ""
  )
  
  return(list(
    theme_name = recommended_theme,
    theme = theme,
    reason = theme$recommendation_reason
  ))
}

#' Create Gradient Colors
#'
#' Creates gradient colors for fold change visualization
#'
#' @param theme_name Character string specifying the theme
#' @param n_colors Number of colors in the gradient
#' @param diverging Whether to create a diverging gradient (for fold changes)
#' @return A vector of colors
#' @export
create_gradient_colors <- function(theme_name = "default", n_colors = 11, diverging = TRUE) {
  
  theme <- get_color_theme(theme_name)
  
  if (n_colors == 1) {
    # Special case for single color
    colors <- theme$group_colors[1]
  } else if (diverging) {
    # Create diverging gradient for fold changes
    low_color <- theme$fold_change_colors["negative"]
    mid_color <- theme$fold_change_colors["neutral"] 
    high_color <- theme$fold_change_colors["positive"]
    
    if (n_colors == 2) {
      colors <- c(low_color, high_color)
    } else {
      # Create gradient
      n_half <- (n_colors - 1) / 2
      low_gradient <- colorRampPalette(c(low_color, mid_color))(n_half + 1)
      high_gradient <- colorRampPalette(c(mid_color, high_color))(n_half + 1)
      
      colors <- c(low_gradient[1:n_half], mid_color, high_gradient[2:(n_half + 1)])
    }
  } else {
    # Create single-direction gradient
    colors <- colorRampPalette(theme$group_colors[1:2])(n_colors)
  }
  
  return(colors)
}

#' Preview Color Theme
#'
#' Creates a visual preview of a color theme
#'
#' @param theme_name Character string specifying the theme name
#' @param save_plot Whether to save the preview plot
#' @param filename Filename for saved plot
#' @return A ggplot object showing the color preview
#' @export
preview_color_theme <- function(theme_name = "default", save_plot = FALSE, filename = NULL) {
  
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("ggplot2 is required for theme preview")
  }
  
  theme <- get_color_theme(theme_name, 8)
  
  # Create preview data
  preview_data <- data.frame(
    type = rep(c("Group Colors", "Pathway Class Colors", "Fold Change Colors"), 
               c(8, 20, 3)),
    color = c(theme$group_colors[1:8], 
              theme$pathway_class_colors[1:20],
              unlist(theme$fold_change_colors)),
    index = c(1:8, 1:20, 1:3),
    label = c(paste("Group", 1:8),
              paste("Class", 1:20), 
              c("Negative", "Neutral", "Positive"))
  )
  
  # Create the plot
  p <- ggplot2::ggplot(preview_data, ggplot2::aes(x = index, y = type, fill = color)) +
    ggplot2::geom_tile(color = "white", linewidth = 0.5) +
    ggplot2::scale_fill_identity() +
    ggplot2::geom_text(ggplot2::aes(label = label), size = 3, angle = 45) +
    ggplot2::labs(
      title = paste("Color Theme Preview:", theme_name),
      subtitle = theme$description,
      x = "Color Index",
      y = "Color Category"
    ) +
    ggplot2::theme_minimal() +
    ggplot2::theme(
      axis.text.x = ggplot2::element_blank(),
      axis.ticks = ggplot2::element_blank(),
      panel.grid = ggplot2::element_blank(),
      plot.title = ggplot2::element_text(hjust = 0.5, size = 14, face = "bold"),
      plot.subtitle = ggplot2::element_text(hjust = 0.5, size = 12)
    )
  
  if (save_plot) {
    if (is.null(filename)) {
      filename <- paste0("color_theme_preview_", theme_name, ".pdf")
    }
    ggplot2::ggsave(filename, p, width = 12, height = 6)
    message(paste("Theme preview saved as", filename))
  }
  
  return(p)
}

# Export all functions
utils::globalVariables(c("type", "color", "index", "label"))

#' Generate Abundance Statistics Table for Pathway Analysis
#'
#' This function generates a table containing mean relative abundance, 
#' standard deviation, and log2 fold change statistics for pathways, 
#' similar to the data used in pathway_errorbar plots but returned as a 
#' data frame instead of a plot.
#'
#' @param abundance A data frame or matrix containing predicted functional 
#'        pathway abundance, with pathways/features as rows and samples as 
#'        columns. The column names should match the sample names in metadata.
#' @param daa_results_df A data frame containing differential abundance 
#'        analysis results from pathway_daa function. Must contain columns: 
#'        feature, group1, group2, p_adjust.
#' @param Group A vector containing group assignments for each sample in the
#'        same order as the columns in abundance matrix. Alternatively, if
#'        metadata is provided, this should match the order of samples in metadata.
#' @param ko_to_kegg Logical value indicating whether to use KO to KEGG 
#'        conversion. Default is FALSE.
#' @param p_values_threshold Numeric value for p-value threshold to filter 
#'        significant features. Default is 0.05.
#' @param select Character vector of specific features to include. If NULL, 
#'        all significant features are included.
#' @param max_features Maximum number of features to include in the table.
#'        Default is 30.
#' @param metadata Optional data frame containing sample metadata. If provided,
#'        the Group vector will be reordered to match the abundance column order.
#' @param sample_col Character string specifying the column name in metadata
#'        that contains sample identifiers. Default is NULL, which triggers
#'        auto-detection via the same logic used by \code{pathway_daa()}
#'        (common names like "sample", "Sample", "sample_id", "sample_name",
#'        or metadata rownames).
#'
#' @return A data frame containing the following columns:
#' \itemize{
#'   \item \code{feature}: Feature/pathway identifier
#'   \item \code{group1}: Reference group name
#'   \item \code{group2}: Comparison group name
#'   \item \code{mean_rel_abundance_group1}: Mean relative abundance for group1
#'   \item \code{sd_rel_abundance_group1}: Standard deviation of relative
#'         abundance for group1
#'   \item \code{mean_rel_abundance_group2}: Mean relative abundance for group2
#'   \item \code{sd_rel_abundance_group2}: Standard deviation of relative
#'         abundance for group2
#'   \item \code{log2_fold_change}: Log2 fold change (group2/group1)
#'   \item \code{p_adjust}: Adjusted p-value from differential analysis
#'   \item Additional annotation columns (e.g., \code{description},
#'         \code{pathway_name}, \code{pathway_class}) if present in the input
#'         daa_results_df
#' }
#'
#' @examples
#' \dontrun{
#' # Load example data
#' data("ko_abundance")
#' data("metadata")
#' 
#' # Convert KO abundance to KEGG pathways
#' kegg_abundance <- ko2kegg_abundance(data = ko_abundance)
#' 
#' # Perform differential abundance analysis
#' daa_results_df <- pathway_daa(
#'   abundance = kegg_abundance,
#'   metadata = metadata,
#'   group = "Environment",
#'   daa_method = "ALDEx2"
#' )
#' 
#' # Filter for specific method
#' daa_sub_method_results_df <- daa_results_df[
#'   daa_results_df$method == "ALDEx2_Welch's t test", 
#' ]
#' 
#' # Annotate results
#' daa_annotated_sub_method_results_df <- pathway_annotation(
#'   pathway = "KO",
#'   daa_results_df = daa_sub_method_results_df,
#'   ko_to_kegg = TRUE
#' )
#' 
#' # Generate abundance statistics table
#' abundance_stats_table <- pathway_errorbar_table(
#'   abundance = kegg_abundance,
#'   daa_results_df = daa_annotated_sub_method_results_df,
#'   Group = metadata$Environment,
#'   ko_to_kegg = TRUE,
#'   p_values_threshold = 0.05
#' )
#'
#' # View the results
#' head(abundance_stats_table)
#' }
#'
#' @importFrom stats setNames
#' @export
pathway_errorbar_table <- function(abundance,
                                  daa_results_df,
                                  Group,
                                  ko_to_kegg = FALSE,
                                  p_values_threshold = 0.05,
                                  select = NULL,
	                                  max_features = 30,
	                                  metadata = NULL,
	                                  sample_col = NULL) {
  ko_to_kegg <- normalize_logical_flag(ko_to_kegg, "ko_to_kegg")

	  # Input validation
  if (!is.matrix(abundance) && !is.data.frame(abundance)) {
    stop("'abundance' must be a matrix or data frame")
  }

  if (!is.data.frame(daa_results_df)) {
    stop("'daa_results_df' must be a data frame")
  }

  # Handle Group vector ordering.
  #
  # Two supported input shapes:
  #   1. metadata is NULL -- Group is assumed to already be in the same
  #      order as colnames(abundance); we just length-check it below.
  #   2. metadata is provided -- Group is in metadata-row order and we need
  #      to realign to abundance-column order. Reuse the package-wide
  #      align_samples() helper for sample-column detection and reordering
  #      instead of re-implementing it here; this keeps accepted metadata
  #      shapes (sample / Sample / sample_id / sample_name / rownames)
  #      identical to pathway_daa() and ggpicrust2().
  if (!is.null(metadata)) {
    metadata <- as.data.frame(metadata)
    if (length(Group) != nrow(metadata)) {
      stop("Length of Group (", length(Group),
           ") does not match number of rows in metadata (",
           nrow(metadata), ")")
    }
    # Attach Group as a private column so align_samples() carries it
    # through the intersect+reorder step without us needing to track
    # indices by hand. Column name is prefixed to avoid colliding with
    # any user column.
    metadata[[".pet_group"]] <- Group
    aligned <- align_samples(abundance, metadata,
                             sample_col = sample_col, verbose = FALSE)
    abundance <- aligned$abundance
    Group <- aligned$metadata[[".pet_group"]]
  }

  required_cols <- c("feature", "group1", "group2", "p_adjust")
  missing_cols <- setdiff(required_cols, colnames(daa_results_df))
  if (length(missing_cols) > 0) {
    stop("Missing required columns in daa_results_df: ",
         paste(missing_cols, collapse = ", "))
  }

  if (length(Group) != ncol(abundance)) {
    stop("Length of Group (", length(Group),
         ") must match number of columns in abundance (",
         ncol(abundance), ")")
  }

	  # Validate single method and group pair
	  validate_daa_results(daa_results_df)
  if (ko_to_kegg && !"pathway_class" %in% colnames(daa_results_df)) {
    stop(
      "The 'pathway_class' column is missing but ko_to_kegg = TRUE. ",
      "Please use pathway_annotation(..., ko_to_kegg = TRUE) to annotate the data, ",
      "or set ko_to_kegg = FALSE if you don't need pathway class annotations.",
      call. = FALSE
    )
  }
  
  # Filter for significant features
  daa_results_filtered_df <- daa_results_df[
    daa_results_df$p_adjust < p_values_threshold, 
  ]
  
  if (!is.null(select)) {
    daa_results_filtered_sub_df <- daa_results_filtered_df[
      daa_results_filtered_df$feature %in% select, 
    ]
  } else {
    daa_results_filtered_sub_df <- daa_results_filtered_df
  }
  
  if (nrow(daa_results_filtered_sub_df) > max_features) {
    warning("The number of features with statistical significance exceeds ", 
            max_features, ". Consider using 'select' parameter or increasing ",
            "'max_features' to include all features.")
  }
  
  if (nrow(daa_results_filtered_sub_df) == 0) {
    stop("No features with statistical significance found. ",
         "Consider adjusting p_values_threshold.")
  }
  
  # Get group names. validate_daa_results() above hard-rejects multi-pair
  # input, so daa_results_filtered_sub_df is guaranteed to carry a single
  # (group1, group2) pair here. Avoid the `unique(...)[1]` idiom because
  # it LOOKS like "silently pick the first of many" behavior -- that is
  # exactly the bug we rely on the validator to prevent, and leaving the
  # idiom in place would mask a validator bypass with silently truncated
  # results instead of failing fast.
  group1_name <- daa_results_filtered_sub_df$group1[1]
  group2_name <- daa_results_filtered_sub_df$group2[1]
  
  # Calculate abundance statistics using the helper function.
  # Group is already confirmed (above) to be in the same order as
  # colnames(abundance) with matching length, so wrap it into a minimal
  # metadata frame for calculate_abundance_stats().
  abundance_metadata <- data.frame(
    sample = colnames(abundance),
    group_col = Group,
    stringsAsFactors = FALSE
  )

  abundance_stats <- calculate_abundance_stats(
    abundance = abundance,
    metadata = abundance_metadata,
    group = "group_col",
    features = daa_results_filtered_sub_df$feature,
    group1 = group1_name,
    group2 = group2_name
  )
  
  # Identify annotation columns to include (excluding columns already in abundance_stats)
  abundance_stats_cols <- colnames(abundance_stats)
  annotation_cols <- setdiff(colnames(daa_results_filtered_sub_df),
                             c(abundance_stats_cols, "method", "p_values"))

  # Include p_adjust and annotation columns for merging
  merge_cols <- c("feature", "p_adjust", annotation_cols)
  merge_cols <- intersect(merge_cols, colnames(daa_results_filtered_sub_df))

  # Merge with DAA results to include p-values and other information
  result_table <- merge(
    abundance_stats,
    daa_results_filtered_sub_df[, merge_cols, drop = FALSE],
    by = "feature",
    all.x = TRUE
  )

  # Reorder columns for better readability
  base_column_order <- c(
    "feature", "group1", "group2",
    "mean_rel_abundance_group1", "sd_rel_abundance_group1",
    "mean_rel_abundance_group2", "sd_rel_abundance_group2",
    "log2_fold_change", "p_adjust"
  )

  # Add annotation columns at the end
  # Recalculate annotation columns from the actual result table
  result_annotation_cols <- setdiff(colnames(result_table), base_column_order)
  column_order <- c(base_column_order, result_annotation_cols)

  # Only select columns that actually exist in the result table
  column_order <- intersect(column_order, colnames(result_table))

  result_table <- result_table[, column_order, drop = FALSE]

  # Order by p-value (most significant first)
  result_table <- result_table[order(result_table$p_adjust), ]

  # Reset row names
  rownames(result_table) <- NULL

  return(result_table)
}

#' Volcano Plot for Pathway Differential Abundance Analysis
#'
#' Creates a volcano plot to visualize the results of differential abundance analysis,
#' showing both statistical significance (-log10 p-value) and effect size (log2 fold change).
#'
#' @param daa_results A data frame containing differential abundance analysis results,
#'   typically from \code{\link{pathway_daa}}. Must contain columns for fold change,
#'   p-values, and optionally pathway names.
#' @param fc_col Character string specifying the column name for log2 fold change values.
#'   Default is "log2_fold_change". Legacy name "log2FoldChange" is also accepted.
#' @param p_col Character string specifying the column name for adjusted p-values.
#'   Default is "p_adjust".
#' @param label_col Character string specifying the column name for pathway labels.
#'   Default is "pathway_name". If NULL, no labels will be shown.
#' @param fc_threshold Numeric. Absolute fold change threshold for significance.
#'   Default is 1 (2-fold change). Pathways with |log2FC| > fc_threshold are considered
#'   biologically significant.
#' @param p_threshold Numeric. P-value threshold for statistical significance.
#'   Default is 0.05.
#' @param label_top_n Integer. Number of top significant pathways to label.
#'   Default is 10. Set to 0 to disable labels.
#' @param point_size Numeric. Size of points in the plot. Default is 2.
#' @param point_alpha Numeric. Transparency of points (0-1). Default is 0.6.
#' @param colors Named character vector with colors for "Down", "Not Significant", and "Up".
#'   Default uses blue for down-regulated, grey for non-significant, and red for up-regulated.
#' @param show_threshold_lines Logical. Whether to show dashed lines for fold change
#'   and p-value thresholds. Default is TRUE.
#' @param title Character string for plot title. Default is
#'   "Volcano Plot: Pathway Differential Abundance".
#' @param x_lab Character string for x-axis label. Default is "log2 Fold Change".
#' @param y_lab Character string for y-axis label. Default is "-log10(Adjusted P-value)".
#'
#' @return A ggplot2 object that can be further customized or saved.
#'
#' @details
#' The volcano plot is a scatter plot that shows statistical significance (y-axis)

#' versus fold change (x-axis). Points are colored by significance:
#' \itemize{
#'   \item \strong{Up (red)}: Pathways with p < p_threshold AND log2FC > fc_threshold
#'   \item \strong{Down (blue)}: Pathways with p < p_threshold AND log2FC < -fc_threshold
#'   \item \strong{Not Significant (grey)}: All other pathways
#' }
#'
#' The function automatically labels the top N most significant pathways using
#' \code{ggrepel::geom_text_repel()} if the ggrepel package is installed.
#'
#' @examples
#' \dontrun{
#' library(ggpicrust2)
#' library(tibble)
#'
#' # Load example data
#' data("ko_abundance")
#' data("metadata")
#'
#' # Convert KO to KEGG abundance
#' kegg_abundance <- ko2kegg_abundance(data = ko_abundance)
#'
#' # Run differential abundance analysis
#' daa_results <- pathway_daa(
#'   abundance = kegg_abundance,
#'   metadata = metadata,
#'   group = "Environment",
#'   daa_method = "LinDA"
#' )
#'
#' # Annotate results
#' daa_annotated <- pathway_annotation(
#'   pathway = "KO",
#'   ko_to_kegg = TRUE,
#'   daa_results_df = daa_results
#' )
#'
#' # Create volcano plot
#' volcano_plot <- pathway_volcano(daa_annotated)
#' print(volcano_plot)
#'
#' # Customize the plot
#' volcano_plot <- pathway_volcano(
#'   daa_annotated,
#'   fc_threshold = 0.5,
#'   p_threshold = 0.01,
#'   label_top_n = 15,
#'   colors = c("Down" = "darkblue", "Not Significant" = "lightgrey", "Up" = "darkred")
#' )
#' }
#'
#' @seealso \code{\link{pathway_daa}}, \code{\link{pathway_annotation}},
#'   \code{\link{pathway_errorbar}}
#'
#' @importFrom ggplot2 ggplot aes geom_point geom_hline geom_vline geom_text
#'   scale_color_manual labs theme_minimal theme element_text
#' @importFrom dplyr filter mutate case_when slice_max
#' @importFrom stats setNames
#'
#' @export
pathway_volcano <- function(daa_results,
                            fc_col = "log2_fold_change",
                            p_col = "p_adjust",
                            label_col = "pathway_name",
                            fc_threshold = 1,
                            p_threshold = 0.05,
                            label_top_n = 10,
                            point_size = 2,
                            point_alpha = 0.6,
                            colors = c("Down" = "#3182bd",
                                       "Not Significant" = "grey60",
                                       "Up" = "#de2d26"),
                            show_threshold_lines = TRUE,
                            title = "Volcano Plot: Pathway Differential Abundance",
                            x_lab = "log2 Fold Change",
                            y_lab = "-log10(Adjusted P-value)") {

 # Input validation
 if (!is.data.frame(daa_results)) {
   stop("'daa_results' must be a data frame.")
 }

 # Backward compatibility: accept legacy column name
 if (fc_col == "log2_fold_change" && !fc_col %in% colnames(daa_results) &&
     "log2FoldChange" %in% colnames(daa_results)) {
   fc_col <- "log2FoldChange"
 }

 require_column(daa_results, fc_col, "daa_results")
 require_column(daa_results, p_col, "daa_results")

 if (!is.null(label_col) && !label_col %in% colnames(daa_results)) {
   warning(paste0("Column '", label_col, "' not found. Labels will not be shown."))
   label_col <- NULL
 }

 # Prepare data
 df <- daa_results

 # Filter out rows with missing values in key columns
 df <- df[!is.na(df[[fc_col]]) & !is.na(df[[p_col]]), ]

 if (nrow(df) == 0) {
   stop("No valid data rows after removing NA values.")
 }

 # Add significance categories and -log10(p)
 df$neg_log10_p <- -log10(df[[p_col]])
 # Handle infinite values (when p-value is 0)
 if (any(is.infinite(df$neg_log10_p))) {
   max_finite <- max(df$neg_log10_p[is.finite(df$neg_log10_p)], na.rm = TRUE)
   df$neg_log10_p[is.infinite(df$neg_log10_p)] <- max_finite * 1.1
 }
 df$significance <- ifelse(
   df[[p_col]] < p_threshold & df[[fc_col]] > fc_threshold, "Up",
   ifelse(df[[p_col]] < p_threshold & df[[fc_col]] < -fc_threshold, "Down",
          "Not Significant")
 )

 # Create base plot
 p <- ggplot2::ggplot(df, ggplot2::aes(x = .data[[fc_col]], y = .data$neg_log10_p,
                                        color = .data$significance)) +
   ggplot2::geom_point(alpha = point_alpha, size = point_size) +
   ggplot2::scale_color_manual(values = colors) +
   ggplot2::labs(
     x = x_lab,
     y = y_lab,
     color = "Significance",
     title = title
   ) +
   ggplot2::theme_minimal() +
   ggplot2::theme(
     legend.position = "top",
     plot.title = ggplot2::element_text(hjust = 0.5, face = "bold"),
     axis.text = ggplot2::element_text(color = "black"),
     panel.grid.minor = ggplot2::element_blank()
   )

 # Add threshold lines if requested
 if (show_threshold_lines) {
   p <- p +
     ggplot2::geom_hline(yintercept = -log10(p_threshold),
                         linetype = "dashed", alpha = 0.5, color = "grey40") +
     ggplot2::geom_vline(xintercept = c(-fc_threshold, fc_threshold),
                         linetype = "dashed", alpha = 0.5, color = "grey40")
 }

 # Add labels for top significant pathways
 if (!is.null(label_col) && label_top_n > 0) {
   # Get top pathways by significance (excluding non-significant and NA labels)
   top_pathways <- df[df$significance != "Not Significant", ]
   # Filter out rows with NA or empty labels
   top_pathways <- top_pathways[!is.na(top_pathways[[label_col]]) &
                                 top_pathways[[label_col]] != "", ]
   if (nrow(top_pathways) > 0) {
     top_pathways <- top_pathways[order(-top_pathways$neg_log10_p), ]
     top_pathways <- head(top_pathways, label_top_n)

     # Use ggrepel if available for better label placement
     if (requireNamespace("ggrepel", quietly = TRUE)) {
       p <- p + ggrepel::geom_text_repel(
         data = top_pathways,
         ggplot2::aes(label = .data[[label_col]]),
         size = 2.5,
         max.overlaps = 20,
         segment.alpha = 0.5,
         segment.size = 0.3,
         show.legend = FALSE
       )
     } else {
       # Fallback without ggrepel
       p <- p + ggplot2::geom_text(
         data = top_pathways,
         ggplot2::aes(label = .data[[label_col]]),
         size = 2.5,
         vjust = -0.8,
         hjust = 0.5,
         check_overlap = TRUE,
         show.legend = FALSE
       )
     }
   }
 }

 # Add count summary in subtitle
 n_up <- sum(df$significance == "Up")
 n_down <- sum(df$significance == "Down")
 n_ns <- sum(df$significance == "Not Significant")

 p <- p + ggplot2::labs(
   subtitle = paste0("Up: ", n_up, " | Down: ", n_down, " | Not Sig: ", n_ns)
 )

 return(p)
}

#' MetaCyc Abundance Dataset
#'
#' This is a demonstration dataset from the ggpicrust2 package, representing the output of PICRUSt2.
#' Each row represents a MetaCyc pathway, and each column corresponds to a sample.
#'
#' @format A data frame where rownames are MetaCyc pathways and column names include "pathway" and individual sample names, such as:
#' "pathway", "SRR11393730", "SRR11393731", "SRR11393732", "SRR11393733", "SRR11393734", "SRR11393735", "SRR11393736",
#' "SRR11393737", "SRR11393738", "SRR11393739", "SRR11393740", "SRR11393741", "SRR11393742", "SRR11393743",
#' "SRR11393744", "SRR11393745", "SRR11393746", "SRR11393747", "SRR11393748", "SRR11393749", "SRR11393750",
#' "SRR11393751", "SRR11393752", "SRR11393753", "SRR11393754", "SRR11393755", "SRR11393756", "SRR11393757",
#' "SRR11393758", "SRR11393759", "SRR11393760", "SRR11393761", "SRR11393762", "SRR11393763", "SRR11393764",
#' "SRR11393765", "SRR11393766", "SRR11393767", "SRR11393768", "SRR11393769", "SRR11393770", "SRR11393771",
#' "SRR11393772", "SRR11393773", "SRR11393774", "SRR11393775", "SRR11393776", "SRR11393777", "SRR11393778", "SRR11393779"
#'
#' @source From ggpicrust2 package demonstration.
#' @references Douglas GM, Maffei VJ, Zaneveld J, Yurgel SN, Brown JR, Taylor CM, Huttenhower C, Langille MGI. PICRUSt2 for prediction of metagenome functions. Nat Biotechnol. 2020.
"metacyc_abundance"

#' Visualize GSEA results
#'
#' This function creates various visualizations for Gene Set Enrichment Analysis (GSEA) results.
#' It automatically detects whether pathway names are available (from gsea_pathway_annotation())
#' and uses them for better readability, falling back to pathway IDs if names are not available.
#'
#' @param gsea_results A data frame containing GSEA results from the pathway_gsea function
#' @param plot_type A character string specifying the visualization type: "enrichment_plot", "dotplot", "barplot", "network", or "heatmap"
#' @param n_pathways An integer specifying the number of pathways to display
#' @param sort_by A character string specifying the sorting criterion: "NES", "pvalue", or "p.adjust"
#' @param colors A vector of colors for the visualization
#' @param abundance A data frame containing the original abundance data (required for heatmap visualization)
#' @param metadata A data frame containing sample metadata (required for heatmap visualization)
#' @param group A character string specifying the column name in metadata that contains the grouping variable (required for heatmap visualization)
#' @param network_params A list of parameters for network visualization
#' @param heatmap_params A list of parameters for heatmap visualization
#' @param pathway_label_column A character string specifying which column to use for pathway labels.
#'   If NULL (default), the function will automatically use 'pathway_name' if available, otherwise 'pathway_id'.
#'   This allows for custom labeling when using annotated GSEA results.
#' @param scale Optional palette/scale for customizing colors. Accepts: (1) a character vector of colors,
#'   (2) a function that returns colors given an integer (e.g., viridisLite::viridis), or
#'   (3) a ggplot2 scale object (e.g., ggplot2::scale_fill_gradientn(...)).
#'   When NULL, defaults keep current behavior. Applies to: enrichment_plot (fill, continuous),
#'   dotplot (color, continuous), barplot (fill, discrete Positive/Negative), network (color, diverging around 0),
#'   heatmap (main heatmap col; row annotation stays default unless overridden in heatmap_params).

#'
#' @return A ggplot2 object or ComplexHeatmap object
#' @export
#'
#' @examples
#' \dontrun{
#' # Load example data
#' data(ko_abundance)
#' data(metadata)
#'
#' # Prepare abundance data
#' abundance_data <- as.data.frame(ko_abundance)
#' rownames(abundance_data) <- abundance_data[, "#NAME"]
#' abundance_data <- abundance_data[, -1]
#'
#' # Run GSEA analysis (using camera method - recommended)
#' gsea_results <- pathway_gsea(
#'   abundance = abundance_data,
#'   metadata = metadata,
#'   group = "Environment",
#'   pathway_type = "KEGG",
#'   method = "camera"
#' )
#'
#' # Create enrichment plot with pathway IDs (default)
#' visualize_gsea(gsea_results, plot_type = "enrichment_plot", n_pathways = 10)
#'
#' # Annotate results for better pathway names
#' annotated_results <- gsea_pathway_annotation(
#'   gsea_results = gsea_results,
#'   pathway_type = "KEGG"
#' )
#'
#' # Create plots with readable pathway names
#' visualize_gsea(annotated_results, plot_type = "dotplot", n_pathways = 20)
#' visualize_gsea(annotated_results, plot_type = "barplot", n_pathways = 15)
#'
#' # Create network plot with custom labels
#' visualize_gsea(annotated_results, plot_type = "network", n_pathways = 15)
#'
#' # Use custom column for labels (if available)
#' visualize_gsea(annotated_results, plot_type = "barplot",
#'                pathway_label_column = "pathway_name", n_pathways = 10)
#'
#' # Create heatmap
#' visualize_gsea(
#'   annotated_results,
#'   plot_type = "heatmap",
#'   n_pathways = 15,
#'   abundance = abundance_data,
#'   metadata = metadata,
#'   group = "Environment"
#' )
#' }
visualize_gsea <- function(gsea_results,
                          plot_type = "enrichment_plot",
                          n_pathways = 20,
                          sort_by = "p.adjust",
                          colors = NULL,
                          abundance = NULL,
                          metadata = NULL,
                          group = NULL,
                          network_params = list(),
                          heatmap_params = list(),
                          pathway_label_column = NULL,
                          scale = NULL) {

  # Input validation using unified functions
  validate_dataframe(gsea_results, param_name = "gsea_results")
  validate_choice(plot_type, c("enrichment_plot", "dotplot", "barplot", "network", "heatmap"), "plot_type")
  validate_choice(sort_by, c("NES", "pvalue", "p.adjust"), "sort_by")
  validate_dataframe(gsea_results, required_cols = sort_by, param_name = "gsea_results")

  if (!is.null(colors) && !is.character(colors)) {
    stop("colors must be NULL or a character vector")
  }
  if (!is.null(pathway_label_column) && !is.character(pathway_label_column)) {
    stop("pathway_label_column must be NULL or a character string")
  }
  if (!is.numeric(n_pathways) || length(n_pathways) != 1 || is.na(n_pathways)) {
    stop("n_pathways must be a single numeric value")
  }

  # Convert to integer if it's not already
  n_pathways <- as.integer(n_pathways)

  # Note: enrichment_plot / dotplot / barplot are built entirely with
  # ggplot2 (see the branches below); they do not call into enrichplot
  # at runtime, so we don't require that Bioconductor package here.
  # Network and heatmap branches still depend on their own stacks and
  # are checked below.
  if (plot_type == "network") {
    require_package("igraph", "network plots")
    require_package("ggraph", "network plots")
    require_package("tidygraph", "network plots")
    validate_dataframe(
      gsea_results,
      required_cols = c("pathway_id", "NES", "pvalue", "p.adjust", "size", "leading_edge"),
      param_name = "gsea_results"
    )
  }

  if (plot_type == "heatmap") {
    require_package("ComplexHeatmap", "heatmap plots")
    require_package("circlize", "heatmap plots")
    validate_dataframe(
      gsea_results,
      required_cols = c("pathway_id", "NES", "leading_edge"),
      param_name = "gsea_results"
    )

    # Check if required parameters are provided
    if (is.null(abundance) || is.null(metadata) || is.null(group)) {
      stop("For heatmap visualization, 'abundance', 'metadata', and 'group' parameters are required")
    }
  }

  # Set default colors if not provided
  if (is.null(colors)) {
    colors <- c("#E41A1C", "#377EB8", "#4DAF4A", "#984EA3", "#FF7F00", "#FFFF33", "#A65628", "#F781BF", "#999999")
  }

  # Determine which column to use for pathway labels
  if (!is.null(pathway_label_column)) {
    # User specified a custom column
    require_column(gsea_results, pathway_label_column, "gsea_results")
    pathway_label_col <- pathway_label_column
  } else {
    # Auto-detect: prefer pathway_name if available, otherwise use pathway_id
    if ("pathway_name" %in% colnames(gsea_results)) {
      pathway_label_col <- "pathway_name"
    } else if ("pathway_id" %in% colnames(gsea_results)) {
      pathway_label_col <- "pathway_id"
    } else {
      # Check if we have any results first
      if (nrow(gsea_results) == 0) {
        return(create_empty_plot(plot_type))
      }
      stop("GSEA results must contain either 'pathway_name' or 'pathway_id' column")
    }
  }

  # Create a standardized pathway_label column for consistent use throughout the function
  gsea_results$pathway_label <- gsea_results[[pathway_label_col]]

  # Sort results based on the specified criterion
  if (sort_by == "NES") {
    gsea_results <- gsea_results[order(abs(gsea_results$NES), decreasing = TRUE), ]
  } else if (sort_by == "pvalue") {
    gsea_results <- gsea_results[order(gsea_results$pvalue), ]
  } else if (sort_by == "p.adjust") {
    gsea_results <- gsea_results[order(gsea_results$p.adjust), ]
  }

  # Handle boundary conditions for n_pathways
  if (n_pathways <= 0) {
    return(create_empty_plot(plot_type))
  }

  # Check if we have any results after filtering
  if (nrow(gsea_results) == 0) {
    return(create_empty_plot(plot_type))
  }

  # Limit to top n_pathways
  if (nrow(gsea_results) > n_pathways) {
    gsea_results <- head(gsea_results, n_pathways)
  }

  # Create visualization based on plot_type
  if (plot_type == "enrichment_plot") {
    # Create enrichment plot
    # For this, we need to convert our results to a format compatible with enrichplot

    # Check if we have the necessary data
    if (!all(c("pathway_id", "NES", "pvalue", "p.adjust") %in% colnames(gsea_results))) {
      stop("GSEA results missing required columns for enrichment plot")
    }

    # Create a simple enrichment plot using ggplot2
    # In a real implementation, we would use enrichplot::gseaplot2
    # But for simplicity, we'll create a basic version

    # Sort by NES
    gsea_results <- gsea_results[order(gsea_results$NES), ]

    # Create a basic barplot of NES values
    p <- ggplot2::ggplot(gsea_results, ggplot2::aes(x = reorder(.data$pathway_label, .data$NES), y = .data$NES, fill = .data$p.adjust)) +
      ggplot2::geom_bar(stat = "identity") +
      ggplot2::coord_flip() +
      # apply user-provided scale if available (fallback to default)
      {
        sc <- .build_continuous_scale(aes = "fill", scale = scale, diverging = FALSE, name = "Adjusted p-value")
        if (is.null(sc)) ggplot2::scale_fill_gradient(low = "red", high = "blue") else sc
      } +
      ggplot2::labs(
        title = "GSEA Enrichment Results",
        x = "Pathway",
        y = "Normalized Enrichment Score (NES)",
        fill = "Adjusted p-value"
      ) +
      ggplot2::theme_minimal() +
      ggplot2::theme(
        axis.text.y = ggplot2::element_text(size = 8),
        plot.title = ggplot2::element_text(hjust = 0.5)
      )

  } else if (plot_type == "dotplot") {
    # Create dotplot
    # Sort by NES
    gsea_results$pathway_label <- factor(gsea_results$pathway_label,
                                      levels = gsea_results$pathway_label[order(gsea_results$NES)])

    p <- ggplot2::ggplot(gsea_results,
                       ggplot2::aes(x = .data$NES, y = .data$pathway_label, color = .data$p.adjust, size = .data$size)) +
      ggplot2::geom_point() +
      # apply user-provided scale if available (fallback to default)
      {
        sc <- .build_continuous_scale(aes = "color", scale = scale, diverging = FALSE, name = "Adjusted p-value")
        if (is.null(sc)) ggplot2::scale_color_gradient(low = "red", high = "blue") else sc
      } +
      ggplot2::labs(
        title = "GSEA Results",
        x = "Normalized Enrichment Score (NES)",
        y = "Pathway",
        color = "Adjusted p-value",
        size = "Gene Set Size"
      ) +
      ggplot2::theme_minimal() +
      ggplot2::theme(
        axis.text.y = ggplot2::element_text(size = 8),
        plot.title = ggplot2::element_text(hjust = 0.5)
      )

  } else if (plot_type == "barplot") {
    # Create barplot
    # Sort by NES
    gsea_results$pathway_label <- factor(gsea_results$pathway_label,
                                      levels = gsea_results$pathway_label[order(gsea_results$NES)])

    # Add color based on NES direction
    gsea_results$direction <- ifelse(gsea_results$NES > 0, "Positive", "Negative")

    p <- ggplot2::ggplot(gsea_results,
                       ggplot2::aes(x = .data$pathway_label, y = .data$NES, fill = .data$direction)) +
      ggplot2::geom_bar(stat = "identity") +
      {
        sc <- .build_discrete_fill_for_direction(scale = scale)
        if (is.null(sc)) ggplot2::scale_fill_manual(values = c("Positive" = "#E41A1C", "Negative" = "#377EB8")) else sc
      } +
      ggplot2::coord_flip() +
      ggplot2::labs(
        title = "GSEA Results",
        x = "Pathway",
        y = "Normalized Enrichment Score (NES)",
        fill = "Direction"
      ) +
      ggplot2::theme_minimal() +
      ggplot2::theme(
        axis.text.y = ggplot2::element_text(size = 8),
        plot.title = ggplot2::element_text(hjust = 0.5)
      )

	  } else if (plot_type == "network") {
    # Set default network parameters
    default_params <- list(
      similarity_measure = "jaccard",
      similarity_cutoff = 0.3,
      layout = "fruchterman",
      node_color_by = "NES",
      edge_width_by = "similarity"
    )

	    # Merge with user-provided parameters
	    network_params <- utils::modifyList(default_params, network_params)
	    validate_choice(network_params$similarity_measure,
	                    c("jaccard", "overlap", "correlation"),
	                    "network_params$similarity_measure")
	    validate_choice(network_params$layout,
	                    c("fruchterman", "kamada", "circle"),
	                    "network_params$layout")
	    validate_choice(network_params$node_color_by,
	                    c("NES", "pvalue", "p.adjust"),
	                    "network_params$node_color_by")
	    validate_choice(network_params$edge_width_by,
	                    c("similarity", "constant"),
	                    "network_params$edge_width_by")
	    if (!is.numeric(network_params$similarity_cutoff) ||
	        length(network_params$similarity_cutoff) != 1 ||
	        is.na(network_params$similarity_cutoff) ||
	        network_params$similarity_cutoff < 0 ||
	        network_params$similarity_cutoff > 1) {
	      stop("network_params$similarity_cutoff must be a single numeric value between 0 and 1",
	           call. = FALSE)
	    }

    # Create network plot
    p <- create_network_plot(
      gsea_results = gsea_results,
      n_pathways = n_pathways,
      similarity_measure = network_params$similarity_measure,
      similarity_cutoff = network_params$similarity_cutoff,
      layout = network_params$layout,
      node_color_by = network_params$node_color_by,
      edge_width_by = network_params$edge_width_by,
      scale = scale
    )

  } else if (plot_type == "heatmap") {
    # Set default heatmap parameters
    default_params <- list(
      cluster_rows = TRUE,
      cluster_columns = TRUE,
      show_rownames = TRUE,
      annotation_colors = list(Group = stats::setNames(colors[seq_along(unique(metadata[[group]]))], unique(metadata[[group]])))
    )

	    # Merge with user-provided parameters
	    heatmap_params <- utils::modifyList(default_params, heatmap_params)
	    if (is.null(heatmap_params$annotation_colors)) {
	      heatmap_params$annotation_colors <- default_params$annotation_colors
	    }

    # Create heatmap
    p <- create_heatmap_plot(
      gsea_results = gsea_results,
      abundance = abundance,
      metadata = metadata,
      group = group,
      n_pathways = n_pathways,
      cluster_rows = heatmap_params$cluster_rows,
      cluster_columns = heatmap_params$cluster_columns,
      show_rownames = heatmap_params$show_rownames,
      annotation_colors = heatmap_params$annotation_colors,
      col_fun = {
        # Prefer explicit col_fun if provided in heatmap_params; else build from `scale`
        if (!is.null(heatmap_params$col_fun)) heatmap_params$col_fun else .build_heatmap_col_fun(scale)
      }
    )
  }

  return(p)
}

#' Create empty plot for edge cases
#'
#' @param plot_type A character string specifying the visualization type
#'
#' @return A ggplot2 object
#' @keywords internal
create_empty_plot <- function(plot_type) {
  # Create consistent empty plots for each visualization type
  base_plot <- ggplot2::ggplot() +
    ggplot2::theme_void() +
    ggplot2::labs(title = paste("No pathways to display for", plot_type))

  if (plot_type %in% c("network", "heatmap")) {
    # Special handling for complex plot types
    base_plot + ggplot2::annotate("text", x = 0, y = 0,
                                  label = "No significant pathways found",
                                  size = 4, color = "gray50")
  } else {
    base_plot + ggplot2::annotate("text", x = 0, y = 0,
                                  label = "No pathways to display",
                                  size = 4, color = "gray50")
  }
}

#' Internal: detect if an object is a ggplot2 Scale
#' @keywords internal
.is_ggplot_scale <- function(x) {
  inherits(x, "Scale")
}

#' Internal: coerce user 'scale' input to a vector of colors (or NULL)
#' Accepts character vector or function(n)->colors. Returns character vector or NULL.
#' @keywords internal
.as_color_vector <- function(scale) {
  if (is.null(scale)) return(NULL)
  if (is.character(scale)) return(scale)
  if (is.function(scale)) {
    # Try to get 100 colors as a reasonable default resolution
    cols <- tryCatch(scale(100), error = function(e) NULL)
    if (is.character(cols) && length(cols) > 1) return(cols)
    return(NULL)
  }
  NULL
}

#' Internal: build a continuous ggplot2 scale layer from colors or ggplot2 scale
#' @keywords internal
.build_continuous_scale <- function(aes = c("fill", "color"), scale = NULL,
                                   diverging = FALSE, midpoint = NULL, name = NULL) {
  aes <- match.arg(aes)
  # If a ggplot2 scale object is provided, return as is
  if (.is_ggplot_scale(scale)) return(scale)
  cols <- .as_color_vector(scale)
  if (is.null(cols)) return(NULL)
  if (diverging && length(cols) >= 3) {
    low <- cols[1]
    mid <- cols[ceiling(length(cols) / 2)]
    high <- cols[length(cols)]
    if (aes == "fill") {
      return(ggplot2::scale_fill_gradient2(low = low, mid = mid, high = high,
                                           midpoint = if (is.null(midpoint)) 0 else midpoint,
                                           name = name))
    } else {
      return(ggplot2::scale_color_gradient2(low = low, mid = mid, high = high,
                                            midpoint = if (is.null(midpoint)) 0 else midpoint,
                                            name = name))
    }
  } else {
    # General gradientn for sequential palettes
    if (aes == "fill") {
      return(ggplot2::scale_fill_gradientn(colors = cols, name = name))
    } else {
      return(ggplot2::scale_color_gradientn(colors = cols, name = name))
    }
  }
}

#' Internal: build a discrete fill scale for barplot direction
#' @keywords internal
.build_discrete_fill_for_direction <- function(scale = NULL) {
  # direction levels are c("Positive", "Negative") in code, but mapping uses both
  cols <- .as_color_vector(scale)
  if (is.null(cols) || length(cols) < 2) return(NULL)
  values <- c("Positive" = cols[length(cols)], "Negative" = cols[1])
  ggplot2::scale_fill_manual(values = values)
}

#' Internal: build a circlize colorRamp2 function for ComplexHeatmap from user scale
#' @keywords internal
.build_heatmap_col_fun <- function(scale = NULL) {
  # Get color vector from scale parameter (handles NULL, vector, function, ggplot scale)
  cols <- .as_color_vector(scale)

  # If no colors provided or couldn't convert, return NULL (use defaults)
  if (is.null(cols)) return(NULL)

  # Create a diverging color function for heatmap
  # Typical heatmap shows z-scores, so we use -2, 0, 2 as breakpoints
  if (length(cols) >= 3) {
    # Use low, mid, high colors from the palette
    low_col <- cols[1]
    mid_col <- cols[ceiling(length(cols) / 2)]
    high_col <- cols[length(cols)]
    return(circlize::colorRamp2(c(-2, 0, 2), c(low_col, mid_col, high_col)))
  } else if (length(cols) == 2) {
    # If only 2 colors, create a gradient without midpoint
    return(circlize::colorRamp2(c(-2, 2), c(cols[1], cols[2])))
  } else {
    # Single color or invalid, return NULL
    return(NULL)
  }
}

#' Create network visualization of GSEA results
#'
#' @param gsea_results A data frame containing GSEA results from the pathway_gsea function
#' @param similarity_measure A character string specifying the similarity measure: "jaccard", "overlap", or "correlation"
#' @param similarity_cutoff A numeric value specifying the similarity threshold for filtering connections
#' @param n_pathways An integer specifying the number of pathways to display
#' @param layout A character string specifying the network layout algorithm: "fruchterman", "kamada", or "circle"
#' @param node_color_by A character string specifying the node color mapping: "NES", "pvalue", or "p.adjust"
#' @param edge_width_by A character string specifying the edge width mapping: "similarity" or "constant"
#' @param scale Optional palette/scale for customizing node color mapping (same conventions as visualize_gsea)
#'
#' @return A ggplot2 object
#' @keywords internal
create_network_plot <- function(gsea_results,
                               similarity_measure = "jaccard",
                               similarity_cutoff = 0.3,
                               n_pathways = 20,
                               layout = "fruchterman",
                               node_color_by = "NES",
                               edge_width_by = "similarity",
                               scale = NULL) {
  # Note: Input validation (packages, n_pathways, nrow) is done in visualize_gsea()

  # Extract leading edge genes
  leading_edges <- strsplit(gsea_results$leading_edge, ";")
  names(leading_edges) <- gsea_results$pathway_id

  # Calculate pathway similarity
  n <- length(leading_edges)
  pathway_ids <- names(leading_edges)
  similarity_matrix <- matrix(0, nrow = n, ncol = n)
  rownames(similarity_matrix) <- pathway_ids
  colnames(similarity_matrix) <- pathway_ids

  for (i in seq_len(n)) {
    for (j in seq_len(n)) {
      if (i != j) {
        set1 <- leading_edges[[i]]
        set2 <- leading_edges[[j]]

        # Handle empty sets
        if (length(set1) == 0 || length(set2) == 0) {
          similarity_matrix[i, j] <- 0
          next
        }

        if (similarity_measure == "jaccard") {
          # Jaccard similarity: |A∩B|/|A∪B|
          similarity_matrix[i, j] <- length(intersect(set1, set2)) / length(union(set1, set2))
        } else if (similarity_measure == "overlap") {
          # Overlap coefficient: |A∩B|/min(|A|,|B|)
          similarity_matrix[i, j] <- length(intersect(set1, set2)) / min(length(set1), length(set2))
        } else if (similarity_measure == "correlation") {
          # Simplified correlation measure
          similarity_matrix[i, j] <- length(intersect(set1, set2)) / sqrt(length(set1) * length(set2))
        }
      }
    }
  }

  # Apply similarity cutoff
  similarity_matrix[similarity_matrix < similarity_cutoff] <- 0

  # Check if there are any connections after applying cutoff
  if (sum(similarity_matrix) == 0) {
    return(create_empty_plot("network"))
  }

  # Create graph object
  graph <- igraph::graph_from_adjacency_matrix(
    similarity_matrix,
    mode = "undirected",
    weighted = TRUE,
    diag = FALSE
  )

  # Check if graph is empty
  if (igraph::vcount(graph) == 0) {
    return(create_empty_plot("network"))
  }

  # Add node attributes
  vertex_attr <- data.frame(
    name = pathway_ids,
    NES = gsea_results$NES[match(pathway_ids, gsea_results$pathway_id)],
    pvalue = gsea_results$pvalue[match(pathway_ids, gsea_results$pathway_id)],
    p.adjust = gsea_results$p.adjust[match(pathway_ids, gsea_results$pathway_id)],
    size = gsea_results$size[match(pathway_ids, gsea_results$pathway_id)],
    pathway_label = gsea_results$pathway_label[match(pathway_ids, gsea_results$pathway_id)],
    stringsAsFactors = FALSE
  )

  # Create a tidygraph object
  tbl_graph <- tidygraph::as_tbl_graph(graph) %>%
    tidygraph::activate("nodes") %>%
    dplyr::mutate(
      name = vertex_attr$name,
      NES = vertex_attr$NES,
      pvalue = vertex_attr$pvalue,
      p.adjust = vertex_attr$p.adjust,
      size = vertex_attr$size,
      pathway_label = vertex_attr$pathway_label
    )

  # Select layout algorithm
  if (layout == "fruchterman") {
    layout_name <- "fr"
  } else if (layout == "kamada") {
    layout_name <- "kk"
  } else if (layout == "circle") {
    layout_name <- "circle"
  } else {
    layout_name <- "fr"
  }

	  edge_layer <- if (edge_width_by == "similarity") {
	    ggraph::geom_edge_link(ggplot2::aes(width = .data$weight, alpha = .data$weight))
	  } else {
	    ggraph::geom_edge_link(ggplot2::aes(alpha = .data$weight), width = 0.5)
	  }
	  edge_width_scale <- if (edge_width_by == "similarity") {
	    ggraph::scale_edge_width(range = c(0.1, 2))
	  } else {
	    NULL
	  }

	  # Create ggraph visualization
	  p <- ggraph::ggraph(tbl_graph, layout = layout_name) +
	    edge_layer +
	    ggraph::geom_node_point(ggplot2::aes(color = .data[[node_color_by]], size = .data$size)) +
	    ggraph::geom_node_text(ggplot2::aes(label = .data$pathway_label), repel = TRUE, size = 3) +
	    edge_width_scale +
    ggraph::scale_edge_alpha(range = c(0.1, 0.8)) +
    # apply user-provided diverging scale for node color if available (fallback to default)
    {
      sc <- .build_continuous_scale(aes = "color", scale = scale, diverging = TRUE, midpoint = 0, name = node_color_by)
      if (is.null(sc)) ggplot2::scale_color_gradient2(low = "blue", mid = "white", high = "red", midpoint = 0, name = node_color_by) else sc
    } +
    ggplot2::scale_size(range = c(2, 8), name = "Gene Set Size") +
    ggraph::theme_graph() +
    ggplot2::labs(
      title = "GSEA Pathway Network",
      subtitle = paste("Similarity measure:", similarity_measure, "| Cutoff:", similarity_cutoff)
    )

  return(p)
}

#' Create heatmap visualization of GSEA results
#'
#' @param gsea_results A data frame containing GSEA results from the pathway_gsea function
#' @param abundance A data frame containing the original abundance data
#' @param metadata A data frame containing sample metadata
#' @param group A character string specifying the column name in metadata that contains the grouping variable
#' @param n_pathways An integer specifying the number of pathways to display
#' @param cluster_rows A logical value indicating whether to cluster rows
#' @param cluster_columns A logical value indicating whether to cluster columns
#' @param show_rownames A logical value indicating whether to show row names
#' @param annotation_colors A list of colors for annotations
#' @param col_fun A color function (e.g., circlize::colorRamp2) to control the main heatmap colors (optional)
#'
#' @return A ComplexHeatmap object
#' @keywords internal
create_heatmap_plot <- function(gsea_results,
                               abundance,
                               metadata,
                               group,
                               n_pathways = 20,
                               cluster_rows = TRUE,
                               cluster_columns = TRUE,
                               show_rownames = TRUE,
                               annotation_colors = NULL,
                               col_fun = NULL) {
  # Note: Input validation (packages, n_pathways, nrow) is done in visualize_gsea()

  # Extract leading edge genes
  leading_edges <- lapply(strsplit(gsea_results$leading_edge, ";"), function(x) x[x != ""])
  names(leading_edges) <- gsea_results$pathway_id

  # Check if any leading edges are empty
  if (all(lengths(leading_edges) == 0)) {
    return(ComplexHeatmap::Heatmap(
      matrix(0, nrow = 1, ncol = 1),
      name = "Empty",
      show_row_names = FALSE,
      show_column_names = FALSE,
      row_title = "No leading edge genes found",
      column_title = "No gene expression data"
    ))
  }

  # Align abundance columns with metadata rows using the package-wide
  # sample-alignment utility. Previously this branch relied solely on
  # `rownames(metadata)` to locate samples, which silently broke on the
  # common case of metadata carrying a `sample_name`/`sample_id` column
  # with default integer rownames -- the column annotation then came
  # out as all-NA, diverging from how every other function in the
  # package matches abundance to metadata.
  aligned <- align_samples(abundance, metadata, verbose = FALSE)
  abundance <- as.matrix(aligned$abundance)
  metadata <- aligned$metadata
  validate_group(metadata, group, min_groups = 1)

  # Create heatmap data matrix
  # For each pathway, calculate the average expression of leading edge genes
  heatmap_data <- matrix(0, nrow = length(leading_edges), ncol = ncol(abundance))
  rownames(heatmap_data) <- names(leading_edges)
  colnames(heatmap_data) <- colnames(abundance)

  for (i in seq_along(leading_edges)) {
    genes <- leading_edges[[i]]

    # Ensure all genes are in abundance data
    genes <- genes[genes %in% rownames(abundance)]

    if (length(genes) > 0) {
      # Calculate average abundance
      heatmap_data[i, ] <- colMeans(abundance[genes, , drop = FALSE])
    }
  }

  # Scale data. Constant rows produce NA after t(scale(t(.))); coerce
  # them to 0 so ComplexHeatmap doesn't crash on clustering.
  heatmap_data_scaled <- t(scale(t(heatmap_data)))
  heatmap_data_scaled[!is.finite(heatmap_data_scaled)] <- 0

  # Column annotation now derives from the aligned metadata, which is
  # guaranteed to match `colnames(heatmap_data)` by construction.
  column_annotation <- metadata[[group]]
  names(column_annotation) <- colnames(heatmap_data)

  # Create column annotation object
  ha <- ComplexHeatmap::HeatmapAnnotation(
    Group = column_annotation,
    col = list(Group = annotation_colors$Group),
    show_legend = TRUE
  )

  # Create row annotation (pathway enrichment scores)
  row_annotation <- data.frame(
    NES = gsea_results$NES,
    row.names = gsea_results$pathway_id
  )

  # Ensure row annotation matches heatmap rows
  row_annotation <- row_annotation[rownames(heatmap_data), , drop = FALSE]

  # Create row annotation object
  # Use symmetric breaks centered at 0 to ensure colorRamp2 gets
  # strictly increasing values even when all NES are same sign
  nes_abs_max <- max(abs(row_annotation$NES), 0.1)
  ra <- ComplexHeatmap::rowAnnotation(
    NES = row_annotation$NES,
    col = list(NES = circlize::colorRamp2(
      c(-nes_abs_max, 0, nes_abs_max),
      c("blue", "white", "red")
    )),
    show_legend = TRUE
  )

  # Create heatmap
  heatmap <- ComplexHeatmap::Heatmap(
    heatmap_data_scaled,
    name = "Z-score",
    col = {
      # Use user-provided col_fun if present; else default blue-white-red
      if (!is.null(col_fun)) col_fun else circlize::colorRamp2(c(-2, 0, 2), c("blue", "white", "red"))
    },
    cluster_rows = cluster_rows,
    cluster_columns = cluster_columns,
    show_row_names = show_rownames,
    row_names_gp = grid::gpar(fontsize = 8),
    top_annotation = ha,
    right_annotation = ra,
    row_title = "Pathways",
    column_title = "Samples",
    row_names_max_width = grid::unit(15, "cm")
  )

  return(heatmap)
}

# Test for visualize_gsea function
# create_gsea_test_results() is defined in helper-gsea.R

test_that("visualize_gsea creates different plot types correctly", {
  skip_if_not_installed("enrichplot")

  gsea_results <- create_gsea_test_results()

  p_bar <- visualize_gsea(gsea_results, plot_type = "barplot")
  expect_s3_class(p_bar, "ggplot")

  p_dot <- visualize_gsea(gsea_results, plot_type = "dotplot")
  expect_s3_class(p_dot, "ggplot")

  p_enrich <- visualize_gsea(gsea_results, plot_type = "enrichment_plot")
  expect_s3_class(p_enrich, "ggplot")
})

test_that("visualize_gsea creates network plot correctly", {
  skip_if_not_installed("igraph")

  gsea_results <- create_gsea_test_results()
  p <- visualize_gsea(gsea_results, plot_type = "network")
  expect_s3_class(p, "ggplot")
})

test_that("visualize_gsea creates heatmap plot correctly", {
  skip_if_not_installed("ComplexHeatmap")
  skip_if_not_installed("circlize")

  gsea_results <- create_gsea_test_results()

  set.seed(123)
  abundance <- as.data.frame(matrix(
    runif(100 * 10, 1, 100), nrow = 100, ncol = 10,
    dimnames = list(paste0("K", sprintf("%05d", 1:100)), paste0("Sample", 1:10))
  ))

  metadata <- data.frame(
    sample = paste0("Sample", 1:10),
    group = rep(c("Group1", "Group2"), each = 5),
    stringsAsFactors = FALSE
  )
  rownames(metadata) <- metadata$sample

  p <- visualize_gsea(
    gsea_results = gsea_results,
    plot_type = "heatmap",
    abundance = abundance,
    metadata = metadata,
    group = "group"
  )
  expect_s4_class(p, "Heatmap")
})

test_that("visualize_gsea validates inputs correctly", {
  skip_if_not_installed("enrichplot")

  gsea_results <- create_gsea_test_results()

  expect_error(visualize_gsea(gsea_results = "invalid"), "'gsea_results' must be a data frame")
  expect_error(visualize_gsea(gsea_results, plot_type = "invalid"), "plot_type must be one of")
  expect_error(visualize_gsea(gsea_results, sort_by = "invalid"), "sort_by must be one of")
})

test_that("visualize_gsea limits pathways correctly", {
  skip_if_not_installed("enrichplot")

  gsea_results <- create_gsea_test_results(n_pathways = 30)

  p_default <- visualize_gsea(gsea_results, plot_type = "barplot")
  expect_equal(nrow(p_default$data), 20)

  p_custom <- visualize_gsea(gsea_results, plot_type = "barplot", n_pathways = 10)
  expect_equal(nrow(p_custom$data), 10)
})

#' Convert KO abundance in picrust2 export files to KEGG pathway abundance
#'
#' This function takes a file containing KO (KEGG Orthology) abundance data in picrust2 export format and converts it to KEGG pathway abundance data.
#' The input file should be in .tsv, .txt, or .csv format.
#'
#' @param file A character string representing the file path of the input file containing KO abundance data in picrust2 export format. The input file should have KO identifiers in the first column and sample identifiers in the first row. The remaining cells should contain the abundance values for each KO-sample pair.
#' @param data An optional data.frame containing KO abundance data in the same format as the input file. If provided, the function will use this data instead of reading from the file. By default, this parameter is set to NULL.
#' @param method Method for calculating pathway abundance. One of:
#'   \itemize{
#'     \item \code{"abundance"}: (Default) PICRUSt2-style calculation using the mean of upper-half sorted KO abundances. This method is more robust and avoids inflating abundances for pathways with more KOs.
#'     \item \code{"sum"}: Simple summation of all KO abundances. This is the legacy method and may double-count KOs belonging to multiple pathways.
#'   }
#' @param filter_for_prokaryotes Logical. If TRUE (default), filters out KEGG pathways
#'   that are not relevant to prokaryotic (bacterial/archaeal) analysis. The function
#'   always removes non-pathway KEGG buckets before this filter is applied. The
#'   prokaryote filter removes pathways in categories such as:
#'   \itemize{
#'     \item Human diseases (cancer, neurodegenerative diseases, addiction, etc.)
#'     \item Organismal systems (immune system, nervous system, endocrine system, etc.)
#'   }
#'   Bacterial infection pathways and antimicrobial resistance pathways are retained.
#'   Set to FALSE to include all KEGG pathways (for eukaryotic analysis or custom filtering).
#' @param progress Logical. Whether to show a progress bar while aggregating
#'   pathways. Defaults to \code{interactive()} so non-interactive scripts and
#'   tests stay quiet.
#'
#' @return
#' A data frame with KEGG pathway abundance values. Rows represent KEGG pathways, identified by their KEGG pathway IDs. Columns represent samples, identified by their sample IDs from the input file.
#'
#' @details
#' The default \code{"abundance"} method follows PICRUSt2's approach for calculating pathway abundance:
#' \enumerate{
#'   \item For each pathway, collect abundances of all associated KOs present in the data
#'   \item Sort the abundances in ascending order
#'   \item Take the upper half of the sorted values
#'   \item Calculate the mean as the pathway abundance
#' }
#'
#' This approach has several advantages over simple summation:
#' \itemize{
#'   \item Does not inflate abundances for pathways containing more KOs
#'   \item More robust to missing or low-abundance KOs
#'   \item Provides a more accurate representation of pathway activity
#' }
#'
#' The \code{"sum"} method is provided for backward compatibility and simply sums all KO abundances for each pathway.
#'
#' @section Pathway Filtering:
#' Before abundance calculation, KEGG BRITE hierarchies and "Not Included in Pathway or
#' Brite" pseudo-pathways are removed because they are not KEGG pathway maps and cannot
#' be consistently annotated as pathways (for example, \code{ko99980}).
#'
#' When \code{filter_for_prokaryotes = TRUE}, the function excludes KEGG pathways that are
#' biologically irrelevant to prokaryotic organisms. KEGG reference pathways include pathways
#' from all domains of life, and many human/animal-specific pathways would appear in bacterial
#' analysis simply because some KOs are shared across organisms.
#'
#' The following KEGG Level 2 categories are excluded:
#' \itemize{
#'   \item Cancer pathways (overview and specific types)
#'   \item Neurodegenerative diseases (Alzheimer's, Parkinson's, etc.)
#'   \item Substance dependence (addiction pathways)
#'   \item Cardiovascular diseases
#'   \item Endocrine and metabolic diseases
#'   \item Immune diseases
#'   \item Organismal systems (immune, nervous, endocrine, digestive, etc.)
#' }
#'
#' The following are RETAINED even with filtering:
#' \itemize{
#'   \item Infectious disease: bacterial (Salmonella, E. coli, Tuberculosis, etc.)
#'   \item Drug resistance: antimicrobial (antibiotic resistance)
#'   \item All Metabolism pathways
#'   \item Genetic/Environmental Information Processing
#'   \item Cellular Processes
#' }
#'
#' @examples
#' \dontrun{
#' library(ggpicrust2)
#' library(readr)
#'
#' # Example 1: Default - filtered for prokaryotic analysis
#' data(ko_abundance)
#' kegg_abundance <- ko2kegg_abundance(data = ko_abundance)
#'
#' # Example 2: Include all pathways (for eukaryotic analysis)
#' kegg_abundance_all <- ko2kegg_abundance(data = ko_abundance, filter_for_prokaryotes = FALSE)
#'
#' # Example 3: Using legacy sum method with filtering
#' kegg_abundance_sum <- ko2kegg_abundance(data = ko_abundance, method = "sum")
#'
#' # Example 4: From file
#' input_file <- "path/to/your/picrust2/results/pred_metagenome_unstrat.tsv"
#' kegg_abundance <- ko2kegg_abundance(file = input_file)
#' }
#' @export
ko2kegg_abundance <- function (file = NULL, data = NULL, method = c("abundance", "sum"),
                               filter_for_prokaryotes = TRUE,
                               progress = interactive()) {
  method <- match.arg(method)
  filter_for_prokaryotes <- normalize_logical_flag(filter_for_prokaryotes, "filter_for_prokaryotes")
  progress <- normalize_logical_flag(progress, "progress")

  # Basic parameter validation
  if (is.null(file) & is.null(data)) {
    stop("Error: Please provide either a file or a data.frame.")
  }

  if (!is.null(file) && !is.null(data)) {
    warning("Both file and data provided. Using data and ignoring file.")
    file <- NULL
  }

  # Load data from file or use provided data
  if (!is.null(file)) {
    abundance <- read_abundance_file(file)
  } else {
    if (!is.data.frame(data)) {
      stop("'data' must be a data.frame")
    }
    abundance <- data
    if (ncol(abundance) < 2) {
      stop("Data must have at least 2 columns (KO IDs and samples)")
    }
  }

  # Standardize first column name (PICRUSt2 uses various formats)
  valid_ko_column_names <- c("function.", "function", "sequence", "#NAME")
  first_col_name <- colnames(abundance)[1]

  if (first_col_name %in% valid_ko_column_names) {
    colnames(abundance)[1] <- "function."
  } else {
    # Check if first column contains KO IDs
    first_col_values <- as.character(abundance[[1]])
    ko_pattern <- "(^K\\d{5}$)|(^ko:K\\d{5}$)"
    if (any(grepl(ko_pattern, first_col_values))) {
      colnames(abundance)[1] <- "function."
    } else {
      stop(sprintf("First column '%s' does not contain KO IDs (expected: K##### or ko:K#####)", first_col_name))
    }
  }

  if (nrow(abundance) == 0) {
    stop("Data contains no rows")
  }

  # Standardize KO ID format and validate
 abundance <- clean_ko_abundance(abundance, verbose = FALSE)

  # Validate numeric matrix (negative values, NA, duplicates)
  numeric_mat <- as.matrix(abundance[, -1, drop = FALSE])
  validate_numeric_matrix(numeric_mat)
  validate_feature_ids(abundance[[1]], type = "KO")
  
  # Load KEGG reference data using unified loader
  ko_to_kegg_reference <- load_reference_data("ko_to_kegg")

  # Keep only KEGG pathway maps. The bundled KO-to-KEGG reference also contains
  # BRITE hierarchies and "Not Included in Pathway or Brite" buckets such as
  # ko99980; those are KO groupings, not pathways, and downstream pathway
  # annotation should not be asked to treat them as pathway IDs.
  ko_to_kegg_reference <- filter_kegg_reference_to_pathways(ko_to_kegg_reference)

  # Filter for prokaryotic pathways if requested
  if (filter_for_prokaryotes) {
    ko_to_kegg_reference <- filter_kegg_reference_for_prokaryotes(ko_to_kegg_reference)
  }

  # Get all unique pathway IDs
  all_pathways <- unique(ko_to_kegg_reference$pathway_id)

  # Create fast pathway → KOs lookup mapping
  pathway_to_ko <- split(ko_to_kegg_reference$ko_id, ko_to_kegg_reference$pathway_id)

  # Create result data frame
  kegg_abundance <- data.frame(
    row.names = all_pathways  # Set pathways as row names
  )

  sample_names <- colnames(abundance)[-1]
  kegg_abundance[sample_names] <- 0

  # Calculate pathway abundances. Progress is interactive-only by default so
  # scripts and tests do not get flooded with carriage-return progress output.
  pb <- NULL
  if (progress) {
    pb <- utils::txtProgressBar(min = 0, max = nrow(kegg_abundance), style = 3)
    on.exit(close(pb), add = TRUE)
  }

  for (i in seq_len(nrow(kegg_abundance))) {
    if (!is.null(pb)) utils::setTxtProgressBar(pb, i)
    current_kegg <- rownames(kegg_abundance)[i]
    relevant_kos <- pathway_to_ko[[current_kegg]]
    matching_rows <- abundance[[1]] %in% relevant_kos

    if (any(matching_rows)) {
      if (method == "sum") {
        kegg_abundance[i, ] <- colSums(abundance[matching_rows, -1, drop = FALSE])
      } else {
        # PICRUSt2-style: upper-half mean
        ko_abundances <- as.matrix(abundance[matching_rows, -1, drop = FALSE])
        kegg_abundance[i, ] <- apply(ko_abundances, 2, function(col) {
          sorted_col <- sort(col)
          mean(sorted_col[ceiling(length(sorted_col) / 2):length(sorted_col)])
        })
      }
    }
  }
  if (!is.null(pb)) {
    close(pb)
    pb <- NULL
  }

  # Remove zero-abundance pathways. Reaching all-zero here means the IDs
  # passed format validation but none are present in the KEGG reference
  # (e.g. outdated or non-standard KO identifiers). Continuing would just
  # hand an empty matrix to downstream DAA and produce a misleading error.
  zero_rows <- rowSums(kegg_abundance, na.rm = TRUE) == 0
  if (all(zero_rows)) {
    stop("No KO IDs in the input matched any KEGG pathway. ",
         "This usually means the KO IDs are outdated or not in the KEGG reference. ",
         "Expected format: K##### (e.g., K00001).")
  }
  kegg_abundance <- kegg_abundance[!zero_rows, , drop = FALSE]

  kegg_abundance
}

#' Filter KO-to-KEGG reference data to true KEGG pathway maps
#'
#' @param ko_to_kegg_reference KO-to-KEGG reference data frame
#' @return Filtered data frame
#' @noRd
filter_kegg_reference_to_pathways <- function(ko_to_kegg_reference) {
  level1 <- as.character(ko_to_kegg_reference$level1)
  is_non_pathway <- grepl("^(09180|09190)\\b", level1)

  ko_to_kegg_reference[!is_non_pathway, , drop = FALSE]
}

#' Filter KO-to-KEGG reference data for prokaryotic analyses
#'
#' KEGG hierarchy labels in the bundled reference include BRITE numeric
#' prefixes (for example, "09160 Human Diseases"). Match on those stable
#' hierarchy IDs instead of exact free-text labels.
#'
#' @param ko_to_kegg_reference KO-to-KEGG reference data frame
#' @return Filtered data frame
#' @noRd
filter_kegg_reference_for_prokaryotes <- function(ko_to_kegg_reference) {
  level1 <- as.character(ko_to_kegg_reference$level1)
  level2 <- as.character(ko_to_kegg_reference$level2)

  is_organismal_system <- grepl("^09150\\b", level1)
  is_human_disease <- grepl("^09160\\b", level1)
  retained_human_disease <- grepl("^(09171|09175)\\b", level2)
  excluded_human_disease <- is_human_disease & !retained_human_disease

  keep <- !(is_organismal_system |
              excluded_human_disease)

  ko_to_kegg_reference[keep, , drop = FALSE]
}

# Test ko2kegg_abundance function

test_that("ko2kegg_abundance works with valid data frame input", {
  mock_ko_data <- data.frame(
    function. = c("K00001", "K00002", "K00003"),
    Sample1 = c(10, 20, 30),
    Sample2 = c(15, 25, 35),
    stringsAsFactors = FALSE
  )

  result <- suppressMessages(ko2kegg_abundance(data = mock_ko_data))

  expect_s3_class(result, "data.frame")
  if (nrow(result) > 0) {
    expect_true(all(result >= 0))
    expect_equal(ncol(result), 2)
  }
})

test_that("ko2kegg_abundance handles file input correctly", {
  temp_file <- tempfile(fileext = ".tsv")
  mock_ko_data <- data.frame(
    function. = c("K00001", "K00002", "K00003"),
    Sample1 = c(10, 20, 30),
    Sample2 = c(15, 25, 35),
    stringsAsFactors = FALSE
  )
  write.table(mock_ko_data, temp_file, sep = "\t", row.names = FALSE)

  result <- suppressMessages(ko2kegg_abundance(file = temp_file))
  unlink(temp_file)

  expect_s3_class(result, "data.frame")
  if (nrow(result) > 0) {
    expect_true(all(result >= 0))
  }
})

test_that("ko2kegg_abundance uses data when both file and data are supplied", {
  bad_file <- tempfile(fileext = ".tsv")
  writeLines(c("bad\tS1", "not_ko\t1"), bad_file)
  on.exit(unlink(bad_file), add = TRUE)

  ko_to_kegg_reference <- ggpicrust2:::load_reference_data("ko_to_kegg")
  real_kos <- head(unique(ko_to_kegg_reference$ko_id), 3)
  valid_data <- data.frame(
    function. = real_kos,
    Sample1 = c(10, 20, 30),
    stringsAsFactors = FALSE
  )

  expect_warning(
    result <- ko2kegg_abundance(file = bad_file, data = valid_data),
    "Using data and ignoring file"
  )
  expect_s3_class(result, "data.frame")
})

test_that("ko2kegg_abundance suppresses progress output by default in non-interactive use", {
  ko_to_kegg_reference <- ggpicrust2:::load_reference_data("ko_to_kegg")
  real_kos <- head(unique(ko_to_kegg_reference$ko_id), 3)
  valid_data <- data.frame(
    function. = real_kos,
    Sample1 = c(10, 20, 30),
    stringsAsFactors = FALSE
  )

  out <- capture.output(invisible(ko2kegg_abundance(data = valid_data)), type = "output")
  expect_equal(out, character(0))
})

test_that("ko2kegg_abundance throws appropriate errors", {
  expect_error(ko2kegg_abundance(), "Please provide either a file or a data.frame")
  expect_error(ko2kegg_abundance(file = "test.pdf"), "File does not exist")
  expect_error(ko2kegg_abundance(data = list(a = 1)), "must be a data.frame")

  # Negative values
  negative_data <- data.frame(function. = c("K00001"), Sample1 = c(-1), stringsAsFactors = FALSE)
  expect_error(ko2kegg_abundance(data = negative_data), "Negative values")

  # Non-numeric columns
  non_numeric <- data.frame(function. = c("K00001"), Sample1 = c("text"), stringsAsFactors = FALSE)
  expect_error(ko2kegg_abundance(data = non_numeric), "non-numeric")
})

test_that("ko2kegg_abundance fails fast when input is not KO data (e.g. EC IDs)", {
  # EC-number-shaped input: passes the first-column name check (column named
  # "function") but no ID matches the KO format. Should stop at
  # validate_feature_ids instead of silently producing an empty matrix.
  ec_shaped_data <- data.frame(
    `function` = c("1.1.1.1", "1.1.1.2", "2.7.1.1"),
    Sample1 = c(10, 20, 30),
    Sample2 = c(15, 25, 35),
    check.names = FALSE,
    stringsAsFactors = FALSE
  )

  expect_error(
    suppressMessages(ko2kegg_abundance(data = ec_shaped_data)),
    "None of the feature IDs match expected KO format"
  )
})

test_that("ko2kegg_abundance fails fast when KO IDs are absent from the KEGG reference", {
  # KO IDs that pass format validation (K#####) but are not in the reference.
  # Use IDs outside the current KEGG space to trigger the all-zero branch.
  unknown_ko_data <- data.frame(
    function. = c("K99991", "K99992", "K99993"),
    Sample1 = c(10, 20, 30),
    Sample2 = c(15, 25, 35),
    stringsAsFactors = FALSE
  )

  expect_error(
    suppressMessages(ko2kegg_abundance(data = unknown_ko_data)),
    "No KO IDs in the input matched any KEGG pathway"
  )
})

test_that("ko2kegg_abundance preserves sample names and removes zero pathways", {
  ko_to_kegg_reference <- ggpicrust2:::load_reference_data("ko_to_kegg")
  real_kos <- head(unique(ko_to_kegg_reference$ko_id), 5)

  mock_ko_data <- data.frame(
    function. = real_kos,
    SampleA = c(100, 200, 150, 180, 120),
    SampleB = c(50, 100, 75, 90, 60),
    stringsAsFactors = FALSE
  )

  result <- suppressMessages(ko2kegg_abundance(data = mock_ko_data))

  if (nrow(result) > 0) {
    expect_equal(colnames(result), c("SampleA", "SampleB"))
    expect_true(all(rowSums(result) > 0))
  }
})

test_that("KEGG pathway filter removes non-pathway buckets", {
  ko_to_kegg_reference <- ggpicrust2:::load_reference_data("ko_to_kegg")
  filtered_reference <- ggpicrust2:::filter_kegg_reference_to_pathways(ko_to_kegg_reference)

  expect_false("ko01001" %in% filtered_reference$pathway_id)
  expect_false("ko99980" %in% filtered_reference$pathway_id)
  expect_false(any(grepl("^(09180|09190)\\b", filtered_reference$level1)))
})

test_that("prokaryote filter removes eukaryotic pathways and keeps microbial disease pathways", {
  ko_to_kegg_reference <- ggpicrust2:::load_reference_data("ko_to_kegg")
  pathway_reference <- ggpicrust2:::filter_kegg_reference_to_pathways(ko_to_kegg_reference)
  filtered_reference <- ggpicrust2:::filter_kegg_reference_for_prokaryotes(pathway_reference)

  expect_false("ko05200" %in% filtered_reference$pathway_id)
  expect_false("ko04910" %in% filtered_reference$pathway_id)
  expect_true("ko05130" %in% filtered_reference$pathway_id)
  expect_true("ko01501" %in% filtered_reference$pathway_id)

  expect_false(any(grepl("^09150\\b", filtered_reference$level1)))
  expect_false(any(
    grepl("^09160\\b", filtered_reference$level1) &
      !grepl("^(09171|09175)\\b", filtered_reference$level2)
  ))
  expect_true(any(grepl("^09171\\b", filtered_reference$level2)))
  expect_true(any(grepl("^09175\\b", filtered_reference$level2)))
})

test_that("ko2kegg_abundance applies pathway and prokaryote filters to output pathways", {
  ko_to_kegg_reference <- ggpicrust2:::load_reference_data("ko_to_kegg")
  real_kos <- head(unique(ko_to_kegg_reference$ko_id), 10)

  mock_ko_data <- data.frame(
    function. = real_kos,
    Sample1 = seq_along(real_kos),
    stringsAsFactors = FALSE
  )

  result <- suppressMessages(ko2kegg_abundance(data = mock_ko_data))

  expect_false("ko01001" %in% rownames(result))
  expect_false("ko99980" %in% rownames(result))
  expect_false("ko05200" %in% rownames(result))
})

test_that("filter_for_prokaryotes = FALSE keeps true eukaryotic pathways but not non-pathway buckets", {
  ko_to_kegg_reference <- ggpicrust2:::load_reference_data("ko_to_kegg")
  kos <- unique(c(
    ko_to_kegg_reference$ko_id[ko_to_kegg_reference$pathway_id == "ko05200"],
    ko_to_kegg_reference$ko_id[ko_to_kegg_reference$pathway_id == "ko99980"]
  ))

  mock_ko_data <- data.frame(
    function. = head(kos, 20),
    Sample1 = seq_len(min(20, length(kos))),
    stringsAsFactors = FALSE
  )

  result <- suppressMessages(
    ko2kegg_abundance(data = mock_ko_data, filter_for_prokaryotes = FALSE)
  )

  expect_true("ko05200" %in% rownames(result))
  expect_false("ko99980" %in% rownames(result))
})

#' Validate group sizes for statistical analysis
#'
#' This function checks group sizes and balance to ensure statistical reliability.
#' Follows Linus principle: fail fast with clear reasons, don't hide problems.
#'
#' @param group_vector A factor vector with group assignments
#' @param group_name A character string with the group variable name for error messages
#' @keywords internal
validate_group_sizes <- function(group_vector, group_name) {
  group_counts <- table(group_vector)
  group_names <- names(group_counts)
  n_groups <- length(group_counts)
  
  # Absolute minimum: need at least 2 samples per group for any comparison
  min_samples_per_group <- 2
  insufficient_groups <- group_counts < min_samples_per_group
  
  if (any(insufficient_groups)) {
    insufficient_names <- paste(group_names[insufficient_groups], ":", group_counts[insufficient_groups], collapse = "; ")
    stop(sprintf("Groups with <2 samples detected in '%s' (%s). Statistical comparison impossible. Need at least 2 samples per group.", 
                 group_name, insufficient_names))
  }
  
  # Statistical power warning: less than 3 samples per group
  small_groups <- group_counts < 3
  if (any(small_groups)) {
    small_names <- paste(group_names[small_groups], ":", group_counts[small_groups], collapse = "; ")
    warning(sprintf("Small group sizes in '%s' (%s). Statistical power severely limited. Recommend n>=3 per group for reliable results.",
                    group_name, small_names), call. = FALSE)
  }
  
  # Group imbalance warning (only for two-group comparisons)
  if (n_groups == 2) {
    imbalance_ratio <- max(group_counts) / min(group_counts)
    if (imbalance_ratio > 3) {
      warning(sprintf("Severe group imbalance in '%s' (ratio %.1f:1). Results may be biased. Consider balancing groups or using robust methods.",
                      group_name, imbalance_ratio), call. = FALSE)
    }
  }
  
  # Multi-group warning
  if (n_groups > 2) {
    warning(sprintf("Multiple groups detected in '%s' (%d groups). GSEA will use pairwise comparisons. Consider running separate two-group analyses.",
                    group_name, n_groups), call. = FALSE)
  }
}

#' Gene Set Enrichment Analysis for PICRUSt2 output
#'
#' This function performs Gene Set Enrichment Analysis (GSEA) on PICRUSt2 predicted functional data
#' to identify enriched pathways between different conditions.
#'
#' @param abundance A data frame containing gene/enzyme abundance data, with features as rows and samples as columns.
#'   For KEGG analysis: features should be KO IDs (e.g., K00001).
#'   For MetaCyc analysis: features should be EC numbers (e.g., EC:1.1.1.1 or 1.1.1.1), NOT pathway IDs.
#'   For GO analysis: features should be KO IDs that will be mapped to GO terms.
#'   NOTE: This function requires gene-level data, not pathway-level abundances.
#'   For pathway abundance analysis, use \code{\link{pathway_daa}} instead
#' @param metadata A data frame containing sample metadata
#' @param group A character string specifying the column name in metadata that contains the grouping variable
#' @param pathway_type A character string specifying the pathway type: "KEGG", "MetaCyc", or "GO"
#' @param method A character string specifying the GSEA method:
#'   \itemize{
#'     \item \code{"camera"}: Competitive gene set test using limma's camera function (recommended).
#'           Accounts for inter-gene correlations and provides more reliable p-values.
#'     \item \code{"fry"}: Fast approximation to rotation gene set testing using limma's fry function.
#'           Self-contained test that is computationally efficient.
#'     \item \code{"fgsea"}: Fast preranked GSEA implementation. Note: preranked methods may produce
#'           unreliable p-values due to not accounting for inter-gene correlations (Wu et al., 2012).
#'     \item \code{"GSEA"} or \code{"clusterProfiler"}: clusterProfiler's GSEA implementation.
#'   }
#' @param covariates A character vector specifying column names in metadata to use as covariates
#'   for adjustment. Only used when method is "camera" or "fry". Default is NULL (no covariates).
#'   Example: \code{covariates = c("age", "sex", "BMI")}
#' @param contrast For multi-group comparisons with "camera" or "fry" methods, specify the contrast
#'   to test. Can be a character string naming a group level, or a numeric vector of contrast weights.
#'   Default is NULL (automatic: compares second group to first).
#' @param inter.gene.cor Numeric value specifying the inter-gene correlation for camera method.
#'   Default is 0.01. Use NA to estimate correlation from data for each gene set.
#' @param rank_method A character string specifying the ranking statistic for preranked methods
#'   (fgsea, GSEA, clusterProfiler): "signal2noise", "t_test", "log2_ratio", or "diff_abundance"
#' @param nperm An integer specifying the number of permutations (for clusterProfiler method only).
#'   The fgsea method uses adaptive multilevel splitting and does not require a fixed permutation count.
#' @param min_size An integer specifying the minimum gene set size
#' @param max_size An integer specifying the maximum gene set size
#' @param p_adjust_method A character string specifying the p-value adjustment method
#' @param p.adjust Deprecated. Use \code{p_adjust_method} instead.
#' @param seed An integer specifying the random seed for reproducibility
#' @param go_category A character string specifying GO category to use.
#'   "all" (default) uses all categories present in the reference data.
#'   Valid categories are determined by the reference data (currently MF and CC).
#'   See \code{table(ko_to_go_reference$category)} for available categories.
#' @param organism A character string specifying the organism for KEGG analysis (default: "ko" for KEGG Orthology)
#'
#' @return A data frame containing GSEA results with columns:
#'   \itemize{
#'     \item \code{pathway_id}: Pathway identifier
#'     \item \code{pathway_name}: Pathway name/description
#'     \item \code{size}: Number of genes in the pathway
#'     \item \code{direction}: Direction of enrichment ("Up" or "Down", for camera/fry)
#'     \item \code{pvalue}: Raw p-value
#'     \item \code{p.adjust}: Adjusted p-value (FDR)
#'     \item \code{method}: The method used for analysis
#'   }
#'   For fgsea/clusterProfiler methods, additional columns include ES, NES, and leading_edge.
#'
#' @details
#' \strong{Method Selection:}
#'
#' The \code{camera} method (default) is recommended for most analyses because:
#' \itemize{
#'   \item It accounts for inter-gene correlations, providing more accurate p-values
#'   \item It supports covariate adjustment through the design matrix
#'   \item It performs a competitive test (genes in set vs. genes not in set)
#' }
#'
#' The \code{fry} method is a fast alternative that:
#' \itemize{
#'   \item Performs a self-contained test (are genes in the set differentially expressed?)
#'   \item Is computationally very efficient for large numbers of gene sets
#'   \item Also supports covariate adjustment
#' }
#'
#' The preranked methods (\code{fgsea}, \code{GSEA}) are included for compatibility but
#' users should be aware that Wu et al. (2012) demonstrated these can produce "spectacularly
#' wrong p-values" even with low inter-gene correlations.
#'
#' \strong{Covariate Adjustment:}
#'
#' When using \code{method = "camera"} or \code{method = "fry"}, you can adjust for
#' confounding variables by specifying them in the \code{covariates} parameter.
#' This is particularly important in microbiome studies where factors like age, sex,
#' BMI, and batch effects can influence results.
#'
#' @references
#' Wu, D., & Smyth, G. K. (2012). Camera: a competitive gene set test accounting for
#' inter-gene correlation. Nucleic Acids Research, 40(17), e133.
#'
#' Wu, D., Lim, E., Vaillant, F., Asselin-Labat, M. L., Visvader, J. E., & Smyth, G. K. (2010).
#' ROAST: rotation gene set tests for complex microarray experiments. Bioinformatics, 26(17), 2176-2182.
#'
#' @export
#'
#' @examples
#' \dontrun{
#' # Load example data
#' data(ko_abundance)
#' data(metadata)
#'
#' # Prepare abundance data
#' abundance_data <- as.data.frame(ko_abundance)
#' rownames(abundance_data) <- abundance_data[, "#NAME"]
#' abundance_data <- abundance_data[, -1]
#'
#' # Method 1: Using camera (recommended) - accounts for inter-gene correlations
#' gsea_results <- pathway_gsea(
#'   abundance = abundance_data,
#'   metadata = metadata,
#'   group = "Environment",
#'   pathway_type = "KEGG",
#'   method = "camera"
#' )
#'
#' # Method 2: Using camera with covariate adjustment
#' gsea_results_adj <- pathway_gsea(
#'   abundance = abundance_data,
#'   metadata = metadata,
#'   group = "Disease",
#'   covariates = c("age", "sex"),
#'   pathway_type = "KEGG",
#'   method = "camera"
#' )
#'
#' # Method 3: Using fry for fast self-contained testing
#' gsea_results_fry <- pathway_gsea(
#'   abundance = abundance_data,
#'   metadata = metadata,
#'   group = "Environment",
#'   pathway_type = "KEGG",
#'   method = "fry"
#' )
#'
#' # Method 4: Using fgsea (preranked, less reliable p-values)
#' gsea_results_fgsea <- pathway_gsea(
#'   abundance = abundance_data,
#'   metadata = metadata,
#'   group = "Environment",
#'   pathway_type = "KEGG",
#'   method = "fgsea"
#' )
#'
#' # Visualize results
#' visualize_gsea(gsea_results, plot_type = "enrichment_plot", n_pathways = 10)
#' }
pathway_gsea <- function(abundance,
                        metadata,
                        group,
                        pathway_type = "KEGG",
                        method = "camera",
                        covariates = NULL,
                        contrast = NULL,
                        inter.gene.cor = 0.01,
                        rank_method = "signal2noise",
                        nperm = 1000,
                        min_size = 5,
                        max_size = 500,
                        p_adjust_method = "BH",
                        seed = 42,
                        go_category = "all",
                        organism = "ko",
                        p.adjust = NULL) {
  # Backward compatibility for deprecated parameter
  if (!is.null(p.adjust)) {
    warning("'p.adjust' parameter is deprecated. Use 'p_adjust_method' instead.", call. = FALSE)
    p_adjust_method <- p.adjust
  }
  
  # Input validation using unified functions
  validate_abundance(abundance, min_samples = 4)
  validate_metadata(metadata)
  validate_group(metadata, group, min_groups = 2)
  
  if (!pathway_type %in% c("KEGG", "MetaCyc", "GO")) {
    stop("pathway_type must be one of 'KEGG', 'MetaCyc', or 'GO'")
  }

  valid_methods <- c("camera", "fry", "fgsea", "GSEA", "clusterProfiler")
  if (!method %in% valid_methods) {
    stop("method must be one of: ", paste(valid_methods, collapse = ", "))
  }

  # Validate rank_method only for preranked methods
  if (method %in% c("fgsea", "GSEA", "clusterProfiler")) {
    if (!rank_method %in% c("signal2noise", "t_test", "log2_ratio", "diff_abundance")) {
      stop("rank_method must be one of 'signal2noise', 't_test', 'log2_ratio', or 'diff_abundance'")
    }
  }

  # Validate covariates
 if (!is.null(covariates)) {
    if (method %in% c("fgsea", "GSEA", "clusterProfiler")) {
      warning("Covariates are only supported for 'camera' and 'fry' methods. ",
              "Covariates will be ignored for method '", method, "'.",
              call. = FALSE)
    } else {
      # Check if all covariates exist in metadata
      missing_covs <- setdiff(covariates, colnames(metadata))
      if (length(missing_covs) > 0) {
        stop("Covariates not found in metadata: ", paste(missing_covs, collapse = ", "))
      }
    }
  }

  # Validate inter.gene.cor for camera (correlation range: -1 to 1)
  if (method == "camera" && !is.null(inter.gene.cor) && !is.na(inter.gene.cor)) {
    if (!is.numeric(inter.gene.cor) || inter.gene.cor < -1 || inter.gene.cor > 1) {
      stop("inter.gene.cor must be a numeric value between -1 and 1, or NA")
    }
  }

  # GO category validation is deferred to prepare_gene_sets() where it is
  # checked against the actual reference data (data-driven, not hardcoded)

  # Check if required package is installed
  method_packages <- list(
    "camera" = "limma",
    "fry" = "limma",
    "fgsea" = "fgsea",
    "GSEA" = "clusterProfiler",
    "clusterProfiler" = "clusterProfiler"
  )
  require_package(method_packages[[method]], purpose = paste("GSEA method", method))

  # Warning for preranked methods about p-value reliability
  if (method %in% c("fgsea", "GSEA", "clusterProfiler")) {
    message("Note: Preranked GSEA methods (fgsea, clusterProfiler) do not account for ",
            "inter-gene correlations, which may lead to unreliable p-values ",
            "(Wu et al., 2012). Consider using method='camera' or method='fry' for ",
            "more reliable statistical inference.")
  }
  
  # Set seed for reproducibility
  set.seed(seed)
  
  # Prepare data
  # Handle #NAME column commonly found in PICRUSt2 output
  if (ncol(abundance) > 0 && colnames(abundance)[1] == "#NAME") {
    # Convert tibble to data.frame if necessary and set proper rownames
    abundance <- as.data.frame(abundance)
    rownames(abundance) <- abundance[, 1]
    abundance <- abundance[, -1]
  }
  
  # Ensure abundance is a matrix with samples as columns
  abundance_mat <- as.matrix(abundance)
  
  # Input validation for MetaCyc pathway data
  if (pathway_type == "MetaCyc") {
    # Check if features look like MetaCyc pathway IDs instead of EC numbers
    feature_names <- rownames(abundance_mat)
    if (length(feature_names) > 0) {
      # Check first 10 features for pathway patterns
      check_features <- feature_names[1:min(10, length(feature_names))]
      if (any(grepl("-PWY$|-PWY[0-9]+$", check_features))) {
        warning(
          "Input appears to contain MetaCyc pathway IDs (e.g., ending in '-PWY'). ",
          "pathway_gsea() requires gene/enzyme level data (EC numbers). ",
          "For pathway-level abundance analysis, use pathway_daa() instead. ",
          "To run GSEA, use EC abundance data from PICRUSt2 output ",
          "(EC_metagenome_out/pred_metagenome_unstrat.tsv). ",
          "See ?pathway_gsea for details.", 
          call. = FALSE, immediate. = TRUE
        )
      }
    }
  }

  # Align samples between abundance and metadata using unified function
  aligned <- align_samples(abundance_mat, metadata)
  abundance_mat <- as.matrix(aligned$abundance)
  metadata <- aligned$metadata

  # Enforce minimum sample requirement
  if (aligned$n_samples < 4) {
    stop("Insufficient samples (", aligned$n_samples,
         "). Need at least 4 samples for statistical analysis.")
  }

  # Extract group information
  Group <- factor(metadata[[group]])
  names(Group) <- colnames(abundance_mat)

  # Validate group balance for statistical reliability
  group_counts <- table(Group)
  if (any(group_counts < 2)) {
    stop("Each group must have at least 2 samples. Current group sizes: ",
         paste(names(group_counts), "=", group_counts, collapse = ", "))
  }
  
  # Prepare gene sets
  gene_sets <- prepare_gene_sets(pathway_type, organism = organism, go_category = go_category)

  # Run analysis based on selected method
  if (method %in% c("camera", "fry")) {
    # =========================================================================
    # Limma-based methods (camera/fry) with covariate support
    # =========================================================================
    results <- run_limma_gsea(
      abundance_mat = abundance_mat,
      metadata = metadata,
      group = group,
      covariates = covariates,
      contrast = contrast,
      gene_sets = gene_sets,
      method = method,
      inter.gene.cor = inter.gene.cor,
      min_size = min_size,
      max_size = max_size,
      p.adjust.method = p_adjust_method
    )

  } else if (method == "fgsea") {
    # =========================================================================
    # Preranked methods (fgsea)
    # =========================================================================

    # Calculate ranking metric for preranked methods
    ranked_list <- calculate_rank_metric(abundance_mat, metadata, group, method = rank_method)
    results <- run_fgsea(ranked_list, gene_sets, min_size, max_size)

  } else if (method == "GSEA" || method == "clusterProfiler") {
    # =========================================================================
    # clusterProfiler GSEA
    # =========================================================================

    # Calculate ranking metric for preranked methods
    ranked_list <- calculate_rank_metric(abundance_mat, metadata, group, method = rank_method)

    # Convert ranked list to format required by clusterProfiler
    gene_list <- sort(ranked_list, decreasing = TRUE)

    # Run GSEA using clusterProfiler
    gsea_result <- clusterProfiler::GSEA(
      geneList = gene_list,
      TERM2GENE = data.frame(
        term = rep(names(gene_sets), sapply(gene_sets, length)),
        gene = unlist(gene_sets)
      ),
      minGSSize = min_size,
      maxGSSize = max_size,
      pvalueCutoff = 1,
      pAdjustMethod = p_adjust_method,
      nPermSimple = nperm,
      seed = seed
    )

    # Convert results to data frame
    if (length(gsea_result) > 0) {
      results <- as.data.frame(gsea_result)

      # Rename columns to match fgsea output
      results <- data.frame(
        pathway_id = results$ID,
        pathway_name = results$Description,
        size = results$setSize,
        ES = results$enrichmentScore,
        NES = results$NES,
        pvalue = results$pvalue,
        p.adjust = results$p.adjust,
        leading_edge = results$core_enrichment,
        stringsAsFactors = FALSE
      )
    } else {
      results <- create_empty_gsea_result(method, full = TRUE)
    }
  }

  # Add method information to populated results
  # (empty results already have method from factory)
  if (nrow(results) > 0) {
    results$method <- method
  }
  
  return(results)
}

#' Prepare gene sets for GSEA
#'
#' @param pathway_type A character string specifying the pathway type: "KEGG", "MetaCyc", or "GO"
#' @param organism A character string specifying the organism (only relevant for KEGG and GO)
#' @param go_category A character string specifying the GO category to use.
#'   "all" (default) uses all categories. Valid values are determined by
#'   the reference data; see \code{table(ko_to_go_reference$category)}.
#'
#' @return A list of pathway gene sets
#' @export
prepare_gene_sets <- function(pathway_type = "KEGG", organism = "ko", go_category = "all") {

  # Validate pathway_type
  valid_types <- c("KEGG", "MetaCyc", "GO")
  if (!pathway_type %in% valid_types) {
    stop("pathway_type must be one of: ", paste(valid_types, collapse = ", "))
  }

  if (pathway_type == "KEGG") {
    # Load KEGG reference using unified loader
    ko_to_kegg_reference <- load_reference_data("ko_to_kegg")

    # Create gene sets: list where each element is a pathway containing KO IDs
    gene_sets <- split(ko_to_kegg_reference$ko_id, ko_to_kegg_reference$pathway_id)

  } else if (pathway_type == "MetaCyc") {
    # Load MetaCyc to EC mapping using unified loader
    metacyc_to_ec_reference <- load_reference_data("metacyc_to_ec")

    # Create gene sets list from MetaCyc pathways
    gene_sets <- list()
    for (i in seq_len(nrow(metacyc_to_ec_reference))) {
      pathway_id <- metacyc_to_ec_reference[i, "pathway"]
      ec_string <- as.character(metacyc_to_ec_reference[i, "ec_numbers"])

      # Skip pathways with no EC mappings
      if (is.na(ec_string) || ec_string == "" || ec_string == "NA") {
        next
      }

      # Split EC numbers by semicolon and clean
      ec_numbers <- strsplit(ec_string, ";")[[1]]
      ec_numbers <- trimws(ec_numbers)
      ec_numbers <- ec_numbers[ec_numbers != ""]

      if (length(ec_numbers) > 0) {
        # Add EC: prefix if not present for consistency
        ec_numbers <- ifelse(grepl("^EC:", ec_numbers), ec_numbers, paste0("EC:", ec_numbers))
        gene_sets[[pathway_id]] <- ec_numbers
      }
    }

  } else if (pathway_type == "GO") {
    # Load GO reference using unified loader
    go_reference <- load_reference_data("ko_to_go")

    # Derive valid categories from the actual reference data
    available_categories <- if ("category" %in% colnames(go_reference)) {
      unique(go_reference$category)
    } else {
      character(0)
    }

    # Validate go_category against what's actually in the data
    if (!is.null(go_category) && go_category != "all" &&
        !go_category %in% available_categories) {
      stop(sprintf(
        "go_category '%s' not found in reference data. Available: %s",
        go_category, paste(c(available_categories, "all"), collapse = ", ")),
        call. = FALSE)
    }

    # Filter by category
    if (!is.null(go_category) && go_category != "all" &&
        "category" %in% colnames(go_reference)) {
      go_reference <- go_reference[go_reference$category == go_category, ]
    }

    # Create gene sets list for each GO term
    gene_sets <- list()

    # Use seq_len to handle empty data frame correctly (1:0 returns c(1,0), not empty)
    for (i in seq_len(nrow(go_reference))) {
      go_id <- go_reference$go_id[i]

      # Get KO members for this GO term
      if ("ko_members" %in% colnames(go_reference)) {
        ko_string <- as.character(go_reference[i, "ko_members"])
        if (!is.na(ko_string) && nchar(ko_string) > 0) {
          ko_ids <- unlist(strsplit(ko_string, ";"))
          ko_ids <- ko_ids[!is.na(ko_ids) & ko_ids != ""]

          if (length(ko_ids) > 0) {
            gene_sets[[go_id]] <- ko_ids
          }
        }
      } else {
        # Fallback: assume other columns contain KO IDs
        ko_ids <- as.character(go_reference[i, -1])
        ko_ids <- ko_ids[!is.na(ko_ids) & ko_ids != ""]

        if (length(ko_ids) > 0) {
          gene_sets[[go_id]] <- ko_ids
        }
      }
    }
  }
  
  return(gene_sets)
}

#' Calculate rank metric for GSEA
#'
#' @param abundance A matrix of abundance data
#' @param metadata A data frame of metadata
#' @param group A character string specifying the grouping variable
#' @param method A character string specifying the ranking method
#'
#' @return A named vector of ranking statistics
#' @keywords internal
calculate_rank_metric <- function(abundance, 
                                 metadata, 
                                 group, 
                                 method = "signal2noise") {
  
  # Extract group information
  Group <- factor(metadata[[group]])
  names(Group) <- rownames(metadata)
  
  # Ensure abundance is a matrix with samples as columns
  abundance <- as.matrix(abundance)
  
  # Subset abundance to include only samples in metadata
  common_samples <- intersect(colnames(abundance), names(Group))
  abundance <- abundance[, common_samples, drop = FALSE]
  Group <- Group[common_samples]

  # Handle problematic values - replace with 0 (undetected features)
  if (any(is.infinite(abundance))) {
    warning("Abundance data contains Inf values. Replacing with 0.", call. = FALSE)
    abundance[is.infinite(abundance)] <- 0
  }
  if (any(is.na(abundance))) {
    warning("Abundance data contains NA values. Replacing with 0.", call. = FALSE)
    abundance[is.na(abundance)] <- 0
  }
  if (any(abundance < 0)) {
    warning("Abundance data contains negative values. Replacing with 0.", call. = FALSE)
    abundance[abundance < 0] <- 0
  }

  # Group size validation already done above in main function
  
  # Get unique group levels
  levels <- levels(Group)
  
  if (length(levels) != 2) {
    stop("GSEA currently only supports two-group comparisons")
  }
  
  # Split samples by group
  group1_samples <- names(Group)[Group == levels[1]]
  group2_samples <- names(Group)[Group == levels[2]]
  
  # Calculate ranking statistic based on method
  if (method == "signal2noise") {
    # Signal-to-noise ratio: (mean1 - mean2) / (sd1 + sd2)
    mean1 <- rowMeans(abundance[, group1_samples, drop = FALSE])
    mean2 <- rowMeans(abundance[, group2_samples, drop = FALSE])
    sd1 <- apply(abundance[, group1_samples, drop = FALSE], 1, stats::sd)
    sd2 <- apply(abundance[, group2_samples, drop = FALSE], 1, stats::sd)
    
    # Handle zero standard deviations using GSEA-style minimum
    # GSEA uses: sigma_min = 0.2 * |mu|, where mu=0 is adjusted to mu=1
    # This ensures the floor is proportional to the data scale
    adjusted_mean1 <- ifelse(mean1 == 0, 1, abs(mean1))
    adjusted_mean2 <- ifelse(mean2 == 0, 1, abs(mean2))
    sd1 <- pmax(sd1, 0.2 * adjusted_mean1)
    sd2 <- pmax(sd2, 0.2 * adjusted_mean2)
    
    metric <- (mean1 - mean2) / (sd1 + sd2)
    # Ensure names are preserved - critical for fgsea
    names(metric) <- rownames(abundance)
    
  } else if (method == "t_test") {
    # t-test statistic
    metric <- numeric(nrow(abundance))
    names(metric) <- rownames(abundance)
    
    for (i in seq_len(nrow(abundance))) {
      t_test <- stats::t.test(abundance[i, group1_samples], abundance[i, group2_samples])
      metric[i] <- t_test$statistic
    }
    
  } else if (method == "log2_ratio") {
    # Log2 fold change
    mean1 <- rowMeans(abundance[, group1_samples, drop = FALSE])
    mean2 <- rowMeans(abundance[, group2_samples, drop = FALSE])

    # Calculate pseudocount using unified function
    # Note: GSEA uses mean1/mean2 direction (group1/group2)
    pseudocount <- calculate_pseudocount(c(mean1, mean2))
    mean1[mean1 == 0] <- pseudocount
    mean2[mean2 == 0] <- pseudocount

    metric <- log2(mean1 / mean2)
    # Ensure names are preserved - critical for fgsea
    names(metric) <- rownames(abundance)
    
  } else if (method == "diff_abundance") {
    # Simple difference in abundance
    mean1 <- rowMeans(abundance[, group1_samples, drop = FALSE])
    mean2 <- rowMeans(abundance[, group2_samples, drop = FALSE])
    metric <- mean1 - mean2
    # Ensure names are preserved - critical for fgsea
    names(metric) <- rownames(abundance)
  }
  
  return(metric)
}

#' Run fgsea using the recommended fgseaMultilevel method
#'
#' @param ranked_list A named vector of ranking statistics
#' @param gene_sets A list of pathway gene sets
#' @param min_size An integer specifying the minimum gene set size
#' @param max_size An integer specifying the maximum gene set size
#'
#' @return A data frame of fgsea results
#' @keywords internal
run_fgsea <- function(ranked_list,
                     gene_sets,
                     min_size = 10,
                     max_size = 500) {
  # Use fgseaMultilevel (default when nperm is NOT passed).
  # fgseaMultilevel uses adaptive multilevel splitting Monte Carlo,
  # which provides accurate p-values without a fixed permutation count.
  # Passing nperm would force the deprecated fgseaSimple method.
  fgsea_result <- fgsea::fgsea(
    pathways = gene_sets,
    stats = ranked_list,
    minSize = min_size,
    maxSize = max_size
  )
  
  # Convert to data frame and handle empty results
  results <- as.data.frame(fgsea_result)
  
  # Check if we have any results
  if (nrow(results) > 0) {
    # Rename columns for consistency
    results <- data.frame(
      pathway_id = results$pathway,
      pathway_name = results$pathway,  # Will be updated later with annotation
      size = results$size,
      ES = results$ES,
      NES = results$NES,
      pvalue = results$pval,
      p.adjust = results$padj,
      leading_edge = sapply(results$leadingEdge, function(x) paste(x, collapse = ";")),
      stringsAsFactors = FALSE
    )
  } else {
    results <- create_empty_gsea_result(full = TRUE)
  }

  return(results)
}

#' Run limma-based gene set analysis (camera/fry)
#'
#' This internal function implements limma's camera and fry methods for gene set
#' enrichment analysis with support for covariates.
#'
#' @param abundance_mat A matrix of abundance data with features as rows and samples as columns
#' @param metadata A data frame containing sample metadata
#' @param group A character string specifying the grouping variable column name
#' @param covariates A character vector of covariate column names (optional)
#' @param contrast Contrast specification for multi-group comparisons (optional)
#' @param gene_sets A named list of gene sets (pathway -> gene IDs)
#' @param method Either "camera" or "fry"
#' @param inter.gene.cor Inter-gene correlation for camera (default 0.01)
#' @param min_size Minimum gene set size
#' @param max_size Maximum gene set size
#' @param p.adjust.method P-value adjustment method
#'
#' @return A data frame containing gene set analysis results
#' @keywords internal
run_limma_gsea <- function(abundance_mat,
                           metadata,
                           group,
                           covariates = NULL,
                           contrast = NULL,
                           gene_sets,
                           method = "camera",
                           inter.gene.cor = 0.01,
                           min_size = 5,
                           max_size = 500,
                           p.adjust.method = "BH") {
  # Samples are already aligned by the caller (pathway_gsea)
  abundance_mat <- as.matrix(abundance_mat)

  # Build design matrix
  design <- build_design_matrix(metadata, group, covariates)

  # Determine which coefficient to test
  # For a design with intercept + group, the group coefficient is typically column 2
  # For a design without intercept (0 + group), we need to construct a contrast
  group_levels <- levels(factor(metadata[[group]]))

  if (is.null(contrast)) {
    # Default: test the group effect (second column for intercept model)
    # or construct a contrast for no-intercept model
    if (ncol(design) >= 2 && grepl(group, colnames(design)[2])) {
      # Intercept model: test second coefficient
      contrast_coef <- 2
    } else {
      # Create contrast: second group vs first group
      contrast_coef <- 2
    }
  } else if (is.numeric(contrast)) {
    contrast_coef <- contrast
  } else if (is.character(contrast)) {
    # Find the column matching the contrast
    contrast_col <- grep(contrast, colnames(design))
    if (length(contrast_col) == 0) {
      stop("Contrast '", contrast, "' not found in design matrix columns: ",
           paste(colnames(design), collapse = ", "))
    }
    contrast_coef <- contrast_col[1]
  } else {
    contrast_coef <- 2
  }

  # Convert gene sets to index format for limma
  # gene_sets is a named list: pathway_id -> vector of gene IDs
  # We need to convert to row indices in abundance_mat
  feature_names <- rownames(abundance_mat)
  gene_set_indices <- lapply(gene_sets, function(genes) {
    which(feature_names %in% genes)
  })

  # Filter gene sets by size
  gene_set_sizes <- sapply(gene_set_indices, length)
  valid_sets <- gene_set_sizes >= min_size & gene_set_sizes <= max_size

  if (sum(valid_sets) == 0) {
    warning("No gene sets passed the size filter (min_size=", min_size,
            ", max_size=", max_size, "). Returning empty results.")
    return(create_empty_gsea_result(method))
  }

  gene_set_indices <- gene_set_indices[valid_sets]
  message(sprintf("Testing %d gene sets (filtered from %d by size constraints)",
                  length(gene_set_indices), length(gene_sets)))

  # Handle problematic values before voom transformation
  # voom/limma do not accept NA, Inf, or negative values in count data

  # Handle Inf/-Inf values first
  if (any(is.infinite(abundance_mat))) {
    inf_count <- sum(is.infinite(abundance_mat))
    warning(sprintf("Abundance data contains %d Inf/-Inf values. ",
                    inf_count),
            "Replacing with 0.",
            call. = FALSE)
    abundance_mat[is.infinite(abundance_mat)] <- 0
  }

  # Handle NA values
  if (any(is.na(abundance_mat))) {
    na_count <- sum(is.na(abundance_mat))
    na_features <- sum(rowSums(is.na(abundance_mat)) > 0)
    warning(sprintf("Abundance data contains %d NA values in %d features. ",
                    na_count, na_features),
            "Replacing NA with 0 (assuming undetected features).",
            call. = FALSE)
    abundance_mat[is.na(abundance_mat)] <- 0
  }

  # Handle negative values (voom expects count-like data)
  if (any(abundance_mat < 0)) {
    neg_count <- sum(abundance_mat < 0)
    warning(sprintf("Abundance data contains %d negative values. ",
                    neg_count),
            "Replacing with 0 (voom expects non-negative count data).",
            call. = FALSE)
    abundance_mat[abundance_mat < 0] <- 0
  }

  # Apply voom transformation for count data
  # First, add a small pseudocount to avoid log(0)
  abundance_mat <- abundance_mat + 0.5

  # Use limma-voom for count data transformation
  # This estimates the mean-variance relationship and computes precision weights
  tryCatch({
    v <- limma::voom(abundance_mat, design, plot = FALSE)
  }, error = function(e) {
    # Fallback: use log2 transformation if voom fails
    warning("voom transformation failed, using log2 transformation instead: ", e$message)
    v <<- list(
      E = log2(abundance_mat),
      weights = matrix(1, nrow = nrow(abundance_mat), ncol = ncol(abundance_mat))
    )
    class(v) <<- "EList"
  })

  # Run the selected method
  if (method == "camera") {
    # Camera: competitive gene set test
    # Accounts for inter-gene correlations
    results <- limma::camera(
      y = v,
      index = gene_set_indices,
      design = design,
      contrast = contrast_coef,
      inter.gene.cor = inter.gene.cor
    )

  } else if (method == "fry") {
    # Fry: fast rotation gene set test (self-contained)
    results <- limma::fry(
      y = v,
      index = gene_set_indices,
      design = design,
      contrast = contrast_coef
    )
  }

  # Format results
  if (nrow(results) > 0) {
    # Add pathway IDs from rownames
    results$pathway_id <- rownames(results)

    # Standardize column names
    formatted_results <- data.frame(
      pathway_id = results$pathway_id,
      pathway_name = results$pathway_id,  # Will be annotated later if mapping available
      size = results$NGenes,
      direction = results$Direction,
      pvalue = results$PValue,
      p.adjust = stats::p.adjust(results$PValue, method = p.adjust.method),
      method = method,
      stringsAsFactors = FALSE
    )

    # Add FDR from results if available (camera/fry compute their own FDR)
    if ("FDR" %in% colnames(results)) {
      formatted_results$FDR_original <- results$FDR
    }

    # Add synthetic NES for visualization compatibility
    # Camera/fry don't produce NES, so we create a synthetic measure based on
    # direction and -log10(pvalue) for visualization purposes
    formatted_results$NES <- ifelse(
      formatted_results$direction == "Up",
      -log10(pmax(formatted_results$pvalue, 1e-300)),  # Positive for Up
      log10(pmax(formatted_results$pvalue, 1e-300))     # Negative for Down
    )

    # Add a leading_edge placeholder for visualization compatibility
    formatted_results$leading_edge <- ""

    # Sort by p-value
    formatted_results <- formatted_results[order(formatted_results$pvalue), ]
    rownames(formatted_results) <- NULL

  } else {
    formatted_results <- create_empty_gsea_result(method)
  }

  return(formatted_results)
}

#' Build design matrix for limma analysis
#'
#' Creates a design matrix incorporating the group variable and optional covariates.
#'
#' @param metadata A data frame containing sample metadata
#' @param group A character string specifying the grouping variable column name
#' @param covariates A character vector of covariate column names (optional)
#'
#' @return A design matrix suitable for limma
#' @keywords internal
build_design_matrix <- function(metadata, group, covariates = NULL) {

  # Ensure group is a factor
  metadata[[group]] <- factor(metadata[[group]])

  # Build formula
  if (is.null(covariates) || length(covariates) == 0) {
    # Simple formula: ~ group
    formula_str <- paste("~", group)
  } else {
    # Formula with covariates: ~ group + cov1 + cov2 + ...
    # Ensure covariates are properly formatted
    for (cov in covariates) {
      if (is.character(metadata[[cov]])) {
        metadata[[cov]] <- factor(metadata[[cov]])
      }
    }
    formula_str <- paste("~", group, "+", paste(covariates, collapse = " + "))
  }

  # Create design matrix
  design <- stats::model.matrix(stats::as.formula(formula_str), data = metadata)

  # Clean up column names for readability
  colnames(design) <- gsub("\\(Intercept\\)", "Intercept", colnames(design))

  return(design)
}

# Tests for pathway_gsea function

# Helper: create standard GSEA test data
create_gsea_test_data <- function(n_features = 50, n_samples = 10) {
  set.seed(123)
  abundance <- matrix(rnorm(n_features * n_samples), nrow = n_features, ncol = n_samples)
  colnames(abundance) <- paste0("Sample", 1:n_samples)
  rownames(abundance) <- paste0("K", sprintf("%05d", 1:n_features))

  metadata <- data.frame(
    sample_name = colnames(abundance),
    group = factor(rep(c("Control", "Treatment"), each = n_samples / 2)),
    stringsAsFactors = FALSE
  )
  rownames(metadata) <- metadata$sample_name

  list(abundance = abundance, metadata = metadata)
}

test_that("pathway_gsea works with fgsea method", {
  skip_if_not_installed("fgsea")

  test_data <- create_gsea_test_data()

  local_mocked_bindings(
    prepare_gene_sets = function(...) {
      list("path:ko00001" = c("K00001", "K00002"), "path:ko00002" = c("K00002", "K00003"))
    },
    run_fgsea = function(...) {
      data.frame(
        pathway_id = c("path:ko00001", "path:ko00002"),
        pathway_name = c("path:ko00001", "path:ko00002"),
        size = c(2, 2), ES = c(0.5, -0.3), NES = c(1.2, -0.8),
        pvalue = c(0.01, 0.05), p.adjust = c(0.02, 0.1),
        leading_edge = c("K00001;K00002", "K00003"),
        stringsAsFactors = FALSE
      )
    }
  )

  result <- pathway_gsea(
    abundance = test_data$abundance,
    metadata = test_data$metadata,
    group = "group",
    method = "fgsea"
  )

  expect_s3_class(result, "data.frame")
  expect_true(all(c("pathway_id", "NES", "pvalue", "method") %in% colnames(result)))
})

test_that("pathway_gsea works with camera method", {
  skip_if_not_installed("limma")

  test_data <- create_gsea_test_data()
  abundance <- abs(test_data$abundance) * 100

  local_mocked_bindings(
    prepare_gene_sets = function(...) {
      list(
        "ko00010" = paste0("K", sprintf("%05d", 1:10)),
        "ko00020" = paste0("K", sprintf("%05d", 11:20))
      )
    }
  )

  result <- pathway_gsea(
    abundance = abundance,
    metadata = test_data$metadata,
    group = "group",
    method = "camera",
    pathway_type = "KEGG"
  )

  expect_s3_class(result, "data.frame")
  expect_true(all(c("pathway_id", "direction", "pvalue") %in% colnames(result)))
  expect_equal(unique(result$method), "camera")
})

test_that("pathway_gsea validates inputs correctly", {
  test_data <- create_gsea_test_data()

  expect_error(pathway_gsea(abundance = "invalid", metadata = test_data$metadata, group = "group"),
               "'abundance' must be a data frame or matrix")
  expect_error(pathway_gsea(abundance = test_data$abundance, metadata = "invalid", group = "group"),
               "'metadata' must be a data frame")
  expect_error(pathway_gsea(abundance = test_data$abundance, metadata = test_data$metadata, group = "invalid_group"),
               "not found in metadata")
  expect_error(pathway_gsea(abundance = test_data$abundance, metadata = test_data$metadata, group = "group", method = "invalid"),
               "method must be one of")
})

test_that("prepare_gene_sets works for KEGG and MetaCyc pathway types", {
  gene_sets_kegg <- prepare_gene_sets("KEGG")
  expect_type(gene_sets_kegg, "list")
  expect_true(length(gene_sets_kegg) > 0)
  expect_false("ko01001" %in% names(gene_sets_kegg))
  expect_false("ko99980" %in% names(gene_sets_kegg))

  gene_sets_metacyc <- prepare_gene_sets("MetaCyc")
  expect_type(gene_sets_metacyc, "list")
})

test_that("calculate_rank_metric aligns by sample column and handles constant t-test rows", {
  calc <- getFromNamespace("calculate_rank_metric", "ggpicrust2")
  abundance <- matrix(
    c(
      1, 1, 1, 1,
      1, 2, 8, 9
    ),
    nrow = 2,
    byrow = TRUE,
    dimnames = list(c("K00001", "K00002"), paste0("S", 1:4))
  )
  metadata <- data.frame(
    sample_name = paste0("S", 1:4),
    group = c("A", "A", "B", "B"),
    stringsAsFactors = FALSE
  )

  metric <- calc(abundance, metadata, "group", method = "t_test")
  expect_named(metric, rownames(abundance))
  expect_equal(unname(metric["K00001"]), 0)
  expect_true(is.finite(metric["K00002"]))
})

test_that("pathway_gsea validates PICRUSt2 #NAME input after stripping the ID column", {
  skip_if_not_installed("fgsea")

  abundance <- data.frame(
    `#NAME` = paste0("K", sprintf("%05d", 1:8)),
    S1 = c(1:8),
    S2 = c(2:9),
    S3 = c(10:17),
    S4 = c(11:18),
    check.names = FALSE
  )
  metadata <- data.frame(
    sample = paste0("S", 1:4),
    group = c("A", "A", "B", "B"),
    stringsAsFactors = FALSE
  )

  local_mocked_bindings(
    prepare_gene_sets = function(...) {
      list("ko00010" = abundance$`#NAME`)
    },
    run_fgsea = function(...) {
      data.frame(
        pathway_id = "ko00010",
        pathway_name = "ko00010",
        size = 8,
        ES = 0.5,
        NES = 1.2,
        pvalue = 0.01,
        p.adjust = 0.02,
        leading_edge = "K00001",
        stringsAsFactors = FALSE
      )
    }
  )

  result <- pathway_gsea(
    abundance = abundance,
    metadata = metadata,
    group = "group",
    method = "fgsea"
  )
  expect_s3_class(result, "data.frame")
  expect_equal(result$pathway_id, "ko00010")
})

test_that("prepare_gene_sets warns when `organism` is set to a non-default value but still returns the same KO gene sets", {
  # Regression: `organism` was documented on both `prepare_gene_sets()`
  # and `pathway_gsea()` but the KEGG and GO branches both read
  # KO-based reference tables (`ko_to_kegg_reference`,
  # `ko_to_go_reference`) that are organism-independent by
  # construction. A user passing `organism = "hsa"` silently got the
  # same gene sets as the default -- a promise/implementation gap --
  # which we now surface as a deprecation warning. The gene-set
  # content must remain identical across organism values (the
  # argument truly has no effect), both for KEGG and GO.
  expect_warning(
    gs_hsa <- prepare_gene_sets("KEGG", organism = "hsa"),
    regexp = "organism.*deprecated.*no effect"
  )
  gs_default <- suppressWarnings(prepare_gene_sets("KEGG", organism = "ko"))
  expect_identical(gs_hsa, gs_default)

  # Default value does not warn.
  expect_warning(
    prepare_gene_sets("KEGG", organism = "ko"),
    regexp = NA
  )

  # GO branch: same contract.
  expect_warning(
    go_hsa <- prepare_gene_sets("GO", organism = "hsa"),
    regexp = "organism.*deprecated.*no effect"
  )
  go_default <- suppressWarnings(prepare_gene_sets("GO", organism = "ko"))
  expect_identical(go_hsa, go_default)
})

# Helper: prepare GSEA data for ridgeplot integration tests
create_ridgeplot_test_data <- function() {
  data(ko_abundance, package = "ggpicrust2")
  data(metadata, package = "ggpicrust2")

  abundance_data <- as.data.frame(ko_abundance)
  rownames(abundance_data) <- abundance_data[, "#NAME"]
  abundance_data <- abundance_data[, -1]

  set.seed(42)
  gsea_results <- suppressMessages(pathway_gsea(
    abundance = abundance_data,
    metadata = metadata,
    group = "Environment",
    pathway_type = "KEGG",
    method = "camera",
    min_size = 5
  ))

  list(gsea_results = gsea_results, abundance = abundance_data, metadata = metadata)
}

test_that("pathway_ridgeplot creates a ggplot object", {
  skip_if_not_installed("ggridges")
  skip_on_cran()

  td <- create_ridgeplot_test_data()

  p <- pathway_ridgeplot(
    gsea_results = td$gsea_results,
    abundance = td$abundance,
    metadata = td$metadata,
    group = "Environment",
    n_pathways = 5
  )

  expect_s3_class(p, "ggplot")
})

test_that("pathway_ridgeplot handles custom parameters", {
  skip_if_not_installed("ggridges")
  skip_on_cran()

  td <- create_ridgeplot_test_data()

  p <- pathway_ridgeplot(
    gsea_results = td$gsea_results,
    abundance = td$abundance,
    metadata = td$metadata,
    group = "Environment",
    n_pathways = 10,
    sort_by = "pvalue",
    show_direction = TRUE,
    colors = c("Down" = "blue", "Up" = "red"),
    title = "Custom Title",
    scale_height = 0.8,
    alpha = 0.5
  )

  expect_s3_class(p, "ggplot")
})

test_that("pathway_ridgeplot handles show_direction = FALSE", {
  skip_if_not_installed("ggridges")
  skip_on_cran()

  td <- create_ridgeplot_test_data()

  p <- pathway_ridgeplot(
    gsea_results = td$gsea_results,
    abundance = td$abundance,
    metadata = td$metadata,
    group = "Environment",
    n_pathways = 5,
    show_direction = FALSE
  )

  expect_s3_class(p, "ggplot")
})

test_that("pathway_ridgeplot errors on invalid input", {
  skip_if_not_installed("ggridges")

  expect_error(
    pathway_ridgeplot(
      gsea_results = "not a data frame",
      abundance = data.frame(),
      metadata = data.frame(),
      group = "group"
    ),
    "must be a data frame"
  )

  expect_error(
    pathway_ridgeplot(
      gsea_results = data.frame(pathway_id = "test"),
      abundance = "not a data frame",
      metadata = data.frame(),
      group = "group"
    ),
    "must be a data frame or matrix"
  )
})

test_that("pathway_ridgeplot errors on missing group column", {
  skip_if_not_installed("ggridges")

  gsea_results <- data.frame(
    pathway_id = "test",
    pvalue = 0.01,
    direction = "Up"
  )

  abundance <- matrix(1:10, nrow = 5)
  rownames(abundance) <- paste0("K", 1:5)
  colnames(abundance) <- c("S1", "S2")

  metadata <- data.frame(
    sample = c("S1", "S2"),
    other_col = c("A", "B"),
    stringsAsFactors = FALSE
  )
  rownames(metadata) <- metadata$sample

  expect_error(
    pathway_ridgeplot(
      gsea_results = gsea_results,
      abundance = abundance,
      metadata = metadata,
      group = "nonexistent"
    ),
    "not found in metadata"
  )
})

# Test GO pathway support in GSEA functionality

test_that("ko_to_go_reference dataset is properly structured", {
  expect_no_error({
    data("ko_to_go_reference", package = "ggpicrust2")
  })

  expect_s3_class(ko_to_go_reference, "data.frame")
  expect_true(all(c("go_id", "go_name", "category", "ko_members") %in% colnames(ko_to_go_reference)))

  # Format validation
  expect_true(all(grepl("^GO:\\d{7}$", ko_to_go_reference$go_id)))
  expect_true(all(ko_to_go_reference$category %in% c("BP", "MF", "CC")))
  expect_true(nrow(ko_to_go_reference) > 0)
})

test_that("prepare_gene_sets works with GO pathway_type", {
  data("ko_to_go_reference", package = "ggpicrust2")
  available_cats <- unique(ko_to_go_reference$category)

  # Test with first available category
  gene_sets <- prepare_gene_sets("GO", go_category = available_cats[1])
  expect_type(gene_sets, "list")
  expect_true(length(gene_sets) > 0)
  expect_true(all(grepl("^GO:\\d{7}$", names(gene_sets))))

  # Test with "all"
  gene_sets_all <- prepare_gene_sets("GO", go_category = "all")
  expect_true(all(names(gene_sets) %in% names(gene_sets_all)))
})

test_that("pathway_gsea works with GO pathway_type", {
  data("ko_abundance", package = "ggpicrust2")
  data("metadata", package = "ggpicrust2")
  data("ko_to_go_reference", package = "ggpicrust2")
  available_cats <- unique(ko_to_go_reference$category)

  abundance_data <- ko_abundance %>%
    tibble::column_to_rownames("#NAME")
  abundance_data <- abundance_data[1:100, ]

  meta_df <- as.data.frame(metadata)
  rownames(meta_df) <- meta_df$sample_name

  expect_no_error({
    gsea_results <- pathway_gsea(
      abundance = abundance_data,
      metadata = meta_df,
      group = "Environment",
      pathway_type = "GO",
      go_category = available_cats[1],
      method = "fgsea",
      nperm = 100,
      min_size = 2,
      max_size = 500
    )
  })
})

#' Compare GSEA and DAA results
#'
#' This function compares the results from Gene Set Enrichment Analysis (GSEA) and
#' Differential Abundance Analysis (DAA) to identify similarities and differences.
#'
#' @param gsea_results A data frame containing GSEA results from the pathway_gsea function
#' @param daa_results A data frame containing DAA results from the pathway_daa function
#' @param plot_type A character string specifying the visualization type: "venn", "upset", or "scatter"
#' @param p_threshold A numeric value specifying the significance threshold
#'
#' @return A list with two elements: \code{plot} (a ggplot2 object, or an
#'   UpSetR object when \code{plot_type = "upset"} and UpSetR is installed)
#'   and \code{results} (a named list with the overlap, GSEA-only, and
#'   DAA-only pathway sets plus their counts).
#' @export
#'
#' @examples
#' \dontrun{
#' # Load example data
#' data(ko_abundance)
#' data(metadata)
#'
#' # Prepare abundance data
#' abundance_data <- as.data.frame(ko_abundance)
#' rownames(abundance_data) <- abundance_data[, "#NAME"]
#' abundance_data <- abundance_data[, -1]
#'
#' # Run GSEA analysis (using camera method - recommended)
#' gsea_results <- pathway_gsea(
#'   abundance = abundance_data,
#'   metadata = metadata,
#'   group = "Environment",
#'   pathway_type = "KEGG",
#'   method = "camera"
#' )
#'
#' # Run DAA analysis
#' daa_results <- pathway_daa(
#'   abundance = abundance_data,
#'   metadata = metadata,
#'   group = "Environment"
#' )
#'
#' # Compare results
#' comparison <- compare_gsea_daa(
#'   gsea_results = gsea_results,
#'   daa_results = daa_results,
#'   plot_type = "venn"
#' )
#' }
compare_gsea_daa <- function(gsea_results,
                            daa_results,
                            plot_type = "venn",
                            p_threshold = 0.05) {

  # Input validation
  if (!is.data.frame(gsea_results)) {
    stop("'gsea_results' must be a data frame")
  }

  if (!is.data.frame(daa_results)) {
    stop("'daa_results' must be a data frame")
  }

  if (length(plot_type) != 1 || !plot_type %in% c("venn", "upset", "scatter")) {
    stop("plot_type must be one of 'venn', 'upset', or 'scatter'")
  }

  # Check if required columns exist. Baseline (for set-membership plots)
  # is just the ID + adjusted p. The scatter variant additionally plots
  # effect sizes (NES vs log2_fold_change), so it needs those columns;
  # checking here rather than only inside the scatter branch means the
  # user gets an up-front, single error naming exactly what's missing
  # instead of a cryptic "undefined columns selected" from merge().
  if (!all(c("pathway_id", "p.adjust") %in% colnames(gsea_results))) {
    stop("GSEA results missing required columns: pathway_id, p.adjust")
  }

  if (!all(c("feature", "p_adjust") %in% colnames(daa_results))) {
    stop("DAA results missing required columns: feature, p_adjust")
  }

  if (plot_type == "scatter") {
    missing_gsea <- setdiff("NES", colnames(gsea_results))
    missing_daa <- setdiff("log2_fold_change", colnames(daa_results))
    if (length(missing_gsea) > 0 || length(missing_daa) > 0) {
      stop(sprintf(
        "plot_type = 'scatter' requires effect-size columns: %s%s%s. Use plot_type = 'venn' or 'upset' if effect sizes are unavailable.",
        if (length(missing_gsea)) paste0("gsea_results needs '", missing_gsea, "'") else "",
        if (length(missing_gsea) && length(missing_daa)) "; " else "",
        if (length(missing_daa)) paste0("daa_results needs '", missing_daa, "'") else ""
      ), call. = FALSE)
    }
  }

  # Extract significant pathways from each analysis
  # Convert to character to ensure consistent types for set operations and ggVennDiagram
  sig_gsea <- unique(as.character(gsea_results$pathway_id[which(gsea_results$p.adjust < p_threshold)]))
  sig_daa <- unique(as.character(daa_results$feature[which(daa_results$p_adjust < p_threshold)]))

  # Find overlapping and unique pathways
  overlap <- intersect(sig_gsea, sig_daa)
  gsea_only <- setdiff(sig_gsea, sig_daa)
  daa_only <- setdiff(sig_daa, sig_gsea)

  # Create comparison results
  comparison_results <- list(
    overlap = overlap,
    gsea_only = gsea_only,
    daa_only = daa_only,
    n_overlap = length(overlap),
    n_gsea_only = length(gsea_only),
    n_daa_only = length(daa_only),
    n_gsea_total = length(sig_gsea),
    n_daa_total = length(sig_daa)
  )

  # Create visualization based on plot_type
  if (plot_type == "venn") {
    # Check if required package is available
    if (!requireNamespace("ggVennDiagram", quietly = TRUE)) {
      warning("Package 'ggVennDiagram' is required for Venn diagrams. Using a basic plot instead.")

      # Create a basic representation
      counts <- c(comparison_results$n_gsea_only,
                 comparison_results$n_overlap,
                 comparison_results$n_daa_only)

      labels <- c("GSEA only", "Overlap", "DAA only")

      p <- ggplot2::ggplot(data.frame(counts = counts, labels = labels),
                         ggplot2::aes(x = labels, y = counts, fill = labels)) +
        ggplot2::geom_bar(stat = "identity") +
        ggplot2::labs(
          title = "Comparison of Significant Pathways",
          x = "",
          y = "Number of Pathways",
          fill = ""
        ) +
        ggplot2::theme_minimal() +
        ggplot2::theme(
          plot.title = ggplot2::element_text(hjust = 0.5)
        )
    } else {
      # Create a proper Venn diagram
      venn_list <- list(
        GSEA = sig_gsea,
        DAA = sig_daa
      )

      p <- ggVennDiagram::ggVennDiagram(venn_list) +
        ggplot2::labs(title = "Overlap of Significant Pathways") +
        ggplot2::theme(plot.title = ggplot2::element_text(hjust = 0.5))
    }

  } else if (plot_type == "upset") {
    # Check if required package is available
    if (!requireNamespace("UpSetR", quietly = TRUE)) {
      warning("Package 'UpSetR' is required for UpSet plots. Using a basic plot instead.")

      # Create a basic representation
      counts <- c(comparison_results$n_gsea_only,
                 comparison_results$n_overlap,
                 comparison_results$n_daa_only)

      labels <- c("GSEA only", "Overlap", "DAA only")

      p <- ggplot2::ggplot(data.frame(counts = counts, labels = labels),
                         ggplot2::aes(x = labels, y = counts, fill = labels)) +
        ggplot2::geom_bar(stat = "identity") +
        ggplot2::labs(
          title = "Comparison of Significant Pathways",
          x = "",
          y = "Number of Pathways",
          fill = ""
        ) +
        ggplot2::theme_minimal() +
        ggplot2::theme(
          plot.title = ggplot2::element_text(hjust = 0.5)
        )
    } else {
      # Create UpSet plot
      all_pathways <- unique(c(sig_gsea, sig_daa))
      upset_matrix <- matrix(0, nrow = length(all_pathways), ncol = 2)
      rownames(upset_matrix) <- all_pathways
      colnames(upset_matrix) <- c("GSEA", "DAA")

      upset_matrix[sig_gsea, "GSEA"] <- 1
      upset_matrix[sig_daa, "DAA"] <- 1

      upset_df <- as.data.frame(upset_matrix)
      p <- UpSetR::upset(upset_df, nsets = 2, order.by = "freq")
    }

  } else if (plot_type == "scatter") {
    # Create a scatter plot comparing p-values or effect sizes

    # Merge the results
    merged_results <- merge(
      gsea_results[, c("pathway_id", "NES", "p.adjust")],
      daa_results[, c("feature", "log2_fold_change", "p_adjust")],
      by.x = "pathway_id",
      by.y = "feature",
      all = FALSE
    )

    # If no overlapping pathways, return a message
    if (nrow(merged_results) == 0) {
      warning("No overlapping pathways found for scatter plot")
      p <- ggplot2::ggplot() +
        ggplot2::annotate("text", x = 0, y = 0, label = "No overlapping pathways found") +
        ggplot2::theme_void()
    } else {
      # Create scatter plot
      p <- ggplot2::ggplot(merged_results,
                         ggplot2::aes(x = .data$NES, y = .data$log2_fold_change,
                                    color = -log10(.data$p.adjust))) +
        ggplot2::geom_point() +
        ggplot2::scale_color_gradient(low = "blue", high = "red") +
        ggplot2::labs(
          title = "Comparison of GSEA and DAA Results",
          x = "Normalized Enrichment Score (GSEA)",
          y = "Log2 Fold Change (DAA)",
          color = "-log10(p.adjust)"
        ) +
        ggplot2::theme_minimal() +
        ggplot2::theme(
          plot.title = ggplot2::element_text(hjust = 0.5)
        ) +
        ggplot2::geom_hline(yintercept = 0, linetype = "dashed", color = "gray") +
        ggplot2::geom_vline(xintercept = 0, linetype = "dashed", color = "gray")
    }
  }

  # Return both the plot and the comparison results
  return(list(
    plot = p,
    results = comparison_results
  ))
}

# =============================================================================
# Taxa-Function Contribution Analysis
# =============================================================================
# Parse and aggregate PICRUSt2 per-sequence contribution data to identify
# which taxa drive differentially abundant pathways.

#' Read PICRUSt2 contribution file
#'
#' Parses the \code{pred_metagenome_contrib.tsv} file produced by PICRUSt2's
#' \code{--per_sequence_contrib} flag.
#'
#' @param file Path to the contribution file (.tsv, .txt, or .csv).
#' @param data A data.frame already loaded from the contribution file.
#'   If both \code{file} and \code{data} are provided, \code{data} is used.
#'
#' @return A data.frame with columns: \code{sample}, \code{function_id},
#'   \code{taxon}, \code{taxon_function_abun}, \code{norm_taxon_function_contrib},
#'   and any additional columns from the original file.
#'
#' @details
#' The contribution file records how much each ASV/OTU contributes to the
#' predicted abundance of each gene family in each sample. The
#' \code{norm_taxon_function_contrib} column gives the proportion of a
#' function's abundance attributable to each taxon.
#'
#' @examples
#' \donttest{
#' # From a data.frame
#' contrib_df <- data.frame(
#'   sample = rep(c("S1", "S2"), each = 4),
#'   `function` = rep(c("K00001", "K00002"), 4),
#'   taxon = rep(c("ASV1", "ASV2"), each = 2, times = 2),
#'   taxon_function_abun = runif(8),
#'   norm_taxon_function_contrib = runif(8),
#'   check.names = FALSE
#' )
#' result <- read_contrib_file(data = contrib_df)
#' head(result)
#' }
#'
#' @export
read_contrib_file <- function(file = NULL, data = NULL) {
  if (is.null(file) && is.null(data)) {
    stop("Please provide either a file path or a data.frame.")
  }
  if (!is.null(file) && !is.null(data)) {
    warning("Both file and data provided. Using data and ignoring file.")
  }

  if (!is.null(data)) {
    if (!is.data.frame(data)) stop("'data' must be a data.frame")
    contrib <- data
  } else {
    contrib <- read_abundance_file(file)
  }

  # Validate required columns
  function_col <- if ("function_id" %in% colnames(contrib)) {
    "function_id"
  } else if ("function" %in% colnames(contrib)) {
    "function"
  } else {
    NULL
  }

  required <- c("sample", "taxon",
                "taxon_function_abun", "norm_taxon_function_contrib")
  missing <- setdiff(required, colnames(contrib))
  if (is.null(function_col)) {
    missing <- c(missing, "function")
  }
  if (length(missing) > 0) {
    stop(sprintf(
      "Missing required columns: %s. Expected PICRUSt2 contrib format.",
      paste(missing, collapse = ", ")
    ))
  }

  if (identical(function_col, "function")) {
    # Rename 'function' -> 'function_id' (reserved word in R)
    colnames(contrib)[colnames(contrib) == "function"] <- "function_id"
  }

  # Strip ko: prefix
  contrib$function_id <- gsub("^ko:", "", contrib$function_id)

  contrib <- as.data.frame(contrib)
  contrib
}


#' Read PICRUSt2 stratified abundance file
#'
#' Parses the \code{pred_metagenome_strat.tsv} file produced by PICRUSt2's
#' \code{--strat_out} flag and converts the wide format to tidy long format.
#'
#' @param file Path to the stratified file (.tsv, .txt, or .csv).
#' @param data A data.frame already loaded from the stratified file.
#'   If both \code{file} and \code{data} are provided, \code{data} is used.
#'
#' @return A tidy data.frame with columns: \code{function_id}, \code{taxon},
#'   \code{sample}, \code{abundance}.
#'
#' @details
#' The stratified file has function IDs in the first column, sequence/taxon
#' IDs in the second column, and sample abundances in the remaining columns
#' (wide format). This function pivots to long format for downstream analysis.
#'
#' @examples
#' \donttest{
#' # From a data.frame
#' strat_df <- data.frame(
#'   `function` = c("K00001", "K00001", "K00002"),
#'   sequence = c("ASV1", "ASV2", "ASV1"),
#'   S1 = c(10, 5, 8),
#'   S2 = c(12, 3, 7),
#'   check.names = FALSE
#' )
#' result <- read_strat_file(data = strat_df)
#' head(result)
#' }
#'
#' @export
read_strat_file <- function(file = NULL, data = NULL) {
  if (is.null(file) && is.null(data)) {
    stop("Please provide either a file path or a data.frame.")
  }
  if (!is.null(file) && !is.null(data)) {
    warning("Both file and data provided. Using data and ignoring file.")
  }

  if (!is.null(data)) {
    if (!is.data.frame(data)) stop("'data' must be a data.frame")
    strat <- data
  } else {
    strat <- read_abundance_file(file)
  }

  # Validate minimum structure: function column + sequence column + >= 1 sample
  if (ncol(strat) < 3) {
    stop("Stratified file must have at least 3 columns (function, sequence, samples).")
  }

  function_col <- if ("function_id" %in% colnames(strat)) {
    "function_id"
  } else if ("function" %in% colnames(strat)) {
    "function"
  } else {
    NULL
  }
  taxon_col <- if ("taxon" %in% colnames(strat)) {
    "taxon"
  } else if ("sequence" %in% colnames(strat)) {
    "sequence"
  } else {
    NULL
  }

  if (is.null(function_col)) {
    stop("Expected a 'function' column in the stratified file.")
  }
  if (is.null(taxon_col)) {
    stop("Expected a 'sequence' column in the stratified file.")
  }

  if (identical(function_col, "function")) {
    colnames(strat)[colnames(strat) == "function"] <- "function_id"
  }
  if (identical(taxon_col, "sequence")) {
    colnames(strat)[colnames(strat) == "sequence"] <- "taxon"
  }

  # Pivot sample columns to long format
  sample_cols <- setdiff(colnames(strat), c("function_id", "taxon"))
  id_cols <- c("function_id", "taxon")
  strat <- tidyr::pivot_longer(
    strat,
    cols = -all_of(id_cols),
    names_to = "sample",
    values_to = "abundance"
  )

  # Strip ko: prefix
  strat$function_id <- gsub("^ko:", "", strat$function_id)

  as.data.frame(strat)
}


#' Aggregate taxa contributions for visualization
#'
#' Core aggregation function that bridges PICRUSt2 contribution data with
#' differential abundance analysis results. Optionally maps ASV/OTU IDs to
#' taxonomic names and filters to significant pathways.
#'
#' @param contrib_data A data.frame from \code{\link{read_contrib_file}} or
#'   \code{\link{read_strat_file}}.
#' @param taxonomy Optional data.frame mapping taxon IDs to taxonomy. Supports
#'   QIIME2 format (semicolon-delimited taxonomy strings) or DADA2 format
#'   (separate columns for each rank).
#' @param tax_level Character. Taxonomic rank for aggregation. One of
#'   \code{"Kingdom"}, \code{"Phylum"}, \code{"Class"}, \code{"Order"},
#'   \code{"Family"}, \code{"Genus"}, \code{"Species"}. Default \code{"Genus"}.
#' @param top_n Integer. Number of top taxa to keep; remaining are lumped as
#'   "Other". Default 10.
#' @param daa_results_df Optional data.frame from \code{\link{pathway_daa}},
#'   used to filter contributions to significant pathways.
#' @param pathway_ids Optional character vector of pathway IDs to filter.
#'   Alternative to \code{daa_results_df}.
#' @param p_threshold Numeric. Significance cutoff when using
#'   \code{daa_results_df}. Default 0.05.
#'
#' @return A tidy data.frame with columns: \code{sample}, \code{function_id},
#'   \code{taxon_label}, \code{contribution}.
#'
#' @details
#' When \code{daa_results_df} is provided, the function:
#' \enumerate{
#'   \item Extracts significant pathway IDs from the DAA results
#'   \item Maps pathway IDs to their constituent KO IDs using the internal
#'     ko_to_kegg reference
#'   \item Filters contribution data to only matching KO IDs
#' }
#'
#' Taxonomy can be provided in two formats:
#' \itemize{
#'   \item QIIME2: A column named \code{Taxon} or \code{taxonomy} containing
#'     semicolon-delimited strings (e.g., "k__Bacteria;p__Firmicutes;...")
#'   \item DADA2: Separate columns for each rank (\code{Kingdom}, \code{Phylum}, etc.)
#' }
#'
#' @examples
#' \donttest{
#' # Basic usage with synthetic data
#' contrib <- data.frame(
#'   sample = rep(c("S1", "S2"), each = 6),
#'   function_id = rep(c("K00001", "K00002", "K00003"), 4),
#'   taxon = rep(c("ASV1", "ASV2"), each = 3, times = 2),
#'   taxon_function_abun = runif(12),
#'   norm_taxon_function_contrib = runif(12)
#' )
#' agg <- aggregate_taxa_contributions(contrib, top_n = 2)
#' head(agg)
#' }
#'
#' @export
aggregate_taxa_contributions <- function(contrib_data,
                                         taxonomy = NULL,
                                         tax_level = "Genus",
                                         top_n = 10,
                                         daa_results_df = NULL,
                                         pathway_ids = NULL,
                                         p_threshold = 0.05) {
  # Validate input
  validate_dataframe(contrib_data, param_name = "contrib_data")

  if (!is.null(daa_results_df) && !is.null(pathway_ids)) {
    warning("Both daa_results_df and pathway_ids provided. Using daa_results_df.")
  }

  # Detect input format (contrib vs strat)
  if ("norm_taxon_function_contrib" %in% colnames(contrib_data)) {
    contrib_col <- "norm_taxon_function_contrib"
  } else if ("abundance" %in% colnames(contrib_data)) {
    contrib_col <- "abundance"
  } else {
    stop("contrib_data must contain 'norm_taxon_function_contrib' or 'abundance' column.")
  }

  # Ensure required columns exist
  required <- c("sample", "function_id", "taxon", contrib_col)
  missing <- setdiff(required, colnames(contrib_data))
  if (length(missing) > 0) {
    stop(sprintf("Missing required columns: %s", paste(missing, collapse = ", ")))
  }

  # Filter to significant pathways via DAA results.
  #
  # `daa_results_df$feature` (or `pathway_ids`) may be either:
  #   - pathway IDs (e.g. "ko00010"), which must be mapped back to their
  #     constituent KOs because `contrib_data$function_id` is KO-level; or
  #   - KO IDs (e.g. "K00001"), which match `function_id` directly and
  #     must NOT go through the ko_to_kegg mapping, otherwise every KO
  #     is treated as a (non-existent) pathway and the entire contrib
  #     table is filtered away.
  #
  # We dispatch on ID shape rather than asking the caller, because the
  # upstream ggpicrust2 pipeline legitimately produces DAA results at
  # either level depending on the chosen abundance table.
  if (!is.null(daa_results_df)) {
    sig_features <- daa_results_df$feature[daa_results_df$p_adjust < p_threshold]
    sig_features <- unique(sig_features[!is.na(sig_features)])
    if (length(sig_features) == 0) {
      stop("No significant features found at p_threshold = ", p_threshold)
    }
    contrib_data <- filter_contrib_by_features(contrib_data, sig_features)
  } else if (!is.null(pathway_ids)) {
    contrib_data <- filter_contrib_by_features(contrib_data, pathway_ids)
  }

  if (nrow(contrib_data) == 0) {
    stop("No data remaining after filtering. Check that function IDs match between contrib_data and pathway/DAA results.")
  }

  # Apply taxonomy labels
  if (!is.null(taxonomy)) {
    contrib_data <- apply_taxonomy(contrib_data, taxonomy, tax_level)
  } else {
    contrib_data$taxon_label <- contrib_data$taxon
  }

  # Aggregate by sample, function, taxon_label
  agg <- stats::aggregate(
    stats::as.formula(paste(contrib_col, "~ sample + function_id + taxon_label")),
    data = contrib_data,
    FUN = sum
  )
  colnames(agg)[colnames(agg) == contrib_col] <- "contribution"

  # Rank taxa and keep top_n
  taxa_means <- stats::aggregate(contribution ~ taxon_label, data = agg, FUN = mean)
  taxa_means <- taxa_means[order(-taxa_means$contribution), ]
  top_taxa <- utils::head(taxa_means$taxon_label, top_n)

  agg$taxon_label <- ifelse(agg$taxon_label %in% top_taxa, agg$taxon_label, "Other")

  # Re-aggregate after lumping
  agg <- stats::aggregate(
    contribution ~ sample + function_id + taxon_label,
    data = agg,
    FUN = sum
  )

  agg
}


# =============================================================================
# Internal helpers
# =============================================================================

#' Filter contribution data by a vector of DAA features (KO or pathway IDs)
#'
#' Dispatches on ID shape: KO-like (`K\d{5}`) intersects directly with
#' `function_id`; pathway-like (`ko\d{5}`, `map\d{5}`, `ec\d{5}`) is first
#' expanded to its constituent KOs via the ko_to_kegg reference. Mixed
#' inputs are handled by combining both resolved sets.
#'
#' @noRd
filter_contrib_by_features <- function(contrib_data, features) {
  ko_pattern <- "^K\\d{5}$"
  pathway_pattern <- "^(ko|map|ec)\\d{5}$"

  is_ko <- grepl(ko_pattern, features)
  is_pathway <- grepl(pathway_pattern, features)

  if (!any(is_ko | is_pathway)) {
    stop(
      "None of the features look like KO IDs (K#####) or pathway IDs ",
      "(ko#####/map#####/ec#####). Example: '",
      paste(head(features, 2), collapse = "', '"),
      "'. Check that your DAA input is KEGG-based."
    )
  }

  matched_kos <- character(0)
  if (any(is_ko)) {
    matched_kos <- c(matched_kos, features[is_ko])
  }
  if (any(is_pathway)) {
    ko_to_kegg <- load_reference_data("ko_to_kegg")
    pathway_kos <- ko_to_kegg$ko_id[ko_to_kegg$pathway_id %in% features[is_pathway]]
    matched_kos <- c(matched_kos, pathway_kos)
  }
  matched_kos <- unique(matched_kos)

  filtered <- contrib_data[contrib_data$function_id %in% matched_kos, ]
  if (nrow(filtered) == 0) {
    stop(
      "No data remaining after filtering. Check that function IDs match ",
      "between contrib_data (e.g. '",
      paste(head(unique(contrib_data$function_id), 2), collapse = "', '"),
      "') and the requested feature IDs (e.g. '",
      paste(head(features, 2), collapse = "', '"), "')."
    )
  }
  filtered
}

#' Parse a QIIME2-style taxonomy string
#'
#' @param tax_string Character. Semicolon-delimited taxonomy string.
#' @param level Character. Rank to extract.
#' @return Character. The taxon name at the requested level, or NA.
#' @noRd
parse_taxonomy_string <- function(tax_string, level) {
  prefix_map <- c(
    Kingdom = "k__", Phylum = "p__", Class = "c__", Order = "o__",
    Family = "f__", Genus = "g__", Species = "s__"
  )
  prefix <- prefix_map[[level]]
  if (is.null(prefix)) return(NA_character_)

  parts <- strsplit(tax_string, ";\\s*")[[1]]
  match <- parts[startsWith(parts, prefix)]
  if (length(match) == 0) return(NA_character_)

  name <- sub(paste0("^", prefix), "", match[1])
  name <- trimws(name)
  if (nchar(name) == 0 || name == "__") return(NA_character_)
  name
}

#' Apply taxonomy mapping to contribution data
#'
#' @param contrib_data Data.frame with a \code{taxon} column.
#' @param taxonomy Data.frame with taxonomy information.
#' @param tax_level Character. Rank for labeling.
#' @return Data.frame with added \code{taxon_label} column.
#' @noRd
apply_taxonomy <- function(contrib_data, taxonomy, tax_level) {
  valid_levels <- c("Kingdom", "Phylum", "Class", "Order",
                    "Family", "Genus", "Species")
  if (!tax_level %in% valid_levels) {
    stop(sprintf(
      "Invalid tax_level '%s'. Must be one of: %s",
      tax_level, paste(valid_levels, collapse = ", ")
    ))
  }

  # Detect taxonomy format
  tax_cols <- colnames(taxonomy)
  id_col <- intersect(
    c("Feature.ID", "Feature ID", "feature_id", "ASV", "OTU", "#OTU ID"),
    tax_cols
  )

  # Also check first column as ID
  if (length(id_col) == 0) {
    id_col <- tax_cols[1]
  } else {
    id_col <- id_col[1]
  }

  # QIIME2 format: has a Taxon or taxonomy column with semicolon-delimited strings
  qiime2_col <- intersect(c("Taxon", "taxonomy", "Taxonomy"), tax_cols)

  if (length(qiime2_col) > 0) {
    # QIIME2 format
    qiime2_col <- qiime2_col[1]
    tax_labels <- vapply(
      taxonomy[[qiime2_col]],
      parse_taxonomy_string,
      character(1),
      level = tax_level
    )
    tax_map <- data.frame(
      taxon = taxonomy[[id_col]],
      taxon_label = tax_labels,
      stringsAsFactors = FALSE
    )
  } else if (tax_level %in% tax_cols) {
    # DADA2 format: separate columns per rank
    tax_map <- data.frame(
      taxon = taxonomy[[id_col]],
      taxon_label = taxonomy[[tax_level]],
      stringsAsFactors = FALSE
    )
  } else {
    warning(sprintf(
      "Could not find '%s' in taxonomy. Using raw taxon IDs.", tax_level
    ))
    contrib_data$taxon_label <- contrib_data$taxon
    return(contrib_data)
  }

  tax_map <- tax_map[!is.na(tax_map$taxon), , drop = FALSE]
  duplicate_taxa <- unique(tax_map$taxon[duplicated(tax_map$taxon)])
  if (length(duplicate_taxa) > 0) {
    warning(
      sprintf(
        "Duplicate taxon IDs found in taxonomy. Using the first match for %d taxa.",
        length(duplicate_taxa)
      ),
      call. = FALSE
    )
    tax_map <- tax_map[!duplicated(tax_map$taxon), , drop = FALSE]
  }

  match_idx <- match(contrib_data$taxon, tax_map$taxon)
  contrib_data$taxon_label <- tax_map$taxon_label[match_idx]
  contrib_data$taxon_label[
    is.na(contrib_data$taxon_label) | trimws(contrib_data$taxon_label) == ""
  ] <- "Unclassified"

  contrib_data
}

# =============================================================================
# Tests for taxa contribution analysis
# =============================================================================

# Shared test data generator
create_contrib_test_data <- function() {
  set.seed(123)
  samples <- c("S1", "S2")
  functions <- c("K00001", "K00002", "K00003")
  taxa <- c("ASV1", "ASV2", "ASV3", "ASV4")

  contrib <- expand.grid(
    sample = samples,
    function_id = functions,
    taxon = taxa,
    stringsAsFactors = FALSE
  )
  # Rename function_id -> function for raw input format
  contrib_raw <- contrib
  colnames(contrib_raw)[colnames(contrib_raw) == "function_id"] <- "function"
  contrib_raw$taxon_function_abun <- runif(nrow(contrib_raw))
  contrib_raw$norm_taxon_function_contrib <- runif(nrow(contrib_raw))

  # Stratified format (wide)
  strat_long <- expand.grid(
    function_id = functions,
    taxon = taxa,
    stringsAsFactors = FALSE
  )
  strat_wide <- data.frame(
    `function` = strat_long$function_id,
    sequence = strat_long$taxon,
    S1 = runif(nrow(strat_long)),
    S2 = runif(nrow(strat_long)),
    check.names = FALSE
  )

  # Taxonomy (QIIME2 format)
  taxonomy_qiime2 <- data.frame(
    `Feature.ID` = taxa,
    Taxon = c(
      "k__Bacteria;p__Firmicutes;c__Bacilli;o__Lactobacillales;f__Lactobacillaceae;g__Lactobacillus;s__acidophilus",
      "k__Bacteria;p__Proteobacteria;c__Gammaproteobacteria;o__Enterobacterales;f__Enterobacteriaceae;g__Escherichia;s__coli",
      "k__Bacteria;p__Firmicutes;c__Clostridia;o__Eubacteriales;f__Lachnospiraceae;g__Roseburia;s__intestinalis",
      "k__Bacteria;p__Bacteroidetes;c__Bacteroidia;o__Bacteroidales;f__Bacteroidaceae;g__Bacteroides;s__fragilis"
    ),
    check.names = FALSE,
    stringsAsFactors = FALSE
  )

  # Taxonomy (DADA2 format)
  taxonomy_dada2 <- data.frame(
    ASV = taxa,
    Kingdom = rep("Bacteria", 4),
    Phylum = c("Firmicutes", "Proteobacteria", "Firmicutes", "Bacteroidetes"),
    Class = c("Bacilli", "Gammaproteobacteria", "Clostridia", "Bacteroidia"),
    Order = c("Lactobacillales", "Enterobacterales", "Eubacteriales", "Bacteroidales"),
    Family = c("Lactobacillaceae", "Enterobacteriaceae", "Lachnospiraceae", "Bacteroidaceae"),
    Genus = c("Lactobacillus", "Escherichia", "Roseburia", "Bacteroides"),
    Species = c("acidophilus", "coli", "intestinalis", "fragilis"),
    stringsAsFactors = FALSE
  )

  # Metadata
  metadata <- data.frame(
    sample = c("S1", "S2"),
    group = c("Control", "Treatment"),
    stringsAsFactors = FALSE
  )

  list(
    contrib_raw = contrib_raw,
    strat_wide = strat_wide,
    taxonomy_qiime2 = taxonomy_qiime2,
    taxonomy_dada2 = taxonomy_dada2,
    metadata = metadata
  )
}


# ---- read_contrib_file ----

test_that("read_contrib_file parses contrib data correctly", {
  td <- create_contrib_test_data()
  result <- read_contrib_file(data = td$contrib_raw)

  expect_s3_class(result, "data.frame")
  expect_true("function_id" %in% colnames(result))
  expect_false("function" %in% colnames(result))
  expect_true(all(c("sample", "taxon", "taxon_function_abun",
                     "norm_taxon_function_contrib") %in% colnames(result)))
  expect_equal(nrow(result), nrow(td$contrib_raw))
})

test_that("read_contrib_file strips ko: prefix", {
  td <- create_contrib_test_data()
  td$contrib_raw$`function` <- paste0("ko:", td$contrib_raw$`function`)
  result <- read_contrib_file(data = td$contrib_raw)
  expect_false(any(grepl("^ko:", result$function_id)))
})

test_that("read_contrib_file accepts already standardized function_id", {
  td <- create_contrib_test_data()
  contrib_standardized <- td$contrib_raw
  colnames(contrib_standardized)[colnames(contrib_standardized) == "function"] <- "function_id"

  result <- read_contrib_file(data = contrib_standardized)

  expect_true("function_id" %in% colnames(result))
  expect_false("function" %in% colnames(result))
  expect_equal(nrow(result), nrow(contrib_standardized))
})

test_that("read_contrib_file errors on missing columns", {
  bad_df <- data.frame(sample = "S1", x = 1)
  expect_error(read_contrib_file(data = bad_df), "Missing required columns")
})

test_that("read_contrib_file requires file or data", {
  expect_error(read_contrib_file(), "Please provide either")
})


# ---- read_strat_file ----

test_that("read_strat_file converts wide to long format", {
  td <- create_contrib_test_data()
  result <- read_strat_file(data = td$strat_wide)

  expect_s3_class(result, "data.frame")
  expect_true(all(c("function_id", "taxon", "sample", "abundance")
                  %in% colnames(result)))
  # 3 functions x 4 taxa x 2 samples = 24 rows
  expect_equal(nrow(result), 24)
})

test_that("read_strat_file strips ko: prefix", {
  td <- create_contrib_test_data()
  td$strat_wide$`function` <- paste0("ko:", td$strat_wide$`function`)
  result <- read_strat_file(data = td$strat_wide)
  expect_false(any(grepl("^ko:", result$function_id)))
})

test_that("read_strat_file accepts already standardized column names", {
  td <- create_contrib_test_data()
  strat_standardized <- td$strat_wide
  colnames(strat_standardized)[colnames(strat_standardized) == "function"] <- "function_id"
  colnames(strat_standardized)[colnames(strat_standardized) == "sequence"] <- "taxon"

  result <- read_strat_file(data = strat_standardized)

  expect_true(all(c("function_id", "taxon", "sample", "abundance")
                  %in% colnames(result)))
  expect_equal(nrow(result), 24)
})

test_that("read_strat_file errors on too few columns", {
  bad_df <- data.frame(`function` = "K00001", check.names = FALSE)
  expect_error(read_strat_file(data = bad_df), "at least 3 columns")
})


# ---- aggregate_taxa_contributions ----

test_that("aggregate_taxa_contributions returns expected shape", {
  td <- create_contrib_test_data()
  contrib <- read_contrib_file(data = td$contrib_raw)
  agg <- aggregate_taxa_contributions(contrib, top_n = 4)

  expect_s3_class(agg, "data.frame")
  expect_true(all(c("sample", "function_id", "taxon_label", "contribution")
                  %in% colnames(agg)))
  expect_gt(nrow(agg), 0)
})

test_that("aggregate_taxa_contributions with QIIME2 taxonomy uses genus labels", {
  td <- create_contrib_test_data()
  contrib <- read_contrib_file(data = td$contrib_raw)
  agg <- aggregate_taxa_contributions(
    contrib,
    taxonomy = td$taxonomy_qiime2,
    tax_level = "Genus",
    top_n = 10
  )

  expect_true(any(agg$taxon_label %in%
                    c("Lactobacillus", "Escherichia", "Roseburia", "Bacteroides")))
})

test_that("aggregate_taxa_contributions with DADA2 taxonomy uses genus labels", {
  td <- create_contrib_test_data()
  contrib <- read_contrib_file(data = td$contrib_raw)
  agg <- aggregate_taxa_contributions(
    contrib,
    taxonomy = td$taxonomy_dada2,
    tax_level = "Genus",
    top_n = 10
  )

  expect_true(any(agg$taxon_label %in%
                    c("Lactobacillus", "Escherichia", "Roseburia", "Bacteroides")))
})

test_that("aggregate_taxa_contributions does not duplicate rows for repeated taxonomy IDs", {
  contrib <- data.frame(
    sample = "S1",
    function_id = "K00001",
    taxon = "ASV1",
    taxon_function_abun = 1,
    norm_taxon_function_contrib = 1,
    stringsAsFactors = FALSE
  )
  taxonomy <- data.frame(
    Feature.ID = c("ASV1", "ASV1"),
    Taxon = c("k__Bacteria;g__Alpha", "k__Bacteria;g__Beta"),
    check.names = FALSE,
    stringsAsFactors = FALSE
  )

  expect_warning(
    agg <- aggregate_taxa_contributions(contrib, taxonomy = taxonomy, top_n = 10),
    "Duplicate taxon IDs found"
  )
  expect_equal(nrow(agg), 1)
  expect_equal(sum(agg$contribution), 1)
})

test_that("aggregate_taxa_contributions creates 'Other' category with top_n", {
  td <- create_contrib_test_data()
  contrib <- read_contrib_file(data = td$contrib_raw)
  agg <- aggregate_taxa_contributions(contrib, top_n = 2)

  expect_true("Other" %in% agg$taxon_label)
  # Should have at most 3 unique labels (top 2 + Other)
  expect_lte(length(unique(agg$taxon_label)), 3)
})

test_that("aggregate_taxa_contributions works with strat file input", {
  td <- create_contrib_test_data()
  strat <- read_strat_file(data = td$strat_wide)
  agg <- aggregate_taxa_contributions(strat, top_n = 4)

  expect_s3_class(agg, "data.frame")
  expect_true(all(c("sample", "function_id", "taxon_label", "contribution")
                  %in% colnames(agg)))
})

test_that("aggregate_taxa_contributions with daa_results_df filters pathways", {
  skip_if_not_installed("ggpicrust2")

  td <- create_contrib_test_data()
  contrib <- read_contrib_file(data = td$contrib_raw)

  # Create mock DAA results with a pathway that maps to our KOs
  daa_results <- data.frame(
    feature = "ko00010",
    p_adjust = 0.01,
    stringsAsFactors = FALSE
  )

  # This may error if ko00010 KOs don't match our test KOs, which is expected
  # The important thing is the pathway→KO mapping logic works
  result <- tryCatch(
    aggregate_taxa_contributions(contrib, daa_results_df = daa_results),
    error = function(e) e
  )

  # Should either succeed or fail with informative message about no matching data
  if (inherits(result, "error")) {
    expect_match(result$message, "No data remaining|No significant")
  } else {
    expect_s3_class(result, "data.frame")
  }
})


# ---- taxa_contribution_bar ----

test_that("taxa_contribution_bar returns ggplot", {
  td <- create_contrib_test_data()
  contrib <- read_contrib_file(data = td$contrib_raw)
  agg <- aggregate_taxa_contributions(contrib, top_n = 4)

  p <- taxa_contribution_bar(agg, td$metadata, group = "group")
  expect_s3_class(p, "ggplot")
})

test_that("taxa_contribution_bar accepts facet_by = 'group'", {
  td <- create_contrib_test_data()
  contrib <- read_contrib_file(data = td$contrib_raw)
  agg <- aggregate_taxa_contributions(contrib, top_n = 4)

  p <- taxa_contribution_bar(agg, td$metadata, group = "group",
                             facet_by = "group")
  expect_s3_class(p, "ggplot")
})

test_that("taxa_contribution_bar supports metadata with a single group", {
  td <- create_contrib_test_data()
  contrib <- read_contrib_file(data = td$contrib_raw)
  agg <- aggregate_taxa_contributions(contrib, top_n = 4)
  td$metadata$group <- "Control"

  p <- taxa_contribution_bar(agg, td$metadata, group = "group")
  expect_s3_class(p, "ggplot")
})


# ---- taxa_contribution_heatmap ----

test_that("taxa_contribution_heatmap returns ggplot", {
  td <- create_contrib_test_data()
  contrib <- read_contrib_file(data = td$contrib_raw)
  agg <- aggregate_taxa_contributions(contrib, top_n = 4)

  p <- taxa_contribution_heatmap(agg, cluster_rows = FALSE, cluster_cols = FALSE)
  expect_s3_class(p, "ggplot")
})

test_that("taxa_contribution_heatmap relabels columns using pathway_annotation output", {
  # Regression: the heatmap looked for an `annotation_data$pathway` ID
  # column, but pathway_annotation() emits the ID in `feature` (with
  # `description` as the human-readable label). Under the old code, the
  # left-join returned all-NA names and the axis silently kept the raw
  # function IDs. Accept both `feature`/`description` (non-ko_to_kegg)
  # and `pathway`/`pathway_name` (ko_to_kegg) column naming.
  td <- create_contrib_test_data()
  contrib <- read_contrib_file(data = td$contrib_raw)
  agg <- aggregate_taxa_contributions(contrib, top_n = 4)

  ann <- data.frame(
    feature = c("K00001", "K00002", "K00003"),
    description = c("alpha desc", "beta desc", "gamma desc"),
    stringsAsFactors = FALSE
  )
  p <- taxa_contribution_heatmap(
    agg,
    annotation_data = ann,
    cluster_rows = FALSE,
    cluster_cols = FALSE
  )
  expect_s3_class(p, "ggplot")
  func_levels <- levels(p$data$func)
  # At least one of the labels should be the pathway_annotation description.
  expect_true(any(c("alpha desc", "beta desc", "gamma desc") %in% func_levels))
})

test_that("taxa_contribution_heatmap with clustering returns patchwork", {
  skip_if_not_installed("ggdendro")

  td <- create_contrib_test_data()
  contrib <- read_contrib_file(data = td$contrib_raw)
  agg <- aggregate_taxa_contributions(contrib, top_n = 4)

  p <- taxa_contribution_heatmap(agg, cluster_rows = TRUE, cluster_cols = TRUE)
  # With dendrograms, patchwork wraps the result
  expect_true(inherits(p, "ggplot") || inherits(p, "patchwork"))
})


# ---- Edge cases ----

test_that("aggregate_taxa_contributions errors on empty data after filtering", {
  td <- create_contrib_test_data()
  contrib <- read_contrib_file(data = td$contrib_raw)

  # Use a syntactically valid but non-existent pathway ID so the filter
  # reaches the "no matching KOs in contrib_data" branch rather than the
  # strict ID-shape check.
  expect_error(
    aggregate_taxa_contributions(contrib, pathway_ids = "ko99999"),
    "No data remaining"
  )
})

test_that("aggregate_taxa_contributions errors with invalid tax_level", {
  td <- create_contrib_test_data()
  contrib <- read_contrib_file(data = td$contrib_raw)

  expect_error(
    aggregate_taxa_contributions(contrib, taxonomy = td$taxonomy_qiime2,
                                 tax_level = "Superclass"),
    "Invalid tax_level"
  )
})

#' Create pathway heatmap with support for multiple grouping variables
#'
#' This function creates a heatmap of the predicted functional pathway abundance data
#' with support for single or multiple grouping variables. The function first performs
#' z-score normalization on the abundance data, then converts it to a long format and
#' orders the samples based on the grouping information. The heatmap supports nested
#' faceting for multiple grouping variables and is created using the `ggplot2` library.
#'
#' @name pathway_heatmap
#' @param abundance A matrix or data frame of pathway abundance data, where columns
#'   correspond to samples and rows correspond to pathways. Must contain at least
#'   two samples.
#' @param metadata A data frame of metadata, where each row corresponds to a sample
#'   and each column corresponds to a metadata variable.
#' @param group A character string specifying the column name in the metadata data frame
#'   that contains the primary group variable. Must contain at least two groups.
#' @param secondary_groups A character vector specifying additional grouping variables
#'   for creating nested faceted heatmaps. If NULL, only the primary group will be used.
#'   These variables will be used as secondary levels in the faceting hierarchy.
#' @param colors A vector of colors used for the background of the facet labels in the
#'   heatmap. If NULL or not provided, a default color set is used for the facet strips.
#' @param font_size A numeric value specifying the font size for the heatmap.
#' @param show_row_names A logical value indicating whether to show row names in the heatmap.
#' @param show_legend A logical value indicating whether to show the legend in the heatmap.
#' @param custom_theme A custom theme for the heatmap.
#' @param low_color A character string specifying the color for low values in the heatmap gradient. Default is "#0571b0" (blue).
#' @param mid_color A character string specifying the color for middle values in the heatmap gradient. Default is "white".
#' @param high_color A character string specifying the color for high values in the heatmap gradient. Default is "#ca0020" (red).
#' @param cluster_rows A logical value indicating whether to cluster rows (pathways). Default is FALSE.
#' @param cluster_cols A logical value indicating whether to cluster columns (samples). Default is FALSE.
#' @param clustering_method A character string specifying the clustering method. Options: "complete", "average", "single", "ward.D", "ward.D2", "mcquitty", "median", "centroid". Default is "complete".
#' @param clustering_distance A character string specifying the distance metric. Options: "euclidean", "maximum", "manhattan", "canberra", "binary", "minkowski", "correlation", "spearman". Default is "euclidean".
#' @param dendro_line_size A numeric value specifying the line width of dendrogram branches. Default is 0.5.
#' @param dendro_labels A logical value indicating whether to show dendrogram labels. Default is FALSE.
#' @param facet_by \strong{[Deprecated]} A character string specifying an additional grouping variable for creating faceted heatmaps.
#'   This parameter is deprecated and will be removed in future versions. Use \code{secondary_groups} instead.
#' @param colorbar_title A character string specifying the title for the color bar. Default is "Z Score".
#' @param colorbar_position A character string specifying the position of the color bar. Options: "right", "left", "top", "bottom". Default is "right".
#' @param colorbar_width A numeric value specifying the width of the color bar. Default is 0.6.
#' @param colorbar_height A numeric value specifying the height of the color bar. Default is 9.
#' @param colorbar_breaks An optional numeric vector specifying custom breaks for the color bar.
#'
#' @return A ggplot heatmap object representing the heatmap of the predicted functional
#'   pathway abundance data.
#' @export
#'
#' @import dplyr
#' @import ggplot2
#' @import tidyr
#' @importFrom ggh4x facet_nested strip_nested elem_list_rect
#' @importFrom stats cor as.dist dist hclust
#'
#' @examples
#' \donttest{
#' library(ggpicrust2)
#' library(ggh4x)
#' library(dplyr)
#' library(tidyr)
#' library(tibble)
#' library(magrittr)
#'
#' # Create example functional pathway abundance data
#' kegg_abundance_example <- matrix(rnorm(30), nrow = 3, ncol = 10)
#' colnames(kegg_abundance_example) <- paste0("Sample", 1:10)
#' rownames(kegg_abundance_example) <- c("PathwayA", "PathwayB", "PathwayC")
#'
#' # Create example metadata
#' metadata_example <- data.frame(
#'   sample_name = colnames(kegg_abundance_example),
#'   group = factor(rep(c("Control", "Treatment"), each = 5)),
#'   batch = factor(rep(c("Batch1", "Batch2"), times = 5))
#' )
#'
#' # Custom colors for facet strips
#' custom_colors <- c("skyblue", "salmon")
#'
#' # Example 1: Basic heatmap
#' pathway_heatmap(kegg_abundance_example, metadata_example, "group", colors = custom_colors)
#'
#' # Example 2: Heatmap with row clustering
#' pathway_heatmap(
#'   abundance = kegg_abundance_example,
#'   metadata = metadata_example,
#'   group = "group",
#'   cluster_rows = TRUE,
#'   clustering_method = "complete",
#'   clustering_distance = "euclidean",
#'   dendro_line_size = 0.8
#' )
#'
#' # Example 3: Heatmap with column clustering using correlation distance
#' pathway_heatmap(
#'   abundance = kegg_abundance_example,
#'   metadata = metadata_example,
#'   group = "group",
#'   cluster_cols = TRUE,
#'   clustering_method = "ward.D2",
#'   clustering_distance = "correlation"
#' )
#'
#' # Example 4: Multi-level grouping with secondary_groups (NEW FEATURE)
#' pathway_heatmap(
#'   abundance = kegg_abundance_example,
#'   metadata = metadata_example,
#'   group = "group",
#'   secondary_groups = "batch",
#'   colors = c("lightblue", "lightcoral", "lightgreen", "lightyellow")
#' )
#'
#' # Example 5: Custom colorbar settings
#' pathway_heatmap(
#'   abundance = kegg_abundance_example,
#'   metadata = metadata_example,
#'   group = "group",
#'   colorbar_title = "Expression Level",
#'   colorbar_position = "bottom",
#'   colorbar_width = 8,
#'   colorbar_height = 0.8,
#'   colorbar_breaks = c(-2, -1, 0, 1, 2)
#' )
#'
#' # Example 6: Advanced heatmap with clustering and custom aesthetics
#' pathway_heatmap(
#'   abundance = kegg_abundance_example,
#'   metadata = metadata_example,
#'   group = "group",
#'   cluster_rows = TRUE,
#'   cluster_cols = FALSE,  # Don't cluster columns to preserve group order
#'   clustering_method = "average",
#'   clustering_distance = "manhattan",
#'   dendro_line_size = 1.0,
#'   low_color = "#053061",     # Dark blue
#'   mid_color = "#f7f7f7",     # Light gray
#'   high_color = "#67001f",    # Dark red
#'   colorbar_title = "Z-Score",
#'   colorbar_position = "left"
#' )
#'
#' # Use real dataset
#' data("metacyc_abundance")
#' data("metadata")
#' metacyc_daa_results_df <- pathway_daa(
#'   abundance = metacyc_abundance %>% column_to_rownames("pathway"),
#'   metadata = metadata,
#'   group = "Environment",
#'   daa_method = "LinDA"
#' )
#' annotated_metacyc_daa_results_df <- pathway_annotation(
#'   pathway = "MetaCyc",
#'   daa_results_df = metacyc_daa_results_df,
#'   ko_to_kegg = FALSE
#' )
#' feature_with_p_0.05 <- metacyc_daa_results_df %>% filter(p_adjust < 0.05)
#' 
#' # Example 7: Real data with hierarchical clustering
#' pathway_heatmap(
#'   abundance = metacyc_abundance %>%
#'     right_join(
#'       annotated_metacyc_daa_results_df %>%
#'       select(all_of(c("feature","description"))),
#'       by = c("pathway" = "feature")
#'     ) %>%
#'     filter(pathway %in% feature_with_p_0.05$feature) %>%
#'     select(-"pathway") %>%
#'     filter(!is.na(description)) %>%
#'     distinct(description, .keep_all = TRUE) %>%
#'     column_to_rownames("description"),
#'   metadata = metadata,
#'   group = "Environment",
#'   cluster_rows = TRUE,
#'   clustering_method = "ward.D2",
#'   clustering_distance = "correlation",
#'   colors = custom_colors,
#'   low_color = "#2166ac",  # Custom blue for low values
#'   mid_color = "#f7f7f7",  # Light gray for mid values
#'   high_color = "#b2182b", # Custom red for high values
#'   colorbar_title = "Standardized Abundance"
#' )
#'
#' # Example 8: Multiple grouping variables (NEW FEATURE)
#' # Create extended metadata with additional grouping variables
#' metadata_extended <- metadata_example %>%
#'   mutate(
#'     sex = factor(rep(c("Male", "Female"), times = 5)),
#'     age_group = factor(rep(c("Young", "Old"), each = 5))
#'   )
#'
#' # Multi-level grouping with three variables
#' pathway_heatmap(
#'   abundance = kegg_abundance_example,
#'   metadata = metadata_extended,
#'   group = "group",                    # Primary grouping
#'   secondary_groups = c("batch", "sex"), # Secondary groupings
#'   colors = c("lightblue", "lightcoral")
#' )
#'
#' # Example 9: Migration from facet_by to secondary_groups
#' # OLD WAY (deprecated, will show warning):
#' # pathway_heatmap(abundance, metadata, group = "Environment", facet_by = "Group")
#'
#' # NEW WAY (recommended):
#' # pathway_heatmap(abundance, metadata, group = "Environment", secondary_groups = "Group")
#'
#' # Example 10: Real data with multiple grouping variables
#' pathway_heatmap(
#'   abundance = metacyc_abundance %>%
#'     right_join(
#'       annotated_metacyc_daa_results_df %>%
#'       select(all_of(c("feature","description"))),
#'       by = c("pathway" = "feature")
#'     ) %>%
#'     filter(pathway %in% feature_with_p_0.05$feature) %>%
#'     select(-"pathway") %>%
#'     filter(!is.na(description)) %>%
#'     distinct(description, .keep_all = TRUE) %>%
#'     column_to_rownames("description"),
#'   metadata = metadata,
#'   group = "Environment",              # Primary: Pro-survival vs others
#'   secondary_groups = "Group",         # Secondary: Broad Institute vs Jackson Labs
#'   cluster_rows = TRUE,
#'   clustering_method = "ward.D2",
#'   clustering_distance = "correlation"
#' )
#' }
utils::globalVariables(c("rowname","Sample","Value","quantile","facet_nested","strip_nested","elem_list_rect", "x", "y", "xend", "yend"))

#' Generate colors for nested grouping variables
#'
#' @param metadata A data frame containing metadata
#' @param all_groups A character vector of grouping variables
#' @param colors A character vector of colors or NULL
#' @return A character vector of colors appropriate for the grouping structure
#' @keywords internal
generate_nested_colors <- function(metadata, all_groups, colors = NULL) {
  # Calculate the number of colors needed
  if (length(all_groups) == 1) {
    # Single-level grouping: one color per group level
    n_colors_needed <- length(unique(metadata[[all_groups[1]]]))
  } else {
    # Multi-level grouping: each grouping level needs distinct colors
    # Sum up unique levels across all grouping variables
    # e.g., Diet(2) + Time(4) = 6 colors needed for all facet strips
    n_colors_needed <- sum(sapply(all_groups, function(g) length(unique(metadata[[g]]))))
  }

  # Default color palette
  default_colors <- c("#d93c3e", "#3685bc", "#6faa3e", "#e8a825", "#c973e6", "#ee6b3d", "#2db0a7", "#f25292")

  if (is.null(colors)) {
    # Use default colors, repeat if necessary
    if (n_colors_needed <= length(default_colors)) {
      colors <- default_colors[1:n_colors_needed]
    } else {
      # Generate additional colors using colorRampPalette
      color_func <- grDevices::colorRampPalette(default_colors)
      colors <- color_func(n_colors_needed)
    }
  } else {
    # Use provided colors, repeat if necessary
    if (length(colors) < n_colors_needed) {
      warning(paste("Not enough colors provided. Need", n_colors_needed, "colors but only", length(colors), "provided. Repeating colors."))
      colors <- rep(colors, length.out = n_colors_needed)
    } else if (length(colors) > n_colors_needed) {
      colors <- colors[1:n_colors_needed]
    }
  }

  return(colors)
}

#' Create dendrogram plot from hierarchical clustering
#'
#' @param hclust_obj An hclust object from hierarchical clustering
#' @param dendro_line_size Line width for dendrogram branches
#' @param dendro_labels Whether to show labels on dendrogram
#' @param horizontal Whether to create horizontal dendrogram
#' @return A ggplot dendrogram
#' @keywords internal
create_dendrogram <- function(hclust_obj, dendro_line_size = 0.5, dendro_labels = FALSE, horizontal = FALSE) {
  if (!requireNamespace("ggdendro", quietly = TRUE)) {
    warning("Package 'ggdendro' is required for dendrogram visualization. Skipping dendrogram.")
    return(NULL)
  }
  
  # Convert hclust to dendrogram
  dendro_data <- ggdendro::dendro_data(hclust_obj)
  
  # Create the plot
  p <- ggplot2::ggplot() +
    ggplot2::geom_segment(data = dendro_data$segments, 
                         ggplot2::aes(x = x, y = y, xend = xend, yend = yend),
                         linewidth = dendro_line_size, color = "black") +
    ggplot2::theme_void()
  
  if (dendro_labels) {
    p <- p + ggplot2::geom_text(data = dendro_data$labels, 
                               ggplot2::aes(x = x, y = y, label = label),
                               size = 3, hjust = 0.5, vjust = 1)
  }
  
  if (horizontal) {
    p <- p + ggplot2::coord_flip()
  }
  
  return(p)
}
pathway_heatmap <- function(abundance,
                            metadata,
                            group,
                            secondary_groups = NULL,
                            colors = NULL,
                            font_size = 12,
                            show_row_names = TRUE,
                            show_legend = TRUE,
                            custom_theme = NULL,
                            low_color = "#0571b0",
                            mid_color = "white",
                            high_color = "#ca0020",
                            cluster_rows = FALSE,
                            cluster_cols = FALSE,
                            clustering_method = "complete",
                            clustering_distance = "euclidean",
                            dendro_line_size = 0.5,
                            dendro_labels = FALSE,
                            facet_by = NULL,
                            colorbar_title = "Z Score",
                            colorbar_position = "right",
                            colorbar_width = 0.6,
                            colorbar_height = 9,
                            colorbar_breaks = NULL) {
  # Input validation using unified functions
  validate_abundance(abundance, min_samples = 2)
  validate_metadata(metadata)

  # Ensure abundance is a matrix with names
  abundance <- as.matrix(abundance)
  if (nrow(abundance) < 1) stop("At least one pathway is required")
  if (is.null(colnames(abundance))) colnames(abundance) <- paste0("Sample", seq_len(ncol(abundance)))
  if (is.null(rownames(abundance))) rownames(abundance) <- paste0("Pathway", seq_len(nrow(abundance)))

  # Handle deprecated facet_by parameter
 if (!is.null(facet_by)) {
    warning("'facet_by' is deprecated. Use 'secondary_groups' instead.", call. = FALSE)
    if (is.null(secondary_groups)) secondary_groups <- facet_by
  }

  # Build and validate all grouping variables
  all_groups <- c(group, secondary_groups)
  for (grp in all_groups) {
    validate_group(metadata, grp, min_groups = 2)
  }

  if (!is.null(colors) && !is.character(colors)) {
    stop("colors must be NULL or a character vector")
  }
  
  # Heatmaps use color changes to visualize changes in values. However, if the
  # data for plotting the heat map are too different, for example, if the heat
  # map is plotted using gene expression data, gene1 is expressed above 1000 in
  # all samples and gene2 is expressed between 1-10 in all samples, it is
  # difficult to plot the heat map with small changes in the expression of two
  # genes in different samples by the colors to reflect. Therefore, when
  # plotting a heat map, we usually normalize the gene expression data, that
  # is, we subtract the mean value of each gene expression from the expression
  # of this gene in all samples and divide it by its standard deviation, and
  # this normalization is called standard normalization or Z-score processing.
  # The processed values are reduced equally, and the expression of each gene
  # in all samples becomes a set of values with a mean of 0 and a standard
  # deviation of 1. At this point, the plotted heat map gives a good indication
  # of the variation in expression of all genes across samples.

  # Align samples using unified function
  aligned <- align_samples(abundance, metadata)
  abundance <- as.matrix(aligned$abundance)
  metadata <- aligned$metadata
  metadata$sample_name <- colnames(abundance)

  # Validate group column
  validate_group(metadata, group, min_groups = 2)

  # Perform z-score normalization. A constant (zero-variance) row has
  # sd = 0, so `scale()` divides by zero and returns NA for every sample.
  # The semantically correct value for "this feature is identical across
  # all samples" is 0 (right at the mean), so we coerce those NA back to
  # 0. Leaving them as NA would also crash `dist()`/`hclust()` below
  # whenever `cluster_rows = TRUE`.
  z_abundance <- t(apply(abundance, 1, scale))
  colnames(z_abundance) <- colnames(abundance)
  constant_rows <- apply(z_abundance, 1, function(x) any(!is.finite(x)))
  if (any(constant_rows)) {
    z_abundance[constant_rows, ] <- 0
    if (cluster_rows) {
      message(sprintf(
        "%d constant pathway(s) have zero variance; treated as z-score 0 for clustering.",
        sum(constant_rows)
      ))
    }
  }

  # Convert the abundance matrix to a data frame
  z_df <- as.data.frame(z_abundance)

  metadata <- metadata %>% as.data.frame()

  # Perform clustering if requested
  row_order <- rownames(z_abundance)
  col_order <- colnames(z_abundance)
  
  if (cluster_rows) {
    # Calculate distance matrix for rows
    if (clustering_distance == "correlation") {
      # Handle correlation distance specially
      cor_matrix <- cor(t(z_abundance))
      row_dist <- as.dist(1 - cor_matrix)
    } else if (clustering_distance == "spearman") {
      # Handle spearman correlation distance specially
      cor_matrix <- cor(t(z_abundance), method = "spearman")
      row_dist <- as.dist(1 - cor_matrix)
    } else {
      row_dist <- dist(z_abundance, method = clustering_distance)
    }
    row_hclust <- hclust(row_dist, method = clustering_method)
    row_order <- rownames(z_abundance)[row_hclust$order]
  }
  
  # Always prepare ordered metadata for consistent behavior
  # Order by all grouping variables hierarchically
  if (length(all_groups) == 1) {
    ordered_metadata <- metadata[order(metadata[, all_groups[1]]),]
  } else {
    # Order by multiple grouping variables
    ordered_metadata <- metadata[do.call(order, metadata[all_groups]),]
  }
  ordered_sample_names <- ordered_metadata$sample_name
  ordered_group_levels <- ordered_metadata %>% select(all_of(c(group))) %>% pull()
  
  if (cluster_cols) {
    # Calculate distance matrix for columns
    if (clustering_distance == "correlation") {
      # Handle correlation distance specially
      cor_matrix <- cor(z_abundance)
      col_dist <- as.dist(1 - cor_matrix)
    } else if (clustering_distance == "spearman") {
      # Handle spearman correlation distance specially
      cor_matrix <- cor(z_abundance, method = "spearman")
      col_dist <- as.dist(1 - cor_matrix)
    } else {
      col_dist <- dist(t(z_abundance), method = clustering_distance)
    }
    col_hclust <- hclust(col_dist, method = clustering_method)
    col_order <- colnames(z_abundance)[col_hclust$order]
  } else {
    # Order the samples based on the environment information (default behavior)
    col_order <- ordered_sample_names
  }

  # Convert the abundance data frame to a long format
  # Prepare metadata columns to join - include all grouping variables
  metadata_cols <- c("sample_name", all_groups)
  
  long_df <- z_df %>%
    tibble::rownames_to_column() %>%
    tidyr::pivot_longer(cols = -rowname,
                        names_to = "Sample",
                        values_to = "Value") %>% 
    left_join(metadata %>% select(all_of(metadata_cols)), 
              by = c("Sample" = "sample_name"))

  # Set the order of the samples and pathways in the heatmap
  long_df$Sample <- factor(long_df$Sample, levels = col_order)
  long_df$rowname <- factor(long_df$rowname, levels = row_order)

  # Compute breaks from the data
  breaks <- range(long_df$Value, na.rm = TRUE)

  # Generate appropriate colors for the grouping structure
  colors <- generate_nested_colors(metadata, all_groups, colors)

  # Create the heatmap using ggplot
  p <- ggplot2::ggplot(data = long_df,
                    mapping = ggplot2::aes(x = Sample, y = rowname, fill = Value)) +
    ggplot2::geom_tile() +
    ggplot2::scale_fill_gradient2(
      low = low_color,
      mid = mid_color,
      high = high_color,
      midpoint = 0,
      breaks = colorbar_breaks
    ) +
    ggplot2::labs(x = NULL, y = NULL) +
    ggplot2::scale_y_discrete(expand = c(0, 0), position = "left") +
    ggplot2::scale_x_discrete(expand = c(0, 0)) +
    # Customize the appearance of the heatmap
    ggplot2::theme(
      axis.text.x = ggplot2::element_blank(),
      axis.text.y = ggplot2::element_text(size = font_size, color = "black"),
      axis.ticks = ggplot2::element_blank(),
      axis.text = ggplot2::element_text(
        color = "black",
        size = 10,
        face = "bold"
      ),
      panel.spacing = unit(0, "lines"),
      legend.title = ggplot2::element_text(size = 12, color = "black",face = "bold"),
      legend.text = ggplot2::element_text(size = 12, color = "black",face = "bold"),
      panel.background = ggplot2::element_blank(),
      legend.margin = ggplot2::margin(l = 0, unit = "cm"),
      strip.text = element_text(size = 12, face = "bold")
    ) +
    # Add improved color bar to the heatmap
    ggplot2::guides(
      fill = ggplot2::guide_colorbar(
        direction = if(colorbar_position %in% c("left", "right")) "vertical" else "horizontal",
        reverse = FALSE,
        barwidth = unit(colorbar_width, "cm"),
        barheight = unit(colorbar_height, "cm"),
        title = colorbar_title,
        title.position = if(colorbar_position %in% c("left", "right")) "top" else "left",
        title.hjust = if(colorbar_position %in% c("left", "right")) 0.5 else 0,
        ticks = TRUE,
        label = TRUE
      )
    ) + 
    ggplot2::theme(legend.position = colorbar_position)

  # Add faceting support with multiple grouping levels
  if (length(all_groups) == 1) {
    # Single-level faceting
    p <- p + ggh4x::facet_nested(
      cols = vars(!!sym(all_groups[1])),
      space = "free",
      scale = "free",
      switch = "x",
      strip = ggh4x::strip_nested(
        background_x = ggh4x::elem_list_rect(fill = colors)
      )
    )
  } else {
    # Multi-level nested faceting
    p <- p + ggh4x::facet_nested(
      cols = vars(!!!syms(all_groups)),
      space = "free",
      scale = "free",
      switch = "x",
      strip = ggh4x::strip_nested(
        background_x = ggh4x::elem_list_rect(fill = colors)
      )
    )
  }

  if (!show_row_names) {
    p <- p + theme(axis.text.y = element_blank())
  }
  
  if (!show_legend) {
    p <- p + theme(legend.position = "none")
  }
  
  if (!is.null(custom_theme)) {
    p <- p + custom_theme
  }

  # Create dendrograms if clustering was performed
  if (cluster_rows || cluster_cols) {
    if (!requireNamespace("patchwork", quietly = TRUE)) {
      warning("Package 'patchwork' is required for combining plots with dendrograms. Returning heatmap only.")
      return(p)
    }
    
    # Create row dendrogram if rows were clustered
    if (cluster_rows) {
      row_dendro <- create_dendrogram(row_hclust, dendro_line_size, dendro_labels, horizontal = TRUE)
      if (!is.null(row_dendro)) {
        p <- row_dendro + p + patchwork::plot_layout(widths = c(0.2, 1))
      }
    }
    
    # Create column dendrogram if columns were clustered
    if (cluster_cols) {
      col_dendro <- create_dendrogram(col_hclust, dendro_line_size, dendro_labels, horizontal = FALSE)
      if (!is.null(col_dendro)) {
        p <- col_dendro / p + patchwork::plot_layout(heights = c(0.2, 1))
      }
    }
  }

  # Print the ordered sample names and group levels
  if (!cluster_cols) {
    message("The Sample Names in order from left to right are:")
    message(paste(ordered_sample_names, collapse = ", "))
    message("The Group Levels in order from left to right are:")
    message(paste(ordered_group_levels, collapse = ", "))
  } else {
    message("Samples ordered by hierarchical clustering (",
            clustering_method, " method, ", clustering_distance, " distance)")
    message("Column order: ", paste(col_order, collapse = ", "))
  }

  if (cluster_rows) {
    message("Pathways ordered by hierarchical clustering (",
            clustering_method, " method, ", clustering_distance, " distance)")
    message("Row order: ", paste(row_order, collapse = ", "))
  }

  return(p)
}

# Helper for compact heatmap test data
create_heatmap_test_data <- function() {
  abundance <- matrix(
    c(
      10, 20, 30, 40,
      5, 15, 25, 35,
      8, 18, 28, 38
    ),
    nrow = 3,
    byrow = TRUE
  )
  rownames(abundance) <- c("Pathway1", "Pathway2", "Pathway3")
  colnames(abundance) <- c("S1", "S2", "S3", "S4")

  metadata <- data.frame(
    sample = c("S1", "S2", "S3", "S4"),
    group = c("A", "A", "B", "B"),
    batch = c("X", "Y", "X", "Y"),
    stringsAsFactors = FALSE
  )

  list(abundance = abundance, metadata = metadata)
}

test_that("pathway_heatmap basic functionality works", {
  td <- create_heatmap_test_data()
  p <- pathway_heatmap(
    abundance = td$abundance,
    metadata = td$metadata,
    group = "group"
  )
  expect_s3_class(p, "ggplot")
})

test_that("pathway_heatmap supports secondary_groups", {
  td <- create_heatmap_test_data()
  p <- pathway_heatmap(
    abundance = td$abundance,
    metadata = td$metadata,
    group = "group",
    secondary_groups = "batch"
  )
  expect_s3_class(p, "ggplot")
})

test_that("pathway_heatmap warns for deprecated facet_by", {
  td <- create_heatmap_test_data()
  expect_warning(
    pathway_heatmap(
      abundance = td$abundance,
      metadata = td$metadata,
      group = "group",
      facet_by = "batch"
    ),
    "deprecated"
  )
})

test_that("pathway_heatmap fails fast when sample IDs do not match", {
  td <- create_heatmap_test_data()
  bad_metadata <- td$metadata
  bad_metadata$sample <- paste0("X", seq_len(nrow(bad_metadata)))

  expect_error(
    pathway_heatmap(
      abundance = td$abundance,
      metadata = bad_metadata,
      group = "group"
    ),
    "Cannot find matching sample identifiers"
  )
})

#' Perform Principal Component Analysis (PCA) on functional pathway abundance data
#'
#' This function performs PCA analysis on pathway abundance data and creates an informative visualization
#' that includes a scatter plot of the first two principal components (PC1 vs PC2) with density plots
#' for both PCs. The plot helps to visualize the clustering patterns and distribution of samples
#' across different groups.
#'
#' @param abundance A numeric matrix or data frame containing pathway abundance data.
#'        Rows represent pathways, columns represent samples.
#'        Column names must match the sample names in metadata.
#'        Values must be numeric and cannot contain missing values (NA).
#'
#' @param metadata A data frame containing sample information.
#'        Must include a column for grouping samples (specified by the 'group' parameter).
#'        Sample identifiers are auto-detected from columns named sample_name, Sample_ID,
#'        SampleID, etc., or from rownames.
#'
#' @param group A character string specifying the column name in metadata that contains
#'        group information for samples (e.g., "treatment", "condition", "group").
#'
#' @param colors Optional. A character vector of colors for different groups.
#'        Length must match the number of unique groups.
#'        If NULL, default colors will be used.
#'
#' @param show_marginal Logical. Whether to show marginal density plots for PC1 and PC2.
#'        Default is TRUE. Set to FALSE to show only the PCA scatter plot.
#'
#' @return A ggplot object showing:
#'        \itemize{
#'          \item Center: PCA scatter plot with confidence ellipses (95%)
#'          \item Top: Density plot for PC1
#'          \item Right: Density plot for PC2
#'        }
#'
#' @details
#' The function automatically aligns samples between abundance data and metadata,
#' supporting various sample identifier formats. Samples and pathways with zero
#' variance are filtered before PCA.
#'
#' @examples
#' # Create example abundance data
#' abundance_data <- matrix(rnorm(30), nrow = 3, ncol = 10)
#' colnames(abundance_data) <- paste0("Sample", 1:10)
#' rownames(abundance_data) <- c("PathwayA", "PathwayB", "PathwayC")
#'
#' # Create example metadata
#' metadata <- data.frame(
#'   sample_name = paste0("Sample", 1:10),
#'   group = factor(rep(c("Control", "Treatment"), each = 5))
#' )
#'
#' # Basic PCA plot with default colors
#' pca_plot <- pathway_pca(abundance_data, metadata, "group")
#'
#' # PCA plot with custom colors
#' pca_plot <- pathway_pca(
#'   abundance_data,
#'   metadata,
#'   "group",
#'   colors = c("blue", "red")  # One color per group
#' )
#'
#' # PCA plot without marginal density plots
#' pca_plot <- pathway_pca(
#'   abundance_data,
#'   metadata,
#'   "group",
#'   show_marginal = FALSE
#' )
#'
#' \donttest{
#' # Example with real data
#' data("metacyc_abundance")  # Load example pathway abundance data
#' data("metadata")          # Load example metadata
#'
#' # Generate PCA plot
#' # Prepare abundance data
#' abundance_data <- as.data.frame(metacyc_abundance)
#' rownames(abundance_data) <- abundance_data$pathway
#' abundance_data <- abundance_data[, -which(names(abundance_data) == "pathway")]
#' 
#' # Create PCA plot
#' pathway_pca(
#'   abundance_data,
#'   metadata,
#'   "Environment",
#'   colors = c("green", "purple")
#' )
#' }
#'
#' @importFrom magrittr %>%
#' @importFrom dplyr select
#' @importFrom stats prcomp
#' @importFrom ggplot2 ggplot aes geom_point scale_color_manual stat_ellipse
#'             labs theme_classic theme element_line element_text element_blank
#'             geom_vline geom_hline geom_density scale_fill_manual coord_flip
#' @importFrom aplot insert_top insert_right
#' @importFrom ggplotify as.ggplot
#'
#' @export
pathway_pca <- function(abundance,
                        metadata,
                        group,
                        colors = NULL,
                        show_marginal = TRUE) {
  # Input validation using unified functions
  validate_abundance(abundance, min_samples = 3)
  validate_metadata(metadata)
  validate_group(metadata, group, min_groups = 1)

  # Align samples between abundance and metadata
  aligned <- align_samples(abundance, metadata)
  abundance <- as.matrix(aligned$abundance)
  metadata <- aligned$metadata

  if (aligned$n_samples < 3) {
    stop(sprintf("PCA requires at least 3 matching samples, found %d", aligned$n_samples))
  }

  # PCA-specific: require complete numeric data
  if (!is.numeric(abundance)) {
    stop("Abundance data must contain only numeric values")
  }
  if (any(is.na(abundance))) {
    stop("Abundance matrix contains missing values (NA). PCA requires complete data.")
  }
  if (nrow(abundance) < 2) {
    stop("PCA requires at least 2 pathways (rows)")
  }

  # Filter out samples with zero variance
  sample_var <- apply(abundance, 2, var)
  zero_var_samples <- sample_var == 0
  if (any(zero_var_samples)) {
    warning(paste("Removing", sum(zero_var_samples), "sample(s) with zero variance"))
    abundance <- abundance[, !zero_var_samples, drop = FALSE]
    metadata <- metadata[!zero_var_samples, , drop = FALSE]
  }

  # Filter out pathways with zero variance
  pathway_var <- apply(abundance, 1, var)
  zero_var_pathways <- pathway_var == 0
  if (any(zero_var_pathways)) {
    warning(paste("Removing", sum(zero_var_pathways), "pathway(s) with zero variance"))
    abundance <- abundance[!zero_var_pathways, , drop = FALSE]
  }

  # Post-filter dimension checks
  if (nrow(abundance) < 2) {
    stop("After filtering, less than 2 pathways remain. PCA requires at least 2 pathways.")
  }
  if (ncol(abundance) < 3) {
    stop("After filtering, less than 3 samples remain. PCA requires at least 3 samples.")
  }

  # Convert group to factor if needed
  if (!is.factor(metadata[[group]])) {
    metadata[[group]] <- factor(metadata[[group]])
  }

  # Validate colors if provided
  if (!is.null(colors)) {
    n_groups <- length(levels(metadata[[group]]))
    if (!is.character(colors)) {
      stop("Colors must be a character vector")
    }
    if (!all(sapply(colors, function(x) tryCatch(is.matrix(col2rgb(x)), error = function(e) FALSE)))) {
      stop("Invalid color names provided")
    }
    if (length(colors) != n_groups) {
      stop(sprintf("Number of colors (%d) does not match number of groups (%d)",
                   length(colors), n_groups))
    }
  }

  # due to NSE notes in R CMD check
  PC1 = PC2 = Group = NULL
  
  # Perform PCA on the abundance data
  pca_result <- tryCatch({
    stats::prcomp(t(abundance), center = TRUE, scale = TRUE)
  }, error = function(e) {
    if (grepl("cannot rescale a constant/zero column", e$message)) {
      stop("PCA failed: Some samples or pathways have zero variance. ",
           "This has been checked earlier, but there might still be near-zero variance columns. ",
           "Consider using a more stringent filtering or transforming your data.")
    } else {
      stop("PCA failed: ", e$message)
    }
  })
  
  # Keep the first two principal components
  pca_axis <- pca_result$x[,1:2]

  # Calculate the proportion of total variance explained by each PC
  # Note: variance = sdev^2, so we need to square the standard deviations
  pca_proportion <- (pca_result$sdev[1:2]^2) / sum(pca_result$sdev^2) * 100

  # Combine the PCA results with the metadata information
  pca <- cbind(pca_axis, metadata %>% select(all_of(c(group))))
  pca$Group <- pca[,group]

  levels <- length(levels(factor(pca$Group)))

  # Set default colors if colors are not provided
  if (is.null(colors)) {
    colors <- c("#d93c3e", "#3685bc","#208A42","#89288F","#F47D2B",
                "#FEE500","#8A9FD1","#C06CAB","#E6C2DC","#90D5E4",
                "#89C75F","#F37B7D","#9983BD","#D24B27","#3BBCA8",
                "#6E4B9E","#0C727C", "#7E1416","#D8A767","#3D3D3D")[1:levels]
  }

  # Ensure the number of colors matches the number of levels in Group
  if (length(colors) != levels) {
    stop("The length of colors vector must match the number of levels in Group")
  }


  # Create a ggplot object for the PCA scatter plot
  Fig1a.taxa.pca <- ggplot2::ggplot(pca,ggplot2::aes(PC1,PC2))+
    ggplot2::geom_point(size=4,ggplot2::aes(color=Group),show.legend = T)+
    ggplot2::scale_color_manual(values=colors)+
    ggplot2::stat_ellipse(ggplot2::aes(color = Group),fill="white",geom = "polygon",
                 level=0.95,alpha = 0.01,show.legend = F)+
    ggplot2::labs(x=paste0("PC1(",round(pca_proportion[1],1),"%)"),y=paste0("PC2(",round(pca_proportion[2],1),"%)"),color = group)+
    ggplot2::theme_classic()+
    ggplot2::theme(axis.line=ggplot2::element_line(colour = "black"),
          axis.title=ggplot2::element_text(color="black",face = "bold"),
          panel.grid.major = ggplot2::element_blank(),
          panel.grid.minor = ggplot2::element_blank(),
          panel.background = ggplot2::element_blank(),
          axis.text = ggplot2::element_text(color="black",size=10,face = "bold"),
          legend.text = ggplot2::element_text(size = 16, face = "bold"),
          legend.title = ggplot2::element_text(size = 16, face = "bold"))+
    ggplot2::geom_vline(xintercept = 0, linetype = "dashed", color = "black") +
    ggplot2::geom_hline(yintercept = 0, linetype = "dashed", color = "black")

  # Marginal density for PC1.
  #
  # geom_density() maps `..density..` (a continuous value) to y, so the
  # matching position scale is `scale_y_continuous()`. The previous
  # `scale_y_discrete()` was a latent type mismatch: ggplot2 silently
  # coerced the continuous aesthetic through a discrete scale, and
  # `expand = c(0, 0.001)` ended up interpreted in discrete-category
  # units rather than the intended "0 multiplicative + small additive
  # padding" on a continuous axis. Using the continuous scale makes
  # the grammar-of-graphics contract explicit and keeps the behavior
  # stable across ggplot2 versions.
  Fig1a.taxa.pc1.density <-
    ggplot2::ggplot(pca) +
    ggplot2::geom_density(ggplot2::aes(x=PC1, group=Group, fill=Group),
                 color="black", alpha=1, position = 'identity',show.legend = F) +
    ggplot2::scale_fill_manual(values=colors) +
    ggplot2::theme_classic() +
    ggplot2::scale_y_continuous(expand = c(0, 0.001)) +
    ggplot2::labs(x=NULL, y=NULL) +
    ggplot2::theme(axis.text.x=ggplot2::element_blank(),
          axis.ticks.x = ggplot2::element_blank())

  # Marginal density for PC2 (horizontal after coord_flip()). Same
  # continuous-scale reasoning applies.
  Fig1a.taxa.pc2.density <-
    ggplot2::ggplot(pca) +
    ggplot2::geom_density(ggplot2::aes(x=PC2, group=Group, fill=Group),
                 color="black", alpha=1, position = 'identity',show.legend = F) +
    ggplot2::scale_fill_manual(values=colors) +
    ggplot2::theme_classic() +
    ggplot2::scale_y_continuous(expand = c(0, 0.001)) +
    ggplot2::labs(x=NULL, y=NULL) +
    ggplot2::theme(axis.text.y = ggplot2::element_blank(),
          axis.ticks.y = ggplot2::element_blank()) +
    ggplot2::coord_flip()

  # Return plot with or without marginal density plots
  if (show_marginal) {
    Fig1a.taxa.pca %>%
      aplot::insert_top(Fig1a.taxa.pc1.density, height = 0.3) %>%
      aplot::insert_right(Fig1a.taxa.pc2.density, width=0.3) %>%
      ggplotify::as.ggplot()
  } else {
    Fig1a.taxa.pca
  }
}

# Tests for pathway_pca function

# Helper to create test data
create_pca_test_data <- function(n_pathways = 3, n_samples = 10, n_groups = 2) {
  set.seed(123)
  test_abundance <- matrix(rnorm(n_pathways * n_samples), nrow = n_pathways, ncol = n_samples)
  colnames(test_abundance) <- paste0("Sample", 1:n_samples)
  rownames(test_abundance) <- paste0("Pathway", LETTERS[1:n_pathways])

  groups <- rep(paste0("Group", 1:n_groups), length.out = n_samples)
  test_metadata <- data.frame(
    sample_name = colnames(test_abundance),
    group = factor(groups)
  )

  list(abundance = test_abundance, metadata = test_metadata)
}

test_that("pathway_pca works with basic inputs", {
  data <- create_pca_test_data()
  result <- pathway_pca(data$abundance, data$metadata, "group")
  expect_s3_class(result, "ggplot")
})

test_that("pathway_pca works with custom colors", {
  data <- create_pca_test_data()
  result <- pathway_pca(data$abundance, data$metadata, "group", colors = c("red", "blue"))
  expect_s3_class(result, "ggplot")
})

test_that("pathway_pca works with multiple groups", {
  data <- create_pca_test_data(n_groups = 3)
  result <- pathway_pca(data$abundance, data$metadata, "group")
  expect_s3_class(result, "ggplot")
})

test_that("pathway_pca validates inputs", {
  data <- create_pca_test_data()


  # Invalid input types
  expect_error(pathway_pca(list(1,2,3), data$metadata, "group"), "must be a data frame or matrix")
  expect_error(pathway_pca(data$abundance, list(a=1), "group"), "must be a data frame")

  # Missing group column
  wrong_metadata <- data.frame(sample_name = data$metadata$sample_name, other = 1:10)
  expect_error(pathway_pca(data$abundance, wrong_metadata, "group"), "Group column.*not found")

  # NA values
  data$abundance[1,1] <- NA
  expect_error(pathway_pca(data$abundance, data$metadata, "group"), "NA|missing")
})

test_that("pathway_pca throws error with wrong color count", {
  data <- create_pca_test_data()
  expect_error(
    pathway_pca(data$abundance, data$metadata, "group", colors = c("red", "blue", "green")),
    "Number of colors"
  )
})

test_that("pathway_pca show_marginal parameter works", {
  data <- create_pca_test_data()

  result_with <- pathway_pca(data$abundance, data$metadata, "group", show_marginal = TRUE)
  result_without <- pathway_pca(data$abundance, data$metadata, "group", show_marginal = FALSE)

  expect_s3_class(result_with, "ggplot")
  expect_s3_class(result_without, "ggplot")
})

# Note: read_abundance_file() is defined in data_utils.R
# Note: load_reference_data() is defined in data_utils.R

#' Cache manager for KEGG annotations
#' @noRd
kegg_cache <- new.env(parent = emptyenv())

#' Get KEGG annotation with caching
#' @param ko_id KO identifier
#' @param organism Organism code (e.g., 'hsa' for human, 'eco' for E. coli). If NULL, the generic KO entry is retrieved.
#' @return KEGG annotation
#' @noRd
get_kegg_with_cache <- function(ko_id, organism = NULL) {
  # Create a cache key that includes the organism if specified
  cache_key <- if (!is.null(organism)) {
    paste0(organism, ":", ko_id)
  } else {
    ko_id
  }
  
  if (!exists(cache_key, envir = kegg_cache)) {
    # Add request interval
    Sys.sleep(0.1)  # 100ms delay to avoid too frequent requests
    
    # Different query strategy based on whether organism is specified
    if (!is.null(organism)) {
      # For organism-specific queries, we need a two-step process:
      # 1. Find organism-specific genes linked to this KO
      # 2. Get pathway information for those genes
      
      # Step 1: Find organism-specific genes
      organism_genes <- tryCatch({
        # Use keggLink to find genes in the specified organism that are linked to this KO
        links <- KEGGREST::keggLink(organism, ko_id)
        links
      }, error = function(e) {
        log_message(sprintf("Error finding %s genes for %s: %s", organism, ko_id, e$message), "WARN")
        NULL
      })
      
      # If we found organism-specific genes
      if (!is.null(organism_genes) && length(organism_genes) > 0) {
        # Step 2: Get pathway information for the first gene (as a representative)
        # We use the first gene as most genes linked to the same KO participate in the same pathways
        gene_id <- names(organism_genes)[1]
        
        result <- with_retry({
          KEGGREST::keggGet(gene_id)
        })
        
        # If successful, store in cache
        if (!is.null(result) && !inherits(result, "kegg_error")) {
          assign(cache_key, result, envir = kegg_cache)
        } else {
          # If we couldn't get gene info, fall back to generic KO query
          log_message(sprintf("Could not get info for %s gene %s, falling back to generic KO query", 
                             organism, gene_id), "INFO")
          
          result <- with_retry({
            KEGGREST::keggGet(ko_id)
          })
          
          if (!is.null(result) && !inherits(result, "kegg_error")) {
            assign(cache_key, result, envir = kegg_cache)
          }
        }
      } else {
        # If no organism-specific genes found, fall back to generic KO query
        log_message(sprintf("No %s genes found for %s, falling back to generic KO query", 
                           organism, ko_id), "INFO")
        
        result <- with_retry({
          KEGGREST::keggGet(ko_id)
        })
        
        if (!is.null(result) && !inherits(result, "kegg_error")) {
          assign(cache_key, result, envir = kegg_cache)
        }
      }
    } else {
      # For generic KO queries, directly query KEGG
      result <- with_retry({
        KEGGREST::keggGet(ko_id)
      })
      
      # Handle error objects
      if (inherits(result, "kegg_error")) {
        result$ko_id <- ko_id
        return(result)
      }
      
      if (!is.null(result)) {
        assign(cache_key, result, envir = kegg_cache)
      }
    }
  }
  
  # If exists in cache, retrieve it
  if (exists(cache_key, envir = kegg_cache)) {
    return(get(cache_key, envir = kegg_cache, inherits = FALSE))
  }
  
  # If not in cache, return NULL
  NULL
}

#' Retry mechanism for KEGG queries
#' @param expr Expression or function to retry
#' @param max_attempts Maximum number of retry attempts
#' @return Result or error
#' @noRd
with_retry <- function(expr, max_attempts = getOption("ggpicrust2.max_retries", 3)) {
  for (attempt in seq_len(max_attempts)) {
    result <- tryCatch({
      if (is.function(expr)) expr() else if (is.expression(expr)) eval(expr) else expr
    }, error = function(e) {
      structure(list(
        status = if (grepl("HTTP 404", e$message)) "not_found" else "failed",
        message = e$message,
        ko_id = NULL
      ), class = "kegg_error")
    })

    # Success
    if (!inherits(result, "kegg_error")) return(result)

    # 404 is permanent — retrying won't help
    if (result$status == "not_found") return(result)

    # Transient failure: log and back off before next attempt
    if (attempt < max_attempts) {
      log_message(sprintf("Attempt %d/%d failed: %s, retrying...",
                          attempt, max_attempts, result$message), "WARN")
      Sys.sleep(min(2^attempt, 30))
    }
  }

  # All retries exhausted: return last error (always a kegg_error, never NULL)
  result
}

#' Enhanced logging system
#' @noRd
log_message <- function(msg, level = "INFO") {
  if (getOption("ggpicrust2.verbose", default = TRUE)) {
    timestamp <- format(Sys.time(), "%Y-%m-%d %H:%M:%S")
    message(sprintf("[%s] %s: %s", timestamp, level, msg))
  }
}

#' Create enhanced progress bar
#' @noRd
create_progress_bar <- function(total, format = NULL) {
  if (is.null(format)) {
    format <- paste0(
      "  [:bar] :percent | Elapsed: :elapsed | ETA: :eta\n",
      "  :current/:total (:rate/sec) | :what"
    )
  }
  
  progress::progress_bar$new(
    format = format,
    total = total,
    clear = FALSE,
    width = 80
  )
}

#' Process KEGG annotations with improved error handling and caching
#' @param df Data frame with features to annotate
#' @param organism KEGG organism code (e.g., 'hsa' for human)
#' @return Annotated data frame
#' @noRd
process_kegg_annotations <- function(df, organism = NULL, p_adjust_threshold = 0.05) {
  if (nrow(df) == 0) {
    stop("Empty data frame provided for KEGG annotation")
  }

  # Filter for significant pathways
  filtered_df <- df[which(df$p_adjust < p_adjust_threshold), ]

  # Handle case when no significant pathways are found
  if (nrow(filtered_df) == 0) {
    min_p <- min(df$p_adjust, na.rm = TRUE)

    warning(
      "\n================================================================================\n",
      "NO SIGNIFICANT PATHWAYS FOUND\n",
      "================================================================================\n\n",
      sprintf("pathway_annotation() filters results by p_adjust < %g threshold.\n", p_adjust_threshold),
      "Your data contains no pathways meeting this criterion.\n\n",
      "Statistics:\n",
      "  Total features in input:       ", nrow(df), "\n",
      sprintf("  Significant features (p<%g): 0\n", p_adjust_threshold),
      "  Minimum p_adjust value:        ", sprintf("%.6f", min_p), "\n\n",
      "Possible reasons:\n",
      "  1. Insufficient statistical power (small sample size)\n",
      "  2. Low biological effect size (small differences between groups)\n",
      "  3. High biological variability in your samples\n",
      "  4. DAA method may not be suitable for your data\n\n",
      "Recommendations:\n",
      "  1. Check input data quality and normalization\n",
      "  2. Verify adequate sample size in each group\n",
      "  3. Try alternative DAA methods (LinDA, DESeq2, edgeR)\n",
      "  4. For annotation only, use ko_to_kegg = FALSE (uses local reference)\n\n",
      "Returning data with NA annotation columns.\n",
      "================================================================================\n",
      call. = FALSE
    )

    # Return original data with empty annotation columns for compatibility
    new_cols <- c("pathway_name", "pathway_description", "pathway_class", "pathway_map")
    df[new_cols] <- NA_character_

    log_message("No significant pathways found. Returning data with empty annotation columns.", "WARN")
    return(df)
  }
  
  # Initialize new columns for significant pathways
  new_cols <- c("pathway_name", "pathway_description", "pathway_class", "pathway_map")
  filtered_df[new_cols] <- NA_character_
  
  # Create progress bar
  total_features <- nrow(filtered_df)
  pb <- create_progress_bar(total_features)
  log_message("Starting KEGG annotation process")
  
  # Track statistics
  success_count <- 0
  not_found_count <- 0
  error_count <- 0
  not_found_ids <- character(0)
  error_ids <- character(0)
  
  # Process each feature
  for (i in seq_len(nrow(filtered_df))) {
    ko_id <- filtered_df$feature[i]
    
    # Update progress bar
    pb$tick(tokens = list(what = sprintf("Processing %s", ko_id)))
    
    # Get KEGG annotation
    entry <- tryCatch({
      result <- get_kegg_with_cache(ko_id, organism)
      
      # Handle error objects
      if (inherits(result, "kegg_error")) {
        if (result$status == "not_found") {
          not_found_count <- not_found_count + 1
          not_found_ids <- c(not_found_ids, ko_id)
          log_message(sprintf("KO ID %s not found in KEGG database (HTTP 404)", ko_id), "WARN")
        } else {
          error_count <- error_count + 1
          error_ids <- c(error_ids, ko_id)
          log_message(sprintf("Error querying KEGG for %s: %s", ko_id, result$message), "ERROR")
        }
        next
      }
      
      if (is.null(result)) {
        log_message(sprintf("No KEGG data found for %s", ko_id), "WARN")
        not_found_count <- not_found_count + 1
        not_found_ids <- c(not_found_ids, ko_id)
        next
      }
      
      success_count <- success_count + 1
      result
    }, error = function(e) {
      log_message(sprintf("Error processing %s: %s", ko_id, e$message), "ERROR")
      error_count <- error_count + 1
      error_ids <- c(error_ids, ko_id)
      NULL
    })
    
    # Safely extract data, check if fields exist
    if (!is.null(entry) && length(entry) > 0) {
      filtered_df$pathway_name[i] <- safe_extract(entry[[1]], "NAME", 1)

      # Prefer PATHWAY (KO entries); fall back to DESCRIPTION (pathway entries).
      if ("PATHWAY" %in% names(entry[[1]]) &&
          !is.null(entry[[1]][["PATHWAY"]]) &&
          length(entry[[1]][["PATHWAY"]]) > 0) {
        pathway_names <- as.character(entry[[1]][["PATHWAY"]])
        filtered_df$pathway_description[i] <- paste(pathway_names, collapse = "; ")
      } else if ("DESCRIPTION" %in% names(entry[[1]]) &&
                 !is.null(entry[[1]][["DESCRIPTION"]]) &&
                 length(entry[[1]][["DESCRIPTION"]]) > 0) {
        filtered_df$pathway_description[i] <- paste(as.character(entry[[1]][["DESCRIPTION"]]), collapse = "; ")
      } else {
        filtered_df$pathway_description[i] <- NA_character_
      }

      # Use CLASS field for pathway_class (contains functional classification)
      # KEGG returns CLASS, not BRITE for pathway entries
      if("CLASS" %in% names(entry[[1]]) && !is.null(entry[[1]][["CLASS"]]) && length(entry[[1]][["CLASS"]]) > 0) {
        # Extract CLASS field
        pathway_class <- as.character(entry[[1]][["CLASS"]])
        filtered_df$pathway_class[i] <- paste(pathway_class, collapse = "; ")
      } else if("BRITE" %in% names(entry[[1]]) && !is.null(entry[[1]][["BRITE"]]) && length(entry[[1]][["BRITE"]]) > 0) {
        # Fallback to BRITE if CLASS is not available
        brite_classes <- as.character(entry[[1]][["BRITE"]])
        filtered_df$pathway_class[i] <- paste(head(brite_classes, 3), collapse = "; ")
      } else {
        filtered_df$pathway_class[i] <- NA_character_
      }

      # Use PATHWAY_MAP field for pathway_map
      if("PATHWAY_MAP" %in% names(entry[[1]]) && !is.null(entry[[1]][["PATHWAY_MAP"]]) && length(entry[[1]][["PATHWAY_MAP"]]) > 0) {
        # Extract pathway map IDs (names of the PATHWAY_MAP vector)
        pathway_maps <- names(entry[[1]][["PATHWAY_MAP"]])
        filtered_df$pathway_map[i] <- paste(pathway_maps, collapse = "; ")
      } else if("PATHWAY" %in% names(entry[[1]]) && !is.null(entry[[1]][["PATHWAY"]]) && length(entry[[1]][["PATHWAY"]]) > 0) {
        # Fallback to PATHWAY field if PATHWAY_MAP is not available
        pathway_maps <- names(entry[[1]][["PATHWAY"]])
        filtered_df$pathway_map[i] <- paste(pathway_maps, collapse = "; ")
      } else {
        filtered_df$pathway_map[i] <- NA_character_
      }
    }
  }
  
  log_message("KEGG annotation process completed")
  
  # Provide summary information after processing all KO IDs
  log_message(sprintf("KEGG annotation complete: %d successful, %d not found, %d errors", 
                      success_count, not_found_count, error_count))
  
  if (not_found_count > 0) {
    if (length(not_found_ids) <= 10) {
      log_message(sprintf("KO IDs not found: %s", paste(not_found_ids, collapse = ", ")), "WARN")
    } else {
      log_message(sprintf("First 10 KO IDs not found: %s...", 
                          paste(not_found_ids[1:10], collapse = ", ")), "WARN")
    }
  }
  
  # Only throw an error when all KO IDs fail
  if (success_count == 0) {
    error_msg <- paste0(
      "\n================================================================================\n",
      "FAILED TO RETRIEVE KEGG ANNOTATIONS\n",
      "================================================================================\n\n",
      "Statistics:\n",
      "  Total features queried:  ", total_features, "\n",
      "  Successful queries:      ", success_count, "\n",
      "  Not found (HTTP 404):    ", not_found_count, "\n",
      "  Failed with errors:      ", error_count, "\n\n"
    )

    if (not_found_count > 0 && error_count == 0) {
      ko_list <- if (length(not_found_ids) <= 10) {
        paste("  ", paste(not_found_ids, collapse = ", "))
      } else {
        paste("  ", paste(not_found_ids[1:10], collapse = ", "), "...")
      }

      error_msg <- paste0(error_msg,
        "Problem: All KO IDs were not found in KEGG database (HTTP 404).\n\n",
        "Possible causes:\n",
        "  1. Invalid KO IDs in your data\n",
        "  2. KEGG database structure has changed\n",
        "  3. KO IDs are outdated or deprecated\n\n",
        "KO IDs not found:\n", ko_list, "\n\n",
        "Recommendations:\n",
        "  1. Verify your KO IDs are valid (check KEGG database)\n",
        "  2. Use ko_to_kegg = FALSE for local annotation (no KEGG API needed)\n",
        "  3. Update your PICRUSt2 reference database\n\n"
      )
    } else if (error_count > 0) {
      ko_list <- if (length(error_ids) <= 10) {
        paste("  ", paste(error_ids, collapse = ", "))
      } else {
        paste("  ", paste(error_ids[1:10], collapse = ", "), "...")
      }

      error_msg <- paste0(error_msg,
        "Problem: Network or KEGG API errors.\n\n",
        "Possible causes:\n",
        "  1. No internet connection or firewall blocking KEGG\n",
        "  2. KEGG API is down or rate-limiting your requests\n",
        "  3. Network instability (especially common in China)\n\n",
        "KO IDs with errors:\n", ko_list, "\n\n",
        "Recommendations:\n",
        "  1. Check your internet connection\n",
        "  2. Try again later (KEGG API may be temporarily down)\n",
        "  3. Use VPN if in China or behind restrictive firewall\n",
        "  4. Use ko_to_kegg = FALSE for local annotation (recommended)\n\n"
      )
    } else {
      error_msg <- paste0(error_msg,
        "Problem: Unknown error occurred.\n\n",
        "Recommendations:\n",
        "  1. Check sessionInfo() for package versions\n",
        "  2. Report this issue on GitHub with reproducible example\n",
        "  3. Use ko_to_kegg = FALSE as temporary workaround\n\n"
      )
    }

    error_msg <- paste0(error_msg, "================================================================================\n")

    stop(error_msg, call. = FALSE)
  }
  
  # Check if any annotations were found
  has_annotations <- any(!is.na(filtered_df$pathway_name))
  if (!has_annotations) {
    warning("No valid KEGG annotations found for any features")
  }
  
  filtered_df
}

#' Annotate pathways using reference data
#' @param data Data frame to annotate
#' @param pathway_type Type of pathway
#' @param ref_data Reference data
#' @return Annotated data frame
#' @noRd
annotate_pathways <- function(data, pathway_type, ref_data) {
  message("Starting pathway annotation...")
  
  # Check if data is from DAA results (check for feature column with any p-value column)
  is_daa_results <- "feature" %in% colnames(data) &&
    any(c("p_values", "p_adjust", "pvalue", "p.adjust") %in% colnames(data))
  
  # Extract features
  if (is_daa_results) {
    message("DAA results data frame is not null. Proceeding...")
    features <- data$feature
  } else {
    features <- colnames(data)[-c(1, 2)]  # Skip first two columns (typically ID and description)
  }
  
  # Match features with reference data
  # For EC pathways, try both with and without EC: prefix
  if (pathway_type == "EC") {
    # First try direct match
    matches <- match(features, ref_data$id)

    # For unmatched features, try adding EC: prefix
    unmatched <- is.na(matches)
    if (any(unmatched)) {
      features_with_prefix <- paste0("EC:", features[unmatched])
      matches_with_prefix <- match(features_with_prefix, ref_data$id)
      matches[unmatched] <- matches_with_prefix
    }
  } else {
    matches <- match(features, ref_data$id)
  }

  # Create description column
  descriptions <- rep(NA_character_, length(features))
  valid_matches <- !is.na(matches)
  if (any(valid_matches)) {
    descriptions[valid_matches] <- ref_data$description[matches[valid_matches]]
  }
  
  # Update data frame
  if (is_daa_results) {
    # For DAA results, add description column
    data$description <- descriptions
  } else {
    # For abundance data, update description column
    for (i in seq_along(features)) {
      feature_idx <- which(colnames(data) == features[i])
      if (length(feature_idx) > 0) {
        data$description[feature_idx - 2] <- descriptions[i]  # Adjust for offset
      }
    }
  }
  
  message("Pathway annotation completed.")
  data
}

#' Pathway information annotation
#'
#' @description
#' This function serves two main purposes:
#' 1. Annotating pathway information from PICRUSt2 output files.
#' 2. Annotating pathway information from the output of `pathway_daa` function, and converting KO abundance to KEGG pathway abundance when `ko_to_kegg` is set to TRUE.
#'
#' **Important**: When `ko_to_kegg = TRUE`, this function automatically filters pathways by
#' `p_adjust < p_adjust_threshold`. If no pathways meet this criterion, the function returns
#' the original data with NA annotation columns and issues a detailed warning message with
#' diagnostic information and recommendations.
#'
#' @param file A character string, the path to the PICRUSt2 output file.
#' @param pathway A character string, the type of pathway to annotate. Options are "KO", "EC", or "MetaCyc".
#' @param daa_results_df A data frame, the output from `pathway_daa` function. When `ko_to_kegg = TRUE`,
#'   must contain columns: feature, p_values, p_adjust, and method.
#' @param ko_to_kegg A logical, decide if convert KO abundance to KEGG pathway abundance. Default is FALSE.
#'   Set to TRUE when using the function for the second use case. When TRUE, queries KEGG database for
#'   pathway annotations (requires internet connection) and filters for significant pathways.
#' @param organism A character string specifying the KEGG organism code (e.g., 'hsa' for human, 'eco' for E. coli).
#'   Default is NULL, which retrieves generic KO information not specific to any organism. Only used when ko_to_kegg is TRUE.
#' @param p_adjust_threshold A numeric value specifying the significance threshold for filtering
#'   pathways when `ko_to_kegg = TRUE`. Only pathways with `p_adjust < p_adjust_threshold` will be
#'   annotated via KEGG API. Default is 0.05. Ignored when `ko_to_kegg = FALSE`.
#'
#' @return A data frame with annotated pathway information.
#'
#' If using the function for the first use case (file input), the output data frame will include:
#' \itemize{
#'   \item \code{id}: The pathway ID.
#'   \item \code{description}: The description of the pathway.
#'   \item \code{sample1, sample2, ...}: Abundance values for each sample.
#' }
#'
#' If \code{ko_to_kegg} is set to TRUE, the output data frame will also include:
#' \itemize{
#'   \item \code{pathway_name}: The name of the KEGG pathway.
#'   \item \code{pathway_description}: The description of the KEGG pathway.
#'   \item \code{pathway_class}: The class of the KEGG pathway.
#'   \item \code{pathway_map}: The KEGG pathway map ID.
#' }
#'
#' **Note**: When \code{ko_to_kegg = TRUE}, only pathways with
#' \code{p_adjust < p_adjust_threshold} are processed. If no pathways meet this criterion,
#' all annotation columns will be NA, and a detailed warning message will be issued with
#' diagnostic information.
#'
#' When \code{ko_to_kegg} is TRUE, the function queries the KEGG database for pathway information.
#' By default (organism = NULL), it retrieves generic KO information that is not specific to any organism.
#' If you are interested in organism-specific pathway information, you can specify the KEGG organism code
#' using the \code{organism} parameter.
#'
#' @examples
#' \dontrun{
#' # Example 1: Annotate pathways from PICRUSt2 output file
#' pathway_annotation(file = "path/to/picrust2_output.tsv",
#'                               pathway = "KO")
#' }
#' 
#' \dontrun{
#' # Example 2: Annotate pathways from pathway_daa output
#' # Assuming you have daa_results from pathway_daa function
#' daa_results <- pathway_daa(abundance, metadata, group = "Group")
#' annotated_results <- pathway_annotation(pathway = "KO",
#'                               daa_results_df = daa_results,
#'                               ko_to_kegg = FALSE)
#' }
#' @export
pathway_annotation <- function(file = NULL,
                             pathway = NULL,
                             daa_results_df = NULL,
                             ko_to_kegg = FALSE,
                             organism = NULL,
                             p_adjust_threshold = 0.05) {
  
  # Input validation
  if (is.null(file) && is.null(daa_results_df)) {
    stop("Please input the picrust2 output or results of pathway_daa daa_results_df")
  }
  
  if (!is.null(daa_results_df) && nrow(daa_results_df) == 0) {
    stop("Input data frame is empty")
  }
  
  # Process file input
  if (!is.null(file)) {
    abundance <- read_abundance_file(file)
    abundance <- tibble::add_column(abundance, description = NA, .after = 1)
    ref_data <- load_reference_data(pathway)
    return(annotate_pathways(abundance, pathway, ref_data))
  }
  
  # Process DAA results
  if (!is.null(daa_results_df)) {
    if (!ko_to_kegg) {
      ref_data <- load_reference_data(pathway)
      return(annotate_pathways(daa_results_df, pathway, ref_data))
    } else {
      message("KO to KEGG is set to TRUE. Proceeding with KEGG pathway annotations...")
      if (!is.null(organism)) {
        message("Using organism code: ", organism, " for species-specific pathway information.")
      } else {
        message("No organism specified. Using generic KO information across all species.")
      }
      message("Connecting to KEGG database...")
      return(process_kegg_annotations(daa_results_df, organism, p_adjust_threshold))
    }
  }
}

#' Safely Extract Elements from a List
#'
#' Safely extracts elements from a list, returning NA if the extraction fails
#'
#' @param list A list object from which to extract elements
#' @param field The name of the field to extract from the list
#' @param index The index position to extract from the field. Default is 1
#'
#' @return The extracted element if successful, NA if extraction fails
#'
#' @examples
#' # Create a sample list
#' my_list <- list(
#'   a = list(x = 1:3),
#'   b = list(y = 4:6)
#' )
#'
#' # Extract existing element
#' safe_extract(my_list, "a", 1)
#'
#' # Extract non-existing element (returns NA)
#' safe_extract(my_list, "c", 1)
#' @export
safe_extract <- function(list, field, index = 1) {
  tryCatch({
    if (is.null(list) || !field %in% names(list) || is.null(list[[field]]) || length(list[[field]]) == 0) {
      NA_character_
    } else {
      as.character(list[[field]][index])
    }
  }, error = function(e) {
    message(paste("Error in safe_extract:", e$message))
    NA_character_
  })
}

# Tests for pathway_annotation function

# Regression: file mode silently ignored ko_to_kegg and organism. A caller
# who passed file + ko_to_kegg = TRUE expected KEGG API annotations but got
# local reference annotations with no indication. Now we warn.
test_that("pathway_annotation warns when file mode receives KEGG-specific params", {
  temp_file <- tempfile(fileext = ".tsv")
  test_data <- data.frame(
    feature = c("K00001", "K00002"),
    sample1 = c(1, 2),
    sample2 = c(3, 4)
  )
  suppressMessages(write.table(test_data, temp_file, sep = "\t", row.names = FALSE))
  on.exit(unlink(temp_file))

  expect_warning(
    suppressMessages(pathway_annotation(file = temp_file, pathway = "KO", ko_to_kegg = TRUE)),
    "ko_to_kegg is ignored in file mode"
  )
  expect_warning(
    suppressMessages(pathway_annotation(file = temp_file, pathway = "KO", organism = "hsa")),
    "organism is ignored in file mode"
  )
})

test_that("pathway_annotation basic functionality works", {
  temp_file <- tempfile(fileext = ".tsv")
  test_data <- data.frame(
    feature = c("K00001", "K00002"),
    sample1 = c(1, 2),
    sample2 = c(3, 4)
  )
  suppressMessages(write.table(test_data, temp_file, sep = "\t", row.names = FALSE))

  result <- suppressMessages(pathway_annotation(file = temp_file, pathway = "KO", ko_to_kegg = FALSE))

  expect_s3_class(result, "data.frame")
  expect_true("description" %in% colnames(result))
  expect_equal(nrow(result), 2)

  # Regression: the file-mode branch previously took sample column names as
  # features, so `description` came back all-NA even for well-known KOs.
  # After the fix, rows whose feature IDs live in the reference data should
  # carry non-NA descriptions.
  expect_true(any(!is.na(result$description)))

  unlink(temp_file)
})

test_that("pathway_annotation works with daa_results_df input", {
  test_daa_df <- data.frame(
    feature = c("K00001", "K00002"),
    p_adjust = c(0.04, 0.03),
    stringsAsFactors = FALSE
  )

  result <- suppressMessages(pathway_annotation(pathway = "KO", daa_results_df = test_daa_df, ko_to_kegg = FALSE))

  expect_s3_class(result, "data.frame")
  expect_true("description" %in% colnames(result))
  expect_equal(nrow(result), 2)
})

test_that("pathway_annotation works with ko_to_kegg", {
  skip_if(
    Sys.getenv("GGPICRUST2_RUN_NETWORK_TESTS", "false") != "true",
    "Set GGPICRUST2_RUN_NETWORK_TESTS=true to run network-dependent KEGG tests."
  )
  skip_if_offline()

  test_daa_df <- data.frame(
    feature = c("K00001", "K00002"),
    p_adjust = c(0.04, 0.03),
    stringsAsFactors = FALSE
  )

  result <- suppressMessages(pathway_annotation(pathway = "KO", daa_results_df = test_daa_df, ko_to_kegg = TRUE))

  expect_s3_class(result, "data.frame")
  expect_true(all(c("pathway_name", "pathway_class") %in% colnames(result)))
})

test_that("pathway_annotation fills description/class/map for pathway IDs with ko_to_kegg", {
  skip_if(
    Sys.getenv("GGPICRUST2_RUN_NETWORK_TESTS", "false") != "true",
    "Set GGPICRUST2_RUN_NETWORK_TESTS=true to run network-dependent KEGG tests."
  )
  skip_if_offline()

  # KEGG pathway IDs from typical ko2kegg_abundance output
  test_daa_df <- data.frame(
    feature = c("ko05340", "ko00564"),
    p_adjust = c(0.01, 0.02),
    stringsAsFactors = FALSE
  )

  result <- suppressMessages(pathway_annotation(pathway = "KO", daa_results_df = test_daa_df, ko_to_kegg = TRUE))

  expect_s3_class(result, "data.frame")
  expect_true(all(c("pathway_name", "pathway_description", "pathway_class", "pathway_map") %in% colnames(result)))
  # pathway_description is optional for some pathway entries in KEGG;
  # regression check is that it is no longer uniformly NA.
  expect_true(any(!is.na(result$pathway_description)))
  expect_true(any(!is.na(result$pathway_class)))
  expect_true(any(!is.na(result$pathway_map)))
})

test_that("pathway_annotation works with all pathway types", {
  # Each annotator gets its own correctly-typed features
  ko_df <- data.frame(feature = c("K00001", "K00002"), p_adjust = c(.04, .03), stringsAsFactors = FALSE)
  ec_df <- data.frame(feature = c("EC:1.1.1.1", "EC:2.7.1.1"), p_adjust = c(.04, .03), stringsAsFactors = FALSE)
  metacyc_df <- data.frame(feature = c("PWY-7219", "GLYCOLYSIS"), p_adjust = c(.04, .03), stringsAsFactors = FALSE)

  ko_result <- suppressMessages(pathway_annotation(pathway = "KO", daa_results_df = ko_df, ko_to_kegg = FALSE))
  expect_s3_class(ko_result, "data.frame")

  ec_result <- suppressMessages(pathway_annotation(pathway = "EC", daa_results_df = ec_df, ko_to_kegg = FALSE))
  expect_s3_class(ec_result, "data.frame")

  metacyc_result <- suppressMessages(pathway_annotation(pathway = "MetaCyc", daa_results_df = metacyc_df, ko_to_kegg = FALSE))
  expect_s3_class(metacyc_result, "data.frame")
})

test_that("pathway_annotation ko_to_kegg returns every input row (merges back)", {
  # Regression: process_kegg_annotations() used to return only the
  # p_adjust < threshold subset, silently dropping non-significant
  # features that downstream code (e.g. ggpicrust2()'s
  # plot_result_list$daa_results_df) expected to still be present.
  # Annotations should be populated for significant rows and NA for
  # the rest, but every row must survive the call.
  cache <- getFromNamespace("kegg_cache", "ggpicrust2")

  # Pre-populate the cache with fake KEGG entries so the function never
  # touches the network.
  fake_entry <- function(name, path_id, path_desc) {
    pathway_vec <- stats::setNames(path_desc, path_id)
    map_vec <- stats::setNames("map_title", path_id)
    list(list(
      NAME = name,
      PATHWAY = pathway_vec,
      PATHWAY_MAP = map_vec,
      CLASS = "Metabolism; Carbohydrate metabolism"
    ))
  }
  assign("K00001", fake_entry("alcohol dehydrogenase", "ko00010", "Glycolysis"), envir = cache)
  assign("K00002", fake_entry("alcohol dehydrogenase NADP+", "ko00010", "Glycolysis"), envir = cache)
  on.exit({
    if (exists("K00001", envir = cache)) rm("K00001", envir = cache)
    if (exists("K00002", envir = cache)) rm("K00002", envir = cache)
    if (exists("K99999", envir = cache)) rm("K99999", envir = cache)
  }, add = TRUE)
  assign("K99999", fake_entry("non-significant placeholder", "ko99999", "Other"),
         envir = cache)

  daa_df <- data.frame(
    feature  = c("K00001", "K00002", "K99999"),
    p_values = c(0.001, 0.002, 0.4),
    p_adjust = c(0.01, 0.02, 0.9),
    method   = "ALDEx2",
    stringsAsFactors = FALSE
  )

  result <- suppressMessages(suppressWarnings(
    pathway_annotation(pathway = "KO", daa_results_df = daa_df,
                       ko_to_kegg = TRUE, p_adjust_threshold = 0.05)
  ))

  # All input rows preserved.
  expect_equal(nrow(result), 3)
  expect_setequal(result$feature, c("K00001", "K00002", "K99999"))

  # Significant rows carry annotations.
  sig_rows <- result[result$feature %in% c("K00001", "K00002"), , drop = FALSE]
  expect_true(all(!is.na(sig_rows$pathway_name)))

  # Non-significant row has NA annotation columns (filtered out of the
  # annotation step, but still present as a row).
  ns_row <- result[result$feature == "K99999", , drop = FALSE]
  expect_true(is.na(ns_row$pathway_name))
})

test_that("pathway_annotation validates inputs", {
  expect_error(pathway_annotation(), "Please input")

  # Test invalid pathway type through daa_results_df path (avoids file-existence check)
  test_df <- data.frame(feature = "K00001", p_adjust = 0.04, stringsAsFactors = FALSE)
  expect_error(
    suppressMessages(pathway_annotation(pathway = "INVALID", daa_results_df = test_df, ko_to_kegg = FALSE)),
    "Unknown reference type"
  )

  empty_df <- data.frame(feature = character(0), p_adjust = numeric(0))
  expect_error(suppressMessages(pathway_annotation(pathway = "KO", daa_results_df = empty_df)), "empty")
})

# Regression: ko_to_kegg = TRUE used to silently ignore `pathway`, so a caller
# that passed pathway = "EC" or "MetaCyc" together with ko_to_kegg = TRUE was
# quietly routed into the KEGG (KO-only) code path. That surfaced either as a
# generic HTTP 404 or as mis-shaped enzyme records. The parameter combination
# is semantically contradictory and should fail at the function boundary.
test_that("pathway_annotation rejects ko_to_kegg = TRUE with non-KO pathway", {
  test_df <- data.frame(
    feature = c("EC:1.1.1.1", "EC:2.7.1.1"),
    p_adjust = c(0.01, 0.02),
    stringsAsFactors = FALSE
  )

  expect_error(
    pathway_annotation(pathway = "EC", daa_results_df = test_df, ko_to_kegg = TRUE),
    "ko_to_kegg = TRUE requires pathway = 'KO'"
  )
  expect_error(
    pathway_annotation(pathway = "MetaCyc", daa_results_df = test_df, ko_to_kegg = TRUE),
    "ko_to_kegg = TRUE requires pathway = 'KO'"
  )
})

test_that("pathway_annotation allows ko_to_kegg = TRUE with pathway = 'KO' or NULL", {
  # Both forms are the legitimate KO-to-KEGG use case. We only exercise input
  # validation here (network calls are gated by GGPICRUST2_RUN_NETWORK_TESTS),
  # so wrap the network hop in tryCatch and assert we get past the parameter
  # check.
  test_df <- data.frame(
    feature = c("K00001"),
    p_adjust = c(0.01),
    stringsAsFactors = FALSE
  )

  past_check <- function(expr) {
    result <- tryCatch(expr, error = function(e) e)
    # If validation rejected the combination it would raise the KO-only
    # message; any other error means we made it past the boundary check.
    if (inherits(result, "error")) {
      expect_false(grepl("ko_to_kegg = TRUE requires pathway = 'KO'",
                         conditionMessage(result), fixed = TRUE))
    }
  }

  past_check(suppressMessages(
    pathway_annotation(pathway = "KO", daa_results_df = test_df, ko_to_kegg = TRUE)
  ))
  past_check(suppressMessages(
    pathway_annotation(pathway = NULL, daa_results_df = test_df, ko_to_kegg = TRUE)
  ))
})

#' Differentially Abundant Analysis Results with Annotation
#'
#' This is a result dataset after processing 'kegg_abundance' through
#' the 'pathway_daa' with the LinDA method and further annotation with 'pathway_annotation'.
#'
#' @format A data frame with 10 variables:
#' \describe{
#'   \item{adj_method}{Method used for adjusting p-values.}
#'   \item{feature}{Feature being tested.}
#'   \item{group1}{One group in the comparison.}
#'   \item{group2}{The other group in the comparison.}
#'   \item{method}{Statistical test used.}
#'   \item{p_adjust}{Adjusted p-value.}
#'   \item{p_values}{P-values from the statistical test.}
#'   \item{pathway_class}{Class of the pathway.}
#'   \item{pathway_description}{Description of the pathway.}
#'   \item{pathway_map}{Map of the pathway.}
#'   \item{pathway_name}{Name of the pathway.}
#' }
#' @source From ggpicrust2 package demonstration.
#' @references Douglas GM, Maffei VJ, Zaneveld J, Yurgel SN, Brown JR, Taylor CM, Huttenhower C, Langille MGI. PICRUSt2 for prediction of metagenome functions. Nat Biotechnol. 2020.
"daa_annotated_results_df"

#' The function pathway_errorbar() is used to visualize the results of functional pathway differential abundance analysis as error bar plots.
#'
#' @name pathway_errorbar
#' @param abundance A data frame with row names representing pathways and column names representing samples. Each element represents the relative abundance of the corresponding pathway in the corresponding sample.
#' @param daa_results_df A data frame containing the results of the differential abundance analysis of the pathways, generated by the pathway_daa function. x_lab should be a column name of daa_results_df.
#' @param Group A data frame or a vector that assigns each sample to a group. The groups are used to color the samples in the figure.
#' @param ko_to_kegg A logical parameter indicating whether there was a convertion that convert ko abundance to kegg abundance.
#' @param p_values_threshold A numeric parameter specifying the threshold for statistical significance of differential abundance. Pathways with p-values below this threshold will be considered significant.
#' @param order A parameter controlling the ordering of the rows in the figure. The options are: "p_values" (order by p-values), "name" (order by pathway name), "group" (order by the group with the highest mean relative abundance), or "pathway_class" (order by the pathway category).
#' @param select A vector of pathway names to be included in the figure. This can be used to limit the number of pathways displayed. If NULL, all pathways will be displayed.
#' @param p_value_bar A logical parameter indicating whether to display a bar showing the p-value threshold for significance. If TRUE, the bar will be displayed.
#' @param colors A vector of colors to be used to represent the groups in the figure. Each color corresponds to a group. If NULL, colors will be selected based on the color_theme.
#' @param x_lab A character string to be used as the x-axis label in the figure. The default value is "description" for KOs'descriptions and "pathway_name" for KEGG pathway names.
#' @param log2_fold_change_color A character string specifying the color for log2 fold change bars. Default is "#87ceeb" (light blue). Can also be "auto" to use theme-based colors.
#' @param max_features A numeric parameter specifying the maximum number of features to display before issuing a warning. Default is 30. Set to a higher value to display more features, or Inf to disable the limit entirely.
#' @param color_theme A character string specifying the color theme to use. Options include: "default", "nature", "science", "cell", "nejm", "lancet", "colorblind_friendly", "viridis", "plasma", "minimal", "high_contrast", "pastel", "bold". Default is "default".
#' @param pathway_class_colors A vector of colors for pathway class annotations. If NULL, colors will be selected from the theme.
#' @param smart_colors A logical parameter indicating whether to use intelligent color selection based on data characteristics. Default is FALSE.
#' @param accessibility_mode A logical parameter indicating whether to use accessibility-friendly colors. Default is FALSE.
#' @param legend_position A character string specifying legend position. Options: "top", "bottom", "left", "right", "none". Default is "top".
#' @param legend_direction A character string specifying legend direction. Options: "horizontal", "vertical". Default is "horizontal".
#' @param legend_title A character string for legend title. If NULL, no title is displayed.
#' @param legend_title_size A numeric value specifying legend title font size. Default is 12.
#' @param legend_text_size A numeric value specifying legend text font size. Default is 10.
#' @param legend_key_size A numeric value specifying legend key size in cm. Default is 0.8.
#' @param legend_ncol A numeric value specifying number of columns in legend. If NULL, automatic layout is used.
#' @param legend_nrow A numeric value specifying number of rows in legend. If NULL, automatic layout is used.
#' @param pvalue_format A character string specifying p-value format. Options: "numeric", "scientific", "smart", "stars_only", "combined". Default is "smart".
#' @param pvalue_stars A logical parameter indicating whether to display significance stars. Default is TRUE.
#' @param pvalue_colors A logical parameter indicating whether to use color coding for significance levels. Default is FALSE.
#' @param pvalue_size A numeric value or "auto" for p-value text size. Default is "auto".
#' @param pvalue_angle A numeric value specifying p-value text angle in degrees. Default is 0.
#' @param pvalue_thresholds A numeric vector of significance thresholds. Default is c(0.001, 0.01, 0.05).
#' @param pvalue_star_symbols A character vector of star symbols for significance levels. Default is c("***", "**", "*").
#' @param pathway_class_text_size A numeric value or "auto" for pathway class text size. Default is "auto".
#' @param pathway_class_text_color A character string for pathway class text color. Use "auto" for theme-based color. Default is "black".
#' @param pathway_class_text_face A character string for pathway class text face. Options: "plain", "bold", "italic". Default is "bold".
#' @param pathway_class_text_angle A numeric value specifying pathway class text angle in degrees. Default is 0.
#' @param pathway_class_position A character string specifying pathway class position. Options: "left", "right", "none". Default is "left".
#' @param pathway_names_text_size A numeric value or "auto" for pathway names (y-axis labels) text size. Default is "auto".
#' @importFrom stats sd
#' @return A ggplot2 plot showing the error bar plot of the differential abundance analysis results for the functional pathways.
#' The plot visualizes the differential abundance results of a specific differential abundance analysis method. The corresponding dataframe contains the results used to create the plot.
#' @export
#'
#' @examples
#' \dontrun{
#' # Example 1: Analyzing KEGG pathway abundance
#' metadata <- read_delim(
#'   "path/to/your/metadata.txt",
#'   delim = "\t",
#'   escape_double = FALSE,
#'   trim_ws = TRUE
#' )
#'
#' # data(metadata)
#'
#' kegg_abundance <- ko2kegg_abundance(
#'   "path/to/your/pred_metagenome_unstrat.tsv"
#' )
#'
#' # data(kegg_abundance)
#'
#' # Please change group to "your_group_column" if you are not using example dataset
#' group <- "Environment"
#'
#' daa_results_df <- pathway_daa(
#'   abundance = kegg_abundance,
#'   metadata = metadata,
#'   group = group,
#'   daa_method = "ALDEx2",
#'   select = NULL,
#'   reference = NULL
#' )
#'
#' # Please check the unique(daa_results_df$method) and choose one
#' daa_sub_method_results_df <- daa_results_df[daa_results_df$method
#' == "ALDEx2_Welch's t test", ]
#'
#' daa_annotated_sub_method_results_df <- pathway_annotation(
#'   pathway = "KO",
#'   daa_results_df = daa_sub_method_results_df,
#'   ko_to_kegg = TRUE
#' )
#'
#' # Please change Group to metadata$your_group_column if you are not using example dataset
#' Group <- metadata$Environment
#'
#' p <- pathway_errorbar(
#'   abundance = kegg_abundance,
#'   daa_results_df = daa_annotated_sub_method_results_df,
#'   Group = Group,
#'   p_values_threshold = 0.05,
#'   order = "pathway_class",
#'   select = daa_annotated_sub_method_results_df %>%
#'   arrange(p_adjust) %>%
#'   slice(1:20) %>%
#'   select("feature") %>% pull(),
#'   ko_to_kegg = TRUE,
#'   p_value_bar = TRUE,
#'   colors = NULL,
#'   x_lab = "pathway_name",
#'   log2_fold_change_color = "#FF5733" # Custom color for log2 fold change bars
#' )
#'
#' # Example 2: Analyzing EC, MetaCyc, KO without conversions
#' metadata <- read_delim(
#'   "path/to/your/metadata.txt",
#'   delim = "\t",
#'   escape_double = FALSE,
#'   trim_ws = TRUE
#' )
#' # data(metadata)
#'
#' metacyc_abundance <- read.delim("path/to/your/metacyc_abundance.tsv")
#'
#' # data(metacyc_abundance)
#'
#' group <- "Environment"
#'
#' daa_results_df <- pathway_daa(
#'   abundance = metacyc_abundance %>% column_to_rownames("pathway"),
#'   metadata = metadata,
#'   group = group,
#'   daa_method = "LinDA",
#'   select = NULL,
#'   reference = NULL
#' )
#'
#'
#' daa_annotated_results_df <- pathway_annotation(
#'   pathway = "MetaCyc",
#'   daa_results_df = daa_results_df,
#'   ko_to_kegg = FALSE
#' )
#'
#' Group <- metadata$Environment
#'
#' p <- pathway_errorbar(
#'   abundance = metacyc_abundance %>% column_to_rownames("pathway"),
#'   daa_results_df = daa_annotated_results_df,
#'   Group = Group,
#'   p_values_threshold = 0.05,
#'   order = "group",
#'   select = NULL,
#'   ko_to_kegg = FALSE,
#'   p_value_bar = TRUE,
#'   colors = NULL,
#'   x_lab = "description",
#'   log2_fold_change_color = "#006400" # Dark green for log2 fold change bars
#' )
#' }
utils::globalVariables(c("group", "name", "value", "feature", "negative_log10_p", "group_nonsense", "nonsense", "pathway_class", "p_adjust", "log2_fold_change", "transform_sample_counts", "column_to_rownames", "txtProgressBar", "setTxtProgressBar", "utils"))
pathway_errorbar <-
  function(abundance,
           daa_results_df,
           Group,
           ko_to_kegg = FALSE,
           p_values_threshold = 0.05,
           order = "group",
           select = NULL,
           p_value_bar = TRUE,
           colors = NULL,
           x_lab = NULL,
           log2_fold_change_color = "#87ceeb",
           max_features = 30,
           color_theme = "default",
           pathway_class_colors = NULL,
           smart_colors = FALSE,
           accessibility_mode = FALSE,
           # New legend control parameters
           legend_position = "top",
           legend_direction = "horizontal",
           legend_title = NULL,
           legend_title_size = 12,
           legend_text_size = 10,
           legend_key_size = 0.8,
           legend_ncol = NULL,
           legend_nrow = NULL,
           # P-value display parameters
           pvalue_format = "numeric",
           pvalue_stars = TRUE,
           pvalue_colors = FALSE,
           pvalue_size = "auto",
           pvalue_angle = 0,
           pvalue_thresholds = c(0.001, 0.01, 0.05),
           pvalue_star_symbols = c("***", "**", "*"),
           # Pathway class annotation parameters
           pathway_class_text_size = "auto",
           pathway_class_text_color = "black",
           pathway_class_text_face = "bold",
           pathway_class_text_angle = 0,
           pathway_class_position = "right",
           # Pathway names text size parameter
           pathway_names_text_size = "auto") {
    # Input validation using unified functions
    validate_abundance(abundance)
    validate_dataframe(daa_results_df,
                       required_cols = c("feature", "method", "group1", "group2", "p_adjust"),
                       param_name = "daa_results_df")

    if (length(Group) != ncol(abundance)) {
      stop("Length of Group must match number of columns in abundance matrix")
    }

    # Check the number of significant features
    # Calculate number of significant features (for reference)
    # sig_features <- sum(daa_results_df$p_adjust < 0.05)

    # Identify pathways with missing annotation
    missing_pathways <- daa_results_df[is.na(daa_results_df$pathway_name), "feature"]

    # Inform the user about the missing annotations
    if(length(missing_pathways) > 0) {
      message(sprintf("Excluded %d pathways with missing annotations. Use 'pathway_annotation' to add them.",
                      length(missing_pathways)))
    }

    # Get the names of all columns in the data frame
    column_names <- colnames(daa_results_df)

    # Check if there are more than two 'group' columns
    group_columns <- grepl("^group", column_names)
    if (sum(group_columns) > 2) {
      p_value_bar <- FALSE
      message(
        "There are more than two 'group' columns in the 'daa_results_df' data frame. As a result, it is not possible to compute the log2 fold values. The 'p_value_bar' has been automatically set to FALSE."
      )
    }

    # Set x_lab if not provided
    if (is.null(x_lab)){
      if (ko_to_kegg == TRUE){
        x_lab <- "pathway_name"
      }else{
        x_lab <- "description"
      }

      if (is.null(daa_results_df$pathway_name) && is.null(daa_results_df$description)){
        stop("No annotation columns found. Use 'pathway_annotation' to annotate daa_results_df first.")
      }
    }

    if (!(x_lab %in% colnames(daa_results_df))){
      stop(sprintf("Column '%s' not found in daa_results_df. Use 'pathway_annotation' to add annotations.", x_lab))
    }

    # Exclude rows with missing pathway annotation (after x_lab is set)
    original_rows <- nrow(daa_results_df)
    daa_results_df <- daa_results_df[!is.na(daa_results_df[,x_lab]),]
    filtered_rows <- nrow(daa_results_df)
    
    # Check if all rows were filtered out due to missing annotations
    if (filtered_rows == 0) {
      warning(
        sprintf("All %d rows were excluded due to missing '%s' annotations. ", original_rows, x_lab),
        "This typically occurs when no statistically significant pathways were found. ",
        "Cannot create error bar plot with empty data. Returning NULL.",
        call. = FALSE
      )
      return(NULL)
    }
    
    if (original_rows > filtered_rows) {
      message(sprintf("Excluded %d rows with missing '%s' annotations.", original_rows - filtered_rows, x_lab))
    }

    validate_daa_results(daa_results_df)

    # Enhanced color selection using the new color theme system
    n_groups <- nlevels(as.factor(Group))

    # Use smart color selection if requested
    if (smart_colors || accessibility_mode) {
      smart_selection <- smart_color_selection(
        n_groups = n_groups,
        has_pathway_class = ko_to_kegg,
        data_type = "abundance",
        accessibility_mode = accessibility_mode
      )
      color_theme <- smart_selection$theme_name
      if (smart_colors) {
        message("Smart color selection: ", smart_selection$reason)
      }
    }
    
    # Get the color theme
    theme_colors <- get_color_theme(color_theme, n_groups)
    
    # Set group colors
    if (is.null(colors)) {
      colors <- theme_colors$group_colors[1:n_groups]
    }
    
    # Set pathway class colors if not provided
    if (is.null(pathway_class_colors)) {
      pathway_class_colors <- theme_colors$pathway_class_colors
    }
    
    # Set log2 fold change color
    if (log2_fold_change_color == "auto") {
      log2_fold_change_color <- theme_colors$fold_change_single
    }

    errorbar_abundance_mat <- as.matrix(abundance)

    daa_results_filtered_df <-
      daa_results_df[which(daa_results_df$p_adjust < p_values_threshold),]

    if (!is.null(select)) {
      daa_results_filtered_sub_df <-
        daa_results_filtered_df[daa_results_filtered_df$feature %in% select, ]
    } else {
      daa_results_filtered_sub_df <- daa_results_filtered_df
    }

    if (nrow(daa_results_filtered_sub_df) > max_features) {
      warning(
        sprintf("Found %d significant features (max: %d). Use 'select' to filter or set max_features=Inf.",
                nrow(daa_results_filtered_sub_df), max_features),
        call. = FALSE
      )
    }

    if (nrow(daa_results_filtered_sub_df) == 0){
      stop(
        "Visualization with 'pathway_errorbar' cannot be performed because there are no features with statistical significance. ",
        "For possible solutions, please check the FAQ section of the tutorial."
      )
    }
    # Convert to relative abundance
    relative_abundance_mat <- apply(t(errorbar_abundance_mat), 1, function(x)
      x / sum(x))

    # Subset to only include the features present in daa_results_filtered_sub_df$feature
    sub_relative_abundance_mat <- relative_abundance_mat[rownames(relative_abundance_mat) %in% daa_results_filtered_sub_df$feature, , drop = FALSE]

    # Create a matrix for the error bars
    # Create a mapping from sample names to groups.
    # Prefer names(Group) when available to avoid order-dependent mismatches.
    if (!is.null(names(Group)) &&
        length(names(Group)) == length(Group) &&
        all(!is.na(names(Group)))) {
      sample_to_group_map <- stats::setNames(as.character(Group), names(Group))
    } else {
      sample_to_group_map <- stats::setNames(as.character(Group), colnames(abundance))
    }
    
    # Get groups for the current abundance columns.
    correct_groups <- sample_to_group_map[colnames(sub_relative_abundance_mat)]
    if (any(is.na(correct_groups))) {
      missing_samples <- colnames(sub_relative_abundance_mat)[is.na(correct_groups)]
      stop(
        "Could not map Group values to abundance samples. ",
        "Missing sample name(s): ",
        paste(missing_samples, collapse = ", "),
        ". Ensure Group is in the same order as abundance columns ",
        "or provide names(Group) matching sample IDs."
      )
    }
    
    # Create the error bar matrix using the correctly mapped groups
    error_bar_matrix <- cbind(
      sample = colnames(sub_relative_abundance_mat),
      group = correct_groups,
      t(sub_relative_abundance_mat)
    )
    error_bar_df <- as.data.frame(error_bar_matrix)

    # Fix for duplicate factor levels issue
    Group_levels <- unique(as.character(Group))
    error_bar_df$group <- factor(correct_groups, levels = Group_levels)

      error_bar_pivot_longer_df <- tidyr::pivot_longer(error_bar_df,-c(sample, group))

    error_bar_pivot_longer_tibble <-
      mutate(error_bar_pivot_longer_df, group = as.factor(group))

    error_bar_pivot_longer_tibble$sample <-
      factor(error_bar_pivot_longer_tibble$sample)

    error_bar_pivot_longer_tibble$name <-
      factor(error_bar_pivot_longer_tibble$name)

    error_bar_pivot_longer_tibble$value <-
      as.numeric(error_bar_pivot_longer_tibble$value)

    error_bar_pivot_longer_tibble_summarised <-
      error_bar_pivot_longer_tibble %>% group_by(name, group) %>%
      summarise(mean = mean(value), sd = stats::sd(value))

    error_bar_pivot_longer_tibble_summarised <-
      error_bar_pivot_longer_tibble_summarised %>% mutate(group2 = "nonsense")

    # When ko_to_kegg = TRUE, validate pathway_class column exists
    # This is required for proper alignment of pathway class annotations
    if (ko_to_kegg == TRUE && !"pathway_class" %in% colnames(daa_results_filtered_sub_df)) {
      stop(
        "The 'pathway_class' column is missing but ko_to_kegg = TRUE. ",
        "Please use pathway_annotation(..., ko_to_kegg = TRUE) to annotate the data, ",
        "or set ko_to_kegg = FALSE if you don't need pathway class annotations."
      )
    }

    switch(
      order,
      "p_values" = {
        if (ko_to_kegg == TRUE) {
          # Nested sorting: first by pathway_class, then by p_adjust within each class
          # This ensures pathway class color blocks align correctly
          order <- order(
            daa_results_filtered_sub_df$pathway_class,
            daa_results_filtered_sub_df$p_adjust
          )
        } else {
          order <- order(daa_results_filtered_sub_df$p_adjust)
        }
      },
      "name" = {
        if (ko_to_kegg == TRUE) {
          # Nested sorting: first by pathway_class, then by feature name within each class
          order <- order(
            daa_results_filtered_sub_df$pathway_class,
            daa_results_filtered_sub_df$feature
          )
        } else {
          order <- order(daa_results_filtered_sub_df$feature)
        }
      },
      "group" = {
        # Initialize pro column with default value 1
        daa_results_filtered_sub_df$pro <- 1

        for (i in levels(error_bar_pivot_longer_tibble_summarised$name)) {
          # Get subset for current feature
          error_bar_pivot_longer_tibble_summarised_sub <-
            error_bar_pivot_longer_tibble_summarised[error_bar_pivot_longer_tibble_summarised$name == i,]

          # Find group with maximum mean abundance
          pro_group <-
            error_bar_pivot_longer_tibble_summarised_sub[error_bar_pivot_longer_tibble_summarised_sub$mean ==
                                                            max(error_bar_pivot_longer_tibble_summarised_sub$mean),]$group
          pro_group <- as.vector(pro_group)

          # Find indices of rows matching the current feature, excluding NA values
          idx <- which(daa_results_filtered_sub_df$feature == i & !is.na(daa_results_filtered_sub_df$feature))

          # Only assign values if valid indices exist
          if (length(idx) > 0) {
            daa_results_filtered_sub_df$pro[idx] <- pro_group
          }
        }

        if (ko_to_kegg == TRUE) {
          # Nested sorting: first by pathway_class, then by group and p_adjust within each class
          order <- order(
            daa_results_filtered_sub_df$pathway_class,
            daa_results_filtered_sub_df$pro,
            daa_results_filtered_sub_df$p_adjust
          )
        } else {
          # Order by group and p-value
          order <-
            order(daa_results_filtered_sub_df$pro,
                  daa_results_filtered_sub_df$p_adjust)
        }
      },
      "pathway_class" = {
        if (!"pathway_class" %in% colnames(daa_results_filtered_sub_df)) {
          stop(
            "The 'pathway_class' column is missing in the 'daa_results_filtered_sub_df' data frame. ",
            "Please use the 'pathway_annotation' function to annotate the 'pathway_daa' results."
          )
        }
        order <- order(
          daa_results_filtered_sub_df$pathway_class,
          daa_results_filtered_sub_df$p_adjust
        )
      },
      {
        order <- order
      }
    )

    daa_results_filtered_sub_df <-
      daa_results_filtered_sub_df[order,]

    # Reorder data to match daa_results_filtered_sub_df$feature order
    # Using match() instead of loop rbind() for O(n) vs O(n²) performance
    error_bar_pivot_longer_tibble_summarised$feature_order <- match(
      error_bar_pivot_longer_tibble_summarised$name,
      daa_results_filtered_sub_df$feature
    )
    error_bar_pivot_longer_tibble_summarised_ordered <-
      error_bar_pivot_longer_tibble_summarised[order(error_bar_pivot_longer_tibble_summarised$feature_order), ]
    error_bar_pivot_longer_tibble_summarised_ordered$feature_order <- NULL

    if (ko_to_kegg == FALSE) {
      # Match by feature name using the match function
      matched_indices <- match(
        error_bar_pivot_longer_tibble_summarised_ordered$name,
        daa_results_filtered_sub_df$feature
      )
      error_bar_pivot_longer_tibble_summarised_ordered[, x_lab] <-
        daa_results_filtered_sub_df[matched_indices, x_lab]
    }

    if (ko_to_kegg == TRUE) {
      error_bar_pivot_longer_tibble_summarised_ordered$pathway_class <-
        rep(daa_results_filtered_sub_df$pathway_class,
            each = length(levels(
              factor(error_bar_pivot_longer_tibble_summarised_ordered$group)
            )))
    }

    error_bar_pivot_longer_tibble_summarised_ordered$name <- factor(error_bar_pivot_longer_tibble_summarised_ordered$name, levels = rev(daa_results_filtered_sub_df$feature))

    # Calculate smart text size for pathway names (y-axis labels)
    pathway_names_final_text_size <- if (pathway_names_text_size == "auto") {
      calculate_smart_text_size(nrow(daa_results_filtered_sub_df),
                                base_size = 10, min_size = 8, max_size = 14)
    } else {
      pathway_names_text_size
    }

    # Calculate smart text size for pathway class annotations
    pathway_class_final_text_size <- if (pathway_class_text_size == "auto") {
      calculate_smart_text_size(nrow(daa_results_filtered_sub_df),
                                base_size = 10, min_size = 3, max_size = 4)
    } else {
      as.numeric(pathway_class_text_size)  # Ensure it's numeric
    }


    # Set pathway class text color based on theme if "auto"
    pathway_class_final_text_color <- if (pathway_class_text_color == "auto") {
      # Get theme colors for pathway class
      current_theme <- get_color_theme(color_theme)
      current_theme$pathway_class_colors[1]  # Use first theme color
    } else {
      pathway_class_text_color
    }

    bar_errorbar <-
      ggplot2::ggplot(error_bar_pivot_longer_tibble_summarised_ordered,
             ggplot2::aes(mean, name, fill = group)) +
      ggplot2::geom_errorbar(
        ggplot2::aes(xmax = mean + sd, xmin = 0),
        position = ggplot2::position_dodge(width = 0.8),
        width = 0.5,
        linewidth = 0.5,
        color = "black"
      ) +
      ggplot2::geom_bar(stat = "identity",
               position = ggplot2::position_dodge(width = 0.8),
               width = 0.8) +
      GGally::geom_stripped_cols(width = 10) +
      ggplot2::scale_fill_manual(values = colors) +
      ggplot2::scale_color_manual(values = colors) +
      ggprism::theme_prism(base_size = 12) +
      ggplot2::scale_x_continuous(expand = c(0, 0)) +
      ggplot2::scale_y_discrete(labels = rev(daa_results_filtered_sub_df[, x_lab])) +
      ggplot2::labs(x = "Relative Abundance", y = NULL) +
      ggplot2::theme(
        axis.ticks.y = ggplot2::element_blank(),
        axis.line.y = ggplot2::element_blank(),
        axis.line.x = ggplot2::element_line(linewidth = 0.5),
        axis.ticks.x = ggplot2::element_line(linewidth = 0.5),
        panel.grid.major.y = ggplot2::element_blank(),
        panel.grid.major.x = ggplot2::element_blank(),
        axis.text = ggplot2::element_text(size = 10, color = "black"),
        axis.text.x = ggplot2::element_text(margin = ggplot2::margin(r = 0)),
        axis.text.y = ggplot2::element_text(
          size = pathway_names_final_text_size,
          color = "black",
          margin = ggplot2::margin(b = 6)
        ),
        axis.title.x =  ggplot2::element_text(
          size = 10,
          color = "black",
          hjust = 0.5
        ),
        legend.position = legend_position,
        legend.key.size = ggplot2::unit(legend_key_size, "cm"),
        legend.direction = legend_direction,
        legend.justification = "center",
        legend.text = ggplot2::element_text(size = legend_text_size, face = "bold"),
        legend.box.just = "center",
        legend.title = if (!is.null(legend_title)) {
          ggplot2::element_text(size = legend_title_size, face = "bold")
        } else {
          ggplot2::element_blank()
        },
        plot.margin = ggplot2::margin(0, 0.5, 0.5, 0, unit = "cm")
      ) +
      # Add guide specifications for multi-column/row layouts
      {if (!is.null(legend_ncol) || !is.null(legend_nrow)) {
        ggplot2::guides(fill = ggplot2::guide_legend(
          ncol = legend_ncol,
          nrow = legend_nrow,
          byrow = TRUE
        ))
      } else {
        ggplot2::guides()
      }} +
      ggplot2::coord_cartesian(clip = "off") +
      ggplot2::theme(
        axis.text.x = ggplot2::element_text(angle = 45, hjust = 1),
        plot.margin = ggplot2::unit(c(1, 1, 1, 1), "cm")
      )

    if (ko_to_kegg == TRUE) {
      # Convert table to matrix to preserve names as rownames
      pathway_class_table <- table(daa_results_filtered_sub_df$pathway_class)
      pathway_class_group_mat <- as.data.frame(as.matrix(pathway_class_table))
      colnames(pathway_class_group_mat) <- "Freq"
      
      # Create the pathway_class_group data frame
      pathway_class_group <- data.frame(
        . = unique(daa_results_filtered_sub_df$pathway_class),
        Freq = pathway_class_group_mat[unique(daa_results_filtered_sub_df$pathway_class), "Freq"]
      )
      start <-
        c(1, rev(pathway_class_group$Freq)[1:(length(pathway_class_group$Freq) - 1)]) %>%
        cumsum()
      end <- cumsum(rev(pathway_class_group$Freq))
      ymin <- start - 1 / 2
      ymax <- end + 1 / 2
      nPoints <- length(start)
      pCol <- pathway_class_colors[1:nPoints]
      pFill <- pCol
      for (i in 1:nPoints)  {
        bar_errorbar <- bar_errorbar +
          ggplot2::annotation_custom(
            grob = grid::rectGrob(
              gp = grid::gpar(
                col = pCol[i],
                fill = pFill[i],
                lty = NULL,
                lwd = NULL,
                alpha = 0.2
              )
            ),
            xmin = -2,
            xmax = 0,
            ymin = ymin[i],
            ymax = ymax[i]
          )
      }
    }
    # Add necessary columns
    daa_results_filtered_sub_df$negative_log10_p <- -log10(daa_results_filtered_sub_df$p_adjust)
    daa_results_filtered_sub_df$group_nonsense <- "nonsense"
    
    # Only add log2_fold_change if it doesn't exist
    if (!"log2_fold_change" %in% colnames(daa_results_filtered_sub_df)) {
      daa_results_filtered_sub_df$log2_fold_change <- rep(NA, nrow(daa_results_filtered_sub_df))
    }

    # Calculate pseudocount once using all mean values for consistency
    all_means <- error_bar_pivot_longer_tibble_summarised_ordered$mean
    pseudocount <- calculate_pseudocount(all_means)

    for (i in daa_results_filtered_sub_df$feature){
      # Get mean values for this feature
      feature_means <- error_bar_pivot_longer_tibble_summarised_ordered[error_bar_pivot_longer_tibble_summarised_ordered$name %in% i,]

      # Get group1 and group2 names for this feature from DAA results
      feature_row <- daa_results_filtered_sub_df[daa_results_filtered_sub_df$feature == i, ]
      group1_name <- feature_row$group1[1]
      group2_name <- feature_row$group2[1]

      # Get mean values for each group in the correct order
      mean_group1 <- feature_means[feature_means$group == group1_name, ]$mean
      mean_group2 <- feature_means[feature_means$group == group2_name, ]$mean

      # Calculate log2 fold change using unified function
      if (length(mean_group1) > 0 && length(mean_group2) > 0) {
        log2_fc <- calculate_log2_fold_change(mean_group1, mean_group2, pseudocount = pseudocount)
        daa_results_filtered_sub_df[daa_results_filtered_sub_df$feature==i,]$log2_fold_change <- log2_fc
      } else {
        # Fallback using unified function with auto-calculated pseudocount
        mean_vals <- feature_means$mean
        if (length(mean_vals) >= 2) {
          log2_fc <- calculate_log2_fold_change(mean_vals[1], mean_vals[2])
          daa_results_filtered_sub_df[daa_results_filtered_sub_df$feature==i,]$log2_fold_change <- log2_fc
        }
      }
    }
    daa_results_filtered_sub_df$feature <- factor(daa_results_filtered_sub_df$feature,levels = rev(daa_results_filtered_sub_df$feature))
    p_values_bar <- daa_results_filtered_sub_df %>%
      ggplot2::ggplot(ggplot2::aes(feature, log2_fold_change, fill = group_nonsense)) +
      ggplot2::geom_bar(stat = "identity",
               position = ggplot2::position_dodge(width = 0.8),
               width = 0.8) +
      ggplot2::labs(y = "log2 fold change", x = NULL) +
      GGally::geom_stripped_cols() +
      ggplot2::scale_fill_manual(values = log2_fold_change_color) +
      ggplot2::scale_color_manual(values = log2_fold_change_color) +
      ggplot2::geom_hline(ggplot2::aes(yintercept = 0),
                 linetype = 'dashed',
                 color = 'black') +
      ggprism::theme_prism(base_size = 12) +
      ggplot2::scale_y_continuous(expand = c(0, 0)) +
      ggplot2::theme(
        axis.ticks.y = ggplot2::element_blank(),
        axis.line.y = ggplot2::element_blank(),
        axis.line.x = ggplot2::element_line(linewidth = 0.5),
        axis.ticks.x = ggplot2::element_line(linewidth = 0.5),
        panel.grid.major.y = ggplot2::element_blank(),
        panel.grid.major.x = ggplot2::element_blank(),
        axis.text = ggplot2::element_text(size = 10, color = "black"),
        axis.text.y = ggplot2::element_blank(),
        axis.text.x = ggplot2::element_text(
          size = 10,
          color = "black",
          margin = ggplot2::margin(b = 6)
        ),
        axis.title.x =  ggplot2::element_text(
          size = 11,
          color = "black",
          hjust = 0.5
        ),
        legend.position = "non"
      ) +
      ggplot2::coord_flip()

    if (ko_to_kegg == TRUE) {
      # Calculate label y-position as the center of each rectangle
      # The -0.5 offset was causing misalignment (see demos/alignment_debug_analysis.R)
      pathway_class_y <- (ymax + ymin) / 2
      pathway_class_plot_df <-
        as.data.frame(
          cbind(
            nonsense = "nonsense",
            pathway_class_y = pathway_class_y,
            pathway_class = rev(unique(
              daa_results_filtered_sub_df$pathway_class
            ))
          )
        )
      pathway_class_plot_df$pathway_class_y <-
        as.numeric(pathway_class_plot_df$pathway_class_y)
      # Number of features for y-axis limits
      n_features <- nrow(daa_results_filtered_sub_df)

      pathway_class_annotation <-
        pathway_class_plot_df %>% ggplot2::ggplot(ggplot2::aes(nonsense, pathway_class_y)) + ggplot2::geom_text(
          ggplot2::aes(nonsense, pathway_class_y, label = pathway_class),
          size = pathway_class_final_text_size,
          color = pathway_class_final_text_color,
          fontface = pathway_class_text_face,
          family = "sans",
          angle = pathway_class_text_angle,
          hjust = if (pathway_class_position == "left") 1 else 0
        ) +
        # Use scale_y_continuous to properly align with the discrete bar plot y-axis
        # The bar plot's discrete y-axis maps factor levels to positions 1, 2, ..., n
        # We need matching limits (0.5 to n+0.5) to align the annotation labels
        ggplot2::scale_y_continuous(limits = c(0.5, n_features + 0.5), expand = c(0, 0)) +
        ggprism::theme_prism(base_size = 12) +
        ggplot2::theme(
          axis.ticks = ggplot2::element_blank(),
          axis.line = ggplot2::element_blank(),
          panel.grid.major.y = ggplot2::element_blank(),
          panel.grid.major.x = ggplot2::element_blank(),
          panel.background = ggplot2::element_blank(),
          axis.text = ggplot2::element_blank(),
          # Dynamic margin based on position: more space on the side where text is
          # Margin order is: top, right, bottom, left
          plot.margin = if (pathway_class_position == "left") {
            ggplot2::unit(c(0, 0.2, 0, 2.0), "cm")  # Increased left margin to 2.0cm
          } else {
            ggplot2::unit(c(0, 2.0, 0, 0.2), "cm")  # Increased right margin to 2.0cm
          },
          axis.title.y =  ggplot2::element_blank(),
          axis.title.x = ggplot2::element_blank(),
          legend.position = "non"
        ) +
        ggplot2::coord_cartesian(clip = "off")
    }

    # Enhanced p-value formatting using the new system
    format_p_value <- function(p) {
      if (pvalue_format != "numeric") {
        format_pvalue_smart(p,
                           format = pvalue_format,
                           stars = pvalue_stars,
                           thresholds = pvalue_thresholds,
                           star_symbols = pvalue_star_symbols)
      } else {
        ifelse(p < 0.001, sprintf("%.1e", p), sprintf("%.3f", p))
      }
    }
    



    # Calculate p-value colors if enabled
    pvalue_text_colors <- if (pvalue_colors) {
      get_significance_colors(daa_results_filtered_sub_df$p_adjust,
                            thresholds = pvalue_thresholds,
                            colors = c("#d73027", "#fc8d59", "#fee08b"),
                            default_color = "black")
    } else {
      rep("black", nrow(daa_results_filtered_sub_df))
    }
    
    # Calculate smart text size for p-values
    pvalue_text_size <- if (pvalue_size == "auto") {
      calculate_smart_text_size(nrow(daa_results_filtered_sub_df),
                                base_size = 2.5, min_size = 1.8, max_size = 3.0)
    } else {
      pvalue_size
    }

    daa_results_filtered_sub_df$unique <-
      nrow(daa_results_filtered_sub_df) - seq_len(nrow(daa_results_filtered_sub_df)) + 1
    p_annotation <- daa_results_filtered_sub_df %>%
      ggplot2::ggplot(ggplot2::aes(group_nonsense, p_adjust)) +
      ggplot2::geom_text(
        ggplot2::aes(group_nonsense, unique, 
                    label = format_p_value(p_adjust),
                    color = I(pvalue_text_colors)),
        size = pvalue_text_size,
        fontface = "bold",
        family = "sans",
        angle = pvalue_angle
      ) +
      ggplot2::labs(y = "p-value (adjusted)") +
      ggplot2::scale_y_discrete(position = "right") +
      ggprism::theme_prism(base_size = 12) +
      ggplot2::theme(
        axis.ticks = ggplot2::element_blank(),
        axis.line = ggplot2::element_blank(),
        panel.grid.major.y = ggplot2::element_blank(),
        panel.grid.major.x = ggplot2::element_blank(),
        panel.background = ggplot2::element_blank(),
        axis.text = ggplot2::element_blank(),
        plot.margin = ggplot2::unit(c(0, 0.2, 0, 0), "cm"),
        axis.title.y =  ggplot2::element_text(
          size = 11,
          color = "black",
          vjust = 0
        ),
        axis.title.x = ggplot2::element_blank(),
        legend.position = "non"
      )
    if (p_value_bar == TRUE) {
      if (ko_to_kegg == TRUE) {
        combination_bar_plot <-
          pathway_class_annotation + bar_errorbar + p_values_bar + p_annotation + patchwork::plot_layout(ncol = 4, widths =
                                                                                                c(2.5, 2.0, 0.7, 0.3))
      }
      else{
        combination_bar_plot <-
          bar_errorbar + p_values_bar + p_annotation + patchwork::plot_layout(ncol = 3, widths = c(2.3, 0.7, 0.3))
      }
    }else{
      if (ko_to_kegg == TRUE) {
        combination_bar_plot <-
          pathway_class_annotation + bar_errorbar + p_annotation + patchwork::plot_layout(ncol = 3, widths =
                                                                                                           c(2.5, 2.0, 0.3))
      }
      else{
        combination_bar_plot <-
          bar_errorbar + p_annotation + patchwork::plot_layout(ncol = 2, widths = c(2.5,  0.2))
      }
    }
    return(combination_bar_plot)
  }

#' Ridge Plot for GSEA Results
#'
#' Creates a ridge plot (joy plot) to visualize the distribution of gene/KO
#' abundances or fold changes for enriched pathways from GSEA analysis.
#' This helps interpret whether pathways are predominantly up- or down-regulated.
#'
#' @param gsea_results A data frame containing GSEA results from \code{\link{pathway_gsea}}.
#'   Must contain pathway_id column and either NES or direction column.
#' @param abundance A data frame or matrix containing the original abundance data
#'   (genes/KOs as rows, samples as columns) used in the GSEA analysis.
#' @param metadata A data frame containing sample metadata with group information.
#' @param group Character string specifying the column name in metadata for grouping.
#' @param pathway_reference A data frame containing pathway-to-gene mappings.
#'   Must have columns: pathway_id (or go_id for GO) and a column containing
#'   gene/KO members (semicolon-separated). If NULL, attempts to use built-in
#'   KEGG or GO reference data.
#' @param pathway_type Character string specifying the pathway type: "KEGG", "GO", or "MetaCyc".
#'   Default is "KEGG".
#' @param n_pathways Integer specifying the number of top pathways to display.
#'   Default is 10.
#' @param sort_by Character string specifying how to sort pathways:
#'   "NES" (Normalized Enrichment Score), "pvalue", or "p.adjust".
#'   Default is "p.adjust".
#' @param show_direction Logical. If TRUE, colors ridges by enrichment direction.
#'   Default is TRUE.
#' @param colors Named character vector with colors for "Up" and "Down" directions.
#'   Default is blue for down-regulated and red for up-regulated.
#' @param title Character string for plot title.
#' @param x_lab Character string for x-axis label.
#' @param scale_height Numeric value controlling the overlap of ridges.
#'   Default is 0.9. Higher values create more overlap.
#' @param alpha Numeric value for ridge transparency (0-1). Default is 0.7.
#'
#' @return A ggplot2 object that can be further customized or saved.
#'
#' @details
#' The ridge plot displays the distribution of gene abundances (or fold changes)
#' for genes within each enriched pathway. This visualization helps to:
#' \itemize{
#'   \item Understand the overall direction of change for each pathway
#'   \item Identify pathways with consistent vs. heterogeneous gene expression
#'   \item Compare the magnitude of changes across pathways
#' }
#'
#' The plot requires the \code{ggridges} package to be installed.
#'
#' @examples
#' \dontrun{
#' library(ggpicrust2)
#' library(tibble)
#'
#' # Load example data
#' data("ko_abundance")
#' data("metadata")
#'
#' # Run GSEA (using camera method - recommended)
#' gsea_results <- pathway_gsea(
#'   abundance = ko_abundance %>% column_to_rownames("#NAME"),
#'   metadata = metadata,
#'   group = "Environment",
#'   pathway_type = "KEGG",
#'   method = "camera"
#' )
#'
#' # Create ridge plot
#' ridge_plot <- pathway_ridgeplot(
#'   gsea_results = gsea_results,
#'   abundance = ko_abundance %>% column_to_rownames("#NAME"),
#'   metadata = metadata,
#'   group = "Environment",
#'   n_pathways = 10
#' )
#' print(ridge_plot)
#' }
#'
#' @seealso \code{\link{pathway_gsea}}, \code{\link{visualize_gsea}},
#'   \code{\link{pathway_volcano}}
#'
#' @importFrom ggplot2 ggplot aes labs theme_minimal theme element_text
#'   scale_fill_manual element_blank
#' @importFrom dplyr filter mutate arrange slice_head left_join
#' @importFrom tidyr pivot_longer
#' @importFrom stats setNames
#'
#' @export
pathway_ridgeplot <- function(gsea_results,
                               abundance,
                               metadata,
                               group,
                               pathway_reference = NULL,
                               pathway_type = "KEGG",
                               n_pathways = 10,
                               sort_by = "p.adjust",
                               show_direction = TRUE,
                               colors = c("Down" = "#3182bd", "Up" = "#de2d26"),
                               title = "Ridge Plot: Gene Distribution in Enriched Pathways",
                               x_lab = "log2 Fold Change",
                               scale_height = 0.9,
                               alpha = 0.7) {

  # Check for ggridges package
 if (!requireNamespace("ggridges", quietly = TRUE)) {
   stop("Package 'ggridges' is required for ridge plots. ",
        "Please install it with: install.packages('ggridges')")
 }

 # Input validation
 if (!is.data.frame(gsea_results)) {
   stop("'gsea_results' must be a data frame.")
 }

 if (!is.data.frame(abundance) && !is.matrix(abundance)) {
   stop("'abundance' must be a data frame or matrix.")
 }

 require_column(metadata, group, "metadata")

 # Convert abundance to matrix if needed
 if (is.data.frame(abundance)) {
   abundance <- as.matrix(abundance)
 }

 # Validate required columns (pathway_gsea() output format)
 require_column(gsea_results, "pathway_id", "gsea_results")

 # pathway_name is optional, fallback to pathway_id
 pathway_name_col <- if ("pathway_name" %in% colnames(gsea_results)) "pathway_name" else "pathway_id"

 # direction is optional, can be derived from NES
 has_nes <- "NES" %in% colnames(gsea_results)
 direction_col <- if ("direction" %in% colnames(gsea_results)) "direction" else NULL

 # Sort and select top pathways
 sort_col <- if (sort_by %in% colnames(gsea_results)) {
   sort_by
 } else if ("pvalue" %in% colnames(gsea_results)) {
   "pvalue"
 } else {
   stop("Cannot find sorting column in gsea_results.")
 }

 # Filter and sort
 df <- gsea_results
 df <- df[!is.na(df[[sort_col]]), ]
 df <- df[order(df[[sort_col]]), ]
 df <- head(df, n_pathways)

 if (nrow(df) == 0) {
   stop("No pathways to display after filtering.")
 }

 # Get pathway-gene mappings
 if (is.null(pathway_reference)) {
   # Load reference data using unified loader
   if (pathway_type == "KEGG") {
     kegg_ref <- load_reference_data("ko_to_kegg")
     # Aggregate KO IDs by pathway_id
     pathway_reference <- stats::aggregate(
       ko_id ~ pathway_id + pathway_name,
       data = kegg_ref,
       FUN = function(x) paste(unique(x), collapse = ";")
     )
     colnames(pathway_reference)[colnames(pathway_reference) == "ko_id"] <- "ko_members"
   } else if (pathway_type == "GO") {
     pathway_reference <- load_reference_data("ko_to_go")
   } else {
     stop("Please provide pathway_reference parameter for ", pathway_type, " pathways.")
   }
 }

 # Calculate fold changes between groups
 group_vec <- metadata[[group]]
 group_levels <- unique(group_vec)

 if (length(group_levels) != 2) {
   warning("Ridge plot works best with 2 groups. Using first two groups.")
   group_levels <- group_levels[1:2]
 }

 # Calculate mean abundance per group
 samples_g1 <- which(group_vec == group_levels[1])
 samples_g2 <- which(group_vec == group_levels[2])

 mean_g1 <- rowMeans(abundance[, samples_g1, drop = FALSE], na.rm = TRUE)
 mean_g2 <- rowMeans(abundance[, samples_g2, drop = FALSE], na.rm = TRUE)

 # Calculate log2 fold change using unified function
 pseudocount <- calculate_pseudocount(as.vector(abundance))
 log2fc <- calculate_log2_fold_change(mean_g1, mean_g2, pseudocount = pseudocount)
 names(log2fc) <- rownames(abundance)

 # Determine gene member column in pathway_reference
 # Support both wide format (semicolon-separated) and long format (one gene per row)
 gene_col_candidates <- c("ko_members", "KO", "ko_id", "genes", "gene", "ec_numbers")
 gene_col <- NULL
 for (col in gene_col_candidates) {
   if (col %in% colnames(pathway_reference)) {
     gene_col <- col
     break
   }
 }
 if (is.null(gene_col)) {
   # Fallback: try to find any column with semicolon-separated values
   for (col in colnames(pathway_reference)) {
     if (any(grepl(";", pathway_reference[[col]], fixed = TRUE))) {
       gene_col <- col
       break
     }
   }
 }
 if (is.null(gene_col)) {
   stop("Cannot find gene member column in pathway_reference. ",
        "Expected columns: ", paste(gene_col_candidates, collapse = ", "))
 }

 # Build data for ridge plot
 # Using list + do.call(rbind, ...) for O(n) vs O(n²) performance
 ridge_data_list <- vector("list", nrow(df))

 # Determine ref_id_col once outside the loop
 ref_id_col <- if ("pathway_id" %in% colnames(pathway_reference)) {
   "pathway_id"
 } else if ("go_id" %in% colnames(pathway_reference)) {
   "go_id"
 } else {
   colnames(pathway_reference)[1]
 }

 for (i in seq_len(nrow(df))) {
   pid <- df[["pathway_id"]][i]
   pname <- df[[pathway_name_col]][i]

   # Truncate long names
   if (nchar(pname) > 50) {
     pname <- paste0(substr(pname, 1, 47), "...")
   }

   # Get direction if available
   direction <- if (!is.null(direction_col)) {
     as.character(df[[direction_col]][i])
   } else if (has_nes) {
     ifelse(df$NES[i] > 0, "Up", "Down")
   } else {
     "Unknown"
   }

   ref_rows <- pathway_reference[pathway_reference[[ref_id_col]] == pid, ]

   if (nrow(ref_rows) > 0) {
     # Handle both long format (multiple rows) and wide format (semicolon-separated)
     if (nrow(ref_rows) > 1) {
       # Long format: each row is one gene
       genes <- unique(as.character(ref_rows[[gene_col]]))
     } else {
       # Wide format: genes are semicolon-separated in one row
       genes_str <- ref_rows[[gene_col]][1]
       genes <- unlist(strsplit(as.character(genes_str), ";"))
     }
     genes <- trimws(genes)

     # Get fold changes for these genes
     fc_values <- log2fc[names(log2fc) %in% genes]

     if (length(fc_values) > 0) {
       ridge_data_list[[i]] <- data.frame(
         pathway = pname,
         pathway_id = pid,
         direction = direction,
         log2fc = fc_values,
         stringsAsFactors = FALSE
       )
     }
   }
 }

 # Combine all data frames at once (O(n) instead of O(n²))
 non_null_list <- ridge_data_list[!sapply(ridge_data_list, is.null)]
 if (length(non_null_list) == 0) {
   stop("No gene data found for the selected pathways. ",
        "Check that pathway_reference matches your abundance data.")
 }
 ridge_data <- do.call(rbind, non_null_list)

 if (is.null(ridge_data) || nrow(ridge_data) == 0) {
   stop("No gene data found for the selected pathways. ",
        "Check that pathway_reference matches your abundance data.")
 }

 # Order pathways by their appearance in the sorted results
 pathway_order <- unique(df[[pathway_name_col]])
 pathway_order <- sapply(pathway_order, function(x) {
   if (nchar(x) > 50) paste0(substr(x, 1, 47), "...") else x
 })
 ridge_data$pathway <- factor(ridge_data$pathway, levels = rev(pathway_order))

 # Create ridge plot
 p <- ggplot2::ggplot(ridge_data, ggplot2::aes(x = .data$log2fc, y = .data$pathway))

 if (show_direction && "direction" %in% colnames(ridge_data)) {
   p <- p + ggridges::geom_density_ridges(
     ggplot2::aes(fill = .data$direction),
     alpha = alpha,
     scale = scale_height
   ) +
     ggplot2::scale_fill_manual(values = colors, name = "Direction")
 } else {
   p <- p + ggridges::geom_density_ridges(
     fill = "#756bb1",
     alpha = alpha,
     scale = scale_height
   )
 }

 p <- p +
   ggplot2::geom_vline(xintercept = 0, linetype = "dashed", color = "grey40", alpha = 0.7) +
   ggplot2::labs(
     x = x_lab,
     y = NULL,
     title = title
   ) +
   ggplot2::theme_minimal() +
   ggplot2::theme(
     plot.title = ggplot2::element_text(hjust = 0.5, face = "bold"),
     axis.text.y = ggplot2::element_text(size = 9),
     legend.position = "top",
     panel.grid.minor = ggplot2::element_blank()
   )

 return(p)
}

# =============================================================================
# Taxa-Function Contribution Visualization
# =============================================================================
# Stacked bar plots and heatmaps showing which taxa drive pathway abundances.

#' Stacked bar plot of taxa contributions
#'
#' Creates a stacked bar plot showing taxa contributions to predicted
#' functional abundances, faceted by function or sample group.
#'
#' @param contrib_agg A data.frame from \code{\link{aggregate_taxa_contributions}}.
#' @param metadata A data.frame containing sample metadata.
#' @param group Character. Column name in \code{metadata} for grouping samples.
#' @param function_ids Optional character vector of function IDs to plot.
#'   If NULL (default), the top \code{n_functions} by variance are shown.
#' @param n_functions Integer. Number of functions to show when
#'   \code{function_ids} is NULL. Default 6.
#' @param facet_by Character. Facet by \code{"function"} (default) or
#'   \code{"group"}.
#' @param show_percentage Logical. Normalize bars to 100\%? Default TRUE.
#' @param color_theme Character. Color theme name, passed to
#'   \code{\link{get_color_theme}}. Default \code{"default"}.
#' @param font_size Numeric. Base font size. Default 12.
#' @param legend_position Character. Legend position. Default \code{"right"}.
#' @param custom_title Optional character string for the plot title.
#'
#' @return A \code{ggplot2} object.
#'
#' @examples
#' \donttest{
#' # Synthetic example
#' agg <- data.frame(
#'   sample = rep(c("S1", "S2", "S3", "S4"), each = 3),
#'   function_id = rep(c("K00001", "K00002"), each = 6),
#'   taxon_label = rep(c("Genus_A", "Genus_B", "Other"), 4),
#'   contribution = runif(12)
#' )
#' metadata <- data.frame(
#'   sample = c("S1", "S2", "S3", "S4"),
#'   group = c("Control", "Control", "Treatment", "Treatment")
#' )
#' p <- taxa_contribution_bar(agg, metadata, group = "group")
#' }
#'
#' @export
taxa_contribution_bar <- function(contrib_agg,
                                  metadata,
                                  group,
                                  function_ids = NULL,
                                  n_functions = 6,
                                  facet_by = "function",
                                  show_percentage = TRUE,
                                  color_theme = "default",
                                  font_size = 12,
                                  legend_position = "right",
                                  custom_title = NULL) {
  # Validate inputs
  validate_dataframe(contrib_agg,
                     required_cols = c("sample", "function_id",
                                       "taxon_label", "contribution"),
                     param_name = "contrib_agg")
  validate_metadata(metadata)
  validate_group(metadata, group, min_groups = 1)

  facet_by <- match.arg(facet_by, c("function", "group"))

  # Align samples
  # Build a pseudo-abundance matrix for align_samples()
  sample_ids <- unique(contrib_agg$sample)
  pseudo_abundance <- matrix(1, nrow = 1, ncol = length(sample_ids))
  colnames(pseudo_abundance) <- sample_ids
  rownames(pseudo_abundance) <- "dummy"
  aligned <- align_samples(pseudo_abundance, metadata, verbose = FALSE)
  common_samples <- colnames(aligned$abundance)
  metadata_aligned <- aligned$metadata

  contrib_agg <- contrib_agg[contrib_agg$sample %in% common_samples, ]
  if (nrow(contrib_agg) == 0) {
    stop("No overlapping samples between contrib_agg and metadata.")
  }

  # Select functions to plot
  if (is.null(function_ids)) {
    func_var <- stats::aggregate(
      contribution ~ function_id, data = contrib_agg, FUN = stats::var
    )
    func_var <- func_var[order(-func_var$contribution), ]
    function_ids <- utils::head(func_var$function_id, n_functions)
  }
  contrib_agg <- contrib_agg[contrib_agg$function_id %in% function_ids, ]

  # Add group info
  sample_col <- aligned$sample_col
  if (sample_col == ".rownames") {
    contrib_agg$group_var <- metadata_aligned[contrib_agg$sample, group]
  } else {
    group_map <- stats::setNames(
      metadata_aligned[[group]],
      metadata_aligned[[sample_col]]
    )
    contrib_agg$group_var <- group_map[contrib_agg$sample]
  }

  # Normalize to percentage if requested
  if (show_percentage) {
    contrib_agg <- do.call(rbind, lapply(
      split(contrib_agg, list(contrib_agg$sample, contrib_agg$function_id)),
      function(df) {
        total <- sum(df$contribution, na.rm = TRUE)
        if (total > 0) df$contribution <- df$contribution / total * 100
        df
      }
    ))
    rownames(contrib_agg) <- NULL
    y_label <- "Relative contribution (%)"
  } else {
    y_label <- "Contribution"
  }

  # Get colors
  n_taxa <- length(unique(contrib_agg$taxon_label))
  theme_colors <- get_color_theme(color_theme, n_colors = n_taxa)
  taxa_colors <- theme_colors$pathway_class_colors[seq_len(n_taxa)]
  names(taxa_colors) <- sort(unique(contrib_agg$taxon_label))
  # Ensure "Other" is grey
  if ("Other" %in% names(taxa_colors)) {
    taxa_colors[["Other"]] <- "#999999"
  }

  # Build plot
  p <- ggplot2::ggplot(
    contrib_agg,
    ggplot2::aes(x = .data$sample, y = .data$contribution,
                 fill = .data$taxon_label)
  ) +
    ggplot2::geom_bar(stat = "identity", position = "stack") +
    ggplot2::scale_fill_manual(values = taxa_colors, name = "Taxon") +
    ggplot2::labs(
      x = "Sample",
      y = y_label,
      title = custom_title
    ) +
    ggprism::theme_prism(base_size = font_size) +
    ggplot2::theme(
      axis.text.x = ggplot2::element_text(angle = 45, hjust = 1),
      legend.position = legend_position
    )

  # Faceting
  if (facet_by == "function") {
    p <- p + ggplot2::facet_wrap(~ function_id, scales = "free_x")
  } else {
    p <- p + ggplot2::facet_wrap(~ group_var, scales = "free_x")
  }

  p
}


#' Heatmap of taxa contributions across functions
#'
#' Creates a heatmap showing mean taxa contributions across pathways/functions,
#' with optional clustering and pathway annotations.
#'
#' @param contrib_agg A data.frame from \code{\link{aggregate_taxa_contributions}}.
#' @param annotation_data Optional data.frame from \code{\link{pathway_annotation}}
#'   for replacing function IDs with readable descriptions.
#' @param n_functions Integer. Number of functions to include. Default 20.
#' @param cluster_rows Logical. Cluster rows (taxa)? Default TRUE.
#' @param cluster_cols Logical. Cluster columns (functions)? Default TRUE.
#' @param clustering_method Character. Method for \code{hclust}. Default \code{"complete"}.
#' @param clustering_distance Character. Distance metric. Default \code{"euclidean"}.
#' @param low_color Character. Color for low values. Default \code{"#f7f7f7"}.
#' @param high_color Character. Color for high values. Default \code{"#ca0020"}.
#' @param font_size Numeric. Base font size. Default 12.
#' @param dendro_line_size Numeric. Dendrogram line width. Default 0.5.
#' @param custom_title Optional plot title.
#'
#' @return A \code{ggplot2} or \code{patchwork} object.
#'
#' @examples
#' \donttest{
#' agg <- data.frame(
#'   sample = rep(c("S1", "S2"), each = 6),
#'   function_id = rep(rep(c("K00001", "K00002", "K00003"), each = 2), 2),
#'   taxon_label = rep(c("Genus_A", "Genus_B"), 6),
#'   contribution = runif(12)
#' )
#' p <- taxa_contribution_heatmap(agg)
#' }
#'
#' @export
taxa_contribution_heatmap <- function(contrib_agg,
                                      annotation_data = NULL,
                                      n_functions = 20,
                                      cluster_rows = TRUE,
                                      cluster_cols = TRUE,
                                      clustering_method = "complete",
                                      clustering_distance = "euclidean",
                                      low_color = "#f7f7f7",
                                      high_color = "#ca0020",
                                      font_size = 12,
                                      dendro_line_size = 0.5,
                                      custom_title = NULL) {
  validate_dataframe(contrib_agg,
                     required_cols = c("sample", "function_id",
                                       "taxon_label", "contribution"),
                     param_name = "contrib_agg")

  # Compute mean contribution per taxon x function
  mean_contrib <- stats::aggregate(
    contribution ~ function_id + taxon_label,
    data = contrib_agg,
    FUN = mean
  )

  # Select top functions by total contribution
  func_totals <- stats::aggregate(
    contribution ~ function_id, data = mean_contrib, FUN = sum
  )
  func_totals <- func_totals[order(-func_totals$contribution), ]
  top_funcs <- utils::head(func_totals$function_id, n_functions)
  mean_contrib <- mean_contrib[mean_contrib$function_id %in% top_funcs, ]

  # Pivot to wide matrix: taxon_label (rows) x function_id (cols)
  mat <- tidyr::pivot_wider(
    mean_contrib,
    names_from = "function_id",
    values_from = "contribution",
    values_fill = 0
  )
  taxa_names <- mat$taxon_label
  mat <- as.matrix(mat[, -1, drop = FALSE])
  rownames(mat) <- taxa_names

  # Replace function IDs with annotations if available
  if (!is.null(annotation_data)) {
    desc_map <- stats::setNames(
      annotation_data$description,
      annotation_data$pathway
    )
    new_names <- desc_map[colnames(mat)]
    # Only replace where we found a match, truncate long names
    found <- !is.na(new_names)
    new_names[found] <- substr(new_names[found], 1, 50)
    new_names[!found] <- colnames(mat)[!found]
    colnames(mat) <- new_names
  }

  # Clustering
  row_order <- seq_len(nrow(mat))
  col_order <- seq_len(ncol(mat))
  row_hc <- NULL
  col_hc <- NULL

  if (cluster_rows && nrow(mat) > 1) {
    row_hc <- stats::hclust(stats::dist(mat, method = clustering_distance),
                            method = clustering_method)
    row_order <- row_hc$order
  }
  if (cluster_cols && ncol(mat) > 1) {
    col_hc <- stats::hclust(stats::dist(t(mat), method = clustering_distance),
                            method = clustering_method)
    col_order <- col_hc$order
  }

  # Reorder matrix
  mat <- mat[row_order, col_order, drop = FALSE]

  # Convert to long for ggplot
  plot_df <- data.frame(
    taxon = rep(rownames(mat), ncol(mat)),
    func = rep(colnames(mat), each = nrow(mat)),
    value = as.vector(mat),
    stringsAsFactors = FALSE
  )
  # Maintain clustering order via factor levels
  plot_df$taxon <- factor(plot_df$taxon, levels = rev(rownames(mat)))
  plot_df$func <- factor(plot_df$func, levels = colnames(mat))

  text_size <- calculate_smart_text_size(max(nrow(mat), ncol(mat)),
                                         base_size = font_size)

  # Build heatmap
  p <- ggplot2::ggplot(
    plot_df,
    ggplot2::aes(x = .data$func, y = .data$taxon, fill = .data$value)
  ) +
    ggplot2::geom_tile(color = "white", linewidth = 0.3) +
    ggplot2::scale_fill_gradient(
      low = low_color, high = high_color,
      name = "Mean\ncontribution"
    ) +
    ggplot2::labs(
      x = "Function",
      y = "Taxon",
      title = custom_title
    ) +
    ggplot2::theme_minimal(base_size = font_size) +
    ggplot2::theme(
      axis.text.x = ggplot2::element_text(
        angle = 45, hjust = 1, size = text_size
      ),
      axis.text.y = ggplot2::element_text(size = text_size),
      panel.grid = ggplot2::element_blank()
    )

  # Add dendrograms with patchwork if clustering is enabled
  if (!is.null(row_hc) || !is.null(col_hc)) {
    if (!is.null(row_hc)) {
      row_dendro <- create_dendrogram(row_hc, dendro_line_size = dendro_line_size,
                                      horizontal = TRUE)
    }
    if (!is.null(col_hc)) {
      col_dendro <- create_dendrogram(col_hc, dendro_line_size = dendro_line_size,
                                      horizontal = FALSE)
    }

    if (!is.null(row_hc) && !is.null(col_hc)) {
      # Both dendrograms
      if (!is.null(row_dendro) && !is.null(col_dendro)) {
        p <- (patchwork::plot_spacer() + col_dendro +
                row_dendro + p) +
          patchwork::plot_layout(
            ncol = 2,
            widths = c(0.15, 1),
            heights = c(0.15, 1)
          )
      } else {
        # Fallback if ggdendro not available
      }
    } else if (!is.null(row_hc) && !is.null(row_dendro)) {
      p <- (row_dendro + p) +
        patchwork::plot_layout(widths = c(0.15, 1))
    } else if (!is.null(col_hc) && !is.null(col_dendro)) {
      p <- (col_dendro / p) +
        patchwork::plot_layout(heights = c(0.15, 1))
    }
  }

  p
}

#' Annotate GSEA results with pathway information
#'
#' This function adds pathway annotations to GSEA results, including pathway names,
#' descriptions, and classifications.
#'
#' @param gsea_results A data frame containing GSEA results from the pathway_gsea function
#' @param pathway_type A character string specifying the pathway type: "KEGG", "MetaCyc", or "GO"
#'
#' @return A data frame with annotated GSEA results
#' @export
#'
#' @examples
#' \dontrun{
#' # Load example data
#' data(ko_abundance)
#' data(metadata)
#'
#' # Prepare abundance data
#' abundance_data <- as.data.frame(ko_abundance)
#' rownames(abundance_data) <- abundance_data[, "#NAME"]
#' abundance_data <- abundance_data[, -1]
#'
#' # Run GSEA analysis (using camera method - recommended)
#' gsea_results <- pathway_gsea(
#'   abundance = abundance_data,
#'   metadata = metadata,
#'   group = "Environment",
#'   pathway_type = "KEGG",
#'   method = "camera"
#' )
#'
#' # Annotate results
#' annotated_results <- gsea_pathway_annotation(
#'   gsea_results = gsea_results,
#'   pathway_type = "KEGG"
#' )
#' }
gsea_pathway_annotation <- function(gsea_results,
                                    pathway_type = "KEGG") {

  # Input validation
  if (!is.data.frame(gsea_results)) {
    stop("'gsea_results' must be a data frame")
  }

  valid_types <- c("KEGG", "MetaCyc", "GO")
  if (length(pathway_type) != 1 || !pathway_type %in% valid_types) {
    stop(sprintf("pathway_type must be one of: %s", paste(valid_types, collapse = ", ")))
  }

  if (!"pathway_id" %in% colnames(gsea_results)) {
    stop("GSEA results missing required column: pathway_id")
  }

  # Annotate based on pathway type
  if (pathway_type == "KEGG") {
    annotated_results <- annotate_kegg_gsea(gsea_results)
  } else if (pathway_type == "MetaCyc") {
    annotated_results <- annotate_metacyc_gsea(gsea_results)
  } else if (pathway_type == "GO") {
    annotated_results <- annotate_go_gsea(gsea_results)
  }

  return(annotated_results)
}

#' Annotate GSEA results with KEGG pathway information
#' @param gsea_results GSEA results data frame
#' @return Annotated results
#' @noRd
annotate_kegg_gsea <- function(gsea_results) {
  # Load KEGG pathway reference using unified loader
  kegg_ref <- load_reference_data("KEGG")

  # Remove pathway_name from gsea_results if present (will be replaced by reference)
  if ("pathway_name" %in% colnames(gsea_results)) {
    gsea_results$pathway_name <- NULL
  }

  # Merge with GSEA results
  annotated_results <- merge(
    gsea_results,
    kegg_ref,
    by.x = "pathway_id",
    by.y = "pathway",
    all.x = TRUE
  )

  # Fill missing pathway names with pathway_id
  if ("pathway_name" %in% colnames(annotated_results)) {
    annotated_results$pathway_name[is.na(annotated_results$pathway_name)] <-
      annotated_results$pathway_id[is.na(annotated_results$pathway_name)]
  } else {
    annotated_results$pathway_name <- annotated_results$pathway_id
  }

  annotated_results
}

#' Annotate GSEA results with MetaCyc pathway information
#' @param gsea_results GSEA results data frame
#' @return Annotated results
#' @noRd
annotate_metacyc_gsea <- function(gsea_results) {
  # Load MetaCyc reference using unified loader
  metacyc_ref <- load_reference_data("MetaCyc")

  # Remove pathway_name from gsea_results if present (will be replaced by reference)
  if ("pathway_name" %in% colnames(gsea_results)) {
    gsea_results$pathway_name <- NULL
  }

  # Merge with GSEA results
  annotated_results <- merge(
    gsea_results,
    metacyc_ref,
    by.x = "pathway_id",
    by.y = "id",
    all.x = TRUE
  )

  # Use description as pathway_name
  annotated_results$pathway_name <- ifelse(
    is.na(annotated_results$description) | annotated_results$description == "",
    annotated_results$pathway_id,
    annotated_results$description
  )
  annotated_results$description <- NULL

  annotated_results
}

#' Annotate GSEA results with GO term information
#' @param gsea_results GSEA results data frame
#' @return Annotated results
#' @noRd
annotate_go_gsea <- function(gsea_results) {
  # Load GO reference using unified loader
  go_ref <- load_reference_data("ko_to_go")

  # Remove pathway_name from gsea_results if present (will be replaced by reference)
  if ("pathway_name" %in% colnames(gsea_results)) {
    gsea_results$pathway_name <- NULL
  }

  # Create lookup data frame for merging (unique GO IDs with names)
  go_lookup <- unique(data.frame(
    pathway_id = go_ref$go_id,
    pathway_name = go_ref$go_name,
    stringsAsFactors = FALSE
  ))

  # Merge with GSEA results
  annotated_results <- merge(
    gsea_results,
    go_lookup,
    by = "pathway_id",
    all.x = TRUE
  )

  # Fill missing pathway names with pathway_id
  if ("pathway_name" %in% colnames(annotated_results)) {
    annotated_results$pathway_name[is.na(annotated_results$pathway_name)] <-
      annotated_results$pathway_id[is.na(annotated_results$pathway_name)]
  } else {
    annotated_results$pathway_name <- annotated_results$pathway_id
  }

  annotated_results
}

#' Compare the Consistency of Statistically Significant Features
#'
#' This function compares the consistency and inconsistency of statistically significant features obtained
#' using different methods in `pathway_daa` from the `ggpicrust2` package. It creates a report showing the number of common and
#' different features identified by each method, and the features themselves.
#' @name compare_daa_results
#' @param daa_results_list A list of data frames containing statistically significant features obtained using different methods.
#' @param method_names A character vector of names for each method used.
#' @param p_values_threshold A numeric value representing the threshold for the p-values. Features with p-values less than this
#' threshold are considered statistically significant. Default is 0.05.
#'
#' @return A data frame with the comparison results. The data frame has the following columns:
#' \itemize{
#'   \item \code{method}: The name of the method.
#'   \item \code{num_features}: The total number of statistically significant features obtained by the method.
#'   \item \code{num_common_features}: The number of features that are common to other methods.
#'   \item \code{num_diff_features}: The number of features that are different from other methods.
#'   \item \code{common_features}: The names of the features that are common to all methods.
#'   \item \code{diff_features}: The names of the features that are different from other methods.
#' }
#'
#' @examples
#' # Minimal DAA-like results from three methods (no external dependencies required)
#' deseq2_df <- data.frame(
#'   feature = c("ko00010", "ko00020", "ko00564"),
#'   group1 = c("A", "A", "A"),
#'   group2 = c("B", "B", "B"),
#'   p_adjust = c(0.01, 0.20, 0.03),
#'   stringsAsFactors = FALSE
#' )
#'
#' edgeR_df <- data.frame(
#'   feature = c("ko00010", "ko00680", "ko00564"),
#'   group1 = c("A", "A", "A"),
#'   group2 = c("B", "B", "B"),
#'   p_adjust = c(0.02, 0.04, 0.01),
#'   stringsAsFactors = FALSE
#' )
#'
#' maaslin2_df <- data.frame(
#'   feature = c("ko00010", "ko03030", "ko00564"),
#'   group1 = c("A", "A", "A"),
#'   group2 = c("B", "B", "B"),
#'   p_adjust = c(0.03, 0.02, 0.04),
#'   stringsAsFactors = FALSE
#' )
#'
#' daa_results_list <- list(DESeq2 = deseq2_df, edgeR = edgeR_df, Maaslin2 = maaslin2_df)
#' comparison_results <- compare_daa_results(
#'   daa_results_list = daa_results_list,
#'   method_names = c("DESeq2", "edgeR", "Maaslin2"),
#'   p_values_threshold = 0.05
#' )
#' comparison_results
#' @export
utils::globalVariables(c("group1","group2"))
compare_daa_results <- function(daa_results_list, method_names, p_values_threshold = 0.05) {
  # Compare the consistency and inconsistency of statistically significant features obtained using different methods in pathway_daa.

  # Arguments:
  # daa_results_list -- A list containing statistically significant features obtained using different methods.
  # method_names -- A character vector representing the names of each method.
  # p_values_threshold -- A numeric value representing the threshold for the p-values. Features with p-values less than this threshold are considered statistically significant.


  # Input validation with clear error messages
  if (!is.list(daa_results_list)) {
    stop("'daa_results_list' must be a list of DAA result data frames")
  }
  if (length(daa_results_list) == 0) {
    stop("'daa_results_list' cannot be empty")
  }
  if (!is.character(method_names)) {
    stop("'method_names' must be a character vector")
  }
  if (length(method_names) != length(daa_results_list)) {
    stop(sprintf("'method_names' length (%d) must match 'daa_results_list' length (%d)",
                 length(method_names), length(daa_results_list)))
  }
  if (!is.numeric(p_values_threshold) || length(p_values_threshold) != 1) {
    stop("'p_values_threshold' must be a single numeric value")
  }
  if (p_values_threshold <= 0 || p_values_threshold > 1) {
    stop("'p_values_threshold' must be between 0 and 1")
  }

  # Initialize a list to store the features obtained by each method
  features <- list()

  # Get the features obtained by each method
  for (i in seq_along(daa_results_list)) {
    if (!all(c("group1", "group2") %in% colnames(daa_results_list[[i]]))) {
      stop("DAA results in daa_results_list[[", i, "]] must contain 'group1' and 'group2' columns")
    }
    group_combinations <- unique(daa_results_list[[i]][, c("group1", "group2")])
    features[[i]] <- lapply(seq_len(nrow(group_combinations)), function(j) {
      subset(daa_results_list[[i]], group1 == group_combinations[j, "group1"] & group2 == group_combinations[j, "group2"] & p_adjust < p_values_threshold)$feature
    })
  }

  # Flatten nested list structure (from group combinations) to simple vectors
  features_flat <- lapply(features, function(x) unique(unlist(x)))

  # Calculate the intersection and union of the features obtained by each method
  intersect_features <- Reduce(intersect, features_flat)
  union_features <- unique(unlist(features_flat))

  # "diff_features" for method i = features method i reports whose
  # significance is NOT endorsed by every other method. Equivalently:
  # method_i_features \ intersection_of_all_methods. A feature found by
  # 2 of 3 methods is a disagreement from the perspective of each of
  # those 2 methods, so it MUST appear in both of their diff lists.
  #
  # Previously there was a second filter that kept only features
  # appearing in a single method, which silently collapsed "found by
  # 2/3" into "unanimous" and zeroed out the disagreement signal that
  # this function exists to surface.
  diff_features <- lapply(features_flat, function(x) setdiff(x, intersect_features))

  # Initialize a data frame to store the comparison results
  comparison_results <- data.frame(
    method = character(),
    num_features = integer(),
    num_common_features = integer(),
    num_diff_features = integer(),
    common_features = character(),
    diff_features = character(),
    stringsAsFactors = FALSE
  )

  # Output the comparison results and store them in the data frame
  message("Comparing ", length(daa_results_list), " methods:\n")
  for (i in seq_along(daa_results_list)) {
    num_features <- length(features_flat[[i]])
    num_common_features <- length(intersect_features)
    num_diff_features <- length(diff_features[[i]])
    common_features_names <- paste(intersect_features, collapse = ", ")
    diff_features_names <- paste(diff_features[[i]], collapse = ", ")

    message("The ", method_names[i], " method obtained ", num_features, " statistically significant features.")
    message("The number of features that are common to other methods is ", num_common_features)
    message("The number of features that are different from other methods is ", num_diff_features)
    message("The names of the features that are different from other methods are ", diff_features_names, "\n")

    comparison_results <- rbind(comparison_results, data.frame(
      method = method_names[i],
      num_features = num_features,
      num_common_features = num_common_features,
      num_diff_features = num_diff_features,
      common_features = common_features_names,
      diff_features = diff_features_names,
      stringsAsFactors = FALSE
    ))
  }

  message("The number of features that are common to all methods is ", length(intersect_features))
  message("The names of the features that are common to all methods are ", paste(intersect_features, collapse = ", "))
  message("The number of features that are obtained by any of the methods is ", length(union_features))
  message("The names of the features that are obtained by any of the methods are ", paste(union_features, collapse = ", "))

  return(comparison_results)
}

#' KO to KEGG Pathway Reference Data
#'
#' A comprehensive mapping between KEGG Orthology (KO) identifiers and KEGG pathways.
#' This dataset contains mappings covering 532 pathways and 23,466 unique KO IDs,
#' filtered to include only real KEGG pathway maps (5-digit IDs).
#'
#' @format A data frame with 9 variables:
#' \describe{
#'   \item{pathway_id}{KEGG pathway identifier (e.g., "ko00010")}
#'   \item{pathway_number}{KEGG pathway number}
#'   \item{pathway_name}{Full name of the pathway}
#'   \item{ko_id}{KEGG Orthology identifier (e.g., "K00001")}
#'   \item{ko_description}{Description of the KO}
#'   \item{ec_number}{EC number associated with the KO (if applicable)}
#'   \item{level1}{KEGG pathway hierarchy Level 1 classification}
#'   \item{level2}{KEGG pathway hierarchy Level 2 classification}
#'   \item{level3}{KEGG pathway hierarchy Level 3 classification}
#' }
#'
#' @details
#' This reference data is used by the \code{\link{ko2kegg_abundance}} function to convert
#' KO abundance data to KEGG pathway abundance. The data is stored internally and does not
#' require internet connectivity to use.
#'
#' The dataset covers major KEGG pathway categories including:
#' \itemize{
#'   \item Metabolism
#'   \item Genetic Information Processing
#'   \item Environmental Information Processing
#'   \item Cellular Processes
#'   \item Organismal Systems
#'   \item Human Diseases
#' }
#'
#' @source KEGG database (\url{https://www.kegg.jp/})
#'
#' @examples
#' # Load the reference data
#' data(ko_to_kegg_reference)
#'
#' # View structure
#' str(ko_to_kegg_reference)
#'
#' # Get unique pathways
#' unique_pathways <- unique(ko_to_kegg_reference$pathway_id)
#' length(unique_pathways)
#'
#' # Find KOs for a specific pathway
#' glycolysis_kos <- ko_to_kegg_reference[ko_to_kegg_reference$pathway_id == "ko00010", ]
#' head(glycolysis_kos)
#'
#' @seealso \code{\link{ko2kegg_abundance}} for converting KO abundance to pathway abundance
#'
"ko_to_kegg_reference"

# Shared helper: create mock GSEA results for testing
# pathway_name defaults to pathway_id (unannotated); callers can override
create_gsea_test_results <- function(n_pathways = 10) {
  set.seed(123)
  data.frame(
    pathway_id = paste0("path:ko", sprintf("%05d", 1:n_pathways)),
    pathway_name = paste0("path:ko", sprintf("%05d", 1:n_pathways)),
    size = sample(10:100, n_pathways, replace = TRUE),
    ES = runif(n_pathways, -0.8, 0.8),
    NES = runif(n_pathways, -2, 2),
    pvalue = runif(n_pathways, 0, 0.1),
    p.adjust = runif(n_pathways, 0, 0.2),
    leading_edge = replicate(n_pathways, paste(paste0("K", sprintf("%05d", sample(1:1000, 5))), collapse = ";")),
    method = rep("fgsea", n_pathways),
    stringsAsFactors = FALSE
  )
}

#' Metadata for ggpicrust2 Demonstration
#'
#' This is a demonstration dataset from the ggpicrust2 package. It provides the metadata
#' required for the demonstration functions in the package. The dataset includes
#' environmental information for each sample.
#'
#' @format A tibble with each row representing metadata for a sample.
#' \describe{
#'   \item{Sample1}{Metadata for Sample1, including Environment}
#'   \item{Sample2}{Metadata for Sample2, including Environment}
#'   \item{...}{...}
#' }
#' @source ggpicrust2 package demonstration.
#' @references Douglas GM, Maffei VJ, Zaneveld J, Yurgel SN, Brown JR, Taylor CM, Huttenhower C, Langille MGI. PICRUSt2 for prediction of metagenome functions. Nat Biotechnol. 2020.
"metadata"

#' Import Differential Abundance Analysis (DAA) results from MicrobiomeAnalyst
#'
#' This function imports DAA results from an external platform such as MicrobiomeAnalyst. It can be used to compare the results obtained from different platforms.
#' @name import_MicrobiomeAnalyst_daa_results
#' @param file_path a character string specifying the path to the CSV file containing the DAA results from MicrobiomeAnalyst. If this parameter is NULL and no data frame is provided, an error will be thrown. Default is NULL.
#' @param data a data frame containing the DAA results from MicrobiomeAnalyst. If this parameter is NULL and no file path is provided, an error will be thrown. Default is NULL.
#' @param method a character string specifying the method used for the DAA. This will be added as a new column in the returned data frame. Default is "MicrobiomeAnalyst".
#' @param group_levels a character vector specifying the group levels for the DAA. This will be added as new columns in the returned data frame. Default is c("control", "treatment").
#'
#' @return a data frame containing the DAA results from MicrobiomeAnalyst with additional columns for the method and group levels.
#'
#' @examples
#' \dontrun{
#' # Assuming you have a CSV file named "DAA_results.csv" in your current directory
#' daa_results <- import_MicrobiomeAnalyst_daa_results(file_path = "DAA_results.csv")
#' }
#'
#' @export
import_MicrobiomeAnalyst_daa_results <- function(file_path = NULL,
                                                data = NULL,
                                                method = "MicrobiomeAnalyst",
                                                group_levels = c("control", "treatment")) {
  if (!is.null(data) && !is.null(file_path)) {
    warning("Both file_path and data were provided. Using data and ignoring file_path.",
            call. = FALSE)
  }

  if (is.null(data)) {
    if (is.null(file_path)) {
      stop("Please provide either a file_path or a data frame.")
    }
    if (!file.exists(file_path)) {
      stop("file_path does not exist: ", file_path, call. = FALSE)
    }
    data <- utils::read.csv(file_path, stringsAsFactors = FALSE)
  }

  validate_dataframe(data, param_name = "data")

  if (!is.character(group_levels) || length(group_levels) < 1 || anyNA(group_levels)) {
    stop("group_levels must be a non-empty character vector without NA values.",
         call. = FALSE)
  }

  required_output_cols <- c("feature", "p_values", "p_adjust", "Statistics")
  if (ncol(data) < length(required_output_cols)) {
    stop("MicrobiomeAnalyst DAA results must contain at least four columns: ",
         paste(required_output_cols, collapse = ", "),
         call. = FALSE)
  }

  data <- as.data.frame(data, stringsAsFactors = FALSE)
  names(data)[seq_along(required_output_cols)] <- required_output_cols

  # Create a new column for method
  data$method <- method

  # Create new columns for group levels
  for (i in seq_along(group_levels)) {
    data[paste0("group", i)] <- group_levels[i]
  }

  return(data)
}

# Tests for find_sample_column / samples_match heuristics in data_utils.R

test_that("find_sample_column uses standard names when available", {
  fsc <- getFromNamespace("find_sample_column", "ggpicrust2")
  meta <- data.frame(sample = paste0("S", 1:4),
                     group = c("a", "a", "b", "b"),
                     stringsAsFactors = FALSE)
  expect_equal(fsc(meta, paste0("S", 1:4)), "sample")
})

test_that("find_sample_column rejects a standard-named column with duplicate values", {
  # Regression: Priority 1 used to pick a standard-named column (e.g.
  # "samples") even when its values were duplicated, simply because the
  # column name was on the standard list. That silently misaligned the
  # downstream analysis. Require uniqueness here too, falling through
  # to Priority 2 when the standard-named column is not a valid sample id.
  fsc <- getFromNamespace("find_sample_column", "ggpicrust2")

  meta <- data.frame(
    sample  = c("S1", "S1", "S2", "S2"),   # duplicates -> not a valid id
    real_id = c("S1", "S2", "S3", "S4"),   # correct sample id
    stringsAsFactors = FALSE
  )
  expect_equal(fsc(meta, paste0("S", 1:4)), "real_id")
})

test_that("find_sample_column rejects categorical columns at Priority 2", {
  # Regression: the scan-every-column fallback used a 50% overlap threshold
  # with no uniqueness check, so a categorical column whose levels happened
  # to share a few strings with the sample names would be mistakenly picked
  # as the sample identifier. Require uniqueness + >= 90% overlap.
  fsc <- getFromNamespace("find_sample_column", "ggpicrust2")

  meta <- data.frame(
    subject_label = c("S1", "S1", "S2", "S2"),  # duplicates -> not a sample id
    my_samples    = c("S1", "S2", "S3", "S4"),  # unique, correct
    stringsAsFactors = FALSE
  )
  # `my_samples` is the only real sample column; `subject_label` must be
  # rejected despite sharing values with abundance_samples.
  expect_equal(fsc(meta, paste0("S", 1:4)), "my_samples")
})

test_that("find_sample_column rejects low-overlap coincidental columns", {
  fsc <- getFromNamespace("find_sample_column", "ggpicrust2")

  meta <- data.frame(
    note = c("S1", "other1", "other2", "other3"),  # only 25% overlap
    real_sample = c("S1", "S2", "S3", "S4"),
    stringsAsFactors = FALSE
  )
  expect_equal(fsc(meta, paste0("S", 1:4)), "real_sample")
})

test_that("find_sample_column uses rownames only with near-complete overlap", {
  fsc <- getFromNamespace("find_sample_column", "ggpicrust2")

  meta <- data.frame(group = c("a", "a", "b", "b"),
                     stringsAsFactors = FALSE)
  rownames(meta) <- paste0("S", 1:4)
  expect_equal(fsc(meta, paste0("S", 1:4)), ".rownames")

  # Default integer rownames should not match arbitrary sample names even
  # though intersect() might find a stray match.
  meta2 <- data.frame(group = c("a", "a", "b", "b"),
                      stringsAsFactors = FALSE)
  expect_null(fsc(meta2, paste0("S", 1:4)))
})

---
title: "Gene Set Enrichment Analysis with ggpicrust2"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Gene Set Enrichment Analysis with ggpicrust2}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
```

## Introduction

This vignette demonstrates how to perform Gene Set Enrichment Analysis (GSEA) on PICRUSt2 predicted functional data using the ggpicrust2 package. GSEA is a powerful method for interpreting gene expression data by focusing on gene sets (pathways) rather than individual genes. In the context of microbiome functional prediction, GSEA can help identify pathways that are enriched in different conditions.

## Installation

First, make sure you have the necessary packages installed:

```{r setup, eval=FALSE}
# Install ggpicrust2
if (!requireNamespace("ggpicrust2", quietly = TRUE)) {
  devtools::install_github("cafferychen777/ggpicrust2")
}

# Install required Bioconductor packages
if (!requireNamespace("BiocManager", quietly = TRUE)) {
  install.packages("BiocManager")
}

BiocManager::install(c("limma", "fgsea", "clusterProfiler", "enrichplot", "DOSE", "pathview"))

# Load the package
library(ggpicrust2)
library(dplyr)
library(ggplot2)
```

## Method Selection Guide

The `pathway_gsea()` function supports multiple methods for gene set enrichment analysis:

| Method | Type | Covariate Support | Description |
|--------|------|-------------------|-------------|
| `camera` (default) | Competitive | ✅ Yes | **Recommended.** Accounts for inter-gene correlations, providing more reliable p-values |
| `fry` | Self-contained | ✅ Yes | Fast rotation test, efficient for large gene set collections |
| `fgsea` | Preranked | ❌ No | Fast preranked GSEA, but p-values may be unreliable |
| `clusterProfiler` | Preranked | ❌ No | Traditional GSEA implementation |

**Why use camera/fry?** Wu et al. (2012) demonstrated that preranked GSEA methods can produce "spectacularly wrong p-values" due to not accounting for inter-gene correlations. The `camera` and `fry` methods from limma address this limitation.

## Basic GSEA Analysis (Recommended: camera method)

Let's start with a basic GSEA analysis using the recommended `camera` method:

```{r basic-gsea, eval=FALSE}
# Load example data
data(ko_abundance)
data(metadata)

# Prepare abundance data
abundance_data <- as.data.frame(ko_abundance)
rownames(abundance_data) <- abundance_data[, "#NAME"]
abundance_data <- abundance_data[, -1]

# Run GSEA analysis with camera (recommended)
gsea_results <- pathway_gsea(
  abundance = abundance_data,
  metadata = metadata,
  group = "Environment",
  pathway_type = "KEGG",
  method = "camera",
  min_size = 5,
  max_size = 500,
  p.adjust = "BH"
)

# View the top results
head(gsea_results)
```

## Covariate Adjustment

One of the most powerful features of the `camera` and `fry` methods is the ability to adjust for confounding variables. This is particularly important in microbiome studies where factors like age, sex, BMI, and batch effects can influence results.

```{r covariate-gsea, eval=FALSE}
# Example with covariate adjustment
# Assuming metadata has columns: group, age, sex, BMI

gsea_results_adjusted <- pathway_gsea(
  abundance = abundance_data,
  metadata = metadata,
  group = "Disease",
  covariates = c("age", "sex", "BMI"),  # Adjust for these confounders
  pathway_type = "KEGG",
  method = "camera"
)

# The results now reflect the group effect after adjusting for confounders
head(gsea_results_adjusted)
```

## Fast Analysis with fry

For large datasets or when computational efficiency is important, use the `fry` method:

```{r fry-gsea, eval=FALSE}
# Fast rotation gene set test
gsea_results_fry <- pathway_gsea(
  abundance = abundance_data,
  metadata = metadata,
  group = "Environment",
  pathway_type = "KEGG",
  method = "fry",
  min_size = 5,
  max_size = 500
)

head(gsea_results_fry)
```

## Legacy: Preranked GSEA (fgsea)

For compatibility with existing workflows, preranked GSEA methods are still available:

```{r fgsea, eval=FALSE}
# Note: Preranked methods don't account for inter-gene correlations
# P-values may be less reliable (Wu et al., 2012)
gsea_results_fgsea <- pathway_gsea(
  abundance = abundance_data,
  metadata = metadata,
  group = "Environment",
  pathway_type = "KEGG",
  method = "fgsea",
  rank_method = "signal2noise",
  nperm = 1000,
  min_size = 10,
  max_size = 500,
  p.adjust = "BH",
  seed = 42
)

# View the top results
head(gsea_results_fgsea)
```

## Annotating GSEA Results

To make the results more interpretable, we can annotate them with pathway names and descriptions:

```{r annotate-gsea, eval=FALSE}
# Annotate GSEA results
annotated_results <- gsea_pathway_annotation(
  gsea_results = gsea_results,
  pathway_type = "KEGG"
)

# View the annotated results
head(annotated_results)
```

## Visualizing GSEA Results

The ggpicrust2 package provides several visualization options for GSEA results. The `visualize_gsea()` function automatically detects whether pathway names are available (from `gsea_pathway_annotation()`) and uses them for better readability, falling back to pathway IDs if names are not available.

### Pathway Label Options

The `visualize_gsea()` function offers flexible pathway labeling:

```{r pathway-labels, eval=FALSE}
# Option 1: Use raw GSEA results (shows pathway IDs)
plot_with_ids <- visualize_gsea(
  gsea_results = gsea_results,
  plot_type = "barplot",
  n_pathways = 10
)

# Option 2: Use annotated results (automatically shows pathway names)
plot_with_names <- visualize_gsea(
  gsea_results = annotated_results,
  plot_type = "barplot",
  n_pathways = 10
)

# Option 3: Explicitly specify which column to use for labels
plot_custom_labels <- visualize_gsea(
  gsea_results = annotated_results,
  plot_type = "barplot",
  pathway_label_column = "pathway_name",
  n_pathways = 10
)

# Compare the plots
plot_with_ids
plot_with_names
plot_custom_labels
```

### Barplot

```{r barplot, eval=FALSE}
# Create a barplot of the top enriched pathways
barplot <- visualize_gsea(
  gsea_results = annotated_results,
  plot_type = "barplot",
  n_pathways = 20,
  sort_by = "p.adjust"
)

# Display the plot
barplot
```

### Dotplot

```{r dotplot, eval=FALSE}
# Create a dotplot of the top enriched pathways
dotplot <- visualize_gsea(
  gsea_results = annotated_results,
  plot_type = "dotplot",
  n_pathways = 20,
  sort_by = "p.adjust"
)

# Display the plot
dotplot
```

### Enrichment Plot

```{r enrichment-plot, eval=FALSE}
# Create an enrichment plot for a specific pathway
enrichment_plot <- visualize_gsea(
  gsea_results = annotated_results,
  plot_type = "enrichment_plot",
  n_pathways = 10,
  sort_by = "NES"
)

# Display the plot
enrichment_plot
```

### Ridge Plot

Ridge plots (also known as joy plots) provide a powerful way to visualize the distribution of gene abundances or fold changes within enriched pathways. This helps understand whether pathways are predominantly up- or down-regulated.

```{r ridge-plot, eval=FALSE}
# Create a ridge plot for GSEA results
# Note: Requires ggridges package to be installed
ridge_plot <- pathway_ridgeplot(
  gsea_results = gsea_results,
  abundance = abundance_data,
  metadata = metadata,
  group = "Environment",
  pathway_type = "KEGG",
  n_pathways = 10,
  sort_by = "p.adjust",
  show_direction = TRUE,
  colors = c("Down" = "#3182bd", "Up" = "#de2d26")
)

# Display the plot
ridge_plot
```

The ridge plot shows:
- Each pathway as a density ridge
- Color indicates enrichment direction (Up = red, Down = blue)
- The distribution of log2 fold changes for genes within each pathway
- A vertical dashed line at 0 for reference

## Comparing GSEA and DAA Results

It can be informative to compare the results from GSEA with those from Differential Abundance Analysis (DAA):

```{r compare-gsea-daa, eval=FALSE}
# Run DAA analysis
daa_results <- pathway_daa(
  abundance = abundance_data,
  metadata = metadata,
  group = "Environment",
  daa_method = "ALDEx2"
)

# Annotate DAA results
annotated_daa_results <- pathway_annotation(
  pathway = "KO",
  daa_results_df = daa_results,
  ko_to_kegg = TRUE
)

# Compare GSEA and DAA results
comparison <- compare_gsea_daa(
  gsea_results = annotated_results,
  daa_results = annotated_daa_results,
  plot_type = "venn",
  p_threshold = 0.05
)

# Display the comparison plot
comparison$plot

# View the comparison results
comparison$results
```

## Conclusion

Gene Set Enrichment Analysis provides a complementary approach to Differential Abundance Analysis for interpreting PICRUSt2 predicted functional data. By focusing on pathways rather than individual features, GSEA can help identify biologically meaningful patterns that might be missed by traditional methods.

### Key Recommendations

1. **Use `camera` method (default)** for most analyses - it provides more reliable p-values by accounting for inter-gene correlations

2. **Adjust for covariates** when you have confounding factors like age, sex, or BMI in your study design

3. **Use `fry` method** when you need faster computation for large gene set collections

4. **Use preranked methods (`fgsea`)** only when you need compatibility with existing workflows, and interpret p-values with caution

### References

- Wu, D., & Smyth, G. K. (2012). Camera: a competitive gene set test accounting for inter-gene correlation. *Nucleic Acids Research*, 40(17), e133.
- Wu, D., et al. (2010). ROAST: rotation gene set tests for complex microarray experiments. *Bioinformatics*, 26(17), 2176-2182.

The ggpicrust2 package now offers a comprehensive suite of tools for both DAA and GSEA analyses, making it easier to gain insights from microbiome functional prediction data.

---
title: "Using ggpicrust2"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Using ggpicrust2}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  fig.width = 7,
  fig.height = 5
)
```

## Introduction

`ggpicrust2` provides a practical workflow for PICRUSt2 downstream analysis:

- convert KO profiles to pathway-level abundance when needed
- run differential abundance analysis with multiple methods
- annotate and visualize pathway-level results
- inspect taxa-level contribution using PICRUSt2 per-sequence outputs
- perform GSEA when pathway-set analysis is more appropriate than single-feature testing

This vignette focuses on the general package workflow. For a deeper GSEA walkthrough, see the dedicated `gsea_analysis` vignette.

## Installation

```{r installation, eval = FALSE}
# install.packages("devtools")
devtools::install_github("cafferychen777/ggpicrust2")

library(ggpicrust2)
library(tibble)
```

## One-command workflow

If you want a fast end-to-end run from abundance data to annotated differential abundance output, start with `ggpicrust2()`:

```{r one-command, eval = FALSE}
data("ko_abundance")
data("metadata")

results <- ggpicrust2(
  data = ko_abundance,
  metadata = metadata,
  group = "Environment",
  pathway = "KO",
  daa_method = "LinDA",
  ko_to_kegg = TRUE,
  order = "pathway_class",
  p_values_bar = TRUE,
  x_lab = "pathway_name"
)

# Access the main outputs
results[[1]]$plot
head(results[[1]]$results)
```

Use this route when you want a fast default analysis. Use the stepwise workflow below when you need more control over data preparation or visualization.

## Stepwise pathway workflow

### Convert KO abundance to KEGG pathway abundance

```{r ko-to-kegg, eval = FALSE}
kegg_pathway_abundance <- ko2kegg_abundance(data = ko_abundance)
head(kegg_pathway_abundance[, 1:3])
```

### Run differential abundance analysis

```{r daa, eval = FALSE}
daa_results <- pathway_daa(
  abundance = kegg_pathway_abundance,
  metadata = metadata,
  group = "Environment",
  daa_method = "ALDEx2"
)

head(daa_results)
```

### Annotate pathway results

```{r annotation, eval = FALSE}
annotated_daa <- pathway_annotation(
  pathway = "KO",
  daa_results_df = daa_results,
  ko_to_kegg = TRUE
)

head(annotated_daa)
```

### Visualize pathway-level results

```{r errorbar, eval = FALSE}
pathway_errorbar(
  abundance = kegg_pathway_abundance,
  daa_results_df = annotated_daa,
  Group = "Environment"
)
```

```{r heatmap-pca, eval = FALSE}
sig_pathways <- annotated_daa$feature[annotated_daa$p_adjust < 0.05]

if (length(sig_pathways) > 0) {
  pathway_heatmap(
    abundance = kegg_pathway_abundance[sig_pathways, , drop = FALSE],
    metadata = metadata,
    group = "Environment"
  )
}

pathway_pca(
  abundance = kegg_pathway_abundance,
  metadata = metadata,
  group = "Environment"
)
```

## Taxa contribution workflow

PICRUSt2 can output contribution files that explain which taxa are driving predicted functional shifts. `ggpicrust2` supports both gene-family-level and pathway-level contribution workflows.

### Read PICRUSt2 contribution files

```{r contrib-readers, eval = FALSE}
# For pred_metagenome_contrib.tsv
contrib_data <- read_contrib_file("pred_metagenome_contrib.tsv")

# For path_abun_contrib.tsv.gz; PICRUSt2 pathway output commonly uses MetaCyc IDs.
path_contrib_data <- read_pathway_contrib_file("path_abun_contrib.tsv.gz")

# For pred_metagenome_strat.tsv
strat_data <- read_strat_file("pred_metagenome_strat.tsv")
```

### Aggregate to a taxonomic level

```{r contrib-aggregate, eval = FALSE}
taxa_contrib <- aggregate_taxa_contributions(
  contrib_data = contrib_data,
  taxonomy = your_taxonomy_table,
  tax_level = "Genus",
  top_n = 10,
  daa_results_df = daa_results
)

head(taxa_contrib)
```

`aggregate_taxa_contributions()` accepts either:

- normalized contribution data from `read_contrib_file()`
- pathway-level contribution data from `read_pathway_contrib_file()`
- stratified abundance data from `read_strat_file()`

Use `daa_results_df` or `pathway_ids` when you want to focus only on pathways that were significant in your pathway-level analysis.

When you already have pathway-level contribution data and do not want to run differential abundance analysis, omit `daa_results_df` and aggregate directly. For PICRUSt2 pathway contribution output, use matching MetaCyc annotations rather than KEGG pathway annotations.

```{r pathway-contrib-no-daa, eval = FALSE}
path_taxa_contrib <- aggregate_taxa_contributions(
  contrib_data = path_contrib_data,
  taxonomy = your_taxonomy_table,
  tax_level = "Genus",
  top_n = 10
)

pathway_annotation_df <- pathway_annotation(
  data = data.frame(function_id = unique(path_taxa_contrib$function_id)),
  pathway = "MetaCyc"
)
```

### Visualize taxa-level drivers

```{r contrib-plots, eval = FALSE}
taxa_contribution_bar(
  contrib_agg = taxa_contrib,
  metadata = metadata,
  group = "Environment",
  facet_by = "function"
)

taxa_contribution_heatmap(
  contrib_agg = taxa_contrib,
  n_functions = 20
)
```

This step is useful when pathway-level significance is not enough and you need to identify which taxa are contributing to the change.

## GSEA workflow

Use GSEA when you want pathway-set level inference from KO or EC abundance rather than testing each pathway independently.

```{r gsea, eval = FALSE}
gsea_results <- pathway_gsea(
  abundance = ko_abundance %>% column_to_rownames("#NAME"),
  metadata = metadata,
  group = "Environment",
  pathway_type = "KEGG",
  method = "camera"
)

annotated_gsea <- gsea_pathway_annotation(
  gsea_results = gsea_results,
  pathway_type = "KEGG"
)

visualize_gsea(
  gsea_results = annotated_gsea,
  plot_type = "barplot",
  n_pathways = 15
)
```

For a method-by-method GSEA explanation, covariate adjustment, and comparison with DAA, see the `gsea_analysis` vignette.

## Summary

The package is easiest to use when you choose the shortest path that matches your question:

- use `ggpicrust2()` for a fast default pathway workflow
- use the stepwise DAA functions when you need more control
- use the taxa contribution workflow when you need taxon-level attribution
- use `pathway_gsea()` when pathway-set enrichment is the primary question

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/color_themes.R
\name{create_gradient_colors}
\alias{create_gradient_colors}
\title{Create Gradient Colors}
\usage{
create_gradient_colors(theme_name = "default", n_colors = 11, diverging = TRUE)
}
\arguments{
\item{theme_name}{Character string specifying the theme}

\item{n_colors}{Number of colors in the gradient}

\item{diverging}{Whether to create a diverging gradient (for fold changes)}
}
\value{
A vector of colors
}
\description{
Creates gradient colors for fold change visualization
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/taxa_contribution.R
\name{aggregate_taxa_contributions}
\alias{aggregate_taxa_contributions}
\title{Aggregate taxa contributions for visualization}
\usage{
aggregate_taxa_contributions(
  contrib_data,
  taxonomy = NULL,
  tax_level = "Genus",
  top_n = 10,
  daa_results_df = NULL,
  pathway_ids = NULL,
  p_threshold = 0.05
)
}
\arguments{
\item{contrib_data}{A data.frame from \code{\link{read_contrib_file}} or
\code{\link{read_strat_file}}.}

\item{taxonomy}{Optional data.frame mapping taxon IDs to taxonomy. Supports
QIIME2 format (semicolon-delimited taxonomy strings) or DADA2 format
(separate columns for each rank).}

\item{tax_level}{Character. Taxonomic rank for aggregation. One of
\code{"Kingdom"}, \code{"Phylum"}, \code{"Class"}, \code{"Order"},
\code{"Family"}, \code{"Genus"}, \code{"Species"}. Default \code{"Genus"}.}

\item{top_n}{Integer. Number of top taxa to keep; remaining are lumped as
"Other". Default 10.}

\item{daa_results_df}{Optional data.frame from \code{\link{pathway_daa}},
used to filter contributions to significant pathways.}

\item{pathway_ids}{Optional character vector of pathway IDs to filter.
Alternative to \code{daa_results_df}.}

\item{p_threshold}{Numeric. Significance cutoff when using
\code{daa_results_df}. Default 0.05.}
}
\value{
A tidy data.frame with columns: \code{sample}, \code{function_id},
  \code{taxon_label}, \code{contribution}.
}
\description{
Core aggregation function that bridges PICRUSt2 contribution data with
differential abundance analysis results. Optionally maps ASV/OTU IDs to
taxonomic names and filters to significant pathways.
}
\details{
When \code{daa_results_df} is provided, the function:
\enumerate{
  \item Extracts significant pathway IDs from the DAA results
  \item Maps pathway IDs to their constituent KO IDs using the internal
    ko_to_kegg reference
  \item Filters contribution data to only matching KO IDs
}

Taxonomy can be provided in two formats:
\itemize{
  \item QIIME2: A column named \code{Taxon} or \code{taxonomy} containing
    semicolon-delimited strings (e.g., "k__Bacteria;p__Firmicutes;...")
  \item DADA2: Separate columns for each rank (\code{Kingdom}, \code{Phylum}, etc.)
}
}
\examples{
\donttest{
# Basic usage with synthetic data
contrib <- data.frame(
  sample = rep(c("S1", "S2"), each = 6),
  function_id = rep(c("K00001", "K00002", "K00003"), 4),
  taxon = rep(c("ASV1", "ASV2"), each = 3, times = 2),
  taxon_function_abun = runif(12),
  norm_taxon_function_contrib = runif(12)
)
agg <- aggregate_taxa_contributions(contrib, top_n = 2)
head(agg)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/daa_results_df.R
\docType{data}
\name{daa_results_df}
\alias{daa_results_df}
\title{DAA Results Dataset}
\format{
A data frame with columns:
\describe{
  \item{adj_method}{Method used for p-value adjustment.}
  \item{feature}{The feature (pathway) being compared.}
  \item{group1}{The first group in the comparison.}
  \item{group2}{The second group in the comparison.}
  \item{method}{The method used for the comparison.}
  \item{p_adjust}{The adjusted p-value from the comparison.}
  \item{p_values}{The raw p-value from the comparison.}
}
}
\source{
From ggpicrust2 package demonstration.
}
\usage{
daa_results_df
}
\description{
This dataset is the result of processing 'kegg_abundance' through the 'LinDA' method in the 'pathway_daa' function.
It includes information about the feature, groups compared, p values, and method used.
}
\references{
Douglas GM, Maffei VJ, Zaneveld J, Yurgel SN, Brown JR, Taylor CM, Huttenhower C, Langille MGI. PICRUSt2 for prediction of metagenome functions. Nat Biotechnol. 2020.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/legend_annotation_utils.R
\name{calculate_smart_text_size}
\alias{calculate_smart_text_size}
\title{Smart Text Size Calculator}
\usage{
calculate_smart_text_size(n_items, base_size = 10, min_size = 8, max_size = 14)
}
\arguments{
\item{n_items}{Number of items to display}

\item{base_size}{Base text size}

\item{min_size}{Minimum text size}

\item{max_size}{Maximum text size}
}
\value{
Calculated text size
}
\description{
Smart Text Size Calculator
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/legend_annotation_utils.R
\name{create_legend_theme}
\alias{create_legend_theme}
\title{Create Enhanced Legend Theme}
\usage{
create_legend_theme(
  position = "top",
  direction = "horizontal",
  title = NULL,
  title_size = 12,
  text_size = 10,
  key_size = 0.8,
  key_width = NULL,
  key_height = NULL,
  ncol = NULL,
  nrow = NULL,
  box_just = "center",
  margin = ggplot2::margin(0, 0, 0, 0)
)
}
\arguments{
\item{position}{Legend position ("top", "bottom", "left", "right", "none")}

\item{direction}{Legend direction ("horizontal", "vertical")}

\item{title}{Legend title}

\item{title_size}{Title font size}

\item{text_size}{Text font size}

\item{key_size}{Key size in cm}

\item{key_width}{Key width}

\item{key_height}{Key height}

\item{ncol}{Number of columns}

\item{nrow}{Number of rows}

\item{box_just}{Legend box justification}

\item{margin}{Legend margin}
}
\value{
ggplot2 theme elements
}
\description{
Create Enhanced Legend Theme
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathway_gsea.R
\name{build_design_matrix}
\alias{build_design_matrix}
\title{Build design matrix for limma analysis}
\usage{
build_design_matrix(metadata, group, covariates = NULL)
}
\arguments{
\item{metadata}{A data frame containing sample metadata}

\item{group}{A character string specifying the grouping variable column name}

\item{covariates}{A character vector of covariate column names (optional)}
}
\value{
A design matrix suitable for limma
}
\description{
Creates a design matrix incorporating the group variable and optional covariates.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_utils.R
\name{validate_daa_results}
\alias{validate_daa_results}
\title{Validate DAA results data frame}
\usage{
validate_daa_results(
  daa_results_df,
  require_single_method = TRUE,
  require_single_group_pair = TRUE
)
}
\arguments{
\item{daa_results_df}{Data frame containing DAA results}

\item{require_single_method}{Logical. If TRUE, requires exactly one method}

\item{require_single_group_pair}{Logical. If TRUE, requires exactly one group1/group2 pair}
}
\value{
Invisible TRUE if validation passes
}
\description{
Validates that a DAA results data frame meets requirements for visualization
functions (single method, single group pair).
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_utils.R
\name{data_utils}
\alias{data_utils}
\title{Data Utilities for ggpicrust2}
\description{
Internal utility functions for data preprocessing, sample matching,
and format standardization.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggpicrust2.R
\name{ggpicrust2}
\alias{ggpicrust2}
\title{This function integrates pathway name/description annotations, ten of the most advanced differential abundance (DA) methods, and visualization of DA results.}
\usage{
ggpicrust2(
  file = NULL,
  data = NULL,
  metadata,
  group,
  pathway,
  daa_method = "ALDEx2",
  ko_to_kegg = FALSE,
  filter_for_prokaryotes = TRUE,
  p_adjust_method = "BH",
  order = "group",
  p_values_bar = TRUE,
  x_lab = NULL,
  select = NULL,
  reference = NULL,
  colors = NULL,
  p_values_threshold = 0.05,
  p.adjust = NULL
)
}
\arguments{
\item{file}{A character string representing the file path of the input file containing KO abundance data in picrust2 export format. The input file should have KO identifiers in the first column and sample identifiers in the first row. The remaining cells should contain the abundance values for each KO-sample pair.}

\item{data}{An optional data.frame containing KO abundance data in the same format as the input file. If provided, the function will use this data instead of reading from the file. By default, this parameter is set to NULL.}

\item{metadata}{A tibble, consisting of sample information}

\item{group}{A character, name of the group}

\item{pathway}{A character, consisting of "EC", "KO", "MetaCyc"}

\item{daa_method}{a character specifying the method for differential abundance analysis, default is "ALDEx2", choices are:
- "ALDEx2": ANOVA-Like Differential Expression tool for high throughput sequencing data
- "DESeq2": Differential expression analysis based on the negative binomial distribution using DESeq2
- "edgeR": Exact test for differences between two groups of negative-binomially distributed counts using edgeR
- "limma voom": Limma-voom framework for the analysis of RNA-seq data
- "metagenomeSeq": Fit logistic regression models to test for differential abundance between groups using metagenomeSeq
- "LinDA": Linear models for differential abundance analysis of microbiome compositional data
- "Maaslin2": Multivariate Association with Linear Models (MaAsLin2) for differential abundance analysis}

\item{ko_to_kegg}{Logical or logical-like string controlling conversion of
KO abundance to KEGG pathway abundance.}

\item{filter_for_prokaryotes}{Logical. If TRUE (default), filters out KEGG pathways
that are specific to eukaryotes (e.g., human diseases, organismal systems) when
ko_to_kegg = TRUE. Set to FALSE to include all KEGG pathways.}

\item{p_adjust_method}{A character specifying the method for p-value adjustment,
default is "BH".}

\item{order}{A character to control the order of the main plot rows}

\item{p_values_bar}{A character to control if the main plot has the p_values bar}

\item{x_lab}{A character to control the x-axis label name, you can choose from "feature","pathway_name" and "description"}

\item{select}{A vector consisting of pathway names to be selected}

\item{reference}{A character, a reference group level for several DA methods}

\item{colors}{A vector consisting of colors number}

\item{p_values_threshold}{A numeric value specifying the threshold for statistical
significance of differential abundance. Pathways with adjusted p-values below this
threshold will be displayed in the plot. Default is 0.05.}

\item{p.adjust}{a character specifying the method for p-value adjustment, default is "BH", choices are:
- "BH": Benjamini-Hochberg correction
- "holm": Holm's correction
- "bonferroni": Bonferroni correction
- "hochberg": Hochberg's correction
- "fdr": False discovery rate correction
- "none": No p-value adjustment.}
}
\value{
A list containing:
\itemize{
  \item Numbered elements (1, 2, ...): Sub-lists for each DA method, each containing:
    \itemize{
      \item \code{plot}: A ggplot2 error bar plot visualizing the differential abundance results
      \item \code{results}: A data frame of differential abundance results for that method
    }
  \item \code{abundance}: The processed abundance data (KEGG pathway or original) for downstream analysis
  \item \code{metadata}: The metadata data frame
  \item \code{group}: The group variable name used in the analysis
  \item \code{daa_results_df}: The complete annotated DAA results data frame
  \item \code{ko_to_kegg}: Logical indicating whether KO to KEGG conversion was performed
}
These additional fields allow seamless integration with \code{\link{pathway_pca}} and
\code{\link{pathway_heatmap}} for further visualization without re-preparing data.
}
\description{
This function integrates pathway name/description annotations, ten of the most advanced differential abundance (DA) methods, and visualization of DA results.
}
\examples{
\dontrun{
# Load necessary data: abundance data and metadata
abundance_file <- "path/to/your/abundance_file.tsv"
metadata <- read.csv("path/to/your/metadata.csv")

# Run ggpicrust2 with input file path
results_file_input <- ggpicrust2(file = abundance_file,
                                 metadata = metadata,
                                 group = "your_group_column",
                                 pathway = "KO",
                                 daa_method = "LinDA",
                                 ko_to_kegg = "TRUE",
                                 order = "pathway_class",
                                 p_values_bar = TRUE,
                                 x_lab = "pathway_name")

# Run ggpicrust2 with imported data.frame
abundance_data <- read_delim(abundance_file, delim="\t", col_names=TRUE, trim_ws=TRUE)

# Run ggpicrust2 with input data
results_data_input <- ggpicrust2(data = abundance_data,
                                 metadata = metadata,
                                 group = "your_group_column",
                                 pathway = "KO",
                                 daa_method = "LinDA",
                                 ko_to_kegg = "TRUE",
                                 order = "pathway_class",
                                 p_values_bar = TRUE,
                                 x_lab = "pathway_name")

# Access the plot and results dataframe for the first DA method
example_plot <- results_file_input[[1]]$plot
example_results <- results_file_input[[1]]$results

# Use the example data in ggpicrust2 package
data(ko_abundance)
data(metadata)
results_file_input <- ggpicrust2(data = ko_abundance,
                                 metadata = metadata,
                                 group = "Environment",
                                 pathway = "KO",
                                 daa_method = "LinDA",
                                 ko_to_kegg = TRUE,
                                 order = "pathway_class",
                                 p_values_bar = TRUE,
                                 x_lab = "pathway_name")
# Analyze the EC or MetaCyc pathway
data(metacyc_abundance)
results_file_input <- ggpicrust2(data = metacyc_abundance,
                                 metadata = metadata,
                                 group = "Environment",
                                 pathway = "MetaCyc",
                                 daa_method = "LinDA",
                                 ko_to_kegg = FALSE,
                                 order = "group",
                                 p_values_bar = TRUE,
                                 x_lab = "description")

# Use the returned data for PCA analysis (no need to re-prepare data)
pca_plot <- pathway_pca(
  abundance = results_file_input$abundance,
  metadata = results_file_input$metadata,
  group = results_file_input$group
)

# Use the returned data for heatmap (filter significant pathways first)
sig_features <- results_file_input$daa_results_df \%>\%
  dplyr::filter(p_adjust < 0.05) \%>\%
  dplyr::pull(feature)
if (length(sig_features) > 0) {
  heatmap_plot <- pathway_heatmap(
    abundance = results_file_input$abundance[sig_features, , drop = FALSE],
    metadata = results_file_input$metadata,
    group = results_file_input$group
  )
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathway_daa.R
\name{pathway_daa}
\alias{pathway_daa}
\title{Differential Abundance Analysis for Predicted Functional Pathways}
\usage{
pathway_daa(
  abundance,
  metadata,
  group,
  daa_method = "ALDEx2",
  select = NULL,
  p_adjust_method = "BH",
  reference = NULL,
  include_abundance_stats = FALSE,
  include_effect_size = FALSE,
  p.adjust = NULL,
  ...
)
}
\arguments{
\item{abundance}{A data frame or matrix containing predicted functional pathway abundance,
with pathways/features as rows and samples as columns.
The column names should match the sample names in metadata.
Values should be counts or abundance measurements.}

\item{metadata}{A data frame or tibble containing sample information.
Must include a 'sample' column with sample identifiers matching the column names in abundance data.}

\item{group}{Character string specifying the column name in metadata that contains group information
for differential abundance analysis.}

\item{daa_method}{Character string specifying the method for differential abundance analysis.
Available choices are:
\itemize{
  \item \code{"ALDEx2"}: ANOVA-Like Differential Expression tool
  \item \code{"DESeq2"}: Differential expression analysis based on negative binomial distribution
  \item \code{"edgeR"}: Exact test for differences between groups using negative binomial model
  \item \code{"limma voom"}: Limma-voom framework for RNA-seq analysis
  \item \code{"metagenomeSeq"}: Zero-inflated Gaussian mixture model
  \item \code{"LinDA"}: Linear models for differential abundance analysis
  \item \code{"Maaslin2"}: Multivariate Association with Linear Models
  \item \code{"Lefser"}: Linear discriminant analysis effect size
}
Default is "ALDEx2".}

\item{select}{Vector of sample names to include in the analysis.
If NULL (default), all samples are included.}

\item{p_adjust_method}{Character string specifying the method for p-value adjustment.
Choices are:
\itemize{
  \item \code{"BH"}: Benjamini-Hochberg procedure (default)
  \item \code{"holm"}: Holm's step-down method
  \item \code{"bonferroni"}: Bonferroni correction
  \item \code{"hochberg"}: Hochberg's step-up method
  \item \code{"fdr"}: False Discovery Rate
  \item \code{"none"}: No adjustment
}}

\item{reference}{Character string specifying the reference level for the group comparison.
If NULL (default), the first level is used as reference.}

\item{include_abundance_stats}{Logical value indicating whether to include
abundance statistics (mean relative abundance, standard deviation,
and log2 fold change) in the output. Default is FALSE for backward
compatibility.}

\item{include_effect_size}{Logical value indicating whether to include
effect size information in the output for ALDEx2 analysis. When TRUE,
additional columns are added including effect_size, diff_btw,
log2_fold_change, rab_all, and overlap. Only applicable for two-group
comparisons with ALDEx2 method. Default is FALSE for backward
compatibility.}

\item{p.adjust}{Deprecated. Use \code{p_adjust_method} instead.}

\item{...}{Additional arguments passed to the specific DAA method}
}
\value{
A data frame containing the differential abundance analysis results. The structure of the results
depends on the chosen DAA method. For methods that support multi-group comparisons (like LinDA),
when there are more than two groups, the results will contain separate rows for each feature in each
pairwise comparison between the reference group and each non-reference group. The data frame includes
the following columns:
\itemize{
  \item \code{feature}: Feature/pathway identifier
  \item \code{method}: The DAA method used
  \item \code{group1}: Reference group
  \item \code{group2}: Comparison group
  \item \code{p_values}: P-values for the comparison
  \item \code{p_adjust}: Adjusted p-values
  \item \code{adj_method}: Method used for p-value adjustment
}

When \code{include_abundance_stats = TRUE}, the following additional columns
are included:
\itemize{
  \item \code{mean_rel_abundance_group1}: Mean relative abundance for group1
  \item \code{sd_rel_abundance_group1}: Standard deviation of relative
        abundance for group1
  \item \code{mean_rel_abundance_group2}: Mean relative abundance for group2
  \item \code{sd_rel_abundance_group2}: Standard deviation of relative
        abundance for group2
  \item \code{log2_fold_change}: Log2 fold change (group2/group1)
}

Some methods may provide additional columns, such as \code{log2_fold_change}
for effect size information.

When \code{include_effect_size = TRUE} and using ALDEx2 method with two groups,
the following additional effect size columns are included:
\itemize{
  \item \code{effect_size}: ALDEx2 effect size (median of the ratio of between-group
        difference and within-group variance)
  \item \code{diff_btw}: Median difference between groups in CLR space
  \item \code{log2_fold_change}: Log2 fold change (same as diff_btw for ALDEx2)
  \item \code{rab_all}: Median CLR abundance across all samples
  \item \code{overlap}: Proportion of effect size that is 0 or less
}
}
\description{
Performs differential abundance analysis on predicted functional pathway data using various statistical methods.
This function supports multiple methods for analyzing differences in pathway abundance between groups,
including popular approaches like ALDEx2, DESeq2, edgeR, and others.
}
\examples{
\donttest{
# Load example data
data(ko_abundance)
data(metadata)

# Prepare abundance data
abundance_data <- as.data.frame(ko_abundance)
rownames(abundance_data) <- abundance_data[, "#NAME"]
abundance_data <- abundance_data[, -1]

# Run differential abundance analysis using ALDEx2
results <- pathway_daa(
  abundance = abundance_data,
  metadata = metadata,
  group = "Environment"
)

# Using a different method (DESeq2)
deseq_results <- pathway_daa(
  abundance = abundance_data,
  metadata = metadata,
  group = "Environment",
  daa_method = "DESeq2"
)

# Create example data with more samples
abundance <- data.frame(
  sample1 = c(10, 20, 30),
  sample2 = c(20, 30, 40),
  sample3 = c(30, 40, 50),
  sample4 = c(40, 50, 60),
  sample5 = c(50, 60, 70),
  row.names = c("pathway1", "pathway2", "pathway3")
)

metadata <- data.frame(
  sample = c("sample1", "sample2", "sample3", "sample4", "sample5"),
  group = c("control", "control", "treatment", "treatment", "treatment")
)

# Run differential abundance analysis using ALDEx2
results <- pathway_daa(abundance, metadata, "group")

# Using a different method (limma voom instead of DESeq2 for this small example)
limma_results <- pathway_daa(abundance, metadata, "group",
                            daa_method = "limma voom")

# Analyze specific samples only
subset_results <- pathway_daa(abundance, metadata, "group",
                             select = c("sample1", "sample2", "sample3", "sample4"))

# Include effect size information for ALDEx2 analysis
aldex2_with_effect <- pathway_daa(abundance, metadata, "group",
                                 daa_method = "ALDEx2",
                                 include_effect_size = TRUE)

# The result will include additional columns: effect_size, diff_btw,
# log2_fold_change, rab_all, and overlap
head(aldex2_with_effect)
}

}
\references{
\itemize{
  \item ALDEx2: Fernandes et al. (2014) Unifying the analysis of high-throughput sequencing datasets:
        characterizing RNA-seq, 16S rRNA gene sequencing and selective growth experiments by
        compositional data analysis. Microbiome.
  \item DESeq2: Love et al. (2014) Moderated estimation of fold change and dispersion for
        RNA-seq data with DESeq2. Genome Biology.
  \item edgeR: Robinson et al. (2010) edgeR: a Bioconductor package for differential expression
        analysis of digital gene expression data. Bioinformatics.
  \item limma-voom: Law et al. (2014) voom: precision weights unlock linear model analysis tools
        for RNA-seq read counts. Genome Biology.
  \item metagenomeSeq: Paulson et al. (2013) Differential abundance analysis for microbial
        marker-gene surveys. Nature Methods.
  \item Maaslin2: Mallick et al. (2021) Multivariable Association Discovery in Population-scale
        Meta-omics Studies.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualize_gsea.R
\name{visualize_gsea}
\alias{visualize_gsea}
\title{Visualize GSEA results}
\usage{
visualize_gsea(
  gsea_results,
  plot_type = "enrichment_plot",
  n_pathways = 20,
  sort_by = "p.adjust",
  colors = NULL,
  abundance = NULL,
  metadata = NULL,
  group = NULL,
  network_params = list(),
  heatmap_params = list(),
  pathway_label_column = NULL,
  scale = NULL
)
}
\arguments{
\item{gsea_results}{A data frame containing GSEA results from the pathway_gsea function}

\item{plot_type}{A character string specifying the visualization type: "enrichment_plot", "dotplot", "barplot", "network", or "heatmap"}

\item{n_pathways}{An integer specifying the number of pathways to display}

\item{sort_by}{A character string specifying the sorting criterion: "NES", "pvalue", or "p.adjust"}

\item{colors}{A vector of colors for the visualization}

\item{abundance}{A data frame containing the original abundance data (required for heatmap visualization)}

\item{metadata}{A data frame containing sample metadata (required for heatmap visualization)}

\item{group}{A character string specifying the column name in metadata that contains the grouping variable (required for heatmap visualization)}

\item{network_params}{A list of parameters for network visualization}

\item{heatmap_params}{A list of parameters for heatmap visualization}

\item{pathway_label_column}{A character string specifying which column to use for pathway labels.
If NULL (default), the function will automatically use 'pathway_name' if available, otherwise 'pathway_id'.
This allows for custom labeling when using annotated GSEA results.}

\item{scale}{Optional palette/scale for customizing colors. Accepts: (1) a character vector of colors,
(2) a function that returns colors given an integer (e.g., viridisLite::viridis), or
(3) a ggplot2 scale object (e.g., ggplot2::scale_fill_gradientn(...)).
When NULL, defaults keep current behavior. Applies to: enrichment_plot (fill, continuous),
dotplot (color, continuous), barplot (fill, discrete Positive/Negative), network (color, diverging around 0),
heatmap (main heatmap col; row annotation stays default unless overridden in heatmap_params).}
}
\value{
A ggplot2 object or ComplexHeatmap object
}
\description{
This function creates various visualizations for Gene Set Enrichment Analysis (GSEA) results.
It automatically detects whether pathway names are available (from gsea_pathway_annotation())
and uses them for better readability, falling back to pathway IDs if names are not available.
}
\examples{
\dontrun{
# Load example data
data(ko_abundance)
data(metadata)

# Prepare abundance data
abundance_data <- as.data.frame(ko_abundance)
rownames(abundance_data) <- abundance_data[, "#NAME"]
abundance_data <- abundance_data[, -1]

# Run GSEA analysis (using camera method - recommended)
gsea_results <- pathway_gsea(
  abundance = abundance_data,
  metadata = metadata,
  group = "Environment",
  pathway_type = "KEGG",
  method = "camera"
)

# Create enrichment plot with pathway IDs (default)
visualize_gsea(gsea_results, plot_type = "enrichment_plot", n_pathways = 10)

# Annotate results for better pathway names
annotated_results <- gsea_pathway_annotation(
  gsea_results = gsea_results,
  pathway_type = "KEGG"
)

# Create plots with readable pathway names
visualize_gsea(annotated_results, plot_type = "dotplot", n_pathways = 20)
visualize_gsea(annotated_results, plot_type = "barplot", n_pathways = 15)

# Create network plot with custom labels
visualize_gsea(annotated_results, plot_type = "network", n_pathways = 15)

# Use custom column for labels (if available)
visualize_gsea(annotated_results, plot_type = "barplot",
               pathway_label_column = "pathway_name", n_pathways = 10)

# Create heatmap
visualize_gsea(
  annotated_results,
  plot_type = "heatmap",
  n_pathways = 15,
  abundance = abundance_data,
  metadata = metadata,
  group = "Environment"
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathway_ridgeplot.R
\name{pathway_ridgeplot}
\alias{pathway_ridgeplot}
\title{Ridge Plot for GSEA Results}
\usage{
pathway_ridgeplot(
  gsea_results,
  abundance,
  metadata,
  group,
  pathway_reference = NULL,
  pathway_type = "KEGG",
  n_pathways = 10,
  sort_by = "p.adjust",
  show_direction = TRUE,
  colors = c(Down = "#3182bd", Up = "#de2d26"),
  title = "Ridge Plot: Gene Distribution in Enriched Pathways",
  x_lab = "log2 Fold Change",
  scale_height = 0.9,
  alpha = 0.7
)
}
\arguments{
\item{gsea_results}{A data frame containing GSEA results from \code{\link{pathway_gsea}}.
Must contain pathway_id column and either NES or direction column.}

\item{abundance}{A data frame or matrix containing the original abundance data
(genes/KOs as rows, samples as columns) used in the GSEA analysis.}

\item{metadata}{A data frame containing sample metadata with group information.}

\item{group}{Character string specifying the column name in metadata for grouping.}

\item{pathway_reference}{A data frame containing pathway-to-gene mappings.
Must have columns: pathway_id (or go_id for GO) and a column containing
gene/KO members (semicolon-separated). If NULL, attempts to use built-in
KEGG or GO reference data.}

\item{pathway_type}{Character string specifying the pathway type: "KEGG", "GO", or "MetaCyc".
Default is "KEGG".}

\item{n_pathways}{Integer specifying the number of top pathways to display.
Default is 10.}

\item{sort_by}{Character string specifying how to sort pathways:
"NES" (Normalized Enrichment Score), "pvalue", or "p.adjust".
Default is "p.adjust".}

\item{show_direction}{Logical. If TRUE, colors ridges by enrichment direction.
Default is TRUE.}

\item{colors}{Named character vector with colors for "Up" and "Down" directions.
Default is blue for down-regulated and red for up-regulated.}

\item{title}{Character string for plot title.}

\item{x_lab}{Character string for x-axis label.}

\item{scale_height}{Numeric value controlling the overlap of ridges.
Default is 0.9. Higher values create more overlap.}

\item{alpha}{Numeric value for ridge transparency (0-1). Default is 0.7.}
}
\value{
A ggplot2 object that can be further customized or saved.
}
\description{
Creates a ridge plot (joy plot) to visualize the distribution of gene/KO
abundances or fold changes for enriched pathways from GSEA analysis.
This helps interpret whether pathways are predominantly up- or down-regulated.
}
\details{
The ridge plot displays the distribution of gene abundances (or fold changes)
for genes within each enriched pathway. This visualization helps to:
\itemize{
  \item Understand the overall direction of change for each pathway
  \item Identify pathways with consistent vs. heterogeneous gene expression
  \item Compare the magnitude of changes across pathways
}

The plot requires the \code{ggridges} package to be installed.
}
\examples{
\dontrun{
library(ggpicrust2)
library(tibble)

# Load example data
data("ko_abundance")
data("metadata")

# Run GSEA (using camera method - recommended)
gsea_results <- pathway_gsea(
  abundance = ko_abundance \%>\% column_to_rownames("#NAME"),
  metadata = metadata,
  group = "Environment",
  pathway_type = "KEGG",
  method = "camera"
)

# Create ridge plot
ridge_plot <- pathway_ridgeplot(
  gsea_results = gsea_results,
  abundance = ko_abundance \%>\% column_to_rownames("#NAME"),
  metadata = metadata,
  group = "Environment",
  n_pathways = 10
)
print(ridge_plot)
}

}
\seealso{
\code{\link{pathway_gsea}}, \code{\link{visualize_gsea}},
  \code{\link{pathway_volcano}}
}

\name{kegg_pathway_reference}
\alias{kegg_pathway_reference}
\docType{data}
\title{
KEGG Pathway Name Reference Dataset
}
\description{
A reference dataset mapping KEGG pathway IDs to their human-readable names.
Used internally by ggpicrust2 for pathway annotation in DAA and GSEA results.
}
\usage{data("kegg_pathway_reference")}
\format{
  A data frame with 505 observations and the following columns:
  \describe{
    \item{\code{pathway}}{Character. KEGG pathway ID in the format "koXXXXX" (e.g., "ko00010")}
    \item{\code{pathway_name}}{Character. Human-readable pathway name (e.g., "Glycolysis / Gluconeogenesis")}
  }
}
\source{
KEGG REST API (\url{https://rest.kegg.jp})
}
\examples{
data("kegg_pathway_reference")
head(kegg_pathway_reference)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathway_gsea.R
\name{calculate_rank_metric}
\alias{calculate_rank_metric}
\title{Calculate rank metric for GSEA}
\usage{
calculate_rank_metric(abundance, metadata, group, method = "signal2noise")
}
\arguments{
\item{abundance}{A matrix of abundance data}

\item{metadata}{A data frame of metadata}

\item{group}{A character string specifying the grouping variable}

\item{method}{A character string specifying the ranking method}
}
\value{
A named vector of ranking statistics
}
\description{
Calculate rank metric for GSEA
}
\keyword{internal}

\name{ko_reference}
\alias{ko_reference}
\docType{data}
\title{
KEGG Orthology (KO) Reference Dataset
}
\description{
A comprehensive reference dataset mapping KEGG Orthology (KO) identifiers to their
pathway classifications and descriptions. Each KO entry can appear in multiple rows
if it belongs to multiple pathways.
}
\usage{data("ko_reference")}
\format{
  A data frame with 58693 observations and the following columns:
  \describe{
    \item{\code{id}}{Character. KO identifier (e.g., "K00001")}
    \item{\code{PathwayL1}}{Character. Top-level KEGG pathway category (e.g., "Metabolism")}
    \item{\code{PathwayL2}}{Character. Second-level pathway category (e.g., "Carbohydrate metabolism")}
    \item{\code{Pathway}}{Character. Specific pathway name with ID (e.g., "Glycolysis / Gluconeogenesis [PATH:ko00010]")}
    \item{\code{description}}{Character. KO entry description with gene name and EC number}
  }
}
\source{
KEGG REST API (\url{https://rest.kegg.jp})
}
\examples{
data("ko_reference")
head(ko_reference)

# Check pathway hierarchy
table(ko_reference$PathwayL1)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathway_pca.R
\name{pathway_pca}
\alias{pathway_pca}
\title{Perform Principal Component Analysis (PCA) on functional pathway abundance data}
\usage{
pathway_pca(abundance, metadata, group, colors = NULL, show_marginal = TRUE)
}
\arguments{
\item{abundance}{A numeric matrix or data frame containing pathway abundance data.
Rows represent pathways, columns represent samples.
Column names must match the sample names in metadata.
Values must be numeric and cannot contain missing values (NA).}

\item{metadata}{A data frame containing sample information.
Must include a column for grouping samples (specified by the 'group' parameter).
Sample identifiers are auto-detected from columns named sample_name, Sample_ID,
SampleID, etc., or from rownames.}

\item{group}{A character string specifying the column name in metadata that contains
group information for samples (e.g., "treatment", "condition", "group").}

\item{colors}{Optional. A character vector of colors for different groups.
Length must match the number of unique groups.
If NULL, default colors will be used.}

\item{show_marginal}{Logical. Whether to show marginal density plots for PC1 and PC2.
Default is TRUE. Set to FALSE to show only the PCA scatter plot.}
}
\value{
A ggplot object showing:
       \itemize{
         \item Center: PCA scatter plot with confidence ellipses (95%)
         \item Top: Density plot for PC1
         \item Right: Density plot for PC2
       }
}
\description{
This function performs PCA analysis on pathway abundance data and creates an informative visualization
that includes a scatter plot of the first two principal components (PC1 vs PC2) with density plots
for both PCs. The plot helps to visualize the clustering patterns and distribution of samples
across different groups.
}
\details{
The function automatically aligns samples between abundance data and metadata,
supporting various sample identifier formats. Samples and pathways with zero
variance are filtered before PCA.
}
\examples{
# Create example abundance data
abundance_data <- matrix(rnorm(30), nrow = 3, ncol = 10)
colnames(abundance_data) <- paste0("Sample", 1:10)
rownames(abundance_data) <- c("PathwayA", "PathwayB", "PathwayC")

# Create example metadata
metadata <- data.frame(
  sample_name = paste0("Sample", 1:10),
  group = factor(rep(c("Control", "Treatment"), each = 5))
)

# Basic PCA plot with default colors
pca_plot <- pathway_pca(abundance_data, metadata, "group")

# PCA plot with custom colors
pca_plot <- pathway_pca(
  abundance_data,
  metadata,
  "group",
  colors = c("blue", "red")  # One color per group
)

# PCA plot without marginal density plots
pca_plot <- pathway_pca(
  abundance_data,
  metadata,
  "group",
  show_marginal = FALSE
)

\donttest{
# Example with real data
data("metacyc_abundance")  # Load example pathway abundance data
data("metadata")          # Load example metadata

# Generate PCA plot
# Prepare abundance data
abundance_data <- as.data.frame(metacyc_abundance)
rownames(abundance_data) <- abundance_data$pathway
abundance_data <- abundance_data[, -which(names(abundance_data) == "pathway")]

# Create PCA plot
pathway_pca(
  abundance_data,
  metadata,
  "Environment",
  colors = c("green", "purple")
)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/legend_annotation_utils.R
\name{create_pathway_class_theme}
\alias{create_pathway_class_theme}
\title{Create Pathway Class Annotation Theme}
\usage{
create_pathway_class_theme(
  text_size = "auto",
  text_color = "black",
  text_face = "bold",
  text_family = "sans",
  text_angle = 0,
  text_hjust = 0.5,
  text_vjust = 0.5,
  bg_color = NULL,
  bg_alpha = 0.2,
  position = "left"
)
}
\arguments{
\item{text_size}{Text size}

\item{text_color}{Text color}

\item{text_face}{Text face ("plain", "bold", "italic")}

\item{text_family}{Text family}

\item{text_angle}{Text angle in degrees}

\item{text_hjust}{Horizontal justification (0-1)}

\item{text_vjust}{Vertical justification (0-1)}

\item{bg_color}{Background color}

\item{bg_alpha}{Background alpha}

\item{position}{Annotation position ("left", "right", "none")}
}
\value{
List of annotation styling parameters
}
\description{
Create Pathway Class Annotation Theme
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathway_gsea.R
\name{pathway_gsea}
\alias{pathway_gsea}
\title{Gene Set Enrichment Analysis for PICRUSt2 output}
\usage{
pathway_gsea(
  abundance,
  metadata,
  group,
  pathway_type = "KEGG",
  method = "camera",
  covariates = NULL,
  contrast = NULL,
  inter.gene.cor = 0.01,
  rank_method = "signal2noise",
  nperm = 1000,
  min_size = 5,
  max_size = 500,
  p_adjust_method = "BH",
  seed = 42,
  go_category = "all",
  organism = "ko",
  p.adjust = NULL
)
}
\arguments{
\item{abundance}{A data frame containing gene/enzyme abundance data, with features as rows and samples as columns.
For KEGG analysis: features should be KO IDs (e.g., K00001).
For MetaCyc analysis: features should be EC numbers (e.g., EC:1.1.1.1 or 1.1.1.1), NOT pathway IDs.
For GO analysis: features should be KO IDs that will be mapped to GO terms.
NOTE: This function requires gene-level data, not pathway-level abundances.
For pathway abundance analysis, use \code{\link{pathway_daa}} instead}

\item{metadata}{A data frame containing sample metadata}

\item{group}{A character string specifying the column name in metadata that contains the grouping variable}

\item{pathway_type}{A character string specifying the pathway type: "KEGG", "MetaCyc", or "GO"}

\item{method}{A character string specifying the GSEA method:
\itemize{
  \item \code{"camera"}: Competitive gene set test using limma's camera function (recommended).
        Accounts for inter-gene correlations and provides more reliable p-values.
  \item \code{"fry"}: Fast approximation to rotation gene set testing using limma's fry function.
        Self-contained test that is computationally efficient.
  \item \code{"fgsea"}: Fast preranked GSEA implementation. Note: preranked methods may produce
        unreliable p-values due to not accounting for inter-gene correlations (Wu et al., 2012).
  \item \code{"GSEA"} or \code{"clusterProfiler"}: clusterProfiler's GSEA implementation.
}}

\item{covariates}{A character vector specifying column names in metadata to use as covariates
for adjustment. Only used when method is "camera" or "fry". Default is NULL (no covariates).
Example: \code{covariates = c("age", "sex", "BMI")}}

\item{contrast}{For multi-group comparisons with "camera" or "fry" methods, specify the contrast
to test. Can be a character string naming a group level, or a numeric vector of contrast weights.
Default is NULL (automatic: compares second group to first).}

\item{inter.gene.cor}{Numeric value specifying the inter-gene correlation for camera method.
Default is 0.01. Use NA to estimate correlation from data for each gene set.}

\item{rank_method}{A character string specifying the ranking statistic for preranked methods
(fgsea, GSEA, clusterProfiler): "signal2noise", "t_test", "log2_ratio", or "diff_abundance"}

\item{nperm}{An integer specifying the number of permutations (for clusterProfiler method only).
The fgsea method uses adaptive multilevel splitting and does not require a fixed permutation count.}

\item{min_size}{An integer specifying the minimum gene set size}

\item{max_size}{An integer specifying the maximum gene set size}

\item{p_adjust_method}{A character string specifying the p-value adjustment method}

\item{seed}{An integer specifying the random seed for reproducibility}

\item{go_category}{A character string specifying GO category to use.
"all" (default) uses all categories present in the reference data.
Valid categories are determined by the reference data (currently MF and CC).
See \code{table(ko_to_go_reference$category)} for available categories.}

\item{organism}{A character string specifying the organism for KEGG analysis (default: "ko" for KEGG Orthology)}

\item{p.adjust}{Deprecated. Use \code{p_adjust_method} instead.}
}
\value{
A data frame containing GSEA results with columns:
  \itemize{
    \item \code{pathway_id}: Pathway identifier
    \item \code{pathway_name}: Pathway name/description
    \item \code{size}: Number of genes in the pathway
    \item \code{direction}: Direction of enrichment ("Up" or "Down", for camera/fry)
    \item \code{pvalue}: Raw p-value
    \item \code{p.adjust}: Adjusted p-value (FDR)
    \item \code{method}: The method used for analysis
  }
  For fgsea/clusterProfiler methods, additional columns include ES, NES, and leading_edge.
}
\description{
This function performs Gene Set Enrichment Analysis (GSEA) on PICRUSt2 predicted functional data
to identify enriched pathways between different conditions.
}
\details{
\strong{Method Selection:}

The \code{camera} method (default) is recommended for most analyses because:
\itemize{
  \item It accounts for inter-gene correlations, providing more accurate p-values
  \item It supports covariate adjustment through the design matrix
  \item It performs a competitive test (genes in set vs. genes not in set)
}

The \code{fry} method is a fast alternative that:
\itemize{
  \item Performs a self-contained test (are genes in the set differentially expressed?)
  \item Is computationally very efficient for large numbers of gene sets
  \item Also supports covariate adjustment
}

The preranked methods (\code{fgsea}, \code{GSEA}) are included for compatibility but
users should be aware that Wu et al. (2012) demonstrated these can produce "spectacularly
wrong p-values" even with low inter-gene correlations.

\strong{Covariate Adjustment:}

When using \code{method = "camera"} or \code{method = "fry"}, you can adjust for
confounding variables by specifying them in the \code{covariates} parameter.
This is particularly important in microbiome studies where factors like age, sex,
BMI, and batch effects can influence results.
}
\examples{
\dontrun{
# Load example data
data(ko_abundance)
data(metadata)

# Prepare abundance data
abundance_data <- as.data.frame(ko_abundance)
rownames(abundance_data) <- abundance_data[, "#NAME"]
abundance_data <- abundance_data[, -1]

# Method 1: Using camera (recommended) - accounts for inter-gene correlations
gsea_results <- pathway_gsea(
  abundance = abundance_data,
  metadata = metadata,
  group = "Environment",
  pathway_type = "KEGG",
  method = "camera"
)

# Method 2: Using camera with covariate adjustment
gsea_results_adj <- pathway_gsea(
  abundance = abundance_data,
  metadata = metadata,
  group = "Disease",
  covariates = c("age", "sex"),
  pathway_type = "KEGG",
  method = "camera"
)

# Method 3: Using fry for fast self-contained testing
gsea_results_fry <- pathway_gsea(
  abundance = abundance_data,
  metadata = metadata,
  group = "Environment",
  pathway_type = "KEGG",
  method = "fry"
)

# Method 4: Using fgsea (preranked, less reliable p-values)
gsea_results_fgsea <- pathway_gsea(
  abundance = abundance_data,
  metadata = metadata,
  group = "Environment",
  pathway_type = "KEGG",
  method = "fgsea"
)

# Visualize results
visualize_gsea(gsea_results, plot_type = "enrichment_plot", n_pathways = 10)
}
}
\references{
Wu, D., & Smyth, G. K. (2012). Camera: a competitive gene set test accounting for
inter-gene correlation. Nucleic Acids Research, 40(17), e133.

Wu, D., Lim, E., Vaillant, F., Asselin-Labat, M. L., Visvader, J. E., & Smyth, G. K. (2010).
ROAST: rotation gene set tests for complex microarray experiments. Bioinformatics, 26(17), 2176-2182.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/legend_annotation_utils.R
\name{format_pvalue_smart}
\alias{format_pvalue_smart}
\title{Smart P-value Formatting}
\usage{
format_pvalue_smart(
  p_values,
  format = "smart",
  stars = TRUE,
  thresholds = c(0.001, 0.01, 0.05),
  star_symbols = c("***", "**", "*")
)
}
\arguments{
\item{p_values}{Numeric vector of p-values}

\item{format}{Character string specifying format type}

\item{stars}{Logical, whether to include star symbols}

\item{thresholds}{Numeric vector of significance thresholds}

\item{star_symbols}{Character vector of star symbols}
}
\value{
Character vector of formatted p-values
}
\description{
Smart P-value Formatting
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/taxa_contribution_viz.R
\name{taxa_contribution_bar}
\alias{taxa_contribution_bar}
\title{Stacked bar plot of taxa contributions}
\usage{
taxa_contribution_bar(
  contrib_agg,
  metadata,
  group,
  function_ids = NULL,
  n_functions = 6,
  facet_by = "function",
  show_percentage = TRUE,
  color_theme = "default",
  font_size = 12,
  legend_position = "right",
  custom_title = NULL
)
}
\arguments{
\item{contrib_agg}{A data.frame from \code{\link{aggregate_taxa_contributions}}.}

\item{metadata}{A data.frame containing sample metadata.}

\item{group}{Character. Column name in \code{metadata} for grouping samples.}

\item{function_ids}{Optional character vector of function IDs to plot.
If NULL (default), the top \code{n_functions} by variance are shown.}

\item{n_functions}{Integer. Number of functions to show when
\code{function_ids} is NULL. Default 6.}

\item{facet_by}{Character. Facet by \code{"function"} (default) or
\code{"group"}.}

\item{show_percentage}{Logical. Normalize bars to 100\%? Default TRUE.}

\item{color_theme}{Character. Color theme name, passed to
\code{\link{get_color_theme}}. Default \code{"default"}.}

\item{font_size}{Numeric. Base font size. Default 12.}

\item{legend_position}{Character. Legend position. Default \code{"right"}.}

\item{custom_title}{Optional character string for the plot title.}
}
\value{
A \code{ggplot2} object.
}
\description{
Creates a stacked bar plot showing taxa contributions to predicted
functional abundances, faceted by function or sample group.
}
\examples{
\donttest{
# Synthetic example
agg <- data.frame(
  sample = rep(c("S1", "S2", "S3", "S4"), each = 3),
  function_id = rep(c("K00001", "K00002"), each = 6),
  taxon_label = rep(c("Genus_A", "Genus_B", "Other"), 4),
  contribution = runif(12)
)
metadata <- data.frame(
  sample = c("S1", "S2", "S3", "S4"),
  group = c("Control", "Control", "Treatment", "Treatment")
)
p <- taxa_contribution_bar(agg, metadata, group = "group")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/legend_annotation_utils.R
\name{get_significance_stars}
\alias{get_significance_stars}
\title{Get Significance Stars}
\usage{
get_significance_stars(
  p_values,
  thresholds = c(0.001, 0.01, 0.05),
  symbols = c("***", "**", "*")
)
}
\arguments{
\item{p_values}{Numeric vector of p-values}

\item{thresholds}{Numeric vector of significance thresholds}

\item{symbols}{Character vector of star symbols}
}
\value{
Character vector of star symbols
}
\description{
Get Significance Stars
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_documentation.R
\docType{data}
\name{ko_to_kegg_reference}
\alias{ko_to_kegg_reference}
\title{KO to KEGG Pathway Reference Data}
\format{
A data frame with 9 variables:
\describe{
  \item{pathway_id}{KEGG pathway identifier (e.g., "ko00010")}
  \item{pathway_number}{KEGG pathway number}
  \item{pathway_name}{Full name of the pathway}
  \item{ko_id}{KEGG Orthology identifier (e.g., "K00001")}
  \item{ko_description}{Description of the KO}
  \item{ec_number}{EC number associated with the KO (if applicable)}
  \item{level1}{KEGG pathway hierarchy Level 1 classification}
  \item{level2}{KEGG pathway hierarchy Level 2 classification}
  \item{level3}{KEGG pathway hierarchy Level 3 classification}
}
}
\source{
KEGG database (\url{https://www.kegg.jp/})
}
\usage{
ko_to_kegg_reference
}
\description{
A comprehensive mapping between KEGG Orthology (KO) identifiers and KEGG pathways.
This dataset contains mappings covering 532 pathways and 23,466 unique KO IDs,
filtered to include only real KEGG pathway maps (5-digit IDs).
}
\details{
This reference data is used by the \code{\link{ko2kegg_abundance}} function to convert
KO abundance data to KEGG pathway abundance. The data is stored internally and does not
require internet connectivity to use.

The dataset covers major KEGG pathway categories including:
\itemize{
  \item Metabolism
  \item Genetic Information Processing
  \item Environmental Information Processing
  \item Cellular Processes
  \item Organismal Systems
  \item Human Diseases
}
}
\examples{
# Load the reference data
data(ko_to_kegg_reference)

# View structure
str(ko_to_kegg_reference)

# Get unique pathways
unique_pathways <- unique(ko_to_kegg_reference$pathway_id)
length(unique_pathways)

# Find KOs for a specific pathway
glycolysis_kos <- ko_to_kegg_reference[ko_to_kegg_reference$pathway_id == "ko00010", ]
head(glycolysis_kos)

}
\seealso{
\code{\link{ko2kegg_abundance}} for converting KO abundance to pathway abundance
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathway_gsea.R
\name{validate_group_sizes}
\alias{validate_group_sizes}
\title{Validate group sizes for statistical analysis}
\usage{
validate_group_sizes(group_vector, group_name)
}
\arguments{
\item{group_vector}{A factor vector with group assignments}

\item{group_name}{A character string with the group variable name for error messages}
}
\description{
This function checks group sizes and balance to ensure statistical reliability.
Follows Linus principle: fail fast with clear reasons, don't hide problems.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualize_gsea.R
\name{.build_heatmap_col_fun}
\alias{.build_heatmap_col_fun}
\title{Internal: build a circlize colorRamp2 function for ComplexHeatmap from user scale}
\usage{
.build_heatmap_col_fun(scale = NULL)
}
\description{
Internal: build a circlize colorRamp2 function for ComplexHeatmap from user scale
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualize_gsea.R
\name{.build_discrete_fill_for_direction}
\alias{.build_discrete_fill_for_direction}
\title{Internal: build a discrete fill scale for barplot direction}
\usage{
.build_discrete_fill_for_direction(scale = NULL)
}
\description{
Internal: build a discrete fill scale for barplot direction
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/legend_annotation_utils.R
\name{resolve_annotation_overlaps}
\alias{resolve_annotation_overlaps}
\title{Detect and Resolve Annotation Overlaps}
\usage{
resolve_annotation_overlaps(labels, positions, min_distance = 1)
}
\arguments{
\item{labels}{Character vector of labels}

\item{positions}{Numeric vector of positions}

\item{min_distance}{Minimum distance between labels}
}
\value{
Adjusted positions
}
\description{
Detect and Resolve Annotation Overlaps
}

\name{ko_to_go_reference}
\alias{ko_to_go_reference}
\docType{data}
\title{
KO to GO Reference Mapping Dataset
}
\description{
A comprehensive reference dataset that maps KEGG Orthology (KO) identifiers to Gene Ontology (GO) terms. This dataset enables GO pathway analysis in ggpicrust2 by providing the necessary mappings between functional predictions and GO biological processes, molecular functions, and cellular components.
}
\usage{data("ko_to_go_reference")}
\format{
  A data frame with the following columns:
  \describe{
    \item{\code{go_id}}{Character. GO term identifier in the format "GO:XXXXXXX"}
    \item{\code{go_name}}{Character. Human-readable name of the GO term}
    \item{\code{category}}{Character. GO category code. Use \code{table(ko_to_go_reference$category)} to see available categories.}
    \item{\code{ko_members}}{Character. Semicolon-separated list of KO identifiers associated with this GO term}
  }
}
\details{
This dataset maps KEGG Orthology (KO) identifiers to Gene Ontology (GO) terms, enabling GO-level functional analysis of PICRUSt2 predictions.

The dataset is built from authoritative biological databases:
\itemize{
  \item KEGG REST API DBLINKS field for KO to GO cross-references
  \item EBI QuickGO API for GO term metadata (names and categories)
}

KEGG DBLINKS primarily cross-references Molecular Function (MF) GO terms, because KO entries describe individual gene functions that naturally correspond to molecular activities (enzyme activities, binding functions, etc.). The current dataset contains predominantly MF terms with a small number of CC (Cellular Component) terms.

Each GO term includes at least 3 associated KO identifiers, ensuring statistical utility for enrichment analysis.
}
\source{
\itemize{
  \item KEGG REST API (\url{https://rest.kegg.jp}) - KO entry DBLINKS section
  \item EBI QuickGO (\url{https://www.ebi.ac.uk/QuickGO/}) - GO term metadata
}
}
\references{
\itemize{
  \item Kanehisa, M., & Goto, S. (2000). KEGG: kyoto encyclopedia of genes and genomes. Nucleic acids research, 28(1), 27-30.
  \item Ashburner, M., et al. (2000). Gene ontology: tool for the unification of biology. Nature genetics, 25(1), 25-29.
  \item Chen Yang, et al. (2023). ggpicrust2: an R package for PICRUSt2 predicted functional profile analysis and visualization. Bioinformatics, 39(8), btad470.
}
}
\examples{
# Load the dataset
data("ko_to_go_reference")

# Explore the dataset structure
head(ko_to_go_reference)
str(ko_to_go_reference)

# Check the distribution of GO categories
table(ko_to_go_reference$category)

# Find GO terms related to polymerase activity
polymerase_terms <- ko_to_go_reference[
  grepl("polymerase", ko_to_go_reference$go_name, ignore.case = TRUE), ]
head(polymerase_terms)

# Get KO members for a specific GO term (RNA polymerase activity)
rna_pol <- ko_to_go_reference[ko_to_go_reference$go_id == "GO:0003899", ]
if (nrow(rna_pol) > 0) {
  ko_list <- strsplit(rna_pol$ko_members, ";")[[1]]
  cat("KO identifiers for RNA polymerase activity:", paste(ko_list, collapse = ", "))
}

# Use in pathway analysis
\dontrun{
library(ggpicrust2)
library(tibble)

# Load example data
data("ko_abundance")
data("metadata")

# Perform GO pathway GSEA analysis
gsea_results <- pathway_gsea(
  abundance = ko_abundance \%>\% column_to_rownames("#NAME"),
  metadata = metadata \%>\% column_to_rownames("sample_name"),
  group = "Environment",
  method = "fgsea",
  pathway_type = "GO",
  go_category = "MF",
  rank_method = "signal2noise"
)

# View results
head(gsea_results)
}
}
\seealso{
\code{\link{pathway_gsea}}, \code{\link{ko_abundance}}, \code{\link{metadata}}
}
\keyword{datasets}
\keyword{microbiome}
\keyword{functional-analysis}
\keyword{gene-ontology}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kegg_abundance.R
\docType{data}
\name{kegg_abundance}
\alias{kegg_abundance}
\title{KEGG Abundance Dataset}
\format{
A data frame where rownames are KEGG pathways and column names are individual sample names, including:
"SRR11393730", "SRR11393731", "SRR11393732", "SRR11393733", "SRR11393734", "SRR11393735", "SRR11393736",
"SRR11393737", "SRR11393738", "SRR11393739", "SRR11393740", "SRR11393741", "SRR11393742", "SRR11393743",
"SRR11393744", "SRR11393745", "SRR11393746", "SRR11393747", "SRR11393748", "SRR11393749", "SRR11393750",
"SRR11393751", "SRR11393752", "SRR11393753", "SRR11393754", "SRR11393755", "SRR11393756", "SRR11393757",
"SRR11393758", "SRR11393759", "SRR11393760", "SRR11393761", "SRR11393762", "SRR11393763", "SRR11393764",
"SRR11393765", "SRR11393766", "SRR11393767", "SRR11393768", "SRR11393769", "SRR11393770", "SRR11393771",
"SRR11393772", "SRR11393773", "SRR11393774", "SRR11393775", "SRR11393776", "SRR11393777", "SRR11393778", "SRR11393779"
}
\source{
From ggpicrust2 package demonstration.
}
\usage{
kegg_abundance
}
\description{
A dataset derived from 'ko_abundance' by the function 'ko2kegg_abundance' in the ggpicrust2 package.
Each row corresponds to a KEGG pathway, and each column corresponds to a sample.
}
\references{
Douglas GM, Maffei VJ, Zaneveld J, Yurgel SN, Brown JR, Taylor CM, Huttenhower C, Langille MGI. PICRUSt2 for prediction of metagenome functions. Nat Biotechnol. 2020.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualize_gsea.R
\name{create_empty_plot}
\alias{create_empty_plot}
\title{Create empty plot for edge cases}
\usage{
create_empty_plot(plot_type)
}
\arguments{
\item{plot_type}{A character string specifying the visualization type}
}
\value{
A ggplot2 object
}
\description{
Create empty plot for edge cases
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compare_daa_results.R
\name{compare_daa_results}
\alias{compare_daa_results}
\title{Compare the Consistency of Statistically Significant Features}
\arguments{
\item{daa_results_list}{A list of data frames containing statistically significant features obtained using different methods.}

\item{method_names}{A character vector of names for each method used.}

\item{p_values_threshold}{A numeric value representing the threshold for the p-values. Features with p-values less than this
threshold are considered statistically significant. Default is 0.05.}
}
\value{
A data frame with the comparison results. The data frame has the following columns:
\itemize{
  \item \code{method}: The name of the method.
  \item \code{num_features}: The total number of statistically significant features obtained by the method.
  \item \code{num_common_features}: The number of features that are common to other methods.
  \item \code{num_diff_features}: The number of features that are different from other methods.
  \item \code{common_features}: The names of the features that are common to all methods.
  \item \code{diff_features}: The names of the features that are different from other methods.
}
}
\description{
This function compares the consistency and inconsistency of statistically significant features obtained
using different methods in `pathway_daa` from the `ggpicrust2` package. It creates a report showing the number of common and
different features identified by each method, and the features themselves.
}
\examples{
# Minimal DAA-like results from three methods (no external dependencies required)
deseq2_df <- data.frame(
  feature = c("ko00010", "ko00020", "ko00564"),
  group1 = c("A", "A", "A"),
  group2 = c("B", "B", "B"),
  p_adjust = c(0.01, 0.20, 0.03),
  stringsAsFactors = FALSE
)

edgeR_df <- data.frame(
  feature = c("ko00010", "ko00680", "ko00564"),
  group1 = c("A", "A", "A"),
  group2 = c("B", "B", "B"),
  p_adjust = c(0.02, 0.04, 0.01),
  stringsAsFactors = FALSE
)

maaslin2_df <- data.frame(
  feature = c("ko00010", "ko03030", "ko00564"),
  group1 = c("A", "A", "A"),
  group2 = c("B", "B", "B"),
  p_adjust = c(0.03, 0.02, 0.04),
  stringsAsFactors = FALSE
)

daa_results_list <- list(DESeq2 = deseq2_df, edgeR = edgeR_df, Maaslin2 = maaslin2_df)
comparison_results <- compare_daa_results(
  daa_results_list = daa_results_list,
  method_names = c("DESeq2", "edgeR", "Maaslin2"),
  p_values_threshold = 0.05
)
comparison_results
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualize_gsea.R
\name{.as_color_vector}
\alias{.as_color_vector}
\title{Internal: coerce user 'scale' input to a vector of colors (or NULL)
Accepts character vector or function(n)->colors. Returns character vector or NULL.}
\usage{
.as_color_vector(scale)
}
\description{
Internal: coerce user 'scale' input to a vector of colors (or NULL)
Accepts character vector or function(n)->colors. Returns character vector or NULL.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compare_metagenome_results.R
\name{compare_metagenome_results}
\alias{compare_metagenome_results}
\title{Compare Metagenome Results}
\usage{
compare_metagenome_results(
  metagenomes,
  names,
  daa_method = "ALDEx2",
  p_adjust_method = "BH",
  reference = NULL,
  p.adjust = NULL
)
}
\arguments{
\item{metagenomes}{A list of metagenomes matrices with rows as KOs and columns as samples.
Each matrix in the list should correspond to a different metagenome.}

\item{names}{A vector of names for the metagenomes in the same order as in the `metagenomes` list.}

\item{daa_method}{A character specifying the method for differential abundance analysis (DAA).
Possible choices are: "ALDEx2", "DESeq2", "edgeR", "limma voom", "metagenomeSeq", "LinDA",
"Maaslin2", and "Lefse". The default is "ALDEx2".}

\item{p_adjust_method}{A character specifying the method for p-value adjustment.
Possible choices are: "BH" (Benjamini-Hochberg), "holm", "bonferroni", "hochberg", "fdr", and "none".
The default is "BH".}

\item{reference}{A character specifying the reference group level for DAA.
This parameter is used when there are more than two groups. The default is NULL.}

\item{p.adjust}{Deprecated. Use \code{p_adjust_method} instead.}
}
\value{
A list containing three elements:
\itemize{
\item "daa": a data frame of results from the `pathway_daa` function
containing the differential abundance analysis results.
\item "correlation": a list with two elements: "cor_matrix" and "p_matrix", which are
matrices of per-feature median Spearman correlation coefficients and their
corresponding p-values, respectively, between every pair of metagenomes.
\item "heatmap": a ComplexHeatmap object visualizing the correlation matrix.
Use \code{print()} or \code{draw()} to display it.
}
}
\description{
Compare Metagenome Results
}
\examples{
\donttest{
library(dplyr)
library(ComplexHeatmap)
# Generate example data
set.seed(123)
# First metagenome
metagenome1 <- abs(matrix(rnorm(1000), nrow = 100, ncol = 10))
rownames(metagenome1) <- paste0("KO", 1:100)
colnames(metagenome1) <- paste0("sample", 1:10)
# Second metagenome
metagenome2 <- abs(matrix(rnorm(1000), nrow = 100, ncol = 10))
rownames(metagenome2) <- paste0("KO", 1:100)
colnames(metagenome2) <- paste0("sample", 1:10)
# Put the metagenomes into a list
metagenomes <- list(metagenome1, metagenome2)
# Define names
names <- c("metagenome1", "metagenome2")
# Call the function
results <- compare_metagenome_results(metagenomes, names, daa_method = "LinDA")
# Print the correlation matrix
print(results$correlation$cor_matrix)
# Display the heatmap
print(results$heatmap)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/color_themes.R
\name{get_available_themes}
\alias{get_available_themes}
\title{Get Available Color Themes}
\usage{
get_available_themes()
}
\value{
A character vector of available theme names
}
\description{
Get Available Color Themes
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathway_gsea.R
\name{prepare_gene_sets}
\alias{prepare_gene_sets}
\title{Prepare gene sets for GSEA}
\usage{
prepare_gene_sets(pathway_type = "KEGG", organism = "ko", go_category = "all")
}
\arguments{
\item{pathway_type}{A character string specifying the pathway type: "KEGG", "MetaCyc", or "GO"}

\item{organism}{Deprecated and has no effect; gene sets are KO-based
for both KEGG and GO. See \code{\link{pathway_gsea}} for details.
Retained for signature compatibility only.}

\item{go_category}{A character string specifying the GO category to use.
"all" (default) uses all categories. Valid values are determined by
the reference data; see \code{table(ko_to_go_reference$category)}.}
}
\value{
A list of pathway gene sets
}
\description{
Prepare gene sets for GSEA
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metadata.R
\docType{data}
\name{metadata}
\alias{metadata}
\title{Metadata for ggpicrust2 Demonstration}
\format{
A tibble with each row representing metadata for a sample.
\describe{
  \item{Sample1}{Metadata for Sample1, including Environment}
  \item{Sample2}{Metadata for Sample2, including Environment}
  \item{...}{...}
}
}
\source{
ggpicrust2 package demonstration.
}
\usage{
metadata
}
\description{
This is a demonstration dataset from the ggpicrust2 package. It provides the metadata
required for the demonstration functions in the package. The dataset includes
environmental information for each sample.
}
\references{
Douglas GM, Maffei VJ, Zaneveld J, Yurgel SN, Brown JR, Taylor CM, Huttenhower C, Langille MGI. PICRUSt2 for prediction of metagenome functions. Nat Biotechnol. 2020.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathway_heatmap.R
\name{generate_nested_colors}
\alias{generate_nested_colors}
\title{Generate colors for nested grouping variables}
\usage{
generate_nested_colors(metadata, all_groups, colors = NULL)
}
\arguments{
\item{metadata}{A data frame containing metadata}

\item{all_groups}{A character vector of grouping variables}

\item{colors}{A character vector of colors or NULL}
}
\value{
A character vector of colors appropriate for the grouping structure
}
\description{
Generate colors for nested grouping variables
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathway_annotation.R
\name{pathway_annotation}
\alias{pathway_annotation}
\title{Pathway information annotation}
\usage{
pathway_annotation(
  file = NULL,
  pathway = NULL,
  daa_results_df = NULL,
  ko_to_kegg = FALSE,
  organism = NULL,
  p_adjust_threshold = 0.05
)
}
\arguments{
\item{file}{A character string, the path to the PICRUSt2 output file.}

\item{pathway}{A character string, the type of pathway to annotate. Options are "KO", "EC", or "MetaCyc".}

\item{daa_results_df}{A data frame, the output from `pathway_daa` function. When `ko_to_kegg = TRUE`,
must contain columns: feature, p_values, p_adjust, and method.}

\item{ko_to_kegg}{A logical, decide if convert KO abundance to KEGG pathway abundance. Default is FALSE.
Set to TRUE when using the function for the second use case. When TRUE, queries KEGG database for
pathway annotations (requires internet connection) and filters for significant pathways.}

\item{organism}{A character string specifying the KEGG organism code (e.g., 'hsa' for human, 'eco' for E. coli).
Default is NULL, which retrieves generic KO information not specific to any organism. Only used when ko_to_kegg is TRUE.}

\item{p_adjust_threshold}{A numeric value specifying the significance threshold for filtering
pathways when `ko_to_kegg = TRUE`. Only pathways with `p_adjust < p_adjust_threshold` will be
annotated via KEGG API. Default is 0.05. Ignored when `ko_to_kegg = FALSE`.}
}
\value{
A data frame with annotated pathway information.

If using the function for the first use case (file input), the output data frame will include:
\itemize{
  \item \code{id}: The pathway ID.
  \item \code{description}: The description of the pathway.
  \item \code{sample1, sample2, ...}: Abundance values for each sample.
}

If \code{ko_to_kegg} is set to TRUE, the output data frame will also include:
\itemize{
  \item \code{pathway_name}: The name of the KEGG pathway.
  \item \code{pathway_description}: The description of the KEGG pathway.
  \item \code{pathway_class}: The class of the KEGG pathway.
  \item \code{pathway_map}: The KEGG pathway map ID.
}

**Note**: When \code{ko_to_kegg = TRUE}, only pathways with
\code{p_adjust < p_adjust_threshold} are processed. If no pathways meet this criterion,
all annotation columns will be NA, and a detailed warning message will be issued with
diagnostic information.

When \code{ko_to_kegg} is TRUE, the function queries the KEGG database for pathway information.
By default (organism = NULL), it retrieves generic KO information that is not specific to any organism.
If you are interested in organism-specific pathway information, you can specify the KEGG organism code
using the \code{organism} parameter.
}
\description{
This function serves two main purposes:
1. Annotating pathway information from PICRUSt2 output files.
2. Annotating pathway information from the output of `pathway_daa` function, and converting KO abundance to KEGG pathway abundance when `ko_to_kegg` is set to TRUE.

**Important**: When `ko_to_kegg = TRUE`, this function automatically filters pathways by
`p_adjust < p_adjust_threshold`. If no pathways meet this criterion, the function returns
the original data with NA annotation columns and issues a detailed warning message with
diagnostic information and recommendations.
}
\examples{
\dontrun{
# Example 1: Annotate pathways from PICRUSt2 output file
pathway_annotation(file = "path/to/picrust2_output.tsv",
                              pathway = "KO")
}

\dontrun{
# Example 2: Annotate pathways from pathway_daa output
# Assuming you have daa_results from pathway_daa function
daa_results <- pathway_daa(abundance, metadata, group = "Group")
annotated_results <- pathway_annotation(pathway = "KO",
                              daa_results_df = daa_results,
                              ko_to_kegg = FALSE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathway_errorbar_table.R
\name{pathway_errorbar_table}
\alias{pathway_errorbar_table}
\title{Generate Abundance Statistics Table for Pathway Analysis}
\usage{
pathway_errorbar_table(
  abundance,
  daa_results_df,
  Group,
  ko_to_kegg = FALSE,
  p_values_threshold = 0.05,
  select = NULL,
  max_features = 30,
  metadata = NULL,
  sample_col = "sample_name"
)
}
\arguments{
\item{abundance}{A data frame or matrix containing predicted functional 
pathway abundance, with pathways/features as rows and samples as 
columns. The column names should match the sample names in metadata.}

\item{daa_results_df}{A data frame containing differential abundance 
analysis results from pathway_daa function. Must contain columns: 
feature, group1, group2, p_adjust.}

\item{Group}{A vector containing group assignments for each sample in the
same order as the columns in abundance matrix. Alternatively, if
metadata is provided, this should match the order of samples in metadata.}

\item{ko_to_kegg}{Logical value indicating whether to use KO to KEGG 
conversion. Default is FALSE.}

\item{p_values_threshold}{Numeric value for p-value threshold to filter 
significant features. Default is 0.05.}

\item{select}{Character vector of specific features to include. If NULL, 
all significant features are included.}

\item{max_features}{Maximum number of features to include in the table.
Default is 30.}

\item{metadata}{Optional data frame containing sample metadata. If provided,
the Group vector will be reordered to match the abundance column order.}

\item{sample_col}{Character string specifying the column name in metadata
that contains sample identifiers. Default is "sample_name".}
}
\value{
A data frame containing the following columns:
\itemize{
  \item \code{feature}: Feature/pathway identifier
  \item \code{group1}: Reference group name
  \item \code{group2}: Comparison group name
  \item \code{mean_rel_abundance_group1}: Mean relative abundance for group1
  \item \code{sd_rel_abundance_group1}: Standard deviation of relative
        abundance for group1
  \item \code{mean_rel_abundance_group2}: Mean relative abundance for group2
  \item \code{sd_rel_abundance_group2}: Standard deviation of relative
        abundance for group2
  \item \code{log2_fold_change}: Log2 fold change (group2/group1)
  \item \code{p_adjust}: Adjusted p-value from differential analysis
  \item Additional annotation columns (e.g., \code{description},
        \code{pathway_name}, \code{pathway_class}) if present in the input
        daa_results_df
}
}
\description{
This function generates a table containing mean relative abundance, 
standard deviation, and log2 fold change statistics for pathways, 
similar to the data used in pathway_errorbar plots but returned as a 
data frame instead of a plot.
}
\examples{
\dontrun{
# Load example data
data("ko_abundance")
data("metadata")

# Convert KO abundance to KEGG pathways
kegg_abundance <- ko2kegg_abundance(data = ko_abundance)

# Perform differential abundance analysis
daa_results_df <- pathway_daa(
  abundance = kegg_abundance,
  metadata = metadata,
  group = "Environment",
  daa_method = "ALDEx2"
)

# Filter for specific method
daa_sub_method_results_df <- daa_results_df[
  daa_results_df$method == "ALDEx2_Welch's t test", 
]

# Annotate results
daa_annotated_sub_method_results_df <- pathway_annotation(
  pathway = "KO",
  daa_results_df = daa_sub_method_results_df,
  ko_to_kegg = TRUE
)

# Generate abundance statistics table
abundance_stats_table <- pathway_errorbar_table(
  abundance = kegg_abundance,
  daa_results_df = daa_annotated_sub_method_results_df,
  Group = metadata$Environment,
  ko_to_kegg = TRUE,
  p_values_threshold = 0.05
)

# View the results
head(abundance_stats_table)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualize_gsea.R
\name{.is_ggplot_scale}
\alias{.is_ggplot_scale}
\title{Internal: detect if an object is a ggplot2 Scale}
\usage{
.is_ggplot_scale(x)
}
\description{
Internal: detect if an object is a ggplot2 Scale
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ko2kegg_abundance.R
\name{ko2kegg_abundance}
\alias{ko2kegg_abundance}
\title{Convert KO abundance in picrust2 export files to KEGG pathway abundance}
\usage{
ko2kegg_abundance(
  file = NULL,
  data = NULL,
  method = c("abundance", "sum"),
  filter_for_prokaryotes = TRUE,
  progress = interactive()
)
}
\arguments{
\item{file}{A character string representing the file path of the input file containing KO abundance data in picrust2 export format. The input file should have KO identifiers in the first column and sample identifiers in the first row. The remaining cells should contain the abundance values for each KO-sample pair.}

\item{data}{An optional data.frame containing KO abundance data in the same format as the input file. If provided, the function will use this data instead of reading from the file. By default, this parameter is set to NULL.}

\item{method}{Method for calculating pathway abundance. One of:
\itemize{
  \item \code{"abundance"}: (Default) PICRUSt2-style calculation using the mean of upper-half sorted KO abundances. This method is more robust and avoids inflating abundances for pathways with more KOs.
  \item \code{"sum"}: Simple summation of all KO abundances. This is the legacy method and may double-count KOs belonging to multiple pathways.
}}

\item{filter_for_prokaryotes}{Logical. If TRUE (default), filters out KEGG pathways
that are not relevant to prokaryotic (bacterial/archaeal) analysis. The function
always removes non-pathway KEGG buckets before this filter is applied. The
prokaryote filter removes pathways in categories such as:
\itemize{
  \item Human diseases (cancer, neurodegenerative diseases, addiction, etc.)
  \item Organismal systems (immune system, nervous system, endocrine system, etc.)
}
Bacterial infection pathways and antimicrobial resistance pathways are retained.
Set to FALSE to include all KEGG pathways (for eukaryotic analysis or custom filtering).}

\item{progress}{Logical. Whether to show a progress bar while aggregating
pathways. Defaults to \code{interactive()} so non-interactive scripts and
tests stay quiet.}
}
\value{
A data frame with KEGG pathway abundance values. Rows represent KEGG pathways, identified by their KEGG pathway IDs. Columns represent samples, identified by their sample IDs from the input file.
}
\description{
This function takes a file containing KO (KEGG Orthology) abundance data in picrust2 export format and converts it to KEGG pathway abundance data.
The input file should be in .tsv, .txt, or .csv format.
}
\details{
The default \code{"abundance"} method follows PICRUSt2's approach for calculating pathway abundance:
\enumerate{
  \item For each pathway, collect abundances of all associated KOs present in the data
  \item Sort the abundances in ascending order
  \item Take the upper half of the sorted values
  \item Calculate the mean as the pathway abundance
}

This approach has several advantages over simple summation:
\itemize{
  \item Does not inflate abundances for pathways containing more KOs
  \item More robust to missing or low-abundance KOs
  \item Provides a more accurate representation of pathway activity
}

The \code{"sum"} method is provided for backward compatibility and simply sums all KO abundances for each pathway.
}
\section{Pathway Filtering}{

Before abundance calculation, KEGG BRITE hierarchies and "Not Included in Pathway or
Brite" pseudo-pathways are removed because they are not KEGG pathway maps and cannot
be consistently annotated as pathways (for example, \code{ko99980}).

When \code{filter_for_prokaryotes = TRUE}, the function excludes KEGG pathways that are
biologically irrelevant to prokaryotic organisms. KEGG reference pathways include pathways
from all domains of life, and many human/animal-specific pathways would appear in bacterial
analysis simply because some KOs are shared across organisms.

The following KEGG Level 2 categories are excluded:
\itemize{
  \item Cancer pathways (overview and specific types)
  \item Neurodegenerative diseases (Alzheimer's, Parkinson's, etc.)
  \item Substance dependence (addiction pathways)
  \item Cardiovascular diseases
  \item Endocrine and metabolic diseases
  \item Immune diseases
  \item Organismal systems (immune, nervous, endocrine, digestive, etc.)
}

The following are RETAINED even with filtering:
\itemize{
  \item Infectious disease: bacterial (Salmonella, E. coli, Tuberculosis, etc.)
  \item Drug resistance: antimicrobial (antibiotic resistance)
  \item All Metabolism pathways
  \item Genetic/Environmental Information Processing
  \item Cellular Processes
}
}

\examples{
\dontrun{
library(ggpicrust2)
library(readr)

# Example 1: Default - filtered for prokaryotic analysis
data(ko_abundance)
kegg_abundance <- ko2kegg_abundance(data = ko_abundance)

# Example 2: Include all pathways (for eukaryotic analysis)
kegg_abundance_all <- ko2kegg_abundance(data = ko_abundance, filter_for_prokaryotes = FALSE)

# Example 3: Using legacy sum method with filtering
kegg_abundance_sum <- ko2kegg_abundance(data = ko_abundance, method = "sum")

# Example 4: From file
input_file <- "path/to/your/picrust2/results/pred_metagenome_unstrat.tsv"
kegg_abundance <- ko2kegg_abundance(file = input_file)
}
}

\name{metacyc_reference}
\alias{metacyc_reference}
\docType{data}
\title{
MetaCyc Pathway Reference Dataset
}
\description{
A reference dataset mapping MetaCyc pathway identifiers to their descriptions.
Used internally by ggpicrust2 for annotating MetaCyc pathway analysis results.
}
\usage{data("metacyc_reference")}
\format{
  A data frame with 2714 observations and the following columns:
  \describe{
    \item{\code{id}}{Character. MetaCyc pathway identifier (e.g., "GLYCOLYSIS", "TCA")}
    \item{\code{description}}{Character. Human-readable pathway description}
  }
}
\source{
MetaCyc database (\url{https://metacyc.org})
}
\examples{
data("metacyc_reference")
head(metacyc_reference)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/legend_annotation_utils.R
\name{get_significance_colors}
\alias{get_significance_colors}
\title{Get Significance Colors}
\usage{
get_significance_colors(
  p_values,
  thresholds = c(0.001, 0.01, 0.05),
  colors = c("#d73027", "#fc8d59", "#fee08b"),
  default_color = "#999999"
)
}
\arguments{
\item{p_values}{Numeric vector of p-values}

\item{thresholds}{Numeric vector of significance thresholds}

\item{colors}{Character vector of colors for each significance level}

\item{default_color}{Default color for non-significant values}
}
\value{
Character vector of colors
}
\description{
Get Significance Colors
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathway_gsea.R
\name{run_fgsea}
\alias{run_fgsea}
\title{Run fgsea using the recommended fgseaMultilevel method}
\usage{
run_fgsea(ranked_list, gene_sets, min_size = 10, max_size = 500)
}
\arguments{
\item{ranked_list}{A named vector of ranking statistics}

\item{gene_sets}{A list of pathway gene sets}

\item{min_size}{An integer specifying the minimum gene set size}

\item{max_size}{An integer specifying the maximum gene set size}
}
\value{
A data frame of fgsea results
}
\description{
Run fgsea using the recommended fgseaMultilevel method
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/taxa_contribution.R
\name{read_contrib_file}
\alias{read_contrib_file}
\title{Read PICRUSt2 contribution file}
\usage{
read_contrib_file(file = NULL, data = NULL)
}
\arguments{
\item{file}{Path to the contribution file (.tsv, .txt, or .csv).}

\item{data}{A data.frame already loaded from the contribution file.
If both \code{file} and \code{data} are provided, \code{data} is used.}
}
\value{
A data.frame with columns: \code{sample}, \code{function_id},
  \code{taxon}, \code{taxon_function_abun}, \code{norm_taxon_function_contrib},
  and any additional columns from the original file.
}
\description{
Parses the \code{pred_metagenome_contrib.tsv} file produced by PICRUSt2's
\code{--per_sequence_contrib} flag.
}
\details{
The contribution file records how much each ASV/OTU contributes to the
predicted abundance of each gene family in each sample. The
\code{norm_taxon_function_contrib} column gives the proportion of a
function's abundance attributable to each taxon.
}
\examples{
\donttest{
# From a data.frame
contrib_df <- data.frame(
  sample = rep(c("S1", "S2"), each = 4),
  `function` = rep(c("K00001", "K00002"), 4),
  taxon = rep(c("ASV1", "ASV2"), each = 2, times = 2),
  taxon_function_abun = runif(8),
  norm_taxon_function_contrib = runif(8),
  check.names = FALSE
)
result <- read_contrib_file(data = contrib_df)
head(result)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_utils.R
\name{calculate_log2_fold_change}
\alias{calculate_log2_fold_change}
\title{Calculate log2 fold change with consistent pseudocount handling}
\usage{
calculate_log2_fold_change(
  mean1,
  mean2,
  pseudocount = NULL,
  reference_values = NULL
)
}
\arguments{
\item{mean1}{Numeric. Mean abundance of group 1 (reference/control)}

\item{mean2}{Numeric. Mean abundance of group 2 (comparison/treatment)}

\item{pseudocount}{Optional numeric. If NULL, calculated from reference_values}

\item{reference_values}{Optional numeric vector for calculating pseudocount}
}
\value{
log2(mean2/mean1) with pseudocount protection
}
\description{
Calculates log2 fold change between two groups with proper handling
of zero values using a data-driven pseudocount approach.
}
\details{
The fold change direction is group2/group1, so:
- Positive values indicate higher abundance in group2
- Negative values indicate higher abundance in group1
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/color_themes.R
\name{get_color_theme}
\alias{get_color_theme}
\title{Get Color Theme}
\usage{
get_color_theme(theme_name = "default", n_colors = 8)
}
\arguments{
\item{theme_name}{Character string specifying the theme name}

\item{n_colors}{Integer specifying the number of colors needed}
}
\value{
A list containing theme colors and settings
}
\description{
Get Color Theme
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/taxa_contribution_viz.R
\name{taxa_contribution_heatmap}
\alias{taxa_contribution_heatmap}
\title{Heatmap of taxa contributions across functions}
\usage{
taxa_contribution_heatmap(
  contrib_agg,
  annotation_data = NULL,
  n_functions = 20,
  cluster_rows = TRUE,
  cluster_cols = TRUE,
  clustering_method = "complete",
  clustering_distance = "euclidean",
  low_color = "#f7f7f7",
  high_color = "#ca0020",
  font_size = 12,
  dendro_line_size = 0.5,
  custom_title = NULL
)
}
\arguments{
\item{contrib_agg}{A data.frame from \code{\link{aggregate_taxa_contributions}}.}

\item{annotation_data}{Optional data.frame from \code{\link{pathway_annotation}}
for replacing function IDs with readable descriptions.}

\item{n_functions}{Integer. Number of functions to include. Default 20.}

\item{cluster_rows}{Logical. Cluster rows (taxa)? Default TRUE.}

\item{cluster_cols}{Logical. Cluster columns (functions)? Default TRUE.}

\item{clustering_method}{Character. Method for \code{hclust}. Default \code{"complete"}.}

\item{clustering_distance}{Character. Distance metric. Default \code{"euclidean"}.}

\item{low_color}{Character. Color for low values. Default \code{"#f7f7f7"}.}

\item{high_color}{Character. Color for high values. Default \code{"#ca0020"}.}

\item{font_size}{Numeric. Base font size. Default 12.}

\item{dendro_line_size}{Numeric. Dendrogram line width. Default 0.5.}

\item{custom_title}{Optional plot title.}
}
\value{
A \code{ggplot2} or \code{patchwork} object.
}
\description{
Creates a heatmap showing mean taxa contributions across pathways/functions,
with optional clustering and pathway annotations.
}
\examples{
\donttest{
agg <- data.frame(
  sample = rep(c("S1", "S2"), each = 6),
  function_id = rep(rep(c("K00001", "K00002", "K00003"), each = 2), 2),
  taxon_label = rep(c("Genus_A", "Genus_B"), 6),
  contribution = runif(12)
)
p <- taxa_contribution_heatmap(agg)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/taxa_contribution.R
\name{read_strat_file}
\alias{read_strat_file}
\title{Read PICRUSt2 stratified abundance file}
\usage{
read_strat_file(file = NULL, data = NULL)
}
\arguments{
\item{file}{Path to the stratified file (.tsv, .txt, or .csv).}

\item{data}{A data.frame already loaded from the stratified file.
If both \code{file} and \code{data} are provided, \code{data} is used.}
}
\value{
A tidy data.frame with columns: \code{function_id}, \code{taxon},
  \code{sample}, \code{abundance}.
}
\description{
Parses the \code{pred_metagenome_strat.tsv} file produced by PICRUSt2's
\code{--strat_out} flag and converts the wide format to tidy long format.
}
\details{
The stratified file has function IDs in the first column, sequence/taxon
IDs in the second column, and sample abundances in the remaining columns
(wide format). This function pivots to long format for downstream analysis.
}
\examples{
\donttest{
# From a data.frame
strat_df <- data.frame(
  `function` = c("K00001", "K00001", "K00002"),
  sequence = c("ASV1", "ASV2", "ASV1"),
  S1 = c(10, 5, 8),
  S2 = c(12, 3, 7),
  check.names = FALSE
)
result <- read_strat_file(data = strat_df)
head(result)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualize_gsea.R
\name{.build_continuous_scale}
\alias{.build_continuous_scale}
\title{Internal: build a continuous ggplot2 scale layer from colors or ggplot2 scale}
\usage{
.build_continuous_scale(
  aes = c("fill", "color"),
  scale = NULL,
  diverging = FALSE,
  midpoint = NULL,
  name = NULL
)
}
\description{
Internal: build a continuous ggplot2 scale layer from colors or ggplot2 scale
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/import_MicrobiomeAnalyst_daa_results.R
\name{import_MicrobiomeAnalyst_daa_results}
\alias{import_MicrobiomeAnalyst_daa_results}
\title{Import Differential Abundance Analysis (DAA) results from MicrobiomeAnalyst}
\usage{
import_MicrobiomeAnalyst_daa_results(
  file_path = NULL,
  data = NULL,
  method = "MicrobiomeAnalyst",
  group_levels = c("control", "treatment")
)
}
\arguments{
\item{file_path}{a character string specifying the path to the CSV file containing the DAA results from MicrobiomeAnalyst. If this parameter is NULL and no data frame is provided, an error will be thrown. Default is NULL.}

\item{data}{a data frame containing the DAA results from MicrobiomeAnalyst. If this parameter is NULL and no file path is provided, an error will be thrown. Default is NULL.}

\item{method}{a character string specifying the method used for the DAA. This will be added as a new column in the returned data frame. Default is "MicrobiomeAnalyst".}

\item{group_levels}{a character vector specifying the group levels for the DAA. This will be added as new columns in the returned data frame. Default is c("control", "treatment").}
}
\value{
a data frame containing the DAA results from MicrobiomeAnalyst with additional columns for the method and group levels.
}
\description{
This function imports DAA results from an external platform such as MicrobiomeAnalyst. It can be used to compare the results obtained from different platforms.
}
\examples{
\dontrun{
# Assuming you have a CSV file named "DAA_results.csv" in your current directory
daa_results <- import_MicrobiomeAnalyst_daa_results(file_path = "DAA_results.csv")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathway_daa.R
\name{calculate_abundance_stats}
\alias{calculate_abundance_stats}
\title{Helper function to calculate abundance statistics for differential analysis}
\usage{
calculate_abundance_stats(abundance, metadata, group, features, group1, group2)
}
\arguments{
\item{abundance}{A matrix or data frame with features as rows and samples as columns}

\item{metadata}{A data frame containing sample information}

\item{group}{Character string specifying the group column name in metadata}

\item{features}{Character vector of feature names to calculate statistics for}

\item{group1}{Character string specifying the first group name}

\item{group2}{Character string specifying the second group name}
}
\value{
A data frame with columns:
  \item{feature}{Feature identifier}
  \item{mean_rel_abundance_group1}{Mean relative abundance for group1}
  \item{sd_rel_abundance_group1}{Standard deviation of relative abundance for group1}
  \item{mean_rel_abundance_group2}{Mean relative abundance for group2}
  \item{sd_rel_abundance_group2}{Standard deviation of relative abundance for group2}
  \item{log2_fold_change}{Log2 fold change (group2/group1)}
}
\description{
This function calculates mean relative abundance, standard deviation, and log2 fold change
for each feature between two groups.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_utils.R
\name{require_column}
\alias{require_column}
\title{Require a column exists in a data frame}
\usage{
require_column(df, col, param_name = deparse(substitute(df)))
}
\arguments{
\item{df}{Data frame to check}

\item{col}{Column name to require}

\item{param_name}{Name of the data frame parameter (for error message)}
}
\value{
Invisible TRUE if column exists, otherwise stops with error
}
\description{
Simple check that a single column exists. For checking multiple columns,
use validate_dataframe() with required_cols parameter.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathway_heatmap.R
\name{pathway_heatmap}
\alias{pathway_heatmap}
\title{Create pathway heatmap with support for multiple grouping variables}
\arguments{
\item{abundance}{A matrix or data frame of pathway abundance data, where columns
correspond to samples and rows correspond to pathways. Must contain at least
two samples.}

\item{metadata}{A data frame of metadata, where each row corresponds to a sample
and each column corresponds to a metadata variable.}

\item{group}{A character string specifying the column name in the metadata data frame
that contains the primary group variable. Must contain at least two groups.}

\item{secondary_groups}{A character vector specifying additional grouping variables
for creating nested faceted heatmaps. If NULL, only the primary group will be used.
These variables will be used as secondary levels in the faceting hierarchy.}

\item{colors}{A vector of colors used for the background of the facet labels in the
heatmap. If NULL or not provided, a default color set is used for the facet strips.}

\item{font_size}{A numeric value specifying the font size for the heatmap.}

\item{show_row_names}{A logical value indicating whether to show row names in the heatmap.}

\item{show_legend}{A logical value indicating whether to show the legend in the heatmap.}

\item{custom_theme}{A custom theme for the heatmap.}

\item{low_color}{A character string specifying the color for low values in the heatmap gradient. Default is "#0571b0" (blue).}

\item{mid_color}{A character string specifying the color for middle values in the heatmap gradient. Default is "white".}

\item{high_color}{A character string specifying the color for high values in the heatmap gradient. Default is "#ca0020" (red).}

\item{cluster_rows}{A logical value indicating whether to cluster rows (pathways). Default is FALSE.}

\item{cluster_cols}{A logical value indicating whether to cluster columns (samples). Default is FALSE.}

\item{clustering_method}{A character string specifying the clustering method. Options: "complete", "average", "single", "ward.D", "ward.D2", "mcquitty", "median", "centroid". Default is "complete".}

\item{clustering_distance}{A character string specifying the distance metric. Options: "euclidean", "maximum", "manhattan", "canberra", "binary", "minkowski", "correlation", "spearman". Default is "euclidean".}

\item{dendro_line_size}{A numeric value specifying the line width of dendrogram branches. Default is 0.5.}

\item{dendro_labels}{A logical value indicating whether to show dendrogram labels. Default is FALSE.}

\item{facet_by}{\strong{[Deprecated]} A character string specifying an additional grouping variable for creating faceted heatmaps.
This parameter is deprecated and will be removed in future versions. Use \code{secondary_groups} instead.}

\item{colorbar_title}{A character string specifying the title for the color bar. Default is "Z Score".}

\item{colorbar_position}{A character string specifying the position of the color bar. Options: "right", "left", "top", "bottom". Default is "right".}

\item{colorbar_width}{A numeric value specifying the width of the color bar. Default is 0.6.}

\item{colorbar_height}{A numeric value specifying the height of the color bar. Default is 9.}

\item{colorbar_breaks}{An optional numeric vector specifying custom breaks for the color bar.}
}
\value{
A ggplot heatmap object representing the heatmap of the predicted functional
  pathway abundance data.
}
\description{
This function creates a heatmap of the predicted functional pathway abundance data
with support for single or multiple grouping variables. The function first performs
z-score normalization on the abundance data, then converts it to a long format and
orders the samples based on the grouping information. The heatmap supports nested
faceting for multiple grouping variables and is created using the `ggplot2` library.
}
\examples{
\donttest{
library(ggpicrust2)
library(ggh4x)
library(dplyr)
library(tidyr)
library(tibble)
library(magrittr)

# Create example functional pathway abundance data
kegg_abundance_example <- matrix(rnorm(30), nrow = 3, ncol = 10)
colnames(kegg_abundance_example) <- paste0("Sample", 1:10)
rownames(kegg_abundance_example) <- c("PathwayA", "PathwayB", "PathwayC")

# Create example metadata
metadata_example <- data.frame(
  sample_name = colnames(kegg_abundance_example),
  group = factor(rep(c("Control", "Treatment"), each = 5)),
  batch = factor(rep(c("Batch1", "Batch2"), times = 5))
)

# Custom colors for facet strips
custom_colors <- c("skyblue", "salmon")

# Example 1: Basic heatmap
pathway_heatmap(kegg_abundance_example, metadata_example, "group", colors = custom_colors)

# Example 2: Heatmap with row clustering
pathway_heatmap(
  abundance = kegg_abundance_example,
  metadata = metadata_example,
  group = "group",
  cluster_rows = TRUE,
  clustering_method = "complete",
  clustering_distance = "euclidean",
  dendro_line_size = 0.8
)

# Example 3: Heatmap with column clustering using correlation distance
pathway_heatmap(
  abundance = kegg_abundance_example,
  metadata = metadata_example,
  group = "group",
  cluster_cols = TRUE,
  clustering_method = "ward.D2",
  clustering_distance = "correlation"
)

# Example 4: Multi-level grouping with secondary_groups (NEW FEATURE)
pathway_heatmap(
  abundance = kegg_abundance_example,
  metadata = metadata_example,
  group = "group",
  secondary_groups = "batch",
  colors = c("lightblue", "lightcoral", "lightgreen", "lightyellow")
)

# Example 5: Custom colorbar settings
pathway_heatmap(
  abundance = kegg_abundance_example,
  metadata = metadata_example,
  group = "group",
  colorbar_title = "Expression Level",
  colorbar_position = "bottom",
  colorbar_width = 8,
  colorbar_height = 0.8,
  colorbar_breaks = c(-2, -1, 0, 1, 2)
)

# Example 6: Advanced heatmap with clustering and custom aesthetics
pathway_heatmap(
  abundance = kegg_abundance_example,
  metadata = metadata_example,
  group = "group",
  cluster_rows = TRUE,
  cluster_cols = FALSE,  # Don't cluster columns to preserve group order
  clustering_method = "average",
  clustering_distance = "manhattan",
  dendro_line_size = 1.0,
  low_color = "#053061",     # Dark blue
  mid_color = "#f7f7f7",     # Light gray
  high_color = "#67001f",    # Dark red
  colorbar_title = "Z-Score",
  colorbar_position = "left"
)

# Use real dataset
data("metacyc_abundance")
data("metadata")
metacyc_daa_results_df <- pathway_daa(
  abundance = metacyc_abundance \%>\% column_to_rownames("pathway"),
  metadata = metadata,
  group = "Environment",
  daa_method = "LinDA"
)
annotated_metacyc_daa_results_df <- pathway_annotation(
  pathway = "MetaCyc",
  daa_results_df = metacyc_daa_results_df,
  ko_to_kegg = FALSE
)
feature_with_p_0.05 <- metacyc_daa_results_df \%>\% filter(p_adjust < 0.05)

# Example 7: Real data with hierarchical clustering
pathway_heatmap(
  abundance = metacyc_abundance \%>\%
    right_join(
      annotated_metacyc_daa_results_df \%>\%
      select(all_of(c("feature","description"))),
      by = c("pathway" = "feature")
    ) \%>\%
    filter(pathway \%in\% feature_with_p_0.05$feature) \%>\%
    select(-"pathway") \%>\%
    filter(!is.na(description)) \%>\%
    distinct(description, .keep_all = TRUE) \%>\%
    column_to_rownames("description"),
  metadata = metadata,
  group = "Environment",
  cluster_rows = TRUE,
  clustering_method = "ward.D2",
  clustering_distance = "correlation",
  colors = custom_colors,
  low_color = "#2166ac",  # Custom blue for low values
  mid_color = "#f7f7f7",  # Light gray for mid values
  high_color = "#b2182b", # Custom red for high values
  colorbar_title = "Standardized Abundance"
)

# Example 8: Multiple grouping variables (NEW FEATURE)
# Create extended metadata with additional grouping variables
metadata_extended <- metadata_example \%>\%
  mutate(
    sex = factor(rep(c("Male", "Female"), times = 5)),
    age_group = factor(rep(c("Young", "Old"), each = 5))
  )

# Multi-level grouping with three variables
pathway_heatmap(
  abundance = kegg_abundance_example,
  metadata = metadata_extended,
  group = "group",                    # Primary grouping
  secondary_groups = c("batch", "sex"), # Secondary groupings
  colors = c("lightblue", "lightcoral")
)

# Example 9: Migration from facet_by to secondary_groups
# OLD WAY (deprecated, will show warning):
# pathway_heatmap(abundance, metadata, group = "Environment", facet_by = "Group")

# NEW WAY (recommended):
# pathway_heatmap(abundance, metadata, group = "Environment", secondary_groups = "Group")

# Example 10: Real data with multiple grouping variables
pathway_heatmap(
  abundance = metacyc_abundance \%>\%
    right_join(
      annotated_metacyc_daa_results_df \%>\%
      select(all_of(c("feature","description"))),
      by = c("pathway" = "feature")
    ) \%>\%
    filter(pathway \%in\% feature_with_p_0.05$feature) \%>\%
    select(-"pathway") \%>\%
    filter(!is.na(description)) \%>\%
    distinct(description, .keep_all = TRUE) \%>\%
    column_to_rownames("description"),
  metadata = metadata,
  group = "Environment",              # Primary: Pro-survival vs others
  secondary_groups = "Group",         # Secondary: Broad Institute vs Jackson Labs
  cluster_rows = TRUE,
  clustering_method = "ward.D2",
  clustering_distance = "correlation"
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/color_themes.R
\name{smart_color_selection}
\alias{smart_color_selection}
\title{Smart Color Selection}
\usage{
smart_color_selection(
  n_groups,
  has_pathway_class = FALSE,
  data_type = "abundance",
  accessibility_mode = FALSE
)
}
\arguments{
\item{n_groups}{Number of groups in the data}

\item{has_pathway_class}{Whether pathway class information is available}

\item{data_type}{Type of data ("abundance", "pvalue", "foldchange")}

\item{accessibility_mode}{Whether to use accessibility-friendly colors}
}
\value{
A list with suggested theme and colors
}
\description{
Intelligently selects colors based on data characteristics
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/color_themes.R
\name{preview_color_theme}
\alias{preview_color_theme}
\title{Preview Color Theme}
\usage{
preview_color_theme(theme_name = "default", save_plot = FALSE, filename = NULL)
}
\arguments{
\item{theme_name}{Character string specifying the theme name}

\item{save_plot}{Whether to save the preview plot}

\item{filename}{Filename for saved plot}
}
\value{
A ggplot object showing the color preview
}
\description{
Creates a visual preview of a color theme
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualize_gsea.R
\name{create_network_plot}
\alias{create_network_plot}
\title{Create network visualization of GSEA results}
\usage{
create_network_plot(
  gsea_results,
  similarity_measure = "jaccard",
  similarity_cutoff = 0.3,
  n_pathways = 20,
  layout = "fruchterman",
  node_color_by = "NES",
  edge_width_by = "similarity",
  scale = NULL
)
}
\arguments{
\item{gsea_results}{A data frame containing GSEA results from the pathway_gsea function}

\item{similarity_measure}{A character string specifying the similarity measure: "jaccard", "overlap", or "correlation"}

\item{similarity_cutoff}{A numeric value specifying the similarity threshold for filtering connections}

\item{n_pathways}{An integer specifying the number of pathways to display}

\item{layout}{A character string specifying the network layout algorithm: "fruchterman", "kamada", or "circle"}

\item{node_color_by}{A character string specifying the node color mapping: "NES", "pvalue", or "p.adjust"}

\item{edge_width_by}{A character string specifying the edge width mapping: "similarity" or "constant"}

\item{scale}{Optional palette/scale for customizing node color mapping (same conventions as visualize_gsea)}
}
\value{
A ggplot2 object
}
\description{
Create network visualization of GSEA results
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/legend_annotation_utils.R
\name{legend_annotation_utils}
\alias{legend_annotation_utils}
\title{Legend and Annotation Utilities for ggpicrust2}
\description{
This module provides enhanced legend and annotation functionality for ggpicrust2
visualizations, including intelligent p-value formatting, significance marking,
and customizable legend styling.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathway_errorbar.R
\name{pathway_errorbar}
\alias{pathway_errorbar}
\title{The function pathway_errorbar() is used to visualize the results of functional pathway differential abundance analysis as error bar plots.}
\arguments{
\item{abundance}{A data frame with row names representing pathways and column names representing samples. Each element represents the relative abundance of the corresponding pathway in the corresponding sample.}

\item{daa_results_df}{A data frame containing the results of the differential abundance analysis of the pathways, generated by the pathway_daa function. x_lab should be a column name of daa_results_df.}

\item{Group}{A data frame or a vector that assigns each sample to a group. The groups are used to color the samples in the figure.}

\item{ko_to_kegg}{A logical parameter indicating whether there was a convertion that convert ko abundance to kegg abundance.}

\item{p_values_threshold}{A numeric parameter specifying the threshold for statistical significance of differential abundance. Pathways with p-values below this threshold will be considered significant.}

\item{order}{A parameter controlling the ordering of the rows in the figure. The options are: "p_values" (order by p-values), "name" (order by pathway name), "group" (order by the group with the highest mean relative abundance), or "pathway_class" (order by the pathway category).}

\item{select}{A vector of pathway names to be included in the figure. This can be used to limit the number of pathways displayed. If NULL, all pathways will be displayed.}

\item{p_value_bar}{A logical parameter indicating whether to display a bar showing the p-value threshold for significance. If TRUE, the bar will be displayed.}

\item{colors}{A vector of colors to be used to represent the groups in the figure. Each color corresponds to a group. If NULL, colors will be selected based on the color_theme.}

\item{x_lab}{A character string to be used as the x-axis label in the figure. The default value is "description" for KOs'descriptions and "pathway_name" for KEGG pathway names.}

\item{log2_fold_change_color}{A character string specifying the color for log2 fold change bars. Default is "#87ceeb" (light blue). Can also be "auto" to use theme-based colors.}

\item{max_features}{A numeric parameter specifying the maximum number of features to display before issuing a warning. Default is 30. Set to a higher value to display more features, or Inf to disable the limit entirely.}

\item{color_theme}{A character string specifying the color theme to use. Options include: "default", "nature", "science", "cell", "nejm", "lancet", "colorblind_friendly", "viridis", "plasma", "minimal", "high_contrast", "pastel", "bold". Default is "default".}

\item{pathway_class_colors}{A vector of colors for pathway class annotations. If NULL, colors will be selected from the theme.}

\item{smart_colors}{A logical parameter indicating whether to use intelligent color selection based on data characteristics. Default is FALSE.}

\item{accessibility_mode}{A logical parameter indicating whether to use accessibility-friendly colors. Default is FALSE.}

\item{legend_position}{A character string specifying legend position. Options: "top", "bottom", "left", "right", "none". Default is "top".}

\item{legend_direction}{A character string specifying legend direction. Options: "horizontal", "vertical". Default is "horizontal".}

\item{legend_title}{A character string for legend title. If NULL, no title is displayed.}

\item{legend_title_size}{A numeric value specifying legend title font size. Default is 12.}

\item{legend_text_size}{A numeric value specifying legend text font size. Default is 10.}

\item{legend_key_size}{A numeric value specifying legend key size in cm. Default is 0.8.}

\item{legend_ncol}{A numeric value specifying number of columns in legend. If NULL, automatic layout is used.}

\item{legend_nrow}{A numeric value specifying number of rows in legend. If NULL, automatic layout is used.}

\item{pvalue_format}{A character string specifying p-value format. Options: "numeric", "scientific", "smart", "stars_only", "combined". Default is "smart".}

\item{pvalue_stars}{A logical parameter indicating whether to display significance stars. Default is TRUE.}

\item{pvalue_colors}{A logical parameter indicating whether to use color coding for significance levels. Default is FALSE.}

\item{pvalue_size}{A numeric value or "auto" for p-value text size. Default is "auto".}

\item{pvalue_angle}{A numeric value specifying p-value text angle in degrees. Default is 0.}

\item{pvalue_thresholds}{A numeric vector of significance thresholds. Default is c(0.001, 0.01, 0.05).}

\item{pvalue_star_symbols}{A character vector of star symbols for significance levels. Default is c("***", "**", "*").}

\item{pathway_class_text_size}{A numeric value or "auto" for pathway class text size. Default is "auto".}

\item{pathway_class_text_color}{A character string for pathway class text color. Use "auto" for theme-based color. Default is "black".}

\item{pathway_class_text_face}{A character string for pathway class text face. Options: "plain", "bold", "italic". Default is "bold".}

\item{pathway_class_text_angle}{A numeric value specifying pathway class text angle in degrees. Default is 0.}

\item{pathway_class_position}{A character string specifying pathway class position. Options: "left", "right", "none". Default is "left".}

\item{pathway_names_text_size}{A numeric value or "auto" for pathway names (y-axis labels) text size. Default is "auto".}
}
\value{
A ggplot2 plot showing the error bar plot of the differential abundance analysis results for the functional pathways.
The plot visualizes the differential abundance results of a specific differential abundance analysis method. The corresponding dataframe contains the results used to create the plot.
}
\description{
The function pathway_errorbar() is used to visualize the results of functional pathway differential abundance analysis as error bar plots.
}
\examples{
\dontrun{
# Example 1: Analyzing KEGG pathway abundance
metadata <- read_delim(
  "path/to/your/metadata.txt",
  delim = "\t",
  escape_double = FALSE,
  trim_ws = TRUE
)

# data(metadata)

kegg_abundance <- ko2kegg_abundance(
  "path/to/your/pred_metagenome_unstrat.tsv"
)

# data(kegg_abundance)

# Please change group to "your_group_column" if you are not using example dataset
group <- "Environment"

daa_results_df <- pathway_daa(
  abundance = kegg_abundance,
  metadata = metadata,
  group = group,
  daa_method = "ALDEx2",
  select = NULL,
  reference = NULL
)

# Please check the unique(daa_results_df$method) and choose one
daa_sub_method_results_df <- daa_results_df[daa_results_df$method
== "ALDEx2_Welch's t test", ]

daa_annotated_sub_method_results_df <- pathway_annotation(
  pathway = "KO",
  daa_results_df = daa_sub_method_results_df,
  ko_to_kegg = TRUE
)

# Please change Group to metadata$your_group_column if you are not using example dataset
Group <- metadata$Environment

p <- pathway_errorbar(
  abundance = kegg_abundance,
  daa_results_df = daa_annotated_sub_method_results_df,
  Group = Group,
  p_values_threshold = 0.05,
  order = "pathway_class",
  select = daa_annotated_sub_method_results_df \%>\%
  arrange(p_adjust) \%>\%
  slice(1:20) \%>\%
  select("feature") \%>\% pull(),
  ko_to_kegg = TRUE,
  p_value_bar = TRUE,
  colors = NULL,
  x_lab = "pathway_name",
  log2_fold_change_color = "#FF5733" # Custom color for log2 fold change bars
)

# Example 2: Analyzing EC, MetaCyc, KO without conversions
metadata <- read_delim(
  "path/to/your/metadata.txt",
  delim = "\t",
  escape_double = FALSE,
  trim_ws = TRUE
)
# data(metadata)

metacyc_abundance <- read.delim("path/to/your/metacyc_abundance.tsv")

# data(metacyc_abundance)

group <- "Environment"

daa_results_df <- pathway_daa(
  abundance = metacyc_abundance \%>\% column_to_rownames("pathway"),
  metadata = metadata,
  group = group,
  daa_method = "LinDA",
  select = NULL,
  reference = NULL
)


daa_annotated_results_df <- pathway_annotation(
  pathway = "MetaCyc",
  daa_results_df = daa_results_df,
  ko_to_kegg = FALSE
)

Group <- metadata$Environment

p <- pathway_errorbar(
  abundance = metacyc_abundance \%>\% column_to_rownames("pathway"),
  daa_results_df = daa_annotated_results_df,
  Group = Group,
  p_values_threshold = 0.05,
  order = "group",
  select = NULL,
  ko_to_kegg = FALSE,
  p_value_bar = TRUE,
  colors = NULL,
  x_lab = "description",
  log2_fold_change_color = "#006400" # Dark green for log2 fold change bars
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gsea_pathway_annotation.R
\name{gsea_pathway_annotation}
\alias{gsea_pathway_annotation}
\title{Annotate GSEA results with pathway information}
\usage{
gsea_pathway_annotation(gsea_results, pathway_type = "KEGG")
}
\arguments{
\item{gsea_results}{A data frame containing GSEA results from the pathway_gsea function}

\item{pathway_type}{A character string specifying the pathway type: "KEGG", "MetaCyc", or "GO"}
}
\value{
A data frame with annotated GSEA results
}
\description{
This function adds pathway annotations to GSEA results, including pathway names,
descriptions, and classifications.
}
\examples{
\dontrun{
# Load example data
data(ko_abundance)
data(metadata)

# Prepare abundance data
abundance_data <- as.data.frame(ko_abundance)
rownames(abundance_data) <- abundance_data[, "#NAME"]
abundance_data <- abundance_data[, -1]

# Run GSEA analysis (using camera method - recommended)
gsea_results <- pathway_gsea(
  abundance = abundance_data,
  metadata = metadata,
  group = "Environment",
  pathway_type = "KEGG",
  method = "camera"
)

# Annotate results
annotated_results <- gsea_pathway_annotation(
  gsea_results = gsea_results,
  pathway_type = "KEGG"
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathway_annotation.R
\name{safe_extract}
\alias{safe_extract}
\title{Safely Extract Elements from a List}
\usage{
safe_extract(list, field, index = 1)
}
\arguments{
\item{list}{A list object from which to extract elements}

\item{field}{The name of the field to extract from the list}

\item{index}{The index position to extract from the field. Default is 1}
}
\value{
The extracted element if successful, NA if extraction fails
}
\description{
Safely extracts elements from a list, returning NA if the extraction fails
}
\examples{
# Create a sample list
my_list <- list(
  a = list(x = 1:3),
  b = list(y = 4:6)
)

# Extract existing element
safe_extract(my_list, "a", 1)

# Extract non-existing element (returns NA)
safe_extract(my_list, "c", 1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathway_gsea.R
\name{run_limma_gsea}
\alias{run_limma_gsea}
\title{Run limma-based gene set analysis (camera/fry)}
\usage{
run_limma_gsea(
  abundance_mat,
  metadata,
  group,
  covariates = NULL,
  contrast = NULL,
  gene_sets,
  method = "camera",
  inter.gene.cor = 0.01,
  min_size = 5,
  max_size = 500,
  p.adjust.method = "BH"
)
}
\arguments{
\item{abundance_mat}{A matrix of abundance data with features as rows and samples as columns}

\item{metadata}{A data frame containing sample metadata}

\item{group}{A character string specifying the grouping variable column name}

\item{covariates}{A character vector of covariate column names (optional)}

\item{contrast}{Contrast specification for multi-group comparisons (optional)}

\item{gene_sets}{A named list of gene sets (pathway -> gene IDs)}

\item{method}{Either "camera" or "fry"}

\item{inter.gene.cor}{Inter-gene correlation for camera (default 0.01)}

\item{min_size}{Minimum gene set size}

\item{max_size}{Maximum gene set size}

\item{p.adjust.method}{P-value adjustment method}
}
\value{
A data frame containing gene set analysis results
}
\description{
This internal function implements limma's camera and fry methods for gene set
enrichment analysis with support for covariates.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/color_themes.R
\name{color_themes}
\alias{color_themes}
\title{Color Theme System for ggpicrust2}
\description{
This module provides a comprehensive color theme system for ggpicrust2 visualizations,
including journal-specific themes, colorblind-friendly palettes, and intelligent
color selection based on data characteristics.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_utils.R
\name{calculate_pseudocount}
\alias{calculate_pseudocount}
\title{Calculate data-driven pseudocount for log transformation}
\usage{
calculate_pseudocount(values)
}
\arguments{
\item{values}{Numeric vector of abundance values}
}
\value{
Pseudocount value (half of minimum non-zero value, or 1e-6 fallback)
}
\description{
Calculates a pseudocount based on the data to avoid log(0) issues.
Uses half of the minimum non-zero value to ensure the pseudocount
is smaller than any real value in the data.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathway_heatmap.R
\name{create_dendrogram}
\alias{create_dendrogram}
\title{Create dendrogram plot from hierarchical clustering}
\usage{
create_dendrogram(
  hclust_obj,
  dendro_line_size = 0.5,
  dendro_labels = FALSE,
  horizontal = FALSE
)
}
\arguments{
\item{hclust_obj}{An hclust object from hierarchical clustering}

\item{dendro_line_size}{Line width for dendrogram branches}

\item{dendro_labels}{Whether to show labels on dendrogram}

\item{horizontal}{Whether to create horizontal dendrogram}
}
\value{
A ggplot dendrogram
}
\description{
Create dendrogram plot from hierarchical clustering
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/daa_annotated_results_df.R
\docType{data}
\name{daa_annotated_results_df}
\alias{daa_annotated_results_df}
\title{Differentially Abundant Analysis Results with Annotation}
\format{
A data frame with 10 variables:
\describe{
  \item{adj_method}{Method used for adjusting p-values.}
  \item{feature}{Feature being tested.}
  \item{group1}{One group in the comparison.}
  \item{group2}{The other group in the comparison.}
  \item{method}{Statistical test used.}
  \item{p_adjust}{Adjusted p-value.}
  \item{p_values}{P-values from the statistical test.}
  \item{pathway_class}{Class of the pathway.}
  \item{pathway_description}{Description of the pathway.}
  \item{pathway_map}{Map of the pathway.}
  \item{pathway_name}{Name of the pathway.}
}
}
\source{
From ggpicrust2 package demonstration.
}
\usage{
daa_annotated_results_df
}
\description{
This is a result dataset after processing 'kegg_abundance' through
the 'pathway_daa' with the LinDA method and further annotation with 'pathway_annotation'.
}
\references{
Douglas GM, Maffei VJ, Zaneveld J, Yurgel SN, Brown JR, Taylor CM, Huttenhower C, Langille MGI. PICRUSt2 for prediction of metagenome functions. Nat Biotechnol. 2020.
}
\keyword{datasets}

\name{metacyc_to_ec_reference}
\alias{metacyc_to_ec_reference}
\docType{data}
\title{
MetaCyc Pathway to EC Number Mapping Dataset
}
\description{
A reference dataset mapping MetaCyc pathway identifiers to their associated
Enzyme Commission (EC) numbers. Used internally by ggpicrust2 for MetaCyc
pathway analysis, enabling the mapping between EC-level functional predictions
and MetaCyc pathways.
}
\usage{data("metacyc_to_ec_reference")}
\format{
  A data frame with 575 observations and the following columns:
  \describe{
    \item{\code{pathway}}{Character. MetaCyc pathway identifier (e.g., "1CMET2-PWY")}
    \item{\code{ec_numbers}}{Character. Semicolon-separated list of EC numbers associated with the pathway}
  }
}
\source{
MetaCyc database (\url{https://metacyc.org})
}
\examples{
data("metacyc_to_ec_reference")
head(metacyc_to_ec_reference)

# Count EC numbers per pathway
ec_counts <- sapply(strsplit(metacyc_to_ec_reference$ec_numbers, ";"), length)
summary(ec_counts)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pathway_volcano.R
\name{pathway_volcano}
\alias{pathway_volcano}
\title{Volcano Plot for Pathway Differential Abundance Analysis}
\usage{
pathway_volcano(
  daa_results,
  fc_col = "log2_fold_change",
  p_col = "p_adjust",
  label_col = "pathway_name",
  fc_threshold = 1,
  p_threshold = 0.05,
  label_top_n = 10,
  point_size = 2,
  point_alpha = 0.6,
  colors = c(Down = "#3182bd", `Not Significant` = "grey60", Up = "#de2d26"),
  show_threshold_lines = TRUE,
  title = "Volcano Plot: Pathway Differential Abundance",
  x_lab = "log2 Fold Change",
  y_lab = "-log10(Adjusted P-value)"
)
}
\arguments{
\item{daa_results}{A data frame containing differential abundance analysis results,
typically from \code{\link{pathway_daa}}. Must contain columns for fold change,
p-values, and optionally pathway names.}

\item{fc_col}{Character string specifying the column name for log2 fold change values.
Default is "log2_fold_change". Legacy name "log2FoldChange" is also accepted.}

\item{p_col}{Character string specifying the column name for adjusted p-values.
Default is "p_adjust".}

\item{label_col}{Character string specifying the column name for pathway labels.
Default is "pathway_name". If NULL, no labels will be shown.}

\item{fc_threshold}{Numeric. Absolute fold change threshold for significance.
Default is 1 (2-fold change). Pathways with |log2FC| > fc_threshold are considered
biologically significant.}

\item{p_threshold}{Numeric. P-value threshold for statistical significance.
Default is 0.05.}

\item{label_top_n}{Integer. Number of top significant pathways to label.
Default is 10. Set to 0 to disable labels.}

\item{point_size}{Numeric. Size of points in the plot. Default is 2.}

\item{point_alpha}{Numeric. Transparency of points (0-1). Default is 0.6.}

\item{colors}{Named character vector with colors for "Down", "Not Significant", and "Up".
Default uses blue for down-regulated, grey for non-significant, and red for up-regulated.}

\item{show_threshold_lines}{Logical. Whether to show dashed lines for fold change
and p-value thresholds. Default is TRUE.}

\item{title}{Character string for plot title. Default is
"Volcano Plot: Pathway Differential Abundance".}

\item{x_lab}{Character string for x-axis label. Default is "log2 Fold Change".}

\item{y_lab}{Character string for y-axis label. Default is "-log10(Adjusted P-value)".}
}
\value{
A ggplot2 object that can be further customized or saved.
}
\description{
Creates a volcano plot to visualize the results of differential abundance analysis,
showing both statistical significance (-log10 p-value) and effect size (log2 fold change).
}
\details{
The volcano plot is a scatter plot that shows statistical significance (y-axis)
versus fold change (x-axis). Points are colored by significance:
\itemize{
  \item \strong{Up (red)}: Pathways with p < p_threshold AND log2FC > fc_threshold
  \item \strong{Down (blue)}: Pathways with p < p_threshold AND log2FC < -fc_threshold
  \item \strong{Not Significant (grey)}: All other pathways
}

The function automatically labels the top N most significant pathways using
\code{ggrepel::geom_text_repel()} if the ggrepel package is installed.
}
\examples{
\dontrun{
library(ggpicrust2)
library(tibble)

# Load example data
data("ko_abundance")
data("metadata")

# Convert KO to KEGG abundance
kegg_abundance <- ko2kegg_abundance(data = ko_abundance)

# Run differential abundance analysis
daa_results <- pathway_daa(
  abundance = kegg_abundance,
  metadata = metadata,
  group = "Environment",
  daa_method = "LinDA"
)

# Annotate results
daa_annotated <- pathway_annotation(
  pathway = "KO",
  ko_to_kegg = TRUE,
  daa_results_df = daa_results
)

# Create volcano plot
volcano_plot <- pathway_volcano(daa_annotated)
print(volcano_plot)

# Customize the plot
volcano_plot <- pathway_volcano(
  daa_annotated,
  fc_threshold = 0.5,
  p_threshold = 0.01,
  label_top_n = 15,
  colors = c("Down" = "darkblue", "Not Significant" = "lightgrey", "Up" = "darkred")
)
}

}
\seealso{
\code{\link{pathway_daa}}, \code{\link{pathway_annotation}},
  \code{\link{pathway_errorbar}}
}

\name{ec_reference}
\alias{ec_reference}
\docType{data}
\title{
EC Number Reference Dataset
}
\description{
A reference dataset mapping Enzyme Commission (EC) numbers to their descriptions.
Used internally by ggpicrust2 for annotating enzyme-level functional predictions.
}
\usage{data("ec_reference")}
\format{
  A data frame with 8405 observations and the following columns:
  \describe{
    \item{\code{id}}{Character. EC number in the format "EC:X.X.X.X"}
    \item{\code{description}}{Character. Human-readable enzyme name/description}
  }
}
\source{
KEGG REST API (\url{https://rest.kegg.jp})
}
\examples{
data("ec_reference")
head(ec_reference)
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/visualize_gsea.R
\name{create_heatmap_plot}
\alias{create_heatmap_plot}
\title{Create heatmap visualization of GSEA results}
\usage{
create_heatmap_plot(
  gsea_results,
  abundance,
  metadata,
  group,
  n_pathways = 20,
  cluster_rows = TRUE,
  cluster_columns = TRUE,
  show_rownames = TRUE,
  annotation_colors = NULL,
  col_fun = NULL
)
}
\arguments{
\item{gsea_results}{A data frame containing GSEA results from the pathway_gsea function}

\item{abundance}{A data frame containing the original abundance data}

\item{metadata}{A data frame containing sample metadata}

\item{group}{A character string specifying the column name in metadata that contains the grouping variable}

\item{n_pathways}{An integer specifying the number of pathways to display}

\item{cluster_rows}{A logical value indicating whether to cluster rows}

\item{cluster_columns}{A logical value indicating whether to cluster columns}

\item{show_rownames}{A logical value indicating whether to show row names}

\item{annotation_colors}{A list of colors for annotations}

\item{col_fun}{A color function (e.g., circlize::colorRamp2) to control the main heatmap colors (optional)}
}
\value{
A ComplexHeatmap object
}
\description{
Create heatmap visualization of GSEA results
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compare_gsea_daa.R
\name{compare_gsea_daa}
\alias{compare_gsea_daa}
\title{Compare GSEA and DAA results}
\usage{
compare_gsea_daa(
  gsea_results,
  daa_results,
  plot_type = "venn",
  p_threshold = 0.05
)
}
\arguments{
\item{gsea_results}{A data frame containing GSEA results from the pathway_gsea function}

\item{daa_results}{A data frame containing DAA results from the pathway_daa function}

\item{plot_type}{A character string specifying the visualization type: "venn", "upset", or "scatter"}

\item{p_threshold}{A numeric value specifying the significance threshold}
}
\value{
A list with two elements: \code{plot} (a ggplot2 object, or an
  UpSetR object when \code{plot_type = "upset"} and UpSetR is installed)
  and \code{results} (a named list with the overlap, GSEA-only, and
  DAA-only pathway sets plus their counts).
}
\description{
This function compares the results from Gene Set Enrichment Analysis (GSEA) and
Differential Abundance Analysis (DAA) to identify similarities and differences.
}
\examples{
\dontrun{
# Load example data
data(ko_abundance)
data(metadata)

# Prepare abundance data
abundance_data <- as.data.frame(ko_abundance)
rownames(abundance_data) <- abundance_data[, "#NAME"]
abundance_data <- abundance_data[, -1]

# Run GSEA analysis (using camera method - recommended)
gsea_results <- pathway_gsea(
  abundance = abundance_data,
  metadata = metadata,
  group = "Environment",
  pathway_type = "KEGG",
  method = "camera"
)

# Run DAA analysis
daa_results <- pathway_daa(
  abundance = abundance_data,
  metadata = metadata,
  group = "Environment"
)

# Compare results
comparison <- compare_gsea_daa(
  gsea_results = gsea_results,
  daa_results = daa_results,
  plot_type = "venn"
)
}
}
